package im.zego.effects.sdkmanager;

import android.content.Context;
import android.util.Log;

import java.util.ArrayList;

import im.zego.effects.ZegoEffects;
import im.zego.effects.entity.ZegoEffectsBigEyesParam;
import im.zego.effects.entity.ZegoEffectsEyesBrighteningParam;
import im.zego.effects.entity.ZegoEffectsFaceLiftingParam;
import im.zego.effects.entity.ZegoEffectsLongChinParam;
import im.zego.effects.entity.ZegoEffectsNoseNarrowingParam;
import im.zego.effects.entity.ZegoEffectsRosyParam;
import im.zego.effects.entity.ZegoEffectsSharpenParam;
import im.zego.effects.entity.ZegoEffectsSmallMouthParam;
import im.zego.effects.entity.ZegoEffectsSmoothParam;
import im.zego.effects.entity.ZegoEffectsTeethWhiteningParam;
import im.zego.effects.entity.ZegoEffectsVideoFrameParam;
import im.zego.effects.entity.ZegoEffectsWhitenParam;
import im.zego.effects.enums.ZegoEffectsVideoFrameFormat;

public class EffectsSDKManager {

    public String TAG = "EffectsSDKManager";
    public static ZegoEffects zegoEffects;
    private static volatile EffectsSDKManager singleton = null;
    private int effectVersion = 0;

    public static EffectsSDKManager getInstance() {
        if (singleton == null) {
            synchronized (EffectsSDKManager.class) {
                if (singleton == null) {
                    singleton = new EffectsSDKManager();
                }
            }
        }
        return singleton;
    }


    /**
     * 初始化环境
     *
     * @param context
     * @param aiModeInfoList   ai模型绝对路径列表
     * @param resourceInfoList 资源绝对路径列表，美白，红润，白牙，挂件等都需要使用
     */
    public void initEvn(Context context, ArrayList<String> aiModeInfoList, ArrayList<String> resourceInfoList) {
        Log.v(TAG, "initEvn");
        ZegoEffects.setModels(aiModeInfoList);
        ZegoEffects.setResources(resourceInfoList);

        zegoEffects = ZegoEffects.create(LicenseConstants.EFFECTS_LICENSE, context);

        //开启部分美白的功能
        openAllBeautifyEffects();
    }

    public void onStart() {
        zegoEffects.initEnv(720, 1280);
    }

    public void onStop() {
        zegoEffects.uninitEnv();
    }

    public int gainProcessedTextureID(int textureID, int width, int height) {
        ZegoEffectsVideoFrameParam param = new ZegoEffectsVideoFrameParam();
        param.format = ZegoEffectsVideoFrameFormat.BGRA32;
        param.width = width;
        param.height = height;

        int processedTextureID = zegoEffects.processTexture(textureID, param);

        return processedTextureID;
    }

    /**
     * 一键开启所有美白功能,版本更新可能增加新功能接口，注意版本更新
     */
    private void openAllBeautifyEffects() {
        enableSmooth(true);
        enableWhiten(true);
        enableRosy(true);
        enableSharpen(true);
        enableFaceLifting(true);
        enableBigEye(true);
        enableEyesBrightening(true);
        enableLongChin(true);
        enableSmallMouth(true);
        enableTeethWhitening(true);
        enableNoseNarrowing(true);
    }

    /**
     * 是否开启美白
     *
     * @param enable
     */
    public void enableWhiten(boolean enable) {
        Log.v(TAG, "enableWhiten :enable" + enable);
        zegoEffects.enableWhiten(enable);
    }

    /**
     * 设置美白参数
     *
     * @param whitenEffectStrength
     */
    public void setWhitenParam(int whitenEffectStrength) {
        Log.v(TAG, "setWhitenParam :whitenEffectStrength" + whitenEffectStrength);
        ZegoEffectsWhitenParam zegoEffectsWhitenParam = new ZegoEffectsWhitenParam();
        zegoEffectsWhitenParam.intensity = whitenEffectStrength;
        zegoEffects.setWhitenParam(zegoEffectsWhitenParam);
    }

    /**
     * 开启磨皮
     *
     * @param enable
     */
    public void enableSmooth(boolean enable) {
        Log.v(TAG, "enableSmooth :enable" + enable);
        zegoEffects.enableSmooth(enable);
    }

    /**
     * 设置磨皮参数
     *
     * @param smoothEffectStrength
     */
    public void setSmoothParam(int smoothEffectStrength) {
        Log.v(TAG, "setSmoothParam :smoothEffectStrength" + smoothEffectStrength);
        ZegoEffectsSmoothParam zegoEffectsSmoothParam = new ZegoEffectsSmoothParam();
        zegoEffectsSmoothParam.intensity = smoothEffectStrength;
        zegoEffects.setSmoothParam(zegoEffectsSmoothParam);
    }

    /**
     * 是否开启红润
     *
     * @param enable
     */
    public void enableRosy(boolean enable) {
        Log.v(TAG, "enableRosy :enable" + enable);
        zegoEffects.enableRosy(enable);
    }

    /**
     * 设置红润参数
     *
     * @param rosyEffectStrength
     */
    public void setRosyParam(int rosyEffectStrength) {
        Log.v(TAG, "setRosyParam :rosyEffectStrength" + rosyEffectStrength);
        ZegoEffectsRosyParam zegoEffectsRosyParam = new ZegoEffectsRosyParam();
        zegoEffectsRosyParam.intensity = rosyEffectStrength;
        zegoEffects.setRosyParam(zegoEffectsRosyParam);
    }

    /**
     * 是否开启锐化
     *
     * @param enable
     */
    public void enableSharpen(boolean enable) {
        Log.v(TAG, "enableSharpen :enable" + enable);
        zegoEffects.enableSharpen(enable);
    }

    /**
     * 设置锐化参数
     *
     * @param sharpenEffectStrength
     */
    public void setSharpenParam(int sharpenEffectStrength) {
        Log.v(TAG, "setSharpenParam :sharpenEffectStrength" + sharpenEffectStrength);
        ZegoEffectsSharpenParam zegoEffectsSharpenParam = new ZegoEffectsSharpenParam();
        zegoEffectsSharpenParam.intensity = sharpenEffectStrength;
        zegoEffects.setSharpenParam(zegoEffectsSharpenParam);
    }

    /**
     * 开启大眼
     *
     * @param enable
     */
    public void enableBigEye(boolean enable) {
        Log.v(TAG, "enableBigEye :enable" + enable);
        zegoEffects.enableBigEyes(enable);
    }

    /**
     * 设置大眼参数
     *
     * @param bigEyeEffectStrength
     */
    public void setBigEyeParam(int bigEyeEffectStrength) {
        Log.v(TAG, "setBigEyeParam :bigEyeEffectStrength" + bigEyeEffectStrength);
        ZegoEffectsBigEyesParam zegoEffectsBigEyeParam = new ZegoEffectsBigEyesParam();
        zegoEffectsBigEyeParam.intensity = bigEyeEffectStrength;
        zegoEffects.setBigEyesParam(zegoEffectsBigEyeParam);
    }

    /**
     * 开启瘦脸
     *
     * @param enable
     */
    public void enableFaceLifting(boolean enable) {
        Log.v(TAG, "enableFaceLifting :enable" + enable);
        zegoEffects.enableFaceLifting(enable);
    }

    /**
     * 设置瘦脸参数
     *
     * @param faceLiftingEffectStrength
     */
    public void setFaceLiftingParam(int faceLiftingEffectStrength) {
        Log.v(TAG, "setFaceLiftingParam :faceLiftingEffectStrength" + faceLiftingEffectStrength);
        ZegoEffectsFaceLiftingParam zegoEffectsFaceLiftingParam = new ZegoEffectsFaceLiftingParam();
        zegoEffectsFaceLiftingParam.intensity = faceLiftingEffectStrength;
        zegoEffects.setFaceLiftingParam(zegoEffectsFaceLiftingParam);
    }


    /**
     * 开启小嘴
     *
     * @param enable
     */
    public void enableSmallMouth(boolean enable) {
        Log.v(TAG, "enableSmallMouth :enable" + enable);
        zegoEffects.enableSmallMouth(enable);
    }

    /**
     * 设置小嘴参数
     *
     * @param smallMouthEffectStrength
     */
    public void setSmallMouthParam(int smallMouthEffectStrength) {
        Log.v(TAG, "setSmallMouthParam :smallMouthEffectStrength" + smallMouthEffectStrength);
        ZegoEffectsSmallMouthParam zegoEffectsSmallMouthParam = new ZegoEffectsSmallMouthParam();
        zegoEffectsSmallMouthParam.intensity = smallMouthEffectStrength;
        zegoEffects.setSmallMouthParam(zegoEffectsSmallMouthParam);
    }


    /**
     * 开启亮眼
     *
     * @param enable
     */
    public void enableEyesBrightening(boolean enable) {
        Log.v(TAG, "enableEyesBrightening :enable" + enable);
        zegoEffects.enableEyesBrightening(enable);
    }

    /**
     * 设置亮眼参数
     *
     * @param eyesBrighteningEffectStrength
     */
    public void setEyesBrighteningParam(int eyesBrighteningEffectStrength) {
        Log.v(TAG, "setEyesBrighteningParam :eyesBrighteningEffectStrength" + eyesBrighteningEffectStrength);
        ZegoEffectsEyesBrighteningParam zegoEffectsEyesBrighteningParam = new ZegoEffectsEyesBrighteningParam();
        zegoEffectsEyesBrighteningParam.intensity = eyesBrighteningEffectStrength;
        zegoEffects.setEyesBrighteningParam(zegoEffectsEyesBrighteningParam);
    }

    /**
     * 开启瘦鼻
     *
     * @param enable
     */
    public void enableNoseNarrowing(boolean enable) {
        Log.v(TAG, "enableNoseNarrowing :enable" + enable);
        zegoEffects.enableNoseNarrowing(enable);
    }

    /**
     * 设置瘦鼻参数
     *
     * @param noseNarrowingEffectStrength
     */
    public void setNoseNarrowingParam(int noseNarrowingEffectStrength) {
        Log.v(TAG, "setNoseNarrowingParam :noseNarrowingEffectStrength" + noseNarrowingEffectStrength);
        ZegoEffectsNoseNarrowingParam zegoEffectsNoseNarrowingParam = new ZegoEffectsNoseNarrowingParam();
        zegoEffectsNoseNarrowingParam.intensity = noseNarrowingEffectStrength;
        zegoEffects.setNoseNarrowingParam(zegoEffectsNoseNarrowingParam);
    }

    /**
     * 开启白牙
     *
     * @param enable
     */
    public void enableTeethWhitening(boolean enable) {
        Log.v(TAG, "enableTeethWhitening :enable" + enable);
        zegoEffects.enableTeethWhitening(enable);
    }

    /**
     * 设置白牙参数
     *
     * @param teethWhiteningEffectStrength
     */
    public void setTeethWhiteningParam(int teethWhiteningEffectStrength) {
        Log.v(TAG, "setTeethWhiteningParam :teethWhiteningEffectStrength" + teethWhiteningEffectStrength);
        ZegoEffectsTeethWhiteningParam zegoEffectsTeethWhiteningParam = new ZegoEffectsTeethWhiteningParam();
        zegoEffectsTeethWhiteningParam.intensity = teethWhiteningEffectStrength;
        zegoEffects.setTeethWhiteningParam(zegoEffectsTeethWhiteningParam);
    }

    /**
     * 开启长下巴
     *
     * @param enable
     */
    public void enableLongChin(boolean enable) {
        Log.v(TAG, "enableLongChin :enable" + enable);
        zegoEffects.enableLongChin(enable);
    }

    /**
     * 设置长下巴参数
     *
     * @param longChinEffectStrength
     */
    public void setLongChinParam(int longChinEffectStrength) {
        Log.v(TAG, "setLongChinParam :longChinEffectStrength" + longChinEffectStrength);
        ZegoEffectsLongChinParam zegoEffectsLongChinParam = new ZegoEffectsLongChinParam();
        zegoEffectsLongChinParam.intensity = longChinEffectStrength;
        zegoEffects.setLongChinParam(zegoEffectsLongChinParam);
    }

    public int getEffectVersion() {
        return effectVersion;
    }

    public void setEffectVersion(int effectVersion) {
        this.effectVersion = effectVersion;
    }
}
