package im.zego.liveshow.effects;

import android.app.Application;
import android.util.Log;

import java.util.HashMap;

import im.zego.effects.sdkmanager.EffectsSDKManager;
import im.zego.effects.sdkmanager.utils.EffectsSDKHelp;
import im.zego.liveshow.Constants;

public class EffectsBeautyManager {

    public String TAG = "EffectsBeautyManager";

    private static volatile EffectsBeautyManager singleton = null;

    private HashMap<String, Integer> beautyMap = new HashMap<>(); //美颜
    private HashMap<String, Integer> beautyTypeMap = new HashMap<>(); //美型

    public static EffectsBeautyManager getInstance() {
        if (singleton == null) {
            synchronized (EffectsBeautyManager.class) {
                if (singleton == null) {
                    singleton = new EffectsBeautyManager();
                }
            }
        }
        return singleton;
    }

    /**
     * 初始化
     *
     * @param application
     */
    public void initEffectsSDK(Application application) {
        EffectsSDKHelp effectsSDKHelp = new EffectsSDKHelp();
        effectsSDKHelp.init(application);

        setBeautyInitData();
    }

    /**
     * 美颜美型数据初始化
     */
    public void setBeautyInitData(){
        setBeautyMap();
        setBeautyTypeMap();
        initBeautyData();
        initBeautyTypeData();
    }

    /**
     * 这种美颜初始值
     */
    public void setBeautyMap() {
        beautyMap.clear();
        beautyMap.put(Constants.EFFECT_WHITE, 50);// 美白 初始值为 50
        beautyMap.put(Constants.EFFECT_SMOOTH, 50);//磨皮 初始值为 50
        beautyMap.put(Constants.EFFECT_ROSY, 50);//红润 初始值为 50
        beautyMap.put(Constants.EFFECT_SHARPEN, 50);//锐化 初始值为 50
    }

    /**
     * 设置美型初始值
     */
    public void setBeautyTypeMap() {
        beautyTypeMap.clear();
        beautyTypeMap.put(Constants.EFFECT_BIG_EYE, 50);// 大眼 初始值为 50
        beautyTypeMap.put(Constants.EFFECT_FACE_LIFTING, 50);//瘦脸 初始值为 50
        beautyTypeMap.put(Constants.EFFECT_SMALL_MOUTH, 0);//小嘴 初始值为 100 最大值为200
        beautyTypeMap.put(Constants.EFFECT_EYES_BRIGHTENING, 50);//亮眼 初始值为 50
        beautyTypeMap.put(Constants.EFFECT_NOSE_NARROWING, 50);//瘦鼻 初始值为 50
        beautyTypeMap.put(Constants.EFFECT_TEETH_WHITENING, 50);//白牙 初始值为 50
        beautyTypeMap.put(Constants.EFFECT_LONG_CHIN, 0);//长下巴 初始值为 100 最大值为200
    }

    /**
     * 设置美颜数据
     */
    public void initBeautyData(){
        //美白
        setWhitenParam(50);
        //磨皮
        setSmoothParam(50);
        //锐化
        setSharpenParam(50);
        //红润
        setRosyParam(50);
    }

    /**
     * 设置美型数据
     */
    public void initBeautyTypeData(){
        //大眼
        setBigEyeParam(50);
        //瘦脸
        setFaceLiftingParam(50);
        //小嘴
        setSmallMouthParam(0);
        //亮眼
        setEyesBrighteningParam(50);
        //瘦鼻
        setNoseNarrowingParam(50);
        // 白牙
        setTeethWhiteningParam(50);
        //长下巴
        setLongChinParam(0);
    }

    public HashMap<String, Integer> getBeautyMap() {
        return beautyMap;
    }

    public HashMap<String, Integer> getBeautyTypeMap() {
        return beautyTypeMap;
    }

    /**
     * 设置美白参数
     *
     * @param whitenEffectStrength
     */
    public void setWhitenParam(int whitenEffectStrength) {
        Log.v(TAG, "setWhitenParam :whitenEffectStrength" + whitenEffectStrength);
        EffectsSDKManager.getInstance().setWhitenParam(whitenEffectStrength);
    }

    /**
     * 设置磨皮参数
     *
     * @param smoothEffectStrength
     */
    public void setSmoothParam(int smoothEffectStrength) {
        Log.v(TAG, "setSmoothParam :smoothEffectStrength" + smoothEffectStrength);
        EffectsSDKManager.getInstance().setSmoothParam(smoothEffectStrength);
    }

    /**
     * 设置红润参数
     *
     * @param rosyEffectStrength
     */
    public void setRosyParam(int rosyEffectStrength) {
        Log.v(TAG, "setRosyParam :rosyEffectStrength" + rosyEffectStrength);
        EffectsSDKManager.getInstance().setRosyParam(rosyEffectStrength);
    }

    /**
     * 设置锐化参数
     *
     * @param sharpenEffectStrength
     */
    public void setSharpenParam(int sharpenEffectStrength) {
        Log.v(TAG, "setSharpenParam :sharpenEffectStrength" + sharpenEffectStrength);
        EffectsSDKManager.getInstance().setSharpenParam(sharpenEffectStrength);
    }

    /**
     * 设置大眼参数
     *
     * @param bigEyeEffectStrength
     */
    public void setBigEyeParam(int bigEyeEffectStrength) {
        Log.v(TAG, "setBigEyeParam :bigEyeEffectStrength" + bigEyeEffectStrength);
        EffectsSDKManager.getInstance().setBigEyeParam(bigEyeEffectStrength);
    }

    /**
     * 设置瘦脸参数
     *
     * @param faceLiftingEffectStrength
     */
    public void setFaceLiftingParam(int faceLiftingEffectStrength) {
        Log.v(TAG, "setFaceLiftingParam :faceLiftingEffectStrength" + faceLiftingEffectStrength);
        EffectsSDKManager.getInstance().setFaceLiftingParam(faceLiftingEffectStrength);
    }

    /**
     * 设置小嘴参数
     *
     * @param smallMouthEffectStrength
     */
    public void setSmallMouthParam(int smallMouthEffectStrength) {
        Log.v(TAG, "setSmallMouthParam :smallMouthEffectStrength" + smallMouthEffectStrength);
        EffectsSDKManager.getInstance().setSmallMouthParam(smallMouthEffectStrength);
    }

    /**
     * 设置亮眼参数
     *
     * @param eyesBrighteningEffectStrength
     */
    public void setEyesBrighteningParam(int eyesBrighteningEffectStrength) {
        Log.v(TAG, "setEyesBrighteningParam :eyesBrighteningEffectStrength" + eyesBrighteningEffectStrength);
        EffectsSDKManager.getInstance().setEyesBrighteningParam(eyesBrighteningEffectStrength);
    }

    /**
     * 设置瘦鼻参数
     *
     * @param noseNarrowingEffectStrength
     */
    public void setNoseNarrowingParam(int noseNarrowingEffectStrength) {
        Log.v(TAG, "setNoseNarrowingParam :noseNarrowingEffectStrength" + noseNarrowingEffectStrength);
        EffectsSDKManager.getInstance().setNoseNarrowingParam(noseNarrowingEffectStrength);
    }

    /**
     * 设置白牙参数
     *
     * @param teethWhiteningEffectStrength
     */
    public void setTeethWhiteningParam(int teethWhiteningEffectStrength) {
        Log.v(TAG, "setTeethWhiteningParam :teethWhiteningEffectStrength" + teethWhiteningEffectStrength);
        EffectsSDKManager.getInstance().setTeethWhiteningParam(teethWhiteningEffectStrength);
    }

    /**
     * 设置长下巴参数
     *
     * @param longChinEffectStrength
     */
    public void setLongChinParam(int longChinEffectStrength) {
        Log.v(TAG, "setLongChinParam :longChinEffectStrength" + longChinEffectStrength);
        EffectsSDKManager.getInstance().setLongChinParam(longChinEffectStrength);
    }

    public void clearEffectData() {
        beautyMap.clear();
        beautyTypeMap.clear();
    }


}
