package im.zego.liveshow.liveroom;

import android.content.DialogInterface;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.Log;
import android.view.TextureView;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import java.util.List;

import im.zego.common.util.MiscUtils;
import im.zego.liveshow.Constants;
import im.zego.liveshow.ErrorcodeConstants;
import im.zego.liveshow.R;
import im.zego.liveshow.effects.SoundEffectsManager;
import im.zego.liveshow.liveroom.adapter.AudienceListAdapter;
import im.zego.liveshow.liveroom.callbacks.IAnchorInfoCallback;
import im.zego.liveshow.liveroom.callbacks.IHeartBeatFailCallback;
import im.zego.liveshow.liveroom.callbacks.IRoomQualityInfoCallback;
import im.zego.liveshow.liveroom.utils.HeartBeatUtil;
import im.zego.liveshow.manager.NetWorkConManager;
import im.zego.liveshow.manager.ResolutionManager;
import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.manager.UserManager;
import im.zego.liveshow.manager.callbacks.ICreateRoomFailTimeoutCallback;
import im.zego.liveshow.manager.callbacks.INetworkOperationCallback;
import im.zego.liveshow.manager.callbacks.IRoomConnectionCallback;
import im.zego.liveshow.manager.callbacks.IRoomDisconnectCallback;
import im.zego.liveshow.model.OnlineInfo;
import im.zego.liveshow.model.RoomInfo;
import im.zego.liveshow.model.StatefulListInfo;
import im.zego.liveshow.model.UserInfo;
import im.zego.liveshow.model.rtc.NotifyApplyMicStateInfo;
import im.zego.liveshow.model.rtc.NotifyRefuseApplyMicInfo;
import im.zego.liveshow.model.rtc.NotifyUserListUpdateInfo;
import im.zego.liveshow.model.rtc.NotifyUserStateUpdateInfo;
import im.zego.liveshow.protocol.IApplyMicStateNotifyCallback;
import im.zego.liveshow.protocol.IExecuteCallback;
import im.zego.liveshow.protocol.IGetHostAudiencetCallback;
import im.zego.liveshow.protocol.IGetOnlineInfoCallback;
import im.zego.liveshow.manager.callbacks.IStreamCallback;
import im.zego.liveshow.protocol.RoomAPI;
import im.zego.liveshow.sdk.ZegoSDKManager;
import im.zego.liveshow.sdk.callbacks.IRTCCommonCallback;
import im.zego.liveshow.utils.DialogQueueUtils;
import im.zego.liveshow.utils.GlideLoader;
import im.zego.liveshow.utils.ToastUtil;
import im.zego.liveshow.view.BaseInquiryDialog;
import im.zego.liveshow.view.EffectsBeautyDialog;
import im.zego.liveshow.view.LiveshowRealtimeLayout;
import im.zego.liveshow.view.MoreSettingDialog;
import im.zego.liveshow.view.ProcessView;
import im.zego.liveshow.view.RoomBottomLayerLayout;
import im.zego.liveshow.view.SoundEffectsDialog;

import static im.zego.liveshow.Constants.ROOM_ROLE_AUDIENCE;
import static im.zego.liveshow.Constants.ROOM_ROLE_HOST;

/**
 * 已废弃
 */
@Deprecated
public class LiveRoomActivity extends AppCompatActivity implements MoreSettingDialog.ISettingMoreCallBack, IApplyMicStateNotifyCallback, IStreamCallback {

    public static final String LOGIN_OR_CREATE_ROOM = "login_or_create";
    public static final int LOGIN_ROOM = 0;
    public static final int CREATE_ROOM = 1;

    private TextureView mLiveVideo;
    private ImageView mIvHostHead; //主播关闭摄像头的时候显示的高斯模糊头像
    private TextView mFinishRoomTips;
    private ConstraintLayout mCreateRoom;
    private ConstraintLayout mRoomLayer;

    private RecyclerView mAudienceList; //观众连麦列表
    private AudienceListAdapter mAudienceListAdapter;
    private RoomBottomLayerLayout mRoomBottomLayerLayout; //底部翻转/语言/连麦的按钮
    private ConstraintLayout mMoreSetting; //房主的更多设置按钮
    private ConstraintLayout mBeautySetting; //房主的美颜美型设置按钮
    private ConstraintLayout mMusicSetting; //房主的音效设置按钮

    private View mCreateResolution;
    private View mIvBack;
    private EditText mEditRoomName;
    private View mCreateFlipCamera;

    private View mSelectBeauty;
    private View mClStartLive;
    private View mCloseRoom;
    private TextView mTvOnlineNum;

    private ImageView mAnchorHeadPortrait;

    private TextView mAnchorName;

    private LiveshowRealtimeLayout streamQualityInfo;
    private ProcessView processView;

    private BaseInquiryDialog createRoomNetworkFailDialog;
    private BaseInquiryDialog heartBeatFailDialog;

    private boolean mIsMicMuted = true; //麦克风是否开启
    private boolean mIsCameraMuted = true; //摄像头是否开启

    private MoreSettingDialog moreSettingDialog; //设置弹窗
    private BaseInquiryDialog connectMicDialog;


    private boolean showCreateRoomNetworkFail = false;

    private boolean mRoomClose = false;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.liveshow_activity_live_room);
        initWindowBar();
        initView();
        initListener();
        initData();
        initRecycleView();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        DialogQueueUtils.getInstance().clear();
        NetWorkConManager.getInstance().clearData();
        SoundEffectsManager.getInstance().clearSoundEffectsData();
        HeartBeatUtil.uninit();
        RoomManager.getInstance().exit();
    }

    private void initWindowBar() {
        WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.flags = lp.flags | WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS;
        getWindow().setAttributes(lp);
    }

    private void initView() {
        mLiveVideo = findViewById(R.id.live_video);
        mIvHostHead = findViewById(R.id.iv_host_head);
        mFinishRoomTips = findViewById(R.id.finish_room_tips);
        mAudienceList = findViewById(R.id.audience_list);
        mRoomBottomLayerLayout = findViewById(R.id.bottom_layer);
        mCreateRoom = findViewById(R.id.create_room);
        mRoomLayer = findViewById(R.id.room_layer);
        mCreateResolution = findViewById(R.id.create_resolution);
        mIvBack = findViewById(R.id.iv_back);
        mSelectBeauty = findViewById(R.id.select_beauty);
        mEditRoomName = findViewById(R.id.edit_room_name);
        mClStartLive = findViewById(R.id.cl_start_live);
        mMoreSetting = findViewById(R.id.cl_more_setting);
        mCloseRoom = findViewById(R.id.close_room);
        streamQualityInfo = findViewById(R.id.stream_quality_info);
        processView = new ProcessView(LiveRoomActivity.this);
        mAnchorHeadPortrait = findViewById(R.id.anchor_head_portrait);
        mAnchorName = findViewById(R.id.anchor_name);
        mTvOnlineNum = findViewById(R.id.tv_online_num);
        mMusicSetting = findViewById(R.id.cl_music_setting);
        mBeautySetting = findViewById(R.id.cl_beauty_setting);
        mCreateFlipCamera = findViewById(R.id.create_flip_camera);
    }

    private void initData() {
        int loginOrCreate = getIntent().getIntExtra(LOGIN_OR_CREATE_ROOM, LOGIN_ROOM);

        if (!ZegoSDKManager.getInstance().isInit()) {
            finish();
            return;
        }

        ZegoSDKManager.getInstance().getDeviceService().enableCamera(true);
        RoomManager.getInstance().setFront(true);

        if (loginOrCreate == CREATE_ROOM) {
            showCreateRoom();
            ZegoSDKManager.getInstance().getDeviceService().startPreview(mLiveVideo);
            ResolutionManager.getInstance().reset(ROOM_ROLE_HOST);
            mEditRoomName.setText(String.valueOf(UserManager.getInstance().getUserInfo().getNickName() + "创建的直播间"));
        } else {
            showLiveRoom();
        }

    }

    private void showCreateRoom() {

        mCreateRoom.setVisibility(View.VISIBLE);
        mRoomLayer.setVisibility(View.GONE);
        ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) mCreateRoom.getLayoutParams();
        params.topMargin = MiscUtils.getStatusBarHeight(this);
        mCreateRoom.setLayoutParams(params);
    }

    private void showLiveRoom() {
        if (RoomManager.getInstance().getRoomID() == null || UserManager.getInstance().getUserInfo() == null) {
            finish();
        }

        UserInfo userInfo = UserManager.getInstance().getUserInfo();

        mCreateRoom.setVisibility(View.GONE);
        mRoomLayer.setVisibility(View.VISIBLE);

        ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) mRoomLayer.getLayoutParams();
        params.topMargin = MiscUtils.getStatusBarHeight(this);
        mRoomLayer.setLayoutParams(params);

        if (userInfo.getRole() == ROOM_ROLE_AUDIENCE) {
            //进入房间的是观众
            mRoomBottomLayerLayout.setVisibility(View.VISIBLE);
            mMoreSetting.setVisibility(View.GONE);
            mBeautySetting.setVisibility(View.GONE);
            mMusicSetting.setVisibility(View.GONE);
            mRoomBottomLayerLayout.bottomLayerVisible(false);

        } else if (userInfo.getRole() == ROOM_ROLE_HOST) {
            //进入房间的是主持人
            mMoreSetting.setVisibility(View.VISIBLE);
            mBeautySetting.setVisibility(View.VISIBLE);
            mMusicSetting.setVisibility(View.VISIBLE);
            mRoomBottomLayerLayout.setVisibility(View.GONE);
        }

        RoomManager.getInstance().setConnectionMicListener(this);
        RoomManager.getInstance().setStreamStatusListener(this);
        RoomManager.getInstance().setRoomDisconnectCallback(new IRoomDisconnectCallback() {
            @Override
            public void onDisconnect() {
                finish();
            }
        });

        RoomAPI.getOnlineInfo(RoomManager
                .getInstance().getRoomID(), UserManager.getInstance().getUserId(), new IGetOnlineInfoCallback() {
            @Override
            public void onGetOnlineInfo(int errorCode, OnlineInfo onlineInfo) {
                if (errorCode == 0) {
                    mTvOnlineNum.setText(String.valueOf(onlineInfo.getOnlineNum()));
                }
            }
        });

        getStatefulList();
        HeartBeatUtil.init(UserManager.getInstance().getUserId(), RoomManager.getInstance().getRoomID());
        HeartBeatUtil.setHeartBeatFailCallback(new IHeartBeatFailCallback() {
            @Override
            public void onHeartBeatFail() {
                if (mRoomClose) {
                    return;
                }

                if (heartBeatFailDialog != null && heartBeatFailDialog.isShowing()) {
                    return;
                }

                heartBeatFailDialog = new BaseInquiryDialog(LiveRoomActivity.this);
                heartBeatFailDialog.setMsgTitle(getString(R.string.liveshow_fail_network));
                heartBeatFailDialog.setMsgContent(getString(R.string.liveshow_network_connect_fail));
                heartBeatFailDialog.setLeftButtonContent(getString(R.string.liveshow_exit));
                heartBeatFailDialog.setRightButtonContent(getString(R.string.liveshow_again));
                heartBeatFailDialog.setCancelListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        if (heartBeatFailDialog != null && !isFinishing() && !isDestroyed()) {
                            heartBeatFailDialog.dismiss();
                        }
                        finish();
                    }
                });
                heartBeatFailDialog.setSureListener(v -> {

                    HeartBeatUtil.userRetry(0);

                });
            }

            @Override
            public void onHeartBeatDisconnect(int errorCode) {

                RoomManager.getInstance().CommonErrorCodeToast(errorCode);
                if (heartBeatFailDialog != null && !isFinishing() && !isDestroyed()) {
                    heartBeatFailDialog.dismiss();
                }
                finish();
            }

            @Override
            public void onHeartBeatSuccess() {
                Log.e("HeartBeatUtil", "onHeartBeatSuccess:");
                if (heartBeatFailDialog != null && !isFinishing() && !isDestroyed()) {
                    Log.e("HeartBeatUtil", "heartBeatFailDialog:");
                    heartBeatFailDialog.dismiss();
                }
            }

            @Override
            public void onHeartBeatUserRetryFail(int errorCode) {
                RoomManager.getInstance().CommonErrorCodeToast(errorCode);

                if (errorCode == ErrorcodeConstants.ERROR_USER_NOT_IN_ROOM) {
                    if (heartBeatFailDialog != null && !isFinishing() && !isDestroyed()) {
                        heartBeatFailDialog.dismiss();
                    }
                    finish();
                }
            }
        });

        HeartBeatUtil.setStateSeqCallback(seq -> NetWorkConManager.getInstance().setPersonalStateSeq(seq));

        RoomManager.getInstance().setRoomConnectionCallback(new IRoomConnectionCallback() {
            @Override
            public void onConnected() {
                HeartBeatUtil.retry(0);
            }

            @Override
            public void connecting() {

            }
        });

        initNetwork();
    }

    /**
     * 观众连麦列表
     */
    private void initRecycleView() {
        mAudienceListAdapter = new AudienceListAdapter(this);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(this);
        mAudienceList.setLayoutManager(linearLayoutManager);
        mAudienceList.setAdapter(mAudienceListAdapter);
        //解决刷新闪烁问题
        ((SimpleItemAnimator) mAudienceList.getItemAnimator()).setSupportsChangeAnimations(false);

        mAudienceListAdapter.setUserStageListener(fromType -> {
            if (fromType == Constants.STATE_OFF_STAGE) {
                //下麦成功后显示
                mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
                mRoomBottomLayerLayout.bottomLayerVisible(false);
                mRoomBottomLayerLayout.switchCamera(false);
                mRoomBottomLayerLayout.switchMic(false);
            } else if (fromType == Constants.STATE_ON_STAGE) {
                //成功连麦显示
                mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_SUCCESS);
                mRoomBottomLayerLayout.bottomLayerVisible(true);
                ZegoSDKManager.getInstance().getDeviceService().setRoomData(true);
                ZegoSDKManager.getInstance().getDeviceService().setFrontCamera(true);
            }
        });
    }

    private void initListener() {
        mIvBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onBackPressed();
            }
        });

        mCreateResolution.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                ResolutionManager.getInstance().showBottomSheetDialog(LiveRoomActivity.this);
            }
        });
        mSelectBeauty.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                settingBeauty();
            }
        });

        mEditRoomName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                if (TextUtils.isEmpty(s.toString().trim())) {
                    mClStartLive.setAlpha(0.5f);
                } else {
                    mClStartLive.setAlpha(1f);
                }
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        mClStartLive.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final String roomName = mEditRoomName.getText().toString().trim();
                if (TextUtils.isEmpty(roomName)) {
                    return;
                }
                processView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mClStartLive.getRootView());
                RoomManager.getInstance().createRoom(roomName, new IRTCCommonCallback() {
                    @Override
                    public void onRTCCallback(int errorCode) {
                        if (errorCode == 0) {
                            showCreateRoomNetworkFail = false;
                            runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    processView.dismiss();
                                    showLiveRoom();
                                    if (createRoomNetworkFailDialog != null && !isDestroyed() && !isFinishing()) {
                                        createRoomNetworkFailDialog.dismiss();
                                        createRoomNetworkFailDialog = null;
                                    }
                                }
                            });
                            RoomManager.getInstance().startPublishStream(mLiveVideo, null);

                        } else {
                            processView.dismiss();

                            if (showCreateRoomNetworkFail) {

                                showCreateRoomNetworkFailDialog();

                            }
                        }
                    }
                });

            }
        });

        mMoreSetting.setOnClickListener(v -> {
            //房主的更多设置弹窗
            if (moreSettingDialog == null) {
                moreSettingDialog = new MoreSettingDialog();
                moreSettingDialog.setCallBack(this);
            }
            moreSettingDialog.show(getSupportFragmentManager(), moreSettingDialog.getTag());
            moreSettingDialog.setMicAndCamara(mIsMicMuted, mIsCameraMuted);
        });

        //观众操作摄像头/麦克风/下麦的回调
        mRoomBottomLayerLayout.setCameraMicListener((mIsMicMuted, mIsCameraMuted, type, operator) -> {
            httpSetStatus(mIsMicMuted, mIsCameraMuted, type, operator);
        });

        mCloseRoom.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onBackPressed();
            }
        });

        streamQualityInfo.setCloseListener(new LiveshowRealtimeLayout.CloseListener() {
            @Override
            public void close() {
                streamQualityInfo.setVisibility(View.GONE);
            }
        });

        RoomManager.getInstance().setRoomQualityInfoCallback(new IRoomQualityInfoCallback() {
            @Override
            public void onRoomQualityInfoUpdate(double kbps, double fps, int rtt, double packetLostRate, String rxSpeed, String txSpeed) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        streamQualityInfo.setKbs((int) kbps);
                        streamQualityInfo.setFps((int) fps);
                        streamQualityInfo.setDelay(rtt);
                        streamQualityInfo.setPacketLoss((int) packetLostRate);
                        streamQualityInfo.setDownlinkNetwork(rxSpeed);
                        streamQualityInfo.seUplinkNetwork(txSpeed);
                    }
                });
            }
        });

        RoomManager.getInstance().setAnchorInfoCallback(new IAnchorInfoCallback() {
            @Override
            public void onAnchorInfoUpdate(UserInfo userInfo) {
                GlideLoader.displayCircleImageByAvatar(userInfo.getAvatar(), mAnchorHeadPortrait);
                mAnchorName.setText(userInfo.getNickName());
                setHostCameraImg(userInfo.getCamera(), userInfo.getAvatar());
            }
        });

        RoomManager.getInstance().setCreateRoomFailTimeoutCallback(new ICreateRoomFailTimeoutCallback() {
            @Override
            public void onCreateRoomFailTimeout() {
                showCreateRoomNetworkFail = true;
                showCreateRoomNetworkFailDialog();
            }
        });

        mBeautySetting.setOnClickListener(v -> settingBeauty());
        mMusicSetting.setOnClickListener(v -> {
            SoundEffectsDialog soundEffectsDialog = new SoundEffectsDialog();
            soundEffectsDialog.show(getSupportFragmentManager(), soundEffectsDialog.getTag());
        });

        mCreateFlipCamera.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                setHostFront();
            }
        });

    }

    private void showCreateRoomNetworkFailDialog() {
        if (createRoomNetworkFailDialog != null) {
            return;
        }

        createRoomNetworkFailDialog = new BaseInquiryDialog(LiveRoomActivity.this);
        createRoomNetworkFailDialog.setMsgTitle(getString(R.string.liveshow_fail_network));
        createRoomNetworkFailDialog.setMsgContent(getString(R.string.liveshow_network_connect_fail));
        createRoomNetworkFailDialog.setLeftButtonContent(getString(R.string.liveshow_exit));
        createRoomNetworkFailDialog.setRightButtonContent(getString(R.string.liveshow_again));
        createRoomNetworkFailDialog.setCancelListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showCreateRoomNetworkFail = false;
                createRoomNetworkFailDialog.dismiss();
                createRoomNetworkFailDialog = null;
                finish();
            }
        });
        createRoomNetworkFailDialog.setSureListener(v -> {
            createRoomNetworkFailDialog.dismiss();
            createRoomNetworkFailDialog = null;
            mClStartLive.performClick();
        });
    }

    /**
     * 设置摄像头、麦克风、下麦
     *
     * @param mic    1：关闭  2:打开
     * @param camera 1：关闭  2:打开
     * @param type   1.mic操作  2. camera操作 3. 关闭连麦
     */
    private void httpSetStatus(boolean mic, boolean camera, int type, int operation) {
        RoomManager.getInstance().setStatus(mic, camera, type, new IExecuteCallback() {
            @Override
            public void onSuccess() {
                //是房主关闭摄像头后，显示默认图片
                if (operation == ROOM_ROLE_HOST && type == Constants.OPERATION_TYPE_CAMERA) {
                    if (camera) {
                        mLiveVideo.setVisibility(View.VISIBLE);
                        mIvHostHead.setVisibility(View.GONE);
                        mFinishRoomTips.setVisibility(View.GONE);
                    } else {
                        mLiveVideo.setVisibility(View.GONE);
                        mIvHostHead.setVisibility(View.VISIBLE);
                        mFinishRoomTips.setVisibility(View.VISIBLE);
                        StatefulListInfo.AttendeeListDTO hostInfo = RoomManager.getInstance().getHostInfo();
                        String avatar = hostInfo == null ? "1" : hostInfo.getAvatar();
                        GlideLoader.displayRoundImageByAvatar(avatar, mIvHostHead);
                    }
                } else if (operation == ROOM_ROLE_AUDIENCE && type == Constants.OPERATION_TYPE_STAGE_STATE) {
                    //下麦成功
                    RoomManager.getInstance().setHasConMic(false);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                if (type == Constants.OPERATION_TYPE_MIC) {
                    //麦克风
                    if (operation == ROOM_ROLE_AUDIENCE) {
                        mRoomBottomLayerLayout.switchMic(mRoomBottomLayerLayout.ismIsMicMuted()); //观众
                    } else if (operation == ROOM_ROLE_HOST) {
                        //房主
                        mIsMicMuted = !mIsMicMuted;
                        moreSettingDialog.setMic(mIsMicMuted);
                    }

                } else if (type == Constants.OPERATION_TYPE_CAMERA) {
                    //摄像头
                    if (operation == ROOM_ROLE_AUDIENCE) {
                        //观众
                        mRoomBottomLayerLayout.switchCamera(mRoomBottomLayerLayout.ismIsCameraMuted());
                    } else if (operation == ROOM_ROLE_HOST) {
                        //房主
                        mIsCameraMuted = !mIsCameraMuted;
                        moreSettingDialog.setCamara(mIsCameraMuted);
                    }

                }
            }
        });
    }

    /**
     * 拉取连麦人员列表
     */
    private void getStatefulList() {

        if(RoomManager.getInstance().getRoomInfo() == null){
            return;
        }

        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();

        RoomManager.getInstance().getStatefulList(roomInfo.getRoomId(), userInfo.getUid(), new IGetHostAudiencetCallback() {
            @Override
            public void onGetStatefulData(int errorCode, StatefulListInfo.AttendeeListDTO hostInfo, List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo,
                                          StatefulListInfo statefulListInfo,int attendeeSeq, int stateSeq) {
                //当前是房主自己的页面不需要拉流显示，只需要预览就可以
                if (UserManager.getInstance().getUserInfo().getRole() == ROOM_ROLE_AUDIENCE) {
                    RoomManager.getInstance().startPlayStream(hostInfo, mLiveVideo, null);
                }

                if (audienceStatefulListInfo != null && audienceStatefulListInfo.size() > 0) {
                    mAudienceListAdapter.addData(audienceStatefulListInfo);
                }

                NetWorkConManager.getInstance().setStartFirst(false);
                NetWorkConManager.getInstance().setPersonalStateSeq(stateSeq);
            }
        });
    }

    /**
     * 连麦通知回调（申请连麦、取消申请连麦）
     */
    @Override
    public void onApplyMicStateNotify(int action, NotifyApplyMicStateInfo notifyInfo) {
        if (action == Constants.OPERATION_ACTION_APPLY) {
            if (!isFinishing()) {
                DialogQueueUtils dialogQueueUtils = DialogQueueUtils.getInstance();
                if (dialogQueueUtils.canShow()) {
                    dialogQueueUtils.addData(notifyInfo);
                    setMicStateListener();
                    dialogQueueUtils.showDialog();
                } else {
                    dialogQueueUtils.addData(notifyInfo);
                }
            }
        } else if (action == Constants.OPERATION_ACTION_CANCEL) {
            //取消申请连麦 (当前显示的弹窗正好是取消连麦的通知，就toast提示并且弹窗取消。否则就在队列里面移除)
            NotifyApplyMicStateInfo dialogInfo = DialogQueueUtils.getInstance().getmNotifyApplyMicStateInfo();
            if (dialogInfo != null && notifyInfo.getUid() == dialogInfo.getUid()) {
                ToastUtil.showToast(R.string.liveshow_connect_mic_cancle);
                if (!isFinishing() && connectMicDialog != null && connectMicDialog.isShowing()) {
                    connectMicDialog.dismiss();
                }
            } else {
                DialogQueueUtils.getInstance().remove(notifyInfo);
            }
        }

    }

    /**
     * 主播拒绝你的连麦回调
     *
     * @param notifyInfo
     */
    @Override
    public void onRefuseApplyMicNotify(NotifyRefuseApplyMicInfo notifyInfo) {
        mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
    }

    /**
     * 3.连麦状态变化 连麦的上下台
     */
    @Override
    public void onMyStageNotify(NotifyUserStateUpdateInfo.UsersDTO notifyInfo) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mAudienceListAdapter.onStage(notifyInfo);
            }
        });

    }

    /**
     * 连麦列表的状态变更 1.mic变化 2.cameara
     */
    @Override
    public void onAudienceListNotify(NotifyUserStateUpdateInfo onstageList) {

        StatefulListInfo.AttendeeListDTO hostInfo = RoomManager.getInstance().getHostInfo();
        long userId = RoomManager.getInstance().getRoomInfo().getRoomInfo().getHostId();
        long hostId = hostInfo != null ? hostInfo.getUid() : userId;

        if (onstageList.getOperatorUid() == hostId) {
            if (onstageList.getType() == Constants.OPERATION_TYPE_CAMERA) {
                String avatar = hostInfo == null ? "1" : hostInfo.getAvatar();
                setHostCameraImg(onstageList.getUsers().get(0).getCamera(), avatar);
            }
        } else {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mAudienceListAdapter.refreshData(onstageList);
                }
            });
        }

    }

    /**
     * 上台流添加回调
     */
    @Override
    public void onStreamAdd() {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mAudienceListAdapter.notifyDataSetChanged();
            }
        });
    }

    /**
     * 成员列表变更通知
     */
    @Override
    public void onUserListUpdateNotify(NotifyUserListUpdateInfo notifyInfo) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mTvOnlineNum.setText(String.valueOf(notifyInfo.getOnlineNum()));
                mAudienceListAdapter.exitRoomData(notifyInfo);

                //获取当前退出房间的人,并判断有你没有申请连麦。申请就取消弹窗
                NotifyApplyMicStateInfo dialogInfo = DialogQueueUtils.getInstance().getmNotifyApplyMicStateInfo();
                UserInfo myInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
                if (dialogInfo != null && myInfo.getRole() == ROOM_ROLE_HOST) {
                    for (int i = 0; i < notifyInfo.getUsers().size(); i++) {
                        NotifyUserListUpdateInfo.UsersDTO getUsers = notifyInfo.getUsers().get(i);
                        if(getUsers.getDelta() == -1){
                            onApplyMicStateNotify(Constants.OPERATION_ACTION_CANCEL,gainMicStateInfo(getUsers));
                        }
                    }
                }

            }
        });
    }

    @Override
    public void onRoomCloseNotify() {
        if (!isFinishing() && UserManager.getInstance().getUserInfo().getRole() != ROOM_ROLE_HOST) {
            mRoomClose = true;
            BaseInquiryDialog baseInquiryDialog = new BaseInquiryDialog(this, R.layout.liveshow_dialog_room_finish);
            baseInquiryDialog.setSureListener(v -> {
                baseInquiryDialog.dismiss();
                finish();
            });
        }
    }

    @Override
    public void onClickFlip() {
        //房主-设置-翻转
        setHostFront();
    }

    @Override
    public void onClickCamera() {
        //房主-设置-摄像头 onResponse
        mIsCameraMuted = !mIsCameraMuted;
        moreSettingDialog.setCamara(mIsCameraMuted);
        httpSetStatus(mIsMicMuted, mIsCameraMuted, Constants.OPERATION_TYPE_CAMERA, ROOM_ROLE_HOST);
    }

    @Override
    public void onClickMic() {
        //房主-设置-麦克风
        mIsMicMuted = !mIsMicMuted;
        moreSettingDialog.setMic(mIsMicMuted);
        httpSetStatus(mIsMicMuted, mIsCameraMuted, Constants.OPERATION_TYPE_MIC, ROOM_ROLE_HOST);
    }

    @Override
    public void onClickResolution() {
        //房主-设置-分辨率
        ResolutionManager.getInstance().showBottomSheetDialog(LiveRoomActivity.this);
    }

    @Override
    public void onClickData() {
        //房主-设置-实时数据
        if (streamQualityInfo.getVisibility() == View.VISIBLE) {
            streamQualityInfo.setVisibility(View.GONE);
        } else {
            streamQualityInfo.setVisibility(View.VISIBLE);
        }
    }

    /**
     * 主播是否关闭摄像头-关闭就显示高斯模糊头像
     *
     * @param camera
     * @param avatar
     */
    private void setHostCameraImg(int camera, String avatar) {
        boolean isOpen = camera == Constants.MIC_CAMERA_OPEN;
        mLiveVideo.setVisibility(isOpen ? View.VISIBLE : View.GONE);
        mIvHostHead.setVisibility(isOpen ? View.GONE : View.VISIBLE);
        mFinishRoomTips.setVisibility(isOpen ? View.GONE : View.VISIBLE);
        if (!isOpen) {
            GlideLoader.displayRoundImageByAvatar(avatar, mIvHostHead);
        }
    }

    @Override
    public void onBackPressed() {
        if (UserManager.getInstance().getUserInfo().getRole() == ROOM_ROLE_HOST) {

            BaseInquiryDialog baseInquiryDialog = new BaseInquiryDialog(LiveRoomActivity.this, true);
            baseInquiryDialog.setMsgTitle(getString(R.string.liveshow_end_of_live_tip));
            baseInquiryDialog.setMsgContent(getString(R.string.liveshow_end_of_live_content));
            baseInquiryDialog.setLeftButtonContent(getString(R.string.liveshow_btn_cancle));
            baseInquiryDialog.setRightButtonContent(getString(R.string.liveshow_btn_true));
            baseInquiryDialog.setCancelListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    baseInquiryDialog.dismiss();
                }
            });
            baseInquiryDialog.setSureListener(v -> {
                RoomManager.getInstance().closeRoom(new IExecuteCallback() {
                    @Override
                    public void onSuccess() {
                        if (!isFinishing() && !isDestroyed()) {
                            baseInquiryDialog.dismiss();
                        }
                        finish();
                    }

                    @Override
                    public void onFailed(int errorCode) {
                        if (!isFinishing() && !isDestroyed()) {
                            baseInquiryDialog.dismiss();
                        }
                        finish();
                    }
                });
            });
        } else {
            super.onBackPressed();
        }
    }

    /**
     * 主播设置摄像头是否前置
     */
    private void setHostFront() {
        RoomManager.getInstance().setFront(!RoomManager.getInstance().isFront());
    }

    /**
     * 接受/拒绝 连麦弹窗监听
     */
    private void setMicStateListener() {
        DialogQueueUtils.getInstance().setDialogListener(new DialogQueueUtils.DialogListener() {
            @Override
            public void onData(NotifyApplyMicStateInfo mNotifyApplyMicStateInfo) {
                connectMicDialog = new BaseInquiryDialog(LiveRoomActivity.this, R.layout.liveshow_inquiry_dialog);
                connectMicDialog.setMsgTitle(getString(R.string.liveshow_apply_connect_mic));
                connectMicDialog.setMsgContent(String.format(getString(R.string.liveshow_apply_connect_mic_content), mNotifyApplyMicStateInfo.getNickName()));
                connectMicDialog.setLeftButtonContent(getString(R.string.liveshow_btn_refuse));
                connectMicDialog.setRightButtonContent(getString(R.string.liveshow_btn_agree));
                connectMicDialog.setSureListener(v -> {
                    //接受申请
                    applyMicStateNotifyRequest(Constants.OPERATION_ACTION_ACCEPT, mNotifyApplyMicStateInfo);
                });
                connectMicDialog.setCancelListener(v -> {
                    //拒绝申请
                    applyMicStateNotifyRequest(Constants.OPERATION_ACTION_REFUSE, mNotifyApplyMicStateInfo);
                });

                connectMicDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
                    @Override
                    public void onDismiss(DialogInterface dialog) {
                        //这边设置了dismiss监听,在监听回调中再次调用show方法,可以获取下一个弹窗
                        DialogQueueUtils.getInstance().setInfoNull();
                        DialogQueueUtils.getInstance().showDialog();
                    }
                });

            }
        });
    }

    /**
     * 连麦弹窗的接受或者拒绝
     */
    private void applyMicStateNotifyRequest(int action, NotifyApplyMicStateInfo notifyInfo) {
        RoomManager.getInstance().applyMicStateNotifyRequest(action, notifyInfo, new IExecuteCallback() {
            @Override
            public void onSuccess() {
                if (!isFinishing() && connectMicDialog != null && connectMicDialog.isShowing()) {
                    connectMicDialog.dismiss();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                if (errorCode == ErrorcodeConstants.ErrorFailNetwork) {
                    ToastUtil.showToast(R.string.fail_network);
                } else {
                    if (!isFinishing() && connectMicDialog != null && connectMicDialog.isShowing()) {
                        connectMicDialog.dismiss();
                    }
                    RoomManager.getInstance().CommonErrorCodeToast(errorCode);
                }
            }
        });
    }

    /**
     * 网咯变更初始化
     */
    private void initNetwork() {

        NetWorkConManager.getInstance().registerNetworkCallback(getApplicationContext());

        NetWorkConManager.getInstance().setCallBack(new INetworkOperationCallback() {
            @Override
            public void audienceData(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo, int micType, boolean crashData) {
                //观众端
                if (audienceStatefulListInfo != null && crashData) {
                    mAudienceListAdapter.addData(audienceStatefulListInfo);
                }

                if (micType == RoomBottomLayerLayout.CONNECTION_MIC_PREPARE) {
                    mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
                    mRoomBottomLayerLayout.bottomLayerVisible(false);
                    RoomManager.getInstance().setHasConMic(false);
                } else if (micType == RoomBottomLayerLayout.CONNECTION_MIC_SUCCESS) {
                    mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_SUCCESS);
                    mRoomBottomLayerLayout.bottomLayerVisible(true);
                    ZegoSDKManager.getInstance().getDeviceService().setRoomData(true);
                    ZegoSDKManager.getInstance().getDeviceService().setFrontCamera(true);
                }
            }

            @Override
            public void applyMicStateNotify(int action, NotifyApplyMicStateInfo notifyInfo) {
                //连麦弹窗的处理
                if (action == Constants.OPERATION_ACTION_APPLY) {
                    onApplyMicStateNotify(action, notifyInfo);
                } else if (action == Constants.OPERATION_ACTION_CANCEL) {
                    ToastUtil.showToast(R.string.liveshow_connect_mic_cancle);
                    if (!isFinishing() && connectMicDialog != null && connectMicDialog.isShowing()) {
                        connectMicDialog.dismiss();
                    }
                }
            }

            @Override
            public void hostData(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo, boolean crashData) {
                //房主端
                if (audienceStatefulListInfo != null && crashData) {
                    mAudienceListAdapter.addData(audienceStatefulListInfo);
                }
            }

            @Override
            public void inviteMicAudienc(StatefulListInfo.AttendeeListDTO info) {

            }

            @Override
            public void onlineNum() {

            }
        });

    }

    /**
     * 数据转换（转成连麦申请的数据）
     *
     * @param info
     * @return
     */
    private NotifyApplyMicStateInfo gainMicStateInfo(NotifyUserListUpdateInfo.UsersDTO info) {
        NotifyApplyMicStateInfo notifyInfo = new NotifyApplyMicStateInfo();
        notifyInfo.setUid(info.getUid());
        notifyInfo.setNickName(info.getNickName());
        return notifyInfo;
    }

    private void settingBeauty() {
        EffectsBeautyDialog effectsBeautyDialog = new EffectsBeautyDialog();
        effectsBeautyDialog.show(getSupportFragmentManager(), effectsBeautyDialog.getTag());
    }

    @Override
    public void onInviteMicNotify(NotifyUserStateUpdateInfo notifyInfo) {

    }
}
