package im.zego.liveshow.liveroom;

import static androidx.viewpager2.widget.ViewPager2.SCROLL_STATE_IDLE;
import static im.zego.liveshow.Constants.ROOM_ROLE_AUDIENCE;
import static im.zego.liveshow.Constants.ROOM_ROLE_HOST;
import static im.zego.liveshow.liveroom.LiveRoomActivity.CREATE_ROOM;
import static im.zego.liveshow.liveroom.LiveRoomActivity.LOGIN_OR_CREATE_ROOM;
import static im.zego.liveshow.liveroom.LiveRoomActivity.LOGIN_ROOM;
import static java.security.AccessController.getContext;

import android.content.DialogInterface;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.Log;
import android.view.MotionEvent;
import android.view.TextureView;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.view.WindowManager;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;
import androidx.viewpager2.widget.ViewPager2;

import com.scwang.smart.refresh.layout.SmartRefreshLayout;

import java.util.ArrayList;
import java.util.List;


import im.zego.common.util.MiscUtils;
import im.zego.liveshow.R;

import im.zego.liveshow.effects.SoundEffectsManager;
import im.zego.liveshow.liveroom.adapter.LiveRoomAdapter;

import im.zego.liveshow.liveroom.utils.HeartBeatUtil;
import im.zego.liveshow.liveroom.view.HintCoverView;
import im.zego.liveshow.manager.NetWorkConManager;
import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.manager.UserManager;


import im.zego.liveshow.model.RoomListInfo;
import im.zego.liveshow.protocol.IExecuteCallback;

import im.zego.liveshow.protocol.IGetRoomListCallback;
import im.zego.liveshow.utils.DialogQueueUtils;
import im.zego.liveshow.utils.SharedPreferencesUtil;
import im.zego.liveshow.utils.ToastUtil;
import im.zego.liveshow.view.BaseInquiryDialog;


public class LiveRoomActivity2 extends AppCompatActivity {

    public static final String LOGIN_OR_CREATE_ROOM = "login_or_create";
    public static final int LOGIN_ROOM = 0;
    public static final int CREATE_ROOM = 1;

    public static final String SELECTION_ROOM_ID = "selection_room_id";

    private ViewPager2 vp2;
    private SmartRefreshLayout mSlideLayout;
    LiveRoomAdapter mAdapter;

    private HintCoverView hintCoverView;

    View mTopHint;
    View mBottomHint;
    private int mFirstPosition = DEFAULT_POSITION;
    private boolean mIsFirst = true;


    private int mAlreadyLoginPosition = DEFAULT_POSITION;

    private static final int DEFAULT_POSITION = -1;
    private int currentPosition = DEFAULT_POSITION;
    private int lastPosition = DEFAULT_POSITION;
    private int loginOrCreate ;

    private boolean mCloseRoom = false;

//    private boolean isShowHintCover = true;

    public static boolean isShowHintCover = false;
    public static boolean isShowEndHintCover = false;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        setContentView(R.layout.liveshow_activity_live_room2);
        initWindowBar();
        initView();
        initData();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if(!isFinishing()){
            HeartBeatUtil.uninit();
            NetWorkConManager.getInstance().clearData();
            DialogQueueUtils.getInstance().clear();
            SoundEffectsManager.getInstance().clearSoundEffectsData();
            RoomManager.getInstance().exit();
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        if(isFinishing()){
            HeartBeatUtil.uninit();
            NetWorkConManager.getInstance().clearData();
            DialogQueueUtils.getInstance().clear();
            SoundEffectsManager.getInstance().clearSoundEffectsData();
            RoomManager.getInstance().exit();
        }
    }

    private void initWindowBar() {
        WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.flags = lp.flags | WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS;
        getWindow().setAttributes(lp);
    }

    private void initView(){

        mTopHint = findViewById(R.id.top_hint);
        mBottomHint = findViewById(R.id.bottom_hint);

        ViewGroup.MarginLayoutParams marginLayoutParams = (ViewGroup.MarginLayoutParams)mTopHint .getLayoutParams();
        marginLayoutParams.setMargins(0, MiscUtils.getStatusBarHeight(this)+MiscUtils.dp2px(this,20),0,0);
        mTopHint.setLayoutParams(marginLayoutParams);

        mSlideLayout = findViewById(R.id.slide_layout);
        vp2 = findViewById(R.id.vp2);

        vp2.setOffscreenPageLimit(1);
        vp2.setOrientation(ViewPager2.ORIENTATION_VERTICAL);
        mAdapter = new LiveRoomAdapter();
        vp2.setAdapter(mAdapter);


        ((RecyclerView) vp2.getChildAt(0)).getLayoutManager().setItemPrefetchEnabled(false);
        ( (LinearLayoutManager)((RecyclerView) vp2.getChildAt(0)).getLayoutManager()).setSmoothScrollbarEnabled(false);
        ((RecyclerView) vp2.getChildAt(0)).setItemViewCacheSize(0);
        vp2.registerOnPageChangeCallback(new ViewPager2.OnPageChangeCallback() {
            @Override
            public void onPageSelected(int position) {
                super.onPageSelected(position);

//                lastPosition = currentPosition;
//                currentPosition = position;


                vp2.post(new Runnable() {
                    @Override
                    public void run() {
                        firstLogin();
                    }
                });
                if(position != currentPosition) {

                    if (hintCoverView != null ) {
                        ((ViewGroup) vp2.getRootView()).removeView(hintCoverView);
                        hintCoverView = null;

                        if(isShowHintCover) {
                            isShowHintCover = false;
                            SharedPreferencesUtil.setFirstEnterHintCover(false);
                        }else {
                            isShowEndHintCover = false;
                            SharedPreferencesUtil.setFirstOutSideHintCover(false);
                        }
                    }
                }

            }

            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels)
            {
                super.onPageScrolled(position, positionOffset, positionOffsetPixels);




                int positionHeight = (int) (positionOffsetPixels / positionOffset);
                if (currentPosition > position) {
                    //下滑
                    mAdapter.playStream(getViewByPosition(position),position,false);
                } else if(currentPosition != position || positionOffsetPixels != 0 ) {
                    //上滑
                    mAdapter.playStream(getViewByPosition(position+1),position+1,false);
                }else {
                    mAdapter.playStream(getViewByPosition(position),position,false);
                }

                if(currentPosition-1 > position){
                    ( (LinearLayoutManager)((RecyclerView) vp2.getChildAt(0)).getLayoutManager()).scrollToPosition(position+1);
                }

                if(currentPosition < position){
                    ( (LinearLayoutManager)((RecyclerView) vp2.getChildAt(0)).getLayoutManager()).scrollToPosition(position);
                }

            }



            @Override
            public void onPageScrollStateChanged(int state) {
                super.onPageScrollStateChanged(state);

                if(state == SCROLL_STATE_IDLE){
                    lastPosition = currentPosition;
                    currentPosition = vp2.getCurrentItem();
                    final List<RoomListInfo.RoomListDTO> data = mAdapter.getData();
                    final int size = data.size();



                    int otherPosition1 = -1;
                    int otherPosition2 = -1;
                    if(lastPosition > currentPosition){
                        otherPosition1 = lastPosition+1;
                        otherPosition2 = lastPosition;
                    }else if(lastPosition < currentPosition){
                        otherPosition1 = lastPosition-1;
                        otherPosition2 = lastPosition;
                    }else {
                        otherPosition1 = currentPosition + 1;
                        otherPosition2 = currentPosition -1;
                    }


                    if(otherPosition1 >=0 && otherPosition1 < size){
                        mAdapter.stopStream(getViewByPosition(otherPosition1),otherPosition1);
                    }

                    if(otherPosition2 >=0 && otherPosition2 < size){
                        mAdapter.stopStream(getViewByPosition(otherPosition2),otherPosition2);
                    }

                    mAdapter.playStream(getViewByPosition(currentPosition),currentPosition,true);

                    if(lastPosition != currentPosition && mAlreadyLoginPosition != currentPosition){
                        mAdapter.logoutRoom(getViewByPosition(lastPosition),lastPosition);
                        mAdapter.loginRoom(getViewByPosition(currentPosition),currentPosition);
                        mAlreadyLoginPosition = currentPosition;
                    }



                }

            }
        });


        loginOrCreate = getIntent().getIntExtra(LOGIN_OR_CREATE_ROOM, LOGIN_ROOM);
//        if (loginOrCreate == CREATE_ROOM) {

        mSlideLayout.setEnabled(false);
            //暂时关闭
//        }else {
//            mSlideLayout.setEnabled(true);
//        }

        vp2.setUserInputEnabled(false);

        mSlideLayout.setEnablePureScrollMode(true);
        mSlideLayout.setEnableLoadMore(true);
        mSlideLayout.setEnableOverScrollBounce(true);


    }

    private void initData(){
        isShowHintCover = false;
        isShowEndHintCover = false;
        if (loginOrCreate == CREATE_ROOM) {
            RoomListInfo.RoomListDTO data = new RoomListInfo.RoomListDTO();

            List<RoomListInfo.RoomListDTO> dataList = new ArrayList<>();
            dataList.add(data);
            mAdapter.refreshData(dataList);


            mFirstPosition = 0;
            lastPosition = currentPosition;
            currentPosition = 0;
            vp2.post(new Runnable() {
                @Override
                public void run() {
                    firstLogin();
                }
            });

        }else {

            RoomManager.getInstance().getRoomList(new IGetRoomListCallback() {
                @Override
                public void onGetRoomList(int code, RoomListInfo roomInfoList) {
                    final List<RoomListInfo.RoomListDTO> roomList = roomInfoList.getRoomList();
                    if(roomInfoList == null){
                        return;
                    }

                    mAdapter.refreshData(roomList);

                    int position = getPositionByRoomID(roomList, getIntent().getStringExtra(SELECTION_ROOM_ID));
                    if(position < 0){
                        finish();
                        ToastUtil.showToast(R.string.room_list_join_fail_common);
                        return;
                    }
                    if (position >= 0 && position < roomList.size()) {


                        //暂时关闭
//                        if(roomList.size() > 1 && SharedPreferencesUtil.getFirstEnterHintCover()){
//                            isShowHintCover = SharedPreferencesUtil.getFirstEnterHintCover();
//
//                            if(position == 0){
//                                showHintCover(HintCoverView.TYPE_UP, HintCoverView.TYPE_HINT_ENTER);
//                            }else if(position == (mAdapter.getData().size()-1)){
//                                showHintCover(HintCoverView.TYPE_DOWN, HintCoverView.TYPE_HINT_ENTER);
//                            }else {
//                                showHintCover(HintCoverView.TYPE_UP_AND_DOWN, HintCoverView.TYPE_HINT_ENTER);
//                            }
//                        }

                        mFirstPosition = position;
                        lastPosition = currentPosition;
                        currentPosition = position;
                        vp2.setCurrentItem(position, false);
                    }
                    mTopHint.setVisibility(View.VISIBLE);
                    mBottomHint.setVisibility(View.VISIBLE);
                }
            });
        }
    }

    @Override
    public void onBackPressed() {
        if (UserManager.getInstance().getUserInfo().getRole() == ROOM_ROLE_HOST) {
            BaseInquiryDialog baseInquiryDialog = new BaseInquiryDialog(LiveRoomActivity2.this, true);
            baseInquiryDialog.setMsgTitle(getString(R.string.liveshow_end_of_live_tip));
            baseInquiryDialog.setMsgContent(getString(R.string.liveshow_end_of_live_content));
            baseInquiryDialog.setLeftButtonContent(getString(R.string.liveshow_btn_cancle));
            baseInquiryDialog.setRightButtonContent(getString(R.string.liveshow_btn_true));
            baseInquiryDialog.setCancelListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    baseInquiryDialog.dismiss();
                }
            });
            baseInquiryDialog.setSureListener(v -> {

                if(mCloseRoom)
                {
                    return;
                }

                mCloseRoom = true;

                RoomManager.getInstance().closeRoom(new IExecuteCallback() {
                    @Override
                    public void onSuccess() {
                        if (!isFinishing() && !isDestroyed()) {
                            baseInquiryDialog.dismiss();
                        }
                        finish();
                    }

                    @Override
                    public void onFailed(int errorCode) {
                        if (!isFinishing() && !isDestroyed()) {
                            baseInquiryDialog.dismiss();
                        }
                        finish();
                    }
                });
            });
        }else if(UserManager.getInstance().getUserInfo().getRole() == ROOM_ROLE_AUDIENCE){

            //暂时关闭
//            if(SharedPreferencesUtil.getFirstOutSideHintCover() && mAdapter != null && mAdapter.getData() != null && mAdapter.getData().size() > 1 && (hintCoverView == null)) {
//                isShowEndHintCover = SharedPreferencesUtil.getFirstOutSideHintCover();
//                if(currentPosition == 0){
//                    showHintCover(HintCoverView.TYPE_UP, HintCoverView.TYPE_HINT_OUTSIDE);
//                }else if(currentPosition == (mAdapter.getData().size()-1)){
//                    showHintCover(HintCoverView.TYPE_DOWN, HintCoverView.TYPE_HINT_OUTSIDE);
//                }else {
//                    showHintCover(HintCoverView.TYPE_UP_AND_DOWN, HintCoverView.TYPE_HINT_OUTSIDE);
//                }

//            }else {
                super.onBackPressed();
//            }
        }else {
            super.onBackPressed();
        }
    }


    private int getPositionByRoomID(List<RoomListInfo.RoomListDTO> data,String roomID){
        if(data == null || data.size() <= 0 || TextUtils.isEmpty(roomID))
        {
            return -1;
        }
        int size = data.size();
        for(int i = 0; i < size ;i++){
            if(roomID.equals(data.get(i).getRoomId())){
                return i;
            }
        }
        return -1;
    }

    private View getViewByPosition(int position){
        return ((RecyclerView) vp2.getChildAt(0)).getLayoutManager().findViewByPosition(position);
    }

    private void firstLogin(){
        if(mIsFirst && mFirstPosition == currentPosition ){
            if(getViewByPosition(currentPosition) != null) {

                mIsFirst = false;
                mAdapter.loginRoom(getViewByPosition(currentPosition), currentPosition);
                mAdapter.playStream(getViewByPosition(currentPosition),currentPosition,true);
                mAlreadyLoginPosition = currentPosition;

            }else {

                vp2.post(new Runnable() {
                    @Override
                    public void run() {
                        firstLogin();
                    }
                });
            }
        }
    }

    public void enableSliding(boolean enable){
        //暂时停止滑动，后续移除
        enable = false;
        if(!this.isDestroyed() && !this.isFinishing() && vp2 != null){
            if(!enable){
                vp2.setCurrentItem(vp2.getCurrentItem());
                ( (LinearLayoutManager)((RecyclerView) vp2.getChildAt(0)).getLayoutManager()).scrollToPosition(vp2.getCurrentItem());
            }
            vp2.setUserInputEnabled(enable);
            mSlideLayout.setEnabled(enable);
        }
    }

    private void showHintCover(int type,int hintType){

        if(hintCoverView !=null && hintCoverView.isAttachedToWindow()){
            return;
        }

        hintCoverView = new HintCoverView(this);
        hintCoverView.setHintType(hintType);
        hintCoverView.setType(type);
        ((ViewGroup)vp2.getRootView()).addView(hintCoverView,new FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT,FrameLayout.LayoutParams.MATCH_PARENT));

        hintCoverView.setLeaveListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                hintCoverView = null;
                if(hintType == HintCoverView.TYPE_HINT_ENTER) {
                    isShowHintCover = false;
                    SharedPreferencesUtil.setFirstEnterHintCover(false);
                }else {
                    isShowEndHintCover = false;
                    SharedPreferencesUtil.setFirstOutSideHintCover(false);
                }

                onBackPressed();
            }
        });
    }
}
