package im.zego.liveshow.liveroom;

import android.app.Activity;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.net.Uri;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.TextureView;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.FragmentActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import java.util.List;

import im.zego.common.util.MiscUtils;
import im.zego.liveshow.Constants;
import im.zego.liveshow.ErrorcodeConstants;
import im.zego.liveshow.R;
import im.zego.liveshow.effects.SoundEffectsManager;
import im.zego.liveshow.liveroom.adapter.AudienceListAdapter;
import im.zego.liveshow.liveroom.callbacks.IAnchorInfoCallback;
import im.zego.liveshow.liveroom.callbacks.IHeartBeatFailCallback;
import im.zego.liveshow.liveroom.callbacks.IRoomQualityInfoCallback;
import im.zego.liveshow.liveroom.utils.HeartBeatUtil;
import im.zego.liveshow.liveroom.view.CloseRoomView;
import im.zego.liveshow.liveroom.view.HostInfoView;
import im.zego.liveshow.manager.NetWorkConManager;
import im.zego.liveshow.manager.ResolutionManager;
import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.manager.UserManager;
import im.zego.liveshow.manager.callbacks.ICreateRoomFailTimeoutCallback;
import im.zego.liveshow.manager.callbacks.INetworkOperationCallback;
import im.zego.liveshow.manager.callbacks.IRoomConnectionCallback;
import im.zego.liveshow.manager.callbacks.IRoomDisconnectCallback;
import im.zego.liveshow.manager.callbacks.IRoomManagerCallback;
import im.zego.liveshow.manager.callbacks.IStreamCallback;
import im.zego.liveshow.model.OnlineInfo;
import im.zego.liveshow.model.RoomInfo;
import im.zego.liveshow.model.RoomListInfo;
import im.zego.liveshow.model.StatefulListInfo;
import im.zego.liveshow.model.UserInfo;
import im.zego.liveshow.model.rtc.NotifyApplyMicStateInfo;
import im.zego.liveshow.model.rtc.NotifyRefuseApplyMicInfo;
import im.zego.liveshow.model.rtc.NotifyUserListUpdateInfo;
import im.zego.liveshow.model.rtc.NotifyUserStateUpdateInfo;
import im.zego.liveshow.protocol.IApplyMicStateNotifyCallback;
import im.zego.liveshow.protocol.IExecuteCallback;
import im.zego.liveshow.protocol.IGetHostAudiencetCallback;
import im.zego.liveshow.protocol.IGetOnlineInfoCallback;
import im.zego.liveshow.protocol.RoomAPI;
import im.zego.liveshow.sdk.ZegoSDKManager;
import im.zego.liveshow.sdk.callbacks.IRTCCommonCallback;
import im.zego.liveshow.utils.CheckDoubleClick;
import im.zego.liveshow.utils.DialogQueueUtils;
import im.zego.liveshow.utils.GlideLoader;
import im.zego.liveshow.utils.PermissionHelper;
import im.zego.liveshow.utils.ToastUtil;
import im.zego.liveshow.view.BaseInquiryDialog;
import im.zego.liveshow.view.EffectsBeautyDialog;
import im.zego.liveshow.view.LiveshowRealtimeLayout;
import im.zego.liveshow.view.MicManagerDialog;
import im.zego.liveshow.view.MoreSettingDialog;
import im.zego.liveshow.view.OnlineDialog;
import im.zego.liveshow.view.ProcessView;
import im.zego.liveshow.view.RoomBottomLayerLayout;
import im.zego.liveshow.view.SoundEffectsDialog;

import static im.zego.liveshow.Constants.OPERATION_TYPE_MIC;
import static im.zego.liveshow.Constants.ROOM_ROLE_AUDIENCE;
import static im.zego.liveshow.Constants.ROOM_ROLE_HOST;
import static im.zego.liveshow.ErrorcodeConstants.ERROR_ROOM_NOT_EXIST;

public class LiveRoomView extends FrameLayout implements MoreSettingDialog.ISettingMoreCallBack, IApplyMicStateNotifyCallback, IStreamCallback, IRoomManagerCallback {

    private static final String TAG = "LiveRoomView";

    public static final String LOGIN_OR_CREATE_ROOM = "login_or_create";
    public static final int LOGIN_ROOM = 0;
    public static final int CREATE_ROOM = 1;

    private ImageView mLiveRoomBg;
    private TextureView mLiveVideo;
    private ImageView mIvHostHead; //主播关闭摄像头的时候显示的高斯模糊头像
    private TextView mFinishRoomTips;
    private ConstraintLayout mCreateRoom;
    private ConstraintLayout mRoomLayer;

    private RecyclerView mAudienceList; //观众连麦列表
    private AudienceListAdapter mAudienceListAdapter;
    private RoomBottomLayerLayout mRoomBottomLayerLayout; //底部翻转/语言/连麦的按钮
    private ConstraintLayout mMoreSetting; //房主的更多设置按钮
    private ConstraintLayout mBeautySetting; //房主的美颜美型设置按钮
    private ConstraintLayout mMusicSetting; //房主的音效设置按钮

    private View mCreateResolution;
    private View mIvBack;
    private EditText mEditRoomName;
    private View mSelectBeauty;


    private View mCreateFlipCamera;
    private View mClStartLive;
//    private View mCloseRoom;
//    private TextView mTvOnlineNum;

//    private ImageView mAnchorHeadPortrait;

//    private TextView mAnchorName;

    private FrameLayout mCloseRoom;

    private FrameLayout mFlLiveVideo;

    private LiveshowRealtimeLayout streamQualityInfo;
    private ProcessView processView;

    private BaseInquiryDialog createRoomNetworkFailDialog;
    private BaseInquiryDialog heartBeatFailDialog;

    private boolean mIsMicMuted = true; //麦克风是否开启
    private boolean mIsCameraMuted = true; //摄像头是否开启

    private MoreSettingDialog moreSettingDialog; //设置弹窗
    private MicManagerDialog micManagerDialog; //主播对于上麦人员的管理
    private OnlineDialog onlineDialog; //在线人数弹窗
    private BaseInquiryDialog connectMicDialog;
    private BaseInquiryDialog inviteMicDialog;

    private HostInfoView hostInfoView;

    private CloseRoomView closeRoomView;

    private boolean showCreateRoomNetworkFail = false;

    private boolean mRoomClose = false;

    private boolean networkhasMineData = false; //断网重连过后标志数据是否包含有本人数据

    public LiveRoomView(Context context) {
        this(context, null);
    }

    public LiveRoomView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public LiveRoomView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        View view = View.inflate(context, R.layout.liveshow_view_live_room, this);
        initView(view);
    }

//    @Nullable
//    @Override
//    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
//
//        View view = inflater.inflate(R.layout.liveshow_activity_live_room,null,false);
//        initView(view);
//        initListener();
//        initData();
//        initRecycleView();
//        return view;
//
//    }
//
//    @Override
//    public void onDestroyView() {
//        super.onDestroyView();
//        DialogQueueUtils.getInstance().clear();
//        NetWorkConManager.getInstance().clearData();
//        HeartBeatUtil.uninit();
//        RoomManager.getInstance().exit();
//    }

    private void initView(View view) {
        mLiveRoomBg = view.findViewById(R.id.live_room_bg);
        mFlLiveVideo = view.findViewById(R.id.fl_live_video);
        mLiveVideo = view.findViewById(R.id.live_video);
        mIvHostHead = view.findViewById(R.id.iv_host_head);
        mFinishRoomTips = view.findViewById(R.id.finish_room_tips);
        mAudienceList = view.findViewById(R.id.audience_list);
        mRoomBottomLayerLayout = view.findViewById(R.id.bottom_layer);
        mCreateRoom = view.findViewById(R.id.create_room);
        mRoomLayer = view.findViewById(R.id.room_layer);
        mSelectBeauty = findViewById(R.id.select_beauty);
        mCreateResolution = view.findViewById(R.id.create_resolution);
        mIvBack = view.findViewById(R.id.iv_back);
        mCreateFlipCamera = view.findViewById(R.id.create_flip_camera);
        mEditRoomName = view.findViewById(R.id.edit_room_name);
        mClStartLive = view.findViewById(R.id.cl_start_live);
        mMoreSetting = view.findViewById(R.id.cl_more_setting);
//        mCloseRoom = view.findViewById(R.id.close_room);
        streamQualityInfo = view.findViewById(R.id.stream_quality_info);
        processView = new ProcessView(getContext());
        mCloseRoom = view.findViewById(R.id.close_room);

        mMusicSetting = findViewById(R.id.cl_music_setting);
        mBeautySetting = findViewById(R.id.cl_beauty_setting);
//        mAnchorHeadPortrait = view.findViewById(R.id.anchor_head_portrait);
//        mAnchorName = view.findViewById(R.id.anchor_name);
//        mTvOnlineNum = view.findViewById(R.id.tv_online_num);
    }

    private void initData() {


        if (!ZegoSDKManager.getInstance().isInit()) {
            getActivity().finish();
            return;
        }


    }


    private void showCreateRoom() {

        mCreateRoom.setVisibility(View.VISIBLE);
        mRoomLayer.setVisibility(View.GONE);
        ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) mCreateRoom.getLayoutParams();
        params.topMargin = MiscUtils.getStatusBarHeight(getContext());
        mCreateRoom.setLayoutParams(params);
    }

    private void showLiveRoom() {
        if (RoomManager.getInstance().getRoomID() == null || UserManager.getInstance().getUserInfo() == null) {
            getActivity().finish();
        }

        UserInfo userInfo = UserManager.getInstance().getUserInfo();

        mCreateRoom.setVisibility(View.GONE);
        mRoomLayer.setVisibility(View.VISIBLE);

        ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) mRoomLayer.getLayoutParams();
        params.topMargin = MiscUtils.getStatusBarHeight(getContext());
        mRoomLayer.setLayoutParams(params);

        hostInfoView = new HostInfoView(getContext());
        mRoomLayer.addView(hostInfoView, new ConstraintLayout.LayoutParams(ConstraintLayout.LayoutParams.MATCH_PARENT, ConstraintLayout.LayoutParams.MATCH_PARENT));


        initOtherListener();

        if (userInfo.getRole() == ROOM_ROLE_AUDIENCE) {
            //进入房间的是观众

            mRoomBottomLayerLayout.post(new Runnable() {
                @Override
                public void run() {
                    mRoomBottomLayerLayout.setVisibility(View.VISIBLE);
                    mMoreSetting.setVisibility(View.GONE);
                    mBeautySetting.setVisibility(View.GONE);
                    mMusicSetting.setVisibility(View.GONE);
                    mRoomBottomLayerLayout.bottomLayerVisible(false);
                }
            });


        } else if (userInfo.getRole() == ROOM_ROLE_HOST) {
            //进入房间的是主持人
            mRoomBottomLayerLayout.post(new Runnable() {
                @Override
                public void run() {
                    mMoreSetting.setVisibility(View.VISIBLE);
                    mBeautySetting.setVisibility(View.VISIBLE);
                    mMusicSetting.setVisibility(View.VISIBLE);
                    mRoomBottomLayerLayout.setVisibility(View.GONE);
                }
            });

        }

        RoomManager.getInstance().setConnectionMicListener(this);
        RoomManager.getInstance().setStreamStatusListener(this);
        RoomManager.getInstance().setRoomDisconnectCallback(new IRoomDisconnectCallback() {
            @Override
            public void onDisconnect() {
                getActivity().finish();
            }
        });

        //设置房间人数信息
        setOnlineNum();

        getStatefulList();
        HeartBeatUtil.init(UserManager.getInstance().getUserId(), RoomManager.getInstance().getRoomID());
        HeartBeatUtil.setHeartBeatFailCallback(new IHeartBeatFailCallback() {
            @Override
            public void onHeartBeatFail() {
                if (mRoomClose) {
                    return;
                }

                if (heartBeatFailDialog != null && heartBeatFailDialog.isShowing()) {
                    return;
                }

                heartBeatFailDialog = new BaseInquiryDialog(getActivity());
                heartBeatFailDialog.setMsgTitle(getActivity().getString(R.string.liveshow_fail_network));
                heartBeatFailDialog.setMsgContent(getActivity().getString(R.string.liveshow_network_connect_fail));
                heartBeatFailDialog.setLeftButtonContent(getActivity().getString(R.string.liveshow_exit));
                heartBeatFailDialog.setRightButtonContent(getActivity().getString(R.string.liveshow_again));
                heartBeatFailDialog.setCancelListener(new OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        if (heartBeatFailDialog != null && !getActivity().isFinishing() && !getActivity().isDestroyed()) {
                            heartBeatFailDialog.dismiss();
                        }
                        getActivity().finish();
                    }
                });
                heartBeatFailDialog.setSureListener(v -> {

                    HeartBeatUtil.userRetry(0);

                });
            }

            @Override
            public void onHeartBeatDisconnect(int errorCode) {

                RoomManager.getInstance().CommonErrorCodeToast(errorCode);
                if (heartBeatFailDialog != null && !getActivity().isFinishing() && !getActivity().isDestroyed()) {
                    heartBeatFailDialog.dismiss();
                }
                getActivity().finish();
            }

            @Override
            public void onHeartBeatSuccess() {
                Log.e("HeartBeatUtil", "onHeartBeatSuccess:");
                if (heartBeatFailDialog != null && !getActivity().isFinishing() && !getActivity().isDestroyed()) {
                    Log.e("HeartBeatUtil", "heartBeatFailDialog:");
                    heartBeatFailDialog.dismiss();
                }
            }

            @Override
            public void onHeartBeatUserRetryFail(int errorCode) {
                RoomManager.getInstance().CommonErrorCodeToast(errorCode);

                if (errorCode == ErrorcodeConstants.ERROR_USER_NOT_IN_ROOM) {
                    if (heartBeatFailDialog != null && !getActivity().isFinishing() && !getActivity().isDestroyed()) {
                        heartBeatFailDialog.dismiss();
                    }
                    getActivity().finish();
                }
            }
        });

        HeartBeatUtil.setStateSeqCallback(seq -> NetWorkConManager.getInstance().setPersonalStateSeq(seq));

        RoomManager.getInstance().setRoomConnectionCallback(new IRoomConnectionCallback() {
            @Override
            public void onConnected() {
                HeartBeatUtil.retry(0);
            }

            @Override
            public void connecting() {

            }
        });

        initNetwork();

        //copy 音乐特效的资料
        SoundEffectsManager.getInstance().initLocalAudioEffectlist(getActivity());
    }

    /**
     * 观众连麦列表
     */
    private void initRecycleView() {
        mAudienceListAdapter = new AudienceListAdapter(getContext());
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getContext());
        mAudienceList.setLayoutManager(linearLayoutManager);
        mAudienceList.setAdapter(mAudienceListAdapter);
        //解决刷新闪烁问题
        ((SimpleItemAnimator) mAudienceList.getItemAnimator()).setSupportsChangeAnimations(false);

        mAudienceListAdapter.setUserStageListener(fromType -> {
            if (fromType == Constants.STATE_OFF_STAGE) {
                //下麦成功后显示
                ((LiveRoomActivity2) getActivity()).enableSliding(true);
                mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
                mRoomBottomLayerLayout.bottomLayerVisible(false);
                mRoomBottomLayerLayout.switchCamera(false);
                mRoomBottomLayerLayout.switchMic(false);
            } else if (fromType == Constants.STATE_ON_STAGE) {
                //成功连麦显示
                ((LiveRoomActivity2) getActivity()).enableSliding(false);
                mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_SUCCESS);
                mRoomBottomLayerLayout.bottomLayerVisible(true);
                ZegoSDKManager.getInstance().getDeviceService().setRoomData(true);
                ZegoSDKManager.getInstance().getDeviceService().setFrontCamera(true);
                RoomManager.getInstance().setHasConMic(false);
            }
        });

        mAudienceListAdapter.setUserMicListener(new AudienceListAdapter.UserMicListener() {
            @Override
            public void onUserMicStatus(StatefulListInfo.AttendeeListDTO userInfo) {
                //房主的更多设置弹窗
                if (micManagerDialog == null) {
                    micManagerDialog = new MicManagerDialog();
                }
                if (!micManagerDialog.isAdded()) {
                    micManagerDialog.show(getActivity().getSupportFragmentManager(), micManagerDialog.getTag());
                    micManagerDialog.setUserInfo(userInfo);
                }
            }

            @Override
            public void onUserMicOff() {
                mRoomBottomLayerLayout.switchMic(true);
            }

            @Override
            public void onUserMicOn() {
                mRoomBottomLayerLayout.switchMic(false);
            }

            @Override
            public void onHostDialogMic(int micStatus) {
                if (micManagerDialog != null) {
                    micManagerDialog.setMic(micStatus);
                }
            }
        });
    }

    private void initListener() {
        mIvBack.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (getActivity() != null) {
                    getActivity().onBackPressed();
                }
            }
        });

        mCreateResolution.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                ResolutionManager.getInstance().showBottomSheetDialog(getActivity());
            }
        });
        mCreateFlipCamera.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                setHostFront();
            }
        });

        mSelectBeauty.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                settingBeauty();
            }
        });

        mEditRoomName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                if (TextUtils.isEmpty(s.toString().trim())) {
                    mClStartLive.setAlpha(0.5f);
                } else {
                    mClStartLive.setAlpha(1f);
                }
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        mClStartLive.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                final String roomName = mEditRoomName.getText().toString().trim();
                if (TextUtils.isEmpty(roomName)) {
                    return;
                }
                processView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mClStartLive.getRootView());
                RoomManager.getInstance().createRoom(roomName, new IRTCCommonCallback() {
                    @Override
                    public void onRTCCallback(int errorCode) {
                        if (errorCode == 0) {
                            showCreateRoomNetworkFail = false;
                            getActivity().runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    processView.dismiss();
                                    showLiveRoom();
                                    if (createRoomNetworkFailDialog != null && !getActivity().isDestroyed() && !getActivity().isFinishing()) {
                                        createRoomNetworkFailDialog.dismiss();
                                        createRoomNetworkFailDialog = null;
                                    }
                                }
                            });
                            RoomManager.getInstance().startHostPublishStream(mLiveVideo, null);

                        } else {
                            processView.dismiss();

                            if (showCreateRoomNetworkFail) {

                                showCreateRoomNetworkFailDialog();

                            }
                        }
                    }
                });

            }
        });
        mBeautySetting.setOnClickListener(v -> settingBeauty());
        mMusicSetting.setOnClickListener(v -> {
            if (CheckDoubleClick.isFastDoubleClick()) {
                return;
            }
            SoundEffectsDialog soundEffectsDialog = new SoundEffectsDialog();
            soundEffectsDialog.show(getActivity().getSupportFragmentManager(), soundEffectsDialog.getTag());
        });
        mMoreSetting.setOnClickListener(v -> {
            if (CheckDoubleClick.isFastDoubleClick()) {
                return;
            }
            //房主的更多设置弹窗
            if (moreSettingDialog == null) {
                moreSettingDialog = new MoreSettingDialog();
                moreSettingDialog.setCallBack(this);
            }
            if (!moreSettingDialog.isAdded()) {
                moreSettingDialog.show(getActivity().getSupportFragmentManager(), moreSettingDialog.getTag());
                moreSettingDialog.setMicAndCamara(mIsMicMuted, mIsCameraMuted);
            }
        });

        //观众操作摄像头/麦克风/下麦的回调
        mRoomBottomLayerLayout.setCameraMicListener((mIsMicMuted, mIsCameraMuted, type, operator) -> {
            httpSetStatus(mIsMicMuted, mIsCameraMuted, type, operator);
        });


        streamQualityInfo.setCloseListener(new LiveshowRealtimeLayout.CloseListener() {
            @Override
            public void close() {
                streamQualityInfo.setVisibility(View.GONE);
            }
        });

        RoomManager.getInstance().setRoomQualityInfoCallback(new IRoomQualityInfoCallback() {
            @Override
            public void onRoomQualityInfoUpdate(double kbps, double fps, int rtt, double packetLostRate, String rxSpeed, String txSpeed) {
                getActivity().runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        streamQualityInfo.setKbs((int) kbps);
                        streamQualityInfo.setFps((int) fps);
                        streamQualityInfo.setDelay(rtt);
                        streamQualityInfo.setPacketLoss((int) packetLostRate);
                        streamQualityInfo.setDownlinkNetwork(rxSpeed);
                        streamQualityInfo.seUplinkNetwork(txSpeed);
                    }
                });
            }
        });

        RoomManager.getInstance().setAnchorInfoCallback(new IAnchorInfoCallback() {
            @Override
            public void onAnchorInfoUpdate(UserInfo userInfo) {

                if (hostInfoView != null) {
                    hostInfoView.update(userInfo);
                }
//                GlideLoader.displayCircleImageByAvatar(userInfo.getAvatar(), mAnchorHeadPortrait);
//                mAnchorName.setText(userInfo.getNickName());
                setHostCameraImg(userInfo.getCamera(), userInfo.getAvatar());
            }
        });

        RoomManager.getInstance().setCreateRoomFailTimeoutCallback(new ICreateRoomFailTimeoutCallback() {
            @Override
            public void onCreateRoomFailTimeout() {
                showCreateRoomNetworkFail = true;
                showCreateRoomNetworkFailDialog();
            }
        });
    }

    private void initOtherListener() {
        hostInfoView.setCloseListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (getActivity() != null) {
                    getActivity().onBackPressed();
                }
            }
        });

        hostInfoView.setOnlineListener(v -> {
            if (CheckDoubleClick.isFastDoubleClick()) {
                return;
            }
            if (onlineDialog == null) {
                onlineDialog = new OnlineDialog();
            }
            if (!onlineDialog.isAdded()) {
                onlineDialog.show(getActivity().getSupportFragmentManager(), onlineDialog.getTag());
            }
        });
    }

    private void showCreateRoomNetworkFailDialog() {
        if (createRoomNetworkFailDialog != null) {
            return;
        }

        createRoomNetworkFailDialog = new BaseInquiryDialog(getActivity());
        createRoomNetworkFailDialog.setMsgTitle(getActivity().getString(R.string.liveshow_fail_network));
        createRoomNetworkFailDialog.setMsgContent(getActivity().getString(R.string.liveshow_network_connect_fail));
        createRoomNetworkFailDialog.setLeftButtonContent(getActivity().getString(R.string.liveshow_exit));
        createRoomNetworkFailDialog.setRightButtonContent(getActivity().getString(R.string.liveshow_again));
        createRoomNetworkFailDialog.setCancelListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                showCreateRoomNetworkFail = false;
                createRoomNetworkFailDialog.dismiss();
                createRoomNetworkFailDialog = null;
                getActivity().finish();
            }
        });
        createRoomNetworkFailDialog.setSureListener(v -> {
            createRoomNetworkFailDialog.dismiss();
            createRoomNetworkFailDialog = null;
            mClStartLive.performClick();
        });
    }

    /**
     * 设置摄像头、麦克风、下麦
     *
     * @param mic    1：关闭  2:打开
     * @param camera 1：关闭  2:打开
     * @param type   1.mic操作  2. camera操作 3. 关闭连麦
     */
    private void httpSetStatus(boolean mic, boolean camera, int type, int operation) {
        RoomManager.getInstance().setStatus(mic, camera, type, new IExecuteCallback() {
            @Override
            public void onSuccess() {
                //是房主关闭摄像头后，显示默认图片
                if (operation == ROOM_ROLE_HOST && type == Constants.OPERATION_TYPE_CAMERA) {
                    if (camera) {
                        mLiveVideo.setVisibility(View.VISIBLE);
                        mIvHostHead.setVisibility(View.GONE);
                        mFinishRoomTips.setVisibility(View.GONE);
                    } else {
                        mLiveVideo.setVisibility(View.GONE);
                        mIvHostHead.setVisibility(View.VISIBLE);
                        mFinishRoomTips.setVisibility(View.VISIBLE);
                        StatefulListInfo.AttendeeListDTO hostInfo = RoomManager.getInstance().getHostInfo();
                        String avatar = hostInfo == null ? "1" : hostInfo.getAvatar();
                        GlideLoader.displayRoundImageByAvatar(avatar, mIvHostHead);
                    }
                } else if (operation == ROOM_ROLE_AUDIENCE && type == Constants.OPERATION_TYPE_STAGE_STATE) {
                    //下麦成功
                    RoomManager.getInstance().setHasConMic(false);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                if (type == Constants.OPERATION_TYPE_MIC) {
                    //麦克风
                    if (operation == ROOM_ROLE_AUDIENCE) {
                        mRoomBottomLayerLayout.switchMic(mRoomBottomLayerLayout.ismIsMicMuted()); //观众
                    } else if (operation == ROOM_ROLE_HOST) {
                        //房主
                        mIsMicMuted = !mIsMicMuted;
                        moreSettingDialog.setMic(mIsMicMuted);
                    }

                } else if (type == Constants.OPERATION_TYPE_CAMERA) {
                    //摄像头
                    if (operation == ROOM_ROLE_AUDIENCE) {
                        //观众
                        mRoomBottomLayerLayout.switchCamera(mRoomBottomLayerLayout.ismIsCameraMuted());
                    } else if (operation == ROOM_ROLE_HOST) {
                        //房主
                        mIsCameraMuted = !mIsCameraMuted;
                        moreSettingDialog.setCamara(mIsCameraMuted);
                    }

                }
            }
        });
    }

    /**
     * 拉取连麦人员列表
     */
    private void getStatefulList() {

        if (RoomManager.getInstance().getRoomInfo() == null) {
            return;
        }

        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();

        RoomManager.getInstance().getStatefulList(roomInfo.getRoomId(), userInfo.getUid(), new IGetHostAudiencetCallback() {
            @Override
            public void onGetStatefulData(int errorCode, StatefulListInfo.AttendeeListDTO hostInfo, List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo,
                                          StatefulListInfo statefulListInfo, int attendeeSeq, int stateSeq) {
                //当前是房主自己的页面不需要拉流显示，只需要预览就可以
                if (UserManager.getInstance().getUserInfo().getRole() == ROOM_ROLE_AUDIENCE) {
                    RoomManager.getInstance().startPlayStream(hostInfo, mLiveVideo, null);
                }

                if (audienceStatefulListInfo != null && audienceStatefulListInfo.size() > 0) {
                    mAudienceListAdapter.addData(audienceStatefulListInfo);
                }

                NetWorkConManager.getInstance().setStartFirst(false);
                NetWorkConManager.getInstance().setPersonalStateSeq(stateSeq);
            }
        });
    }

    /**
     * 连麦通知回调（申请连麦、取消申请连麦）
     */
    @Override
    public void onApplyMicStateNotify(int action, NotifyApplyMicStateInfo notifyInfo) {
        if (action == Constants.OPERATION_ACTION_APPLY) {
            if (!getActivity().isFinishing()) {
                DialogQueueUtils dialogQueueUtils = DialogQueueUtils.getInstance();
                if (dialogQueueUtils.canShow()) {
                    dialogQueueUtils.addData(notifyInfo);
                    setMicStateListener();
                    dialogQueueUtils.showDialog();
                } else {
                    dialogQueueUtils.addData(notifyInfo);
                }
            }
        } else if (action == Constants.OPERATION_ACTION_CANCEL) {
            //取消申请连麦 (当前显示的弹窗正好是取消连麦的通知，就toast提示并且弹窗取消。否则就在队列里面移除)
            NotifyApplyMicStateInfo dialogInfo = DialogQueueUtils.getInstance().getmNotifyApplyMicStateInfo();
            if (dialogInfo != null && notifyInfo.getUid() == dialogInfo.getUid()) {
                ToastUtil.showToast(R.string.liveshow_connect_mic_cancle);
                if (!getActivity().isFinishing() && connectMicDialog != null && connectMicDialog.isShowing()) {
                    connectMicDialog.dismiss();
                }
            } else {
                DialogQueueUtils.getInstance().remove(notifyInfo);
            }
        }

    }

    /**
     * 主播拒绝你的连麦回调
     *
     * @param notifyInfo
     */
    @Override
    public void onRefuseApplyMicNotify(NotifyRefuseApplyMicInfo notifyInfo) {
        mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
    }

    /**
     * 3.连麦状态变化 连麦的上下台
     */
    @Override
    public void onMyStageNotify(NotifyUserStateUpdateInfo.UsersDTO notifyInfo) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mAudienceListAdapter.onStage(notifyInfo);
            }
        });

    }

    /**
     * 连麦列表的状态变更 1.mic变化 2.cameara
     */
    @Override
    public void onAudienceListNotify(NotifyUserStateUpdateInfo onstageList) {

        if (RoomManager.getInstance().getRoomInfo() == null) {
            return;
        }

        StatefulListInfo.AttendeeListDTO hostInfo = RoomManager.getInstance().getHostInfo();
        long userId = RoomManager.getInstance().getRoomInfo().getRoomInfo().getHostId();
        long hostId = hostInfo != null ? hostInfo.getUid() : userId;

        if (onstageList.getOperatorUid() == hostId) {
            if (onstageList.getType() == Constants.OPERATION_TYPE_CAMERA) {
                String avatar = hostInfo == null ? "1" : hostInfo.getAvatar();
                setHostCameraImg(onstageList.getUsers().get(0).getCamera(), avatar);
            } else if (onstageList.getType() == Constants.OPERATION_TYPE_MIC) {
                getActivity().runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        mAudienceListAdapter.refreshData(onstageList);
                    }
                });
            }
        } else {
            getActivity().runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mAudienceListAdapter.refreshData(onstageList);
                }
            });
        }

    }

    /**
     * 上台流添加回调
     */
    @Override
    public void onStreamAdd() {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mAudienceListAdapter.notifyDataSetChanged();
            }
        });
    }

    /**
     * 成员列表变更通知
     */
    @Override
    public void onUserListUpdateNotify(NotifyUserListUpdateInfo notifyInfo) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {

                if (RoomManager.getInstance().getRoomInfo() == null) {
                    return;
                }

                if (hostInfoView != null) {
                    hostInfoView.updateOnlineNum(String.valueOf(notifyInfo.getOnlineNum()));
                }
                mAudienceListAdapter.exitRoomData(notifyInfo);

                //房间用户变更后实时更新弹窗的数据
                if (onlineDialog != null && onlineDialog.isVisible()) {
                    onlineDialog.getAttendeeList();
                }

                //获取当前退出房间的人,并判断有你没有申请连麦。申请就取消弹窗
                NotifyApplyMicStateInfo dialogInfo = DialogQueueUtils.getInstance().getmNotifyApplyMicStateInfo();
                UserInfo myInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
                if (dialogInfo != null && myInfo.getRole() == ROOM_ROLE_HOST) {
                    for (int i = 0; i < notifyInfo.getUsers().size(); i++) {
                        NotifyUserListUpdateInfo.UsersDTO getUsers = notifyInfo.getUsers().get(i);
                        if (getUsers.getDelta() == -1) {
                            onApplyMicStateNotify(Constants.OPERATION_ACTION_CANCEL, gainMicStateInfo(getUsers));
                        }
                    }
                }

            }
        });
    }

    @Override
    public void onRoomCloseNotify() {
        if (!getActivity().isFinishing() && UserManager.getInstance().getUserInfo().getRole() != ROOM_ROLE_HOST) {
            mRoomClose = true;
//            BaseInquiryDialog baseInquiryDialog = new BaseInquiryDialog(getActivity(), R.layout.liveshow_dialog_room_finish);
//            baseInquiryDialog.setSureListener(v -> {
//                baseInquiryDialog.dismiss();
//                getActivity().finish();
//            });
            showCloseRoom();
        }
    }

    @Override
    public void onClickFlip() {
        //房主-设置-翻转
        setHostFront();
    }

    @Override
    public void onClickCamera() {
        //房主-设置-摄像头 onResponse
        mIsCameraMuted = !mIsCameraMuted;
        moreSettingDialog.setCamara(mIsCameraMuted);
        httpSetStatus(mIsMicMuted, mIsCameraMuted, Constants.OPERATION_TYPE_CAMERA, ROOM_ROLE_HOST);
    }

    @Override
    public void onClickMic() {
        //房主-设置-麦克风
        mIsMicMuted = !mIsMicMuted;
        moreSettingDialog.setMic(mIsMicMuted);
        httpSetStatus(mIsMicMuted, mIsCameraMuted, Constants.OPERATION_TYPE_MIC, ROOM_ROLE_HOST);
    }

    @Override
    public void onClickResolution() {
        //房主-设置-分辨率
        ResolutionManager.getInstance().showBottomSheetDialog(getActivity());
    }

    @Override
    public void onClickData() {
        //房主-设置-实时数据
        if (streamQualityInfo.getVisibility() == View.VISIBLE) {
            streamQualityInfo.setVisibility(View.GONE);
        } else {
            streamQualityInfo.setVisibility(View.VISIBLE);
        }
    }

    /**
     * 主播是否关闭摄像头-关闭就显示高斯模糊头像
     *
     * @param camera
     * @param avatar
     */
    private void setHostCameraImg(int camera, String avatar) {
        boolean isOpen = camera == Constants.MIC_CAMERA_OPEN;
        mLiveVideo.setVisibility(isOpen ? View.VISIBLE : View.GONE);
        mIvHostHead.setVisibility(isOpen ? View.GONE : View.VISIBLE);
        mFinishRoomTips.setVisibility(isOpen ? View.GONE : View.VISIBLE);
        if (!isOpen) {
            GlideLoader.displayRoundImageByAvatar(avatar, mIvHostHead);
        }
    }


//    @Override
//    public void onBackPressed() {
//        if (UserManager.getInstance().getUserInfo().getRole() == ROOM_ROLE_HOST) {
//
//            BaseInquiryDialog baseInquiryDialog = new BaseInquiryDialog(getActivity(), true);
//            baseInquiryDialog.setMsgTitle(getString(R.string.liveshow_end_of_live_tip));
//            baseInquiryDialog.setMsgContent(getString(R.string.liveshow_end_of_live_content));
//            baseInquiryDialog.setLeftButtonContent(getString(R.string.liveshow_btn_cancle));
//            baseInquiryDialog.setRightButtonContent(getString(R.string.liveshow_btn_true));
//            baseInquiryDialog.setCancelListener(new View.OnClickListener() {
//                @Override
//                public void onClick(View v) {
//                    baseInquiryDialog.dismiss();
//                }
//            });
//            baseInquiryDialog.setSureListener(v -> {
//                RoomManager.getInstance().closeRoom(new IExecuteCallback() {
//                    @Override
//                    public void onSuccess() {
//                        if (!getActivity().isFinishing() && !getActivity().isDestroyed()) {
//                            baseInquiryDialog.dismiss();
//                        }
//                        getActivity().finish();
//                    }
//
//                    @Override
//                    public void onFailed(int errorCode) {
//                        if (!getActivity().isFinishing() && !getActivity().isDestroyed()) {
//                            baseInquiryDialog.dismiss();
//                        }
//                        getActivity().finish();
//                    }
//                });
//            });
//        } else {
//            super.onBackPressed();
//        }
//    }

    /**
     * 主播设置摄像头是否前置
     */
    private void setHostFront() {
        RoomManager.getInstance().setFront(!RoomManager.getInstance().isFront());
    }

    /**
     * 接受/拒绝 连麦弹窗监听
     */
    private void setMicStateListener() {
        DialogQueueUtils.getInstance().setDialogListener(new DialogQueueUtils.DialogListener() {
            @Override
            public void onData(NotifyApplyMicStateInfo mNotifyApplyMicStateInfo) {
                connectMicDialog = new BaseInquiryDialog(getActivity(), R.layout.liveshow_inquiry_dialog);
                connectMicDialog.setMsgTitle(getActivity().getString(R.string.liveshow_apply_connect_mic));
                connectMicDialog.setMsgContent(String.format(getActivity().getString(R.string.liveshow_apply_connect_mic_content), mNotifyApplyMicStateInfo.getNickName()));
                connectMicDialog.setLeftButtonContent(getActivity().getString(R.string.liveshow_btn_refuse));
                connectMicDialog.setRightButtonContent(getActivity().getString(R.string.liveshow_btn_agree));
                connectMicDialog.setSureListener(v -> {
                    //接受申请
                    applyMicStateNotifyRequest(Constants.OPERATION_ACTION_ACCEPT, mNotifyApplyMicStateInfo);
                });
                connectMicDialog.setCancelListener(v -> {
                    //拒绝申请
                    applyMicStateNotifyRequest(Constants.OPERATION_ACTION_REFUSE, mNotifyApplyMicStateInfo);
                });

                connectMicDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
                    @Override
                    public void onDismiss(DialogInterface dialog) {
                        //这边设置了dismiss监听,在监听回调中再次调用show方法,可以获取下一个弹窗
                        DialogQueueUtils.getInstance().setInfoNull();
                        DialogQueueUtils.getInstance().showDialog();
                    }
                });

            }
        });
    }

    /**
     * 连麦弹窗的接受或者拒绝
     */
    private void applyMicStateNotifyRequest(int action, NotifyApplyMicStateInfo notifyInfo) {
        RoomManager.getInstance().applyMicStateNotifyRequest(action, notifyInfo, new IExecuteCallback() {
            @Override
            public void onSuccess() {
                if (!getActivity().isFinishing() && connectMicDialog != null && connectMicDialog.isShowing()) {
                    connectMicDialog.dismiss();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                if (errorCode == ErrorcodeConstants.ErrorFailNetwork) {
                    ToastUtil.showToast(R.string.fail_network);
                } else {
                    if (!getActivity().isFinishing() && connectMicDialog != null && connectMicDialog.isShowing()) {
                        connectMicDialog.dismiss();
                    }
                    RoomManager.getInstance().CommonErrorCodeToast(errorCode);
                }
            }
        });
    }

    /**
     * 网咯变更初始化
     */
    private void initNetwork() {

        UserInfo myInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        NetWorkConManager.getInstance().registerNetworkCallback(getContext());

        NetWorkConManager.getInstance().setCallBack(new INetworkOperationCallback() {
            @Override
            public void audienceData(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo, int micType, boolean crashData) {
                networkhasMineData = false;
                //观众端
                if (audienceStatefulListInfo != null && crashData) {
                    mAudienceListAdapter.addData(audienceStatefulListInfo);
                }

                if (micType == RoomBottomLayerLayout.CONNECTION_MIC_PREPARE) {
                    mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
                    mRoomBottomLayerLayout.bottomLayerVisible(false);
                    RoomManager.getInstance().setHasConMic(false);
                } else if (micType == RoomBottomLayerLayout.CONNECTION_MIC_SUCCESS) {
                    mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_SUCCESS);
                    mRoomBottomLayerLayout.bottomLayerVisible(true);
                    ZegoSDKManager.getInstance().getDeviceService().setRoomData(true);
                    ZegoSDKManager.getInstance().getDeviceService().setFrontCamera(true);
                }

                for (int i = 0; i < audienceStatefulListInfo.size(); i++) {
                    StatefulListInfo.AttendeeListDTO info = audienceStatefulListInfo.get(i);
                    if (info.getUid() == myInfo.getUid()) {
                        networkhasMineData = true;
                        if (info.getOnstageState() == Constants.STATE_ON_STAGE) {
                            //为了解决 观众端断网过后，主播禁言的操作，观众端恢复网络后底部麦克风按钮没有变更的问题
                            mAudienceListAdapter.micStatus(info.getMic());
                        }
                    }
                }



                //主要是解决 观众端断网过后，主播移下麦，众端恢复网络过后底部按钮没有恢复
                if (!networkhasMineData) {
                    mRoomBottomLayerLayout.bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
                    mRoomBottomLayerLayout.bottomLayerVisible(false);
                }

            }

            @Override
            public void applyMicStateNotify(int action, NotifyApplyMicStateInfo notifyInfo) {
                //连麦弹窗的处理
                if (action == Constants.OPERATION_ACTION_APPLY) {
                    onApplyMicStateNotify(action, notifyInfo);
                } else if (action == Constants.OPERATION_ACTION_CANCEL) {
                    ToastUtil.showToast(R.string.liveshow_connect_mic_cancle);
                    if (!getActivity().isFinishing() && connectMicDialog != null && connectMicDialog.isShowing()) {
                        connectMicDialog.dismiss();
                    }
                }
            }

            @Override
            public void hostData(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo, boolean crashData) {
                //房主端
                if (audienceStatefulListInfo != null && crashData) {
                    mAudienceListAdapter.addData(audienceStatefulListInfo);

                    //解决 房主邀请连麦过后断网，然后观众拒绝后房主恢复网络后邀请按钮没有变更
                    if (onlineDialog != null && onlineDialog.isVisible()) {
                        onlineDialog.refuseToast(audienceStatefulListInfo);
                    }

                    //解决主播弹起麦位管理的弹窗后断网，观众操作了麦克风
                    StatefulListInfo.AttendeeListDTO isShowInfo = RoomManager.getInstance().getDialogIsShowInfo();
                    if (micManagerDialog != null && micManagerDialog.isVisible() && isShowInfo != null) {
                        for (int i = 0; i < audienceStatefulListInfo.size(); i++) {
                            StatefulListInfo.AttendeeListDTO info = audienceStatefulListInfo.get(i);
                            if (info.getUid() == isShowInfo.getUid()) {
                                micManagerDialog.setMic(info.getMic());
                            }
                        }
                    }
                }
            }

            @Override
            public void inviteMicAudienc(StatefulListInfo.AttendeeListDTO info) {
                //解决 观众断网但是房主邀请连麦、房主的按钮变更了但是观众恢复网络收不到弹窗的问题
                if (!getActivity().isFinishing()) {
                    //被邀请连麦 弹窗显示
                    if (inviteMicDialog == null) {
                        showInviteDialog();
                    } else {
                        if (!inviteMicDialog.isShowing()) {
                            showInviteDialog();
                        }
                    }
                }
            }

            @Override
            public void onlineNum() {
                //重置房间人数信息
                setOnlineNum();
            }
        });

    }

    /**
     * 数据转换（转成连麦申请的数据）
     *
     * @param info
     * @return
     */
    private NotifyApplyMicStateInfo gainMicStateInfo(NotifyUserListUpdateInfo.UsersDTO info) {
        NotifyApplyMicStateInfo notifyInfo = new NotifyApplyMicStateInfo();
        notifyInfo.setUid(info.getUid());
        notifyInfo.setNickName(info.getNickName());
        return notifyInfo;
    }


    @Override
    public void loginRoom() {
        mCloseRoom.removeAllViews();
        mCloseRoom.setBackground(null);

        initListener();
        initData();
        initRecycleView();


        RoomListInfo.RoomListDTO data = (RoomListInfo.RoomListDTO) getTag(R.id.liveshow_live_room_data);
        String roomID = null;
        if (data != null) {
            roomID = data.getRoomId();
        }


        int loginOrCreate = getActivity().getIntent().getIntExtra(LOGIN_OR_CREATE_ROOM, LOGIN_ROOM);
        ZegoSDKManager.getInstance().getDeviceService().enableCamera(true);
        RoomManager.getInstance().setFront(true);

        if (loginOrCreate == CREATE_ROOM) {
            showCreateRoom();
            ZegoSDKManager.getInstance().getDeviceService().startPreview(mLiveVideo);
            ResolutionManager.getInstance().reset(ROOM_ROLE_HOST);
            mEditRoomName.setText(String.valueOf(UserManager.getInstance().getUserInfo().getNickName() + "创建的直播间"));
        } else {
            final String lastRoomID = RoomManager.getInstance().getRoomID();
            if (roomID != null && RoomManager.getInstance().getRoomID() != null) {
                if (!roomID.equals(RoomManager.getInstance().getRoomID())) {
                    RoomManager.getInstance().switchRoom(RoomManager.getInstance().getRoomID(), roomID, new IRTCCommonCallback() {
                        @Override
                        public void onRTCCallback(int errorCode) {
                            if (errorCode == 0) {
                                showLiveRoom();

//                                if(TextUtils.isEmpty(lastRoomID)){
                                playStream(true);
//                                }

                            } else if (errorCode == ERROR_ROOM_NOT_EXIST) {
                                mRoomLayer.setVisibility(View.VISIBLE);

                                ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) mRoomLayer.getLayoutParams();
                                params.topMargin = MiscUtils.getStatusBarHeight(getContext());
                                mRoomLayer.setLayoutParams(params);

                                showCloseRoom();
                            } else {
                                RoomManager.getInstance().CommonErrorCodeToast(errorCode);
                            }
                        }
                    });
                } else {
                    showLiveRoom();
                }
            }

        }
    }

    @Override
    public void logoutRoom() {

        mCreateRoom.setVisibility(View.GONE);
        mRoomLayer.setVisibility(View.GONE);
        mRoomBottomLayerLayout.setVisibility(View.GONE);
        mMoreSetting.setVisibility(View.GONE);
        mFinishRoomTips.setVisibility(View.GONE);
        mAudienceList.setAdapter(null);

        HeartBeatUtil.uninit();
        NetWorkConManager.getInstance().clearData();
        DialogQueueUtils.getInstance().clear();

        if (hostInfoView != null) {
            mRoomLayer.removeView(hostInfoView);
            hostInfoView = null;
        }

        if (closeRoomView != null) {
            mCloseRoom.removeView(closeRoomView);
            mCloseRoom.setBackground(null);
            closeRoomView = null;
        }

        mIsMicMuted = true;
        mIsCameraMuted = true;

        showCreateRoomNetworkFail = false;

        mRoomClose = false;


        closeAllDialog();
    }

    private void closeAllDialog() {

        if (getActivity() == null) {
            return;
        }

        if (createRoomNetworkFailDialog != null && !getActivity().isFinishing() && !getActivity().isDestroyed()) {
            createRoomNetworkFailDialog.dismiss();
            createRoomNetworkFailDialog = null;
        }

        if (heartBeatFailDialog != null && !getActivity().isFinishing() && !getActivity().isDestroyed()) {
            heartBeatFailDialog.dismiss();
            heartBeatFailDialog = null;
        }


        if (!getActivity().isFinishing() && connectMicDialog != null && connectMicDialog.isShowing()) {
            connectMicDialog.dismiss();
        }

        if (!getActivity().isFinishing() && inviteMicDialog != null && inviteMicDialog.isShowing()) {
            inviteMicDialog.dismiss();
        }
    }

    /**
     * 拉流时是否可以接收指定远端用户的音频数据，“true” 表示接收，“false” 表示禁止
     *
     * @param enable
     */
    @Override
    public void playStream(boolean enable) {
        mLiveVideo.setVisibility(VISIBLE);

        RoomListInfo.RoomListDTO data = (RoomListInfo.RoomListDTO) getTag(R.id.liveshow_live_room_data);

        if (data != null) {
            String avatar = data.getCoverImg();
            GlideLoader.displayRoundImageByAvatar(avatar, mLiveRoomBg);


            String streamID = data.getStreamId();
            if (!TextUtils.isEmpty(streamID) && this.isAttachedToWindow()) {

                ZegoSDKManager.getInstance().getStreamService().enablePlayStreamAudio(streamID, enable);

                if (mLiveVideo.getTag() != null && !mLiveVideo.getTag().equals(streamID)) {
                    RoomManager.getInstance().stopStream((String) mLiveVideo.getTag());
                    mLiveVideo.setTag(null);
                }
                mLiveVideo.setTag(streamID);


                RoomManager.getInstance().startPlayStream(streamID, mLiveVideo, new IRTCCommonCallback() {
                    @Override
                    public void onRTCCallback(int errorCode) {
                        if (errorCode != 0) {
                            mLiveVideo.setTag(null);
                        } else {
                        }
                    }
                });
            }
        }
    }

    @Override
    public void stopStream() {
        if (mLiveVideo.getTag() != null) {

            RoomManager.getInstance().stopStream((String) mLiveVideo.getTag());
            mLiveVideo.setTag(null);
        }
    }

    public FragmentActivity getActivity() {
        Context context = this.getContext();
        while (context instanceof ContextWrapper) {
            if (context instanceof Activity) {
                return (FragmentActivity) context;
            }
            context = ((ContextWrapper) context).getBaseContext();
        }
        return null;

    }

    private void showCloseRoom() {
        RoomManager.getInstance().setRoomID("");
        if (getActivity() != null && !getActivity().isDestroyed() && !getActivity().isFinishing()) {
            closeRoomView = new CloseRoomView(getContext());
            LayoutParams layoutParams = new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT);
            layoutParams.topMargin = MiscUtils.getStatusBarHeight(getContext());
            mCloseRoom.setBackgroundColor(Color.parseColor("#32000000"));
            mCloseRoom.addView(closeRoomView, layoutParams);

            RoomListInfo.RoomListDTO data = (RoomListInfo.RoomListDTO) getTag(R.id.liveshow_live_room_data);
            if (data != null) {
                closeRoomView.update(data.getCoverImg());
            }
            closeRoomView.setCloseListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (getActivity() != null) {
                        getActivity().onBackPressed();
                    }
                }
            });
            mRoomBottomLayerLayout.setVisibility(View.GONE);
            mMoreSetting.setVisibility(View.GONE);
            mFinishRoomTips.setVisibility(View.GONE);
            if (hostInfoView != null) {
                mRoomLayer.removeView(hostInfoView);
                hostInfoView = null;
            }
            mAudienceList.setAdapter(null);

            ((LiveRoomActivity2) getActivity()).enableSliding(true);

            DialogQueueUtils.getInstance().clear();
            closeAllDialog();

            mFlLiveVideo.clearAnimation();
            mFlLiveVideo.removeAllViews();
            mFlLiveVideo.addView(mLiveVideo, new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT));

            mFlLiveVideo.post(new Runnable() {
                @Override
                public void run() {
                    mFlLiveVideo.clearAnimation();
                    mFlLiveVideo.removeAllViews();
                    mFlLiveVideo.addView(mLiveVideo, new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT));

                }
            });

            mLiveVideo.setVisibility(GONE);
        }
    }

    private void settingBeauty() {
        if (CheckDoubleClick.isFastDoubleClick()) {
            return;
        }
        EffectsBeautyDialog effectsBeautyDialog = new EffectsBeautyDialog();
        effectsBeautyDialog.show(getActivity().getSupportFragmentManager(), effectsBeautyDialog.getTag());
    }


    @Override
    public boolean dispatchTouchEvent(MotionEvent ev) {
        if (LiveRoomActivity2.isShowHintCover || LiveRoomActivity2.isShowEndHintCover) {

            return true;
        }

        return super.dispatchTouchEvent(ev);
    }


    /**
     * 邀请连麦逻辑处理
     *
     * @param notifyInfo
     */
    @Override
    public void onInviteMicNotify(NotifyUserStateUpdateInfo notifyInfo) {

        UserInfo myInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();

        if (myInfo.getRole() == Constants.ROOM_ROLE_HOST) { //房主

            //弹窗弹出才有房主的逻辑
            if (onlineDialog != null && onlineDialog.isVisible()) {
                onlineDialog.onInviteMicNotify(notifyInfo);
            }

            if(notifyInfo.getType() == Constants.OPERATION_TYPE_INVITE_REFUSE){
                ToastUtil.showToast(notifyInfo.getOperatorName()+notifyInfo.getExtra());
            }


        } else {  //观众
            if (notifyInfo.getType() == Constants.OPERATION_TYPE_INVITE) {
                List<NotifyUserStateUpdateInfo.UsersDTO> stateUpdateInfo = notifyInfo.getUsers();
                for (int i = 0; i < stateUpdateInfo.size(); i++) {
                    //说明是本人
                    if (stateUpdateInfo.get(i).getUid() == myInfo.getUid() && !getActivity().isFinishing()) {
                        //被邀请连麦 弹窗显示
                        if (inviteMicDialog == null) {
                            showInviteDialog();
                        } else {
                            if (!inviteMicDialog.isShowing()) {
                                showInviteDialog();
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 邀请连麦的弹窗
     */
    private void showInviteDialog() {
        inviteMicDialog = new BaseInquiryDialog(getActivity(), R.layout.liveshow_inquiry_dialog);
        inviteMicDialog.setMsgTitle(getActivity().getString(R.string.liveshow_invite_connect_mic));
        inviteMicDialog.setMsgContent(getActivity().getString(R.string.liveshow_invite_connect_mic_content));
        inviteMicDialog.setLeftButtonContent(getActivity().getString(R.string.liveshow_btn_refuse));
        inviteMicDialog.setRightButtonContent(getActivity().getString(R.string.liveshow_btn_agree));
        inviteMicDialog.setSureListener(v -> {
            //接受邀请
            inviteMicPermission();
        });
        inviteMicDialog.setCancelListener(v -> {
            //拒绝邀请
            inviteMicStateNotifyRequest(Constants.OPERATION_ACTION_REFUSE,"已拒绝连麦邀请");
        });
    }

    /**
     * 接受连麦邀请
     */
    private void inviteMicPermission() {
        PermissionHelper.onCameraAndAudioPermissionGranted((FragmentActivity) getActivity(), new PermissionHelper.GrantResult() {
            @Override
            public void onGrantResult(boolean allGranted) {
                if (allGranted) {
                    inviteMicStateNotifyRequest(Constants.OPERATION_ACTION_ACCEPT,"");
                } else {
                    BaseInquiryDialog baseInquiryDialog = new BaseInquiryDialog(getActivity());
                    baseInquiryDialog.setMsgTitle(getActivity().getString(R.string.liveshow_tips));
                    baseInquiryDialog.setMsgContent(getActivity().getString(R.string.liveshow_need_pemission_content));
                    baseInquiryDialog.setLeftButtonContent(getActivity().getString(R.string.liveshow_btn_cancle));
                    baseInquiryDialog.setRightButtonContent(getActivity().getString(R.string.liveshow_go_setting));
                    baseInquiryDialog.setSureListener(v -> {
                        baseInquiryDialog.dismiss();
                        Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
                                .setData(Uri.fromParts("package", getActivity().getPackageName(), null));
                        ((Activity) getActivity()).startActivityForResult(intent, 666);
                    });
                    baseInquiryDialog.setCancelListener(v -> {
                        inviteMicStateNotifyRequest(Constants.OPERATION_ACTION_REFUSE,"连麦失败");
                        ToastUtil.showToast("连麦失败，请先授权麦克风权限");
                        baseInquiryDialog.dismiss();
                    });
                }
            }
        });
    }

    /**
     * 被邀请连麦的 同意/拒绝
     *
     * @param action
     */
    private void inviteMicStateNotifyRequest(int action,String extra) {
        RoomManager.getInstance().inviteMicStateNotifyRequest(action, extra,new IExecuteCallback() {
            @Override
            public void onSuccess() {
                if (!getActivity().isFinishing() && inviteMicDialog != null && inviteMicDialog.isShowing()) {
                    inviteMicDialog.dismiss();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                if (errorCode == ErrorcodeConstants.ErrorFailNetwork) {
                    ToastUtil.showToast(R.string.fail_network);
                } else {
                    if (!getActivity().isFinishing() && inviteMicDialog != null && inviteMicDialog.isShowing()) {
                        inviteMicDialog.dismiss();
                    }
                    RoomManager.getInstance().CommonErrorCodeToast(errorCode);
                }
            }
        });
    }


    /**
     * 设置房间人数信息
     */
    private void setOnlineNum() {
        RoomManager.getInstance().getOnlineInfo(new IGetOnlineInfoCallback() {
            @Override
            public void onGetOnlineInfo(int errorCode, OnlineInfo onlineInfo) {
                if (errorCode == 0 && hostInfoView != null) {
                    hostInfoView.updateOnlineNum(String.valueOf(onlineInfo.getOnlineNum()));
                }
            }
        });
    }

}
