package im.zego.liveshow.liveroom.adapter;

import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.TextureView;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import im.zego.liveshow.Constants;
import im.zego.liveshow.R;
import im.zego.liveshow.manager.ResolutionManager;
import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.model.StatefulListInfo;
import im.zego.liveshow.model.UserInfo;
import im.zego.liveshow.model.rtc.NotifyUserListUpdateInfo;
import im.zego.liveshow.model.rtc.NotifyUserStateUpdateInfo;
import im.zego.liveshow.sdk.ZegoSDKManager;
import im.zego.liveshow.utils.CheckDoubleClick;
import im.zego.liveshow.utils.GlideLoader;
import im.zego.liveshow.utils.ToastUtil;


public class AudienceListAdapter extends RecyclerView.Adapter<AudienceListAdapter.AudienceViewHolder> {

    private List<StatefulListInfo.AttendeeListDTO> mData = new ArrayList<>();
    private Context context;

    public AudienceListAdapter(Context context) {
        this.context = context;
    }

    @NonNull
    @Override
    public AudienceViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.liveshow_item_audience_list, parent, false);
        return new AudienceViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull AudienceViewHolder holder, int position) {

        StatefulListInfo.AttendeeListDTO userInfo = mData.get(holder.getAdapterPosition());

        if (userInfo.getCamera() == Constants.MIC_CAMERA_CLOSE) {
            holder.mtvAudienceVideo.setVisibility(View.GONE);
            holder.mIvAudienceVideo.setVisibility(View.VISIBLE);
            holder.mIvAudienceHead.setVisibility(View.VISIBLE);

            GlideLoader.displayRoundImageByAvatar(userInfo.getAvatar(), holder.mIvAudienceVideo);
            GlideLoader.displayCircleImageByAvatar(userInfo.getAvatar(), holder.mIvAudienceHead);
        } else {
            holder.mtvAudienceVideo.setVisibility(View.VISIBLE);
            holder.mIvAudienceVideo.setVisibility(View.GONE);
            holder.mIvAudienceHead.setVisibility(View.GONE);
        }

        if (!TextUtils.isEmpty(userInfo.getNickName())) {
            holder.mTvName.setText(userInfo.getNickName());
        }

        if (userInfo.getMic() == Constants.MIC_CAMERA_CLOSE) {
            holder.mIvMicOff.setVisibility(View.VISIBLE);
        } else {
            holder.mIvMicOff.setVisibility(View.GONE);
        }

        if (isMine(userInfo.getUid())) {
            //本人就要预览并且推流
            ZegoSDKManager.getInstance().getDeviceService().startPreview(holder.mtvAudienceVideo);
            ResolutionManager.getInstance().reset(Constants.ROOM_ROLE_AUDIENCE);
            RoomManager.getInstance().startPublishStream(holder.mtvAudienceVideo, null);
        } else {
            //拉流显示页面
            RoomManager.getInstance().startPlayStream(userInfo, holder.mtvAudienceVideo, null);
        }

        if(isHost()){
            holder.mIvMicMore.setVisibility(View.VISIBLE);
            holder.itemView.setOnClickListener(v -> {
                if (CheckDoubleClick.isFastDoubleClick()) {
                    return;
                }
                if (userMicListener != null) {
                    userMicListener.onUserMicStatus(userInfo);
                }
            });
        }else {
            holder.mIvMicMore.setVisibility(View.GONE);
        }

    }

    @Override
    public int getItemCount() {
        if (mData == null) {
            return 0;
        }
        return mData.size() >= 3 ? 3 : mData.size();
    }

    /**
     * 数据变更的时候回调刷新数据（cameara/mic 的变化）
     *
     * @param onstageList
     */
    public void refreshData(NotifyUserStateUpdateInfo onstageList) {
        for (int i = 0; i < onstageList.getUsers().size(); i++) {
            NotifyUserStateUpdateInfo.UsersDTO usersDTO = onstageList.getUsers().get(i);
            if (mData != null && mData.size() > 0) {
                for (int j = 0; j < mData.size(); j++) {
                    StatefulListInfo.AttendeeListDTO attendeeListDTO = mData.get(j);
                    if (usersDTO.getUid() == attendeeListDTO.getUid()) {
                        if (onstageList.getType() == Constants.OPERATION_TYPE_CAMERA) { //cameara 变化
                            attendeeListDTO.setCamera(usersDTO.getCamera());
                            notifyItemChanged(j);
                        } else if (onstageList.getType() == Constants.OPERATION_TYPE_MIC) { //mic变化
                            attendeeListDTO.setMic(usersDTO.getMic());
                            notifyItemChanged(j);
                            //判断是不是被主播禁言
                            if(isMine(usersDTO.getUid()) && !isMine(onstageList.getOperatorUid())){
                                micStatus(usersDTO.getMic());
                            }

                            //当弹窗弹窗但是用户改变麦克风状态的时候 主播的弹窗数据也要变更
                            if(isHost() && RoomManager.getInstance().getDialogIsShowInfo() != null){
                                if(usersDTO.getUid() == RoomManager.getInstance().getDialogIsShowInfo().getUid()){
                                    if(userMicListener != null){
                                        userMicListener.onHostDialogMic(usersDTO.getMic());
                                    }
                                }
                            }

                        }
                    }
                }
            }
        }
    }

    /**
     * 观众退出房间后判断该用户有没有在麦位，有就将其移除
     *
     * @param userListUpdateInfo
     */
    public void exitRoomData(NotifyUserListUpdateInfo userListUpdateInfo) {
        for (int i = 0; i < userListUpdateInfo.getUsers().size(); i++) {
            NotifyUserListUpdateInfo.UsersDTO usersDTO = userListUpdateInfo.getUsers().get(i);
            if (mData != null && mData.size() > 0) {
                Iterator<StatefulListInfo.AttendeeListDTO> it = mData.iterator();
                while (it.hasNext()) {
                    StatefulListInfo.AttendeeListDTO attendeeListDTO = (StatefulListInfo.AttendeeListDTO) it.next();
                    //Delta -1 为退出房间
                    if (usersDTO.getDelta() == -1 && usersDTO.getUid() == attendeeListDTO.getUid()) {
                        it.remove();
                        if (isHost()) {
                            //只有主播端才有提示
                            ToastUtil.showToast(String.format(context.getString(R.string.liveshow_connect_exit_room), attendeeListDTO.getNickName()));
                        }
                    }
                }

            }
        }
        notifyDataSetChanged();
    }

    /**
     * 整体数据的添加
     *
     * @param newData
     */
    public void addData(List<StatefulListInfo.AttendeeListDTO> newData) {
        mData.clear();
        mData.addAll(newData);
        notifyDataSetChanged();
    }

    /**
     * 上下台数据的刷新
     *
     * @param notifyInfo
     */
    public void onStage(NotifyUserStateUpdateInfo.UsersDTO notifyInfo) {
        // 上台状态，1:在台下 2:在台上
        if (notifyInfo.getOnstageState() == Constants.STATE_OFF_STAGE) {
            //在台下的逻辑(要下麦)
            Iterator<StatefulListInfo.AttendeeListDTO> it = mData.iterator();
            while (it.hasNext()) {
                StatefulListInfo.AttendeeListDTO info = (StatefulListInfo.AttendeeListDTO) it.next();
                if (info.getUid() == notifyInfo.getUid()) {
                    it.remove();
                    if (isMine(notifyInfo.getUid())) {
                        //如果是本人下麦了就不需要预览并且推流，所以需要停止预览/暂停推流
                        RoomManager.getInstance().stopPreviewAndPublishStream();
                        //回调回去刷新按钮状态（下麦）
                        if (userStageListener != null) {
                            userStageListener.onStage(Constants.STATE_OFF_STAGE);
                        }
                        //如果是本人下麦就得重置数据
                        RoomManager.getInstance().setMicIsOffFromHost(false);
                    } else {
                        //如果不是本人下台就停止拉流
                        RoomManager.getInstance().stopPlayStream(info);
                    }
                }
            }
            notifyDataSetChanged();

        } else if (notifyInfo.getOnstageState() == Constants.STATE_ON_STAGE) {  //在台上

            //判断当前上麦的人当中有没有该观众，有就不添加
            for (StatefulListInfo.AttendeeListDTO listDTO : mData) {
                if (listDTO.getUid() == notifyInfo.getUid()) {
                    return;
                }
            }

            //添加我的个人上麦信息-位置为第一个
            StatefulListInfo.AttendeeListDTO userInfo = new StatefulListInfo.AttendeeListDTO();
            userInfo.setUid(notifyInfo.getUid());
            userInfo.setAvatar(notifyInfo.getAvatar());
            userInfo.setNickName(notifyInfo.getNickName());
            userInfo.setRole(notifyInfo.getRole());
            userInfo.setLoginTimestamp(notifyInfo.getLoginTimestamp());
            userInfo.setOnstageTimestamp(notifyInfo.getOnstageTimestamp());
            userInfo.setMic(notifyInfo.getMic());
            userInfo.setCamera(notifyInfo.getCamera());
            userInfo.setOnstageState(notifyInfo.getOnstageState());

            if (isMine(notifyInfo.getUid())) {
                //判断是不是本人
                mData.add(0, userInfo);
                if (userStageListener != null) {
                    userStageListener.onStage(Constants.STATE_ON_STAGE);
                }
                notifyItemInserted(0);
            } else {
                mData.add(userInfo);
                notifyItemInserted(mData.size() - 1);
            }

        }
    }

    /**
     * 主播对麦克风的操作
     * @param micType
     */
    public void micStatus(int micType){
        if(micType == 1){
            ToastUtil.showToast("你已被主播禁言");
            RoomManager.getInstance().setMicIsOffFromHost(true);
            ZegoSDKManager.getInstance().getDeviceService().enableMic(false);
            if(userMicListener != null){
                userMicListener.onUserMicOff();
            }
        }else if(micType == 2){
            RoomManager.getInstance().setMicIsOffFromHost(false);
            ZegoSDKManager.getInstance().getDeviceService().enableMic(true);
            if(userMicListener != null){
                userMicListener.onUserMicOn();
            }
        }
    }

    private UserStageListener userStageListener;

    public void setUserStageListener(UserStageListener userStageListener) {
        this.userStageListener = userStageListener;
    }

    public interface UserStageListener {
        void onStage(int state);
    }

    static class AudienceViewHolder extends RecyclerView.ViewHolder {

        private TextureView mtvAudienceVideo;
        private ImageView mIvAudienceVideo;
        private ImageView mIvAudienceHead;
        private TextView mTvName;
        private ImageView mIvMicOff;
        private ImageView mIvMicMore;

        private AudienceViewHolder(View itemView) {
            super(itemView);

            mtvAudienceVideo = itemView.findViewById(R.id.tv_audience_video);
            mIvAudienceVideo = itemView.findViewById(R.id.iv_audience_video);
            mIvAudienceHead = itemView.findViewById(R.id.iv_audience_head);
            mTvName = itemView.findViewById(R.id.tv_name);
            mIvMicOff = itemView.findViewById(R.id.iv_mic_off);
            mIvMicMore = itemView.findViewById(R.id.iv_mic_more);

        }

    }

    /**
     * 是自己本人
     */
    private boolean isMine(long uid) {
        UserInfo myInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        return myInfo.getUid() == uid;
    }

    /**
     * 是主播
     */
    private boolean isHost() {
        UserInfo myInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        return myInfo.getRole() == Constants.ROOM_ROLE_HOST;
    }

    private UserMicListener userMicListener;

    public void setUserMicListener(UserMicListener userMicListener) {
        this.userMicListener = userMicListener;
    }

    public interface UserMicListener {
        void onUserMicStatus(StatefulListInfo.AttendeeListDTO userInfo);

        void onUserMicOff();

        void onUserMicOn();

        void onHostDialogMic(int micStatus);
    }

}
