package im.zego.liveshow.liveroom.adapter;

import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.List;

import im.zego.liveshow.Constants;
import im.zego.liveshow.R;
import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.model.StatefulListInfo;
import im.zego.liveshow.model.UserInfo;
import im.zego.liveshow.utils.CheckDoubleClick;
import im.zego.liveshow.utils.GlideLoader;
import im.zego.liveshow.utils.ToastUtil;

public class OnlineAdapter extends RecyclerView.Adapter<OnlineAdapter.OnlineViewHolder> {

    private List<StatefulListInfo.AttendeeListDTO> mData = new ArrayList<>();
    private Context context;

    public OnlineAdapter(Context context) {
        this.context = context;
    }

    /**
     * 整体数据的添加
     *
     * @param newData
     */
    public void addData(List<StatefulListInfo.AttendeeListDTO> newData) {
        mData.clear();
        mData.addAll(newData);
        notifyDataSetChanged();
    }

    /**
     * 更新在台状态
     *
     * @param userInfo
     */
    public void updateOnstageData(StatefulListInfo.AttendeeListDTO userInfo) {
        if (mData != null && mData.size() > 0) {
            for (int i = 0; i < mData.size(); i++) {
                StatefulListInfo.AttendeeListDTO attendeeListDTO = mData.get(i);
                if (userInfo.getUid() == attendeeListDTO.getUid()) {
                    attendeeListDTO.setOnstageState(userInfo.getOnstageState());
                    notifyItemChanged(i);
                }
            }
        }
    }

    @NonNull
    @Override
    public OnlineViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.liveshow_item_online_list, parent, false);
        return new OnlineViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull OnlineViewHolder holder, int position) {

        StatefulListInfo.AttendeeListDTO userInfo = mData.get(holder.getAdapterPosition());

        GlideLoader.displayCircleImageByAvatar(userInfo.getAvatar(), holder.mTvOnlineHead);

        holder.mTvOnlineName.setText(userInfo.getNickName());

        if (isHost()) {  //主播
            if (isMine(userInfo.getUid())) {
                holder.mTvHostOrMine.setVisibility(View.VISIBLE);
                holder.mClMicStatus.setVisibility(View.GONE);
                holder.mTvHostOrMine.setText("我");
            } else {
                holder.mTvHostOrMine.setVisibility(View.GONE);
                holder.mClMicStatus.setVisibility(View.VISIBLE);

                if (userInfo.getOnstageState() == Constants.STATE_OFF_STAGE) {
                    //未连麦
                    holder.mClMicStatus.setBackgroundResource(R.drawable.liveshow_shape_gradient);
                    holder.mTvMicStatus.setText("邀请连麦");
                    holder.mTvMicStatus.setTextColor(Color.parseColor("#ffffff"));
                    holder.mClMicStatus.setClickable(true);
                    holder.mClMicStatus.setOnClickListener(v -> {
                        if (CheckDoubleClick.isFastDoubleClick()) {
                            return;
                        }
                        if (inviteStageListener != null) {
                            inviteStageListener.onInviteStage(userInfo);
                        }
                    });

                } else if (userInfo.getOnstageState() == Constants.STATE_ON_STAGE) {
                    //已经连麦
                    holder.mClMicStatus.setBackgroundResource(R.drawable.liveshow_shape_1affffff_16);
                    holder.mTvMicStatus.setText("已连麦");
                    holder.mTvMicStatus.setTextColor(Color.parseColor("#ffa7a6b7"));
                    holder.mClMicStatus.setClickable(false);
                } else if (userInfo.getOnstageState() == Constants.STATE_INVITE_CON_MIC) {
                    holder.mClMicStatus.setBackgroundResource(R.drawable.liveshow_shape_1affffff_16);
                    holder.mTvMicStatus.setText("已邀请");
                    holder.mTvMicStatus.setTextColor(Color.parseColor("#ffa7a6b7"));
                    holder.mClMicStatus.setClickable(true);
                }
            }

        } else {  // 观众
            holder.mClMicStatus.setVisibility(View.GONE);
            if (userInfo.getRole() == Constants.ROOM_ROLE_HOST) {
                holder.mTvHostOrMine.setVisibility(View.VISIBLE);
                holder.mTvHostOrMine.setText("主播");
            } else if (isMine(userInfo.getUid())) {
                holder.mTvHostOrMine.setVisibility(View.VISIBLE);
                holder.mTvHostOrMine.setText("我");
            } else {
                holder.mTvHostOrMine.setVisibility(View.GONE);
            }
        }

    }

    @Override
    public int getItemCount() {
        if (mData == null) {
            return 0;
        }
        return mData.size();
    }


    private InviteStageListener inviteStageListener;

    public void setInviteStageListener(InviteStageListener inviteStageListener) {
        this.inviteStageListener = inviteStageListener;
    }

    public interface InviteStageListener {
        void onInviteStage(StatefulListInfo.AttendeeListDTO userInfo);
    }

    /**
     * 是自己本人
     */
    private boolean isMine(long uid) {
        UserInfo myInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        return myInfo.getUid() == uid;
    }

    /**
     * 是主播
     */
    private boolean isHost() {
        UserInfo myInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        return myInfo.getRole() == Constants.ROOM_ROLE_HOST;
    }

    static class OnlineViewHolder extends RecyclerView.ViewHolder {

        private ImageView mTvOnlineHead;
        private TextView mTvOnlineName;
        private TextView mTvHostOrMine;
        private ConstraintLayout mClMicStatus;
        private TextView mTvMicStatus;

        private OnlineViewHolder(View itemView) {
            super(itemView);

            mTvOnlineHead = itemView.findViewById(R.id.tv_online_head);
            mTvOnlineName = itemView.findViewById(R.id.tv_online_name);
            mTvHostOrMine = itemView.findViewById(R.id.tv_host_or_mine);
            mClMicStatus = itemView.findViewById(R.id.cl_mic_status);
            mTvMicStatus = itemView.findViewById(R.id.tv_mic_status);

        }
    }

}
