package im.zego.liveshow.liveroom.utils;

import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.util.Log;

import androidx.annotation.NonNull;

import im.zego.liveshow.liveroom.callbacks.IHeartBeatFailCallback;
import im.zego.liveshow.liveroom.callbacks.IStateSeqCallback;
import im.zego.liveshow.liveroom.callbacks.IUserSeqCallback;
import im.zego.liveshow.model.HeartbeatRespInfo;
import im.zego.liveshow.protocol.ICommonResponseCallback;
import im.zego.liveshow.protocol.IHeartBeatCallback;
import im.zego.liveshow.protocol.RoomAPI;

public class HeartBeatUtil {

    private static final String TAG = "HeartBeatUtil";

    private static final int HEART_BEAT_REQUEST = 876;
    private static final int HEART_BEAT_DISCONNECT = HEART_BEAT_REQUEST+1;
    private static final int HEART_BEAT_RETRY = HEART_BEAT_DISCONNECT +1;
    private static final int HEART_BEAT_USER_RETRY = HEART_BEAT_RETRY +1;

    private static int heartBeatFailCount = 0;
    private static int heartBeatRetryCount = 0;
    private static final int MAX_HEART_BEA_FAIL_COUNT = 3;
    private static final int MAX_HEART_BEA_RETRY_COUNT = 3;
    private static final long DEFAULT_HEART_BEAT_TIME = 10 * 1000;
    private static long sHeartBeatTime = DEFAULT_HEART_BEAT_TIME;
    private static HandlerThread mThread;
    private static Handler heartBeatHandler = null;
    private static long sUid;
    private static String sRoomID;

    private static IHeartBeatFailCallback sHeartBeatFailCallback = null;
    private static IStateSeqCallback sStateSeqCallback = null;
    private static IUserSeqCallback sUserSeqCallback = null;

    public static void init(long uid,String roomID){

        sUid = uid;
        sRoomID = roomID;

        mThread = new HandlerThread("NetworkSpeedUtil");
        mThread.start();

        heartBeatHandler = new Handler(mThread.getLooper()){
            @Override
            public void handleMessage(@NonNull Message msg) {
                super.handleMessage(msg);

                if(msg.what == HEART_BEAT_REQUEST){
                    Log.e(TAG,"HEART_BEAT_REQUEST:");
                    heartBeatRequest(new ICommonResponseCallback() {
                        @Override
                        public void onResponse(int errorCode) {
                            if(errorCode == 0){
                                request(sHeartBeatTime);
                                if(sHeartBeatFailCallback != null){
                                    sHeartBeatFailCallback.onHeartBeatSuccess();
                                }

                            } else {
                                if(heartBeatFailCount >= MAX_HEART_BEA_FAIL_COUNT)
                                {
                                    if(sHeartBeatFailCallback != null)
                                    {
                                        sHeartBeatFailCallback.onHeartBeatFail();
                                    }
                                }else {
                                    request(0);
                                }
                            }
                        }
                    });

                } if(msg.what == HEART_BEAT_DISCONNECT){
                    Log.e(TAG,"HEART_BEAT_DISCONNECT:");
                    heartBeatHandler.removeMessages(HEART_BEAT_REQUEST);
                    heartBeatHandler.removeMessages(HEART_BEAT_RETRY);
                    heartBeatHandler.removeMessages(HEART_BEAT_DISCONNECT);
                    heartBeatHandler.removeMessages(HEART_BEAT_USER_RETRY);

                    heartBeatRequest(new ICommonResponseCallback() {
                        @Override
                        public void onResponse(int errorCode) {
                            if (errorCode == 0) {
                                request(sHeartBeatTime);
                                if(sHeartBeatFailCallback != null){
                                    sHeartBeatFailCallback.onHeartBeatSuccess();
                                }
                            }else {
                                if(sHeartBeatFailCallback != null)
                                {
                                    sHeartBeatFailCallback.onHeartBeatDisconnect(errorCode);
                                }
                            }

                        }
                    });

                }else if(msg.what == HEART_BEAT_RETRY){
                    Log.e(TAG,"HEART_BEAT_RETRY:");
                    heartBeatRequest(new ICommonResponseCallback() {
                        @Override
                        public void onResponse(int errorCode) {

                            if (errorCode == 0) {
                                request(sHeartBeatTime);
                                if(sHeartBeatFailCallback != null){
                                    sHeartBeatFailCallback.onHeartBeatSuccess();
                                }
                            }else {
                                heartBeatRetryCount++;
                                if(heartBeatRetryCount <= MAX_HEART_BEA_RETRY_COUNT){
                                        retry(0);
                                }
                            }

                        }
                    });

                }else if(msg.what == HEART_BEAT_USER_RETRY){
                    Log.e(TAG,"HEART_BEAT_USER_RETRY:");
                    heartBeatRequest(new ICommonResponseCallback() {
                        @Override
                        public void onResponse(int errorCode) {

                            if (errorCode == 0) {
                                request(sHeartBeatTime);
                                if(sHeartBeatFailCallback != null){
                                    sHeartBeatFailCallback.onHeartBeatSuccess();
                                }
                            }else {
                                if(sHeartBeatFailCallback != null){
                                    sHeartBeatFailCallback.onHeartBeatUserRetryFail(errorCode);
                                }
                            }

                        }
                    });
                }
            }
        };
        request(0);
    }

    public static void uninit(){

        if(mThread != null) {
            mThread.quit();
        }
        heartBeatFailCount = 0;
        heartBeatRetryCount = 0;
        sHeartBeatTime = DEFAULT_HEART_BEAT_TIME;
        heartBeatHandler = null;
        sUid =0;
        sRoomID =null;
        sHeartBeatFailCallback = null;
        sStateSeqCallback = null;
        sUserSeqCallback = null;
    }

    private static void heartBeatRequest(ICommonResponseCallback commonResponseCallback){

        RoomAPI.heartBeat(sUid, sRoomID, new IHeartBeatCallback() {
            @Override
            public void onHeartBeat(int errorCode, HeartbeatRespInfo heartbeatRespInfo) {
                Log.e(TAG,"errorCode:" + errorCode);
                if(errorCode == 0){
                    long stateSeq = heartbeatRespInfo.getGbPersonalStateSeq();
                    long userSeq = heartbeatRespInfo.getAttendeeListSeq();

                    reset(heartbeatRespInfo.getInterval() * 1000L);

                    if(sStateSeqCallback != null){
                        sStateSeqCallback.onStateSeqCallback(stateSeq);
                    }

                    if(sUserSeqCallback != null){
                        sUserSeqCallback.onUserSeqCallback(userSeq);
                    }

                }else {
                    heartBeatFailCount++;
                    timeoutLimit();

                }
                commonResponseCallback.onResponse(errorCode);
            }
        });

    }


    public static void reset(long heartBeatTime){
        heartBeatFailCount = 0;
        heartBeatRetryCount = 0;
        sHeartBeatTime = heartBeatTime;
        if(heartBeatHandler != null){
            heartBeatHandler.removeMessages(HEART_BEAT_DISCONNECT);
            heartBeatHandler.removeMessages(HEART_BEAT_REQUEST);
            heartBeatHandler.removeMessages(HEART_BEAT_RETRY);
            heartBeatHandler.removeMessages(HEART_BEAT_USER_RETRY);

        }
    }

    public static void setHeartBeatFailCallback(IHeartBeatFailCallback heartBeatFailCallback) {
        sHeartBeatFailCallback = heartBeatFailCallback;
    }

    public static void setStateSeqCallback(IStateSeqCallback stateSeqCallback) {
        sStateSeqCallback = stateSeqCallback;
    }

    public static void setUserSeqCallback(IUserSeqCallback userSeqCallback) {
        sUserSeqCallback = userSeqCallback;
    }

    public static void retry(long time){
        Log.e(TAG,"retry:");
        if(heartBeatHandler != null) {
            heartBeatHandler.sendEmptyMessageDelayed(HEART_BEAT_RETRY, time);
        }
    }

    public static void userRetry(long time){
        Log.e(TAG,"userRetry:");
        if(heartBeatHandler != null) {
            heartBeatHandler.sendEmptyMessageDelayed(HEART_BEAT_USER_RETRY, time);
        }
    }

    public static void timeoutLimit(){
        Log.e(TAG,"timeoutLimit:");
        if(heartBeatHandler != null) {
            long time = 180*1000;
            if(sHeartBeatTime > 0 && sHeartBeatTime <= 90 * 1000)
            {
                time = time - sHeartBeatTime;
            }
            heartBeatHandler.sendEmptyMessageDelayed(HEART_BEAT_DISCONNECT, time);
        }
    }

    public static void request(long time){
        Log.e(TAG,"request:" + time);
        if(heartBeatHandler != null) {
            heartBeatHandler.sendEmptyMessageDelayed(HEART_BEAT_REQUEST, time);
        }
    }

}
