package im.zego.liveshow.liveroom.view;

import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.animation.ValueAnimator;
import android.content.Context;
import android.util.AttributeSet;
import android.util.Log;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AlphaAnimation;
import android.view.animation.PathInterpolator;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import im.zego.common.util.MiscUtils;
import im.zego.liveshow.R;

public class HintCoverView extends FrameLayout {

    public static final int TYPE_UP = 1;
    public static final int TYPE_DOWN = 2;
    public static final int TYPE_UP_AND_DOWN = 3;

    public static final int TYPE_HINT_ENTER = 1;
    public static final int TYPE_HINT_OUTSIDE = 2;

    private ImageView mGesture;
    private ImageView mArrows;
    private ImageView mArrowsUp;
    private ImageView mArrowsDown;
    private TextView mHintText;

    private int type = TYPE_UP;
    private int hintType = TYPE_HINT_ENTER;


    private View mLeave;

    private OnClickListener mListener;

    public HintCoverView(@NonNull Context context) {
        this(context,null);
    }

    public HintCoverView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public HintCoverView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        View view = View.inflate(context, R.layout.liveshow_live_room_hint,this);
        initView();
    }

    private void initView(){

        mGesture = findViewById(R.id.gesture);
        mArrows = findViewById(R.id.arrows);
        mArrowsUp = findViewById(R.id.arrows_up);
        mArrowsDown = findViewById(R.id.arrows_down);
        mHintText = findViewById(R.id.hint_text);
        mLeave = findViewById(R.id.leave);

        mLeave.setOnTouchListener(new OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {

                ((ViewGroup)getParent()).removeView(HintCoverView.this);

                if(mListener != null){
                    mListener.onClick(mLeave);
                }

                return false;
            }
        });
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        if(type == TYPE_DOWN) {
            mArrows.setVisibility(View.VISIBLE);
            mArrowsUp.setVisibility(GONE);
            mArrowsDown.setVisibility(GONE);
            mArrows.setRotationX(180);
            if(hintType == TYPE_HINT_ENTER) {
                mHintText.setText("下滑可观看更多直播间");
                mLeave.setVisibility(GONE);
            }else {
                mHintText.setText("真的要离开吗？\n试试下滑切换直播间吧");
                mLeave.setVisibility(VISIBLE);
            }

            animDown();
        }else if(type == TYPE_UP){
            mArrows.setVisibility(View.VISIBLE);
            mArrowsUp.setVisibility(GONE);
            mArrowsDown.setVisibility(GONE);
            mArrows.setRotationX(0);
            if(hintType == TYPE_HINT_ENTER) {
                mHintText.setText("上滑可观看更多直播间");
                mLeave.setVisibility(GONE);
            }else {
                mHintText.setText("真的要离开吗？\n试试上滑切换直播间吧");
                mLeave.setVisibility(VISIBLE);
            }
            animUp();
        }else {
            mArrows.setVisibility(View.INVISIBLE);
            mArrowsUp.setVisibility(VISIBLE);
            mArrowsDown.setVisibility(VISIBLE);

            if(hintType == TYPE_HINT_ENTER) {
                mHintText.setText("上下滑动可切换直播间");
                mLeave.setVisibility(GONE);
            }else {
                mHintText.setText("真的要离开吗？\n试试上下滑动切换直播间吧");
                mLeave.setVisibility(VISIBLE);
            }
            animUpAndDown();
        }

    }

    public void animUp(){
        int px = MiscUtils.dp2px(getContext(),92);

        ObjectAnimator alpha = ObjectAnimator.ofFloat(mGesture, "alpha", 0f, 1f);
        alpha.setInterpolator(new PathInterpolator(0.25f,0.1f,0.25f,1f));
        alpha.setDuration(300);

        ObjectAnimator translation = ObjectAnimator.ofFloat(mGesture, "translationY", 0f);
        translation.setDuration(1);


        ObjectAnimator translation2 = ObjectAnimator.ofFloat(mGesture, "translationY", 0f, -px);
        translation2.setInterpolator(new PathInterpolator(0.25f,0.1f,0.25f,1f));
        translation2.setDuration(1100);

        ObjectAnimator alpha2 = ObjectAnimator.ofFloat(mGesture, "alpha", 1f, 0f);
        alpha2.setInterpolator(new PathInterpolator(0f,0f,0.57f,0.26f));
        alpha2.setDuration(1100);



        AnimatorSet animatorSet  = new AnimatorSet();
        animatorSet.play(alpha).with(translation).before(translation2).before(alpha2);
        animatorSet.start();

        if(isAttachedToWindow()) {
            postDelayed(new Runnable() {
                @Override
                public void run() {
                    animUp();
                }
            }, 1500);
        }
    }

    public void animDown(){
        int px = MiscUtils.dp2px(getContext(),92);

        ObjectAnimator alpha = ObjectAnimator.ofFloat(mGesture, "alpha", 0f, 1f);
        alpha.setInterpolator(new PathInterpolator(0.25f,0.1f,0.25f,1f));
        alpha.setDuration(300);

        ObjectAnimator translation = ObjectAnimator.ofFloat(mGesture, "translationY", -px);
        translation.setDuration(1);


        ObjectAnimator translation2 = ObjectAnimator.ofFloat(mGesture, "translationY", -px, 0f);
        translation2.setInterpolator(new PathInterpolator(0.25f,0.1f,0.25f,1f));
        translation2.setDuration(1100);

        ObjectAnimator alpha2 = ObjectAnimator.ofFloat(mGesture, "alpha", 1f, 0f);
        alpha2.setInterpolator(new PathInterpolator(0f,0f,0.57f,0.26f));
        alpha2.setDuration(1100);



        AnimatorSet animatorSet  = new AnimatorSet();
        animatorSet.play(alpha).with(translation).before(translation2).before(alpha2);
        animatorSet.start();

        if(isAttachedToWindow()) {
            postDelayed(new Runnable() {
                @Override
                public void run() {
                    animDown();
                }
            }, 1500);
        }
    }

    public void animUpAndDown(){
        int px = MiscUtils.dp2px(getContext(),92);


        ObjectAnimator translation = ObjectAnimator.ofFloat(mGesture, "translationY", 0f);
        translation.setDuration(1);


        ObjectAnimator translation2 = ObjectAnimator.ofFloat(mGesture, "translationY", 0f,-px);
        translation2.setInterpolator(new PathInterpolator(0.42f,0f,0.58f,1f));
        translation2.setDuration(750);

        ObjectAnimator translation3 = ObjectAnimator.ofFloat(mGesture, "translationY", -px, 0f);
        translation3.setInterpolator(new PathInterpolator(0.42f,0f,0.58f,1f));
        translation3.setDuration(750);
        translation3.setStartDelay(550);


        AnimatorSet animatorSet  = new AnimatorSet();
        animatorSet.play(translation2).with(translation).before(translation3);
        animatorSet.start();

        if(isAttachedToWindow()) {
            postDelayed(new Runnable() {
                @Override
                public void run() {
                    animUpAndDown();
                }
            }, 2600);
        }
    }


    public void setType(int type) {
        this.type = type;
    }

    public void setHintType(int hintType) {
        this.hintType = hintType;
    }

    public void setLeaveListener(OnClickListener onClickListener){
        mListener = onClickListener;
    }
}
