package im.zego.liveshow.liveroom.view;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Color;
import android.util.AttributeSet;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import im.zego.liveshow.R;

public class ResolutionView extends ConstraintLayout {

    private int mResolutionWidth = 0;
    private int mResolutionHeight = 0;
    private int mResolutionFPS = 0;
    private int mResolutionBitrate = 0;
    private boolean mCheck = false;

    private final TextView mResolutionType;
    private final TextView mResolutionName;

    public ResolutionView(@NonNull Context context) {
        this(context,null);
    }

    public ResolutionView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public ResolutionView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.ResolutionView);
        View view = inflate(context,R.layout.liveshow_view_resolution,this);
        String type = typedArray.getString(R.styleable.ResolutionView_resolution_type);
        String name = typedArray.getString(R.styleable.ResolutionView_resolution_name);
        mResolutionWidth = typedArray.getInt(R.styleable.ResolutionView_resolution_width,0);
        mResolutionHeight = typedArray.getInt(R.styleable.ResolutionView_resolution_height,0);
        mResolutionFPS = typedArray.getInt(R.styleable.ResolutionView_resolution_fps,0);
        mResolutionBitrate = typedArray.getInt(R.styleable.ResolutionView_resolution_bitrate,0);
        mCheck = typedArray.getBoolean(R.styleable.ResolutionView_check,false);
        typedArray.recycle();
        mResolutionType = view.findViewById(R.id.resolution_type);
        mResolutionName = view.findViewById(R.id.resolution_name);

        mResolutionType.setText(type);
        mResolutionName.setText(name);
        setCheck(mCheck);
    }


    public int getResolutionWidth() {
        return mResolutionWidth;
    }

    public void setResolutionWidth(int resolutionWidth) {
        this.mResolutionWidth = resolutionWidth;
    }

    public int getResolutionHeight() {
        return mResolutionHeight;
    }

    public void setResolutionHeight(int resolutionHeight) {
        this.mResolutionHeight = resolutionHeight;
    }

    public int getResolutionFPS() {
        return mResolutionFPS;
    }

    public void setResolutionFPS(int resolutionFPS) {
        this.mResolutionFPS = resolutionFPS;
    }

    public int getResolutionBitrate() {
        return mResolutionBitrate;
    }

    public void setResolutionBitrate(int resolutionBitrate) {
        this.mResolutionBitrate = resolutionBitrate;
    }

    public boolean ismCheck() {
        return mCheck;
    }

    public void setCheck(boolean check) {
        this.mCheck = check;
        if(mCheck)
        {
            mResolutionType.setTextColor(Color.parseColor("#A653FF"));
            mResolutionName.setTextColor(Color.parseColor("#A653FF"));
        }else {
            mResolutionType.setTextColor(Color.parseColor("#CCCCCC"));
            mResolutionName.setTextColor(Color.parseColor("#CCCCCC"));
        }
    }
}
