package im.zego.liveshow.manager;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;
import im.zego.liveshow.Constants;
import im.zego.liveshow.R;
import im.zego.liveshow.liveroom.utils.NetworkCallbackImpl;
import im.zego.liveshow.manager.callbacks.INetworkOperationCallback;
import im.zego.liveshow.model.OnstageListInfo;
import im.zego.liveshow.model.RoomInfo;
import im.zego.liveshow.model.StatefulListInfo;
import im.zego.liveshow.model.UserInfo;
import im.zego.liveshow.model.rtc.NotifyApplyMicStateInfo;
import im.zego.liveshow.protocol.IGetConMicListCallback;
import im.zego.liveshow.protocol.IGetHostAudiencetCallback;
import im.zego.liveshow.protocol.RoomAPI;
import im.zego.liveshow.utils.DialogQueueUtils;
import im.zego.liveshow.utils.ToastUtil;
import im.zego.liveshow.view.RoomBottomLayerLayout;

import static im.zego.liveshow.Constants.ROOM_ROLE_AUDIENCE;
import static im.zego.liveshow.Constants.ROOM_ROLE_HOST;

public class NetWorkConManager {
    private static final String TAG = "NetWorkConManager";

    private INetworkOperationCallback mCallback;
    private ConnectivityManager connMgr;
    private NetworkCallbackImpl networkCallback;

    /**
     * 是否是第一次开始连接网咯
     */
    private AtomicBoolean startFirst = new AtomicBoolean(true);
    /**
     * 用户状态seq
     */
    private AtomicLong personalStateSeq = new AtomicLong(0);

    /**
     * 申请连麦的数据
     */
    private List<OnstageListInfo.ReqListDTO> uidList = new ArrayList<>();


    private volatile static NetWorkConManager mInstance;

    public static NetWorkConManager getInstance() {
        if (mInstance == null) {
            synchronized (NetWorkConManager.class) {
                if (mInstance == null) {
                    mInstance = new NetWorkConManager();
                }
            }
        }
        return mInstance;
    }

    /**
     * 网咯监听
     *
     * @param isAvailable
     */
    public void networState(boolean isAvailable) {
        if (isAvailable && !startFirst.get()) {
            getOnstageList();

            //恢复网络的话，要重新设置房间人数信息
            if (mCallback != null) {
                mCallback.onlineNum();
            }
        }
    }

    /**
     * 获取连麦申请列表
     */
    private void getOnstageList() {
        if(RoomManager.getInstance().getRoomInfo() == null){
            return;
        }

        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();

        RoomAPI.getOnstageList(userInfo.getUid(), roomInfo.getRoomId(), new IGetConMicListCallback() {
            @Override
            public void getOnstageList(int retcode, OnstageListInfo responseJsonBean) {
                if (retcode == 0) {
                    uidList.clear();
                    uidList.addAll(responseJsonBean.getReqListDTOList());
                    getStatefulList();
                }
            }
        });

    }

    /**
     * 拉取连麦人员列表
     */
    private void getStatefulList() {
        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();

        RoomManager.getInstance().getStatefulList(roomInfo.getRoomId(), userInfo.getUid(), new IGetHostAudiencetCallback() {
            @Override
            public void onGetStatefulData(int errorCode, StatefulListInfo.AttendeeListDTO hostInfo, List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo,
                                          StatefulListInfo statefulListInfo,int attendeeSeq, int stateSeq) {

                boolean crashData = stateSeq >= personalStateSeq.get();

                if (UserManager.getInstance().getUserInfo().getRole() == ROOM_ROLE_AUDIENCE) {
                    //观众端
                    if (RoomManager.getInstance().isHasConMic() && !uidListIncludeMe()) {
                        //有申请连麦才有接下来的数据处理
                        audienceData(audienceStatefulListInfo, crashData);
                    } else {

                        //这个是观众主动连麦请求
                        callbackAudienceData(audienceStatefulListInfo, 0, crashData);

                        StatefulListInfo.AttendeeListDTO myInfo = returnAudienceListIncludeMe(statefulListInfo.getInvitedList());
                        //解决 观众断网但是房主邀请连麦、房主的按钮变更了但是观众恢复网络收不到弹窗的问题
                        if (myInfo != null && myInfo.getOnstageState() == Constants.STATE_INVITE_CON_MIC) {
                            if (mCallback != null) {
                                mCallback.inviteMicAudienc(myInfo);
                            }
                        }

                    }

                } else if (UserManager.getInstance().getUserInfo().getRole() == ROOM_ROLE_HOST) {
                    //房主端
                    hostData(audienceStatefulListInfo, crashData);
                }

                personalStateSeq.set(stateSeq);
            }
        });
    }

    /**
     * 观众端处理断网重连后的数据处理
     *
     * @param audienceStatefulListInfo
     */
    private void audienceData(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo, boolean crashData) {
        if (audienceListIncludeMe(audienceStatefulListInfo)) {
            //房主同意连申请，要上麦刷新数据，按钮变更为成功上麦
            callbackAudienceData(audienceStatefulListInfo, RoomBottomLayerLayout.CONNECTION_MIC_SUCCESS, crashData);
        } else {
            //房主拒绝连连麦申请，按钮变更回连麦的状态；toast提示
            ToastUtil.showToast(R.string.liveshow_connect_mic_refuse);
            callbackAudienceData(audienceStatefulListInfo, RoomBottomLayerLayout.CONNECTION_MIC_PREPARE, crashData);
        }
    }

    /**
     * 房主端处理断网重连后的数据处理
     */
    private void hostData(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo, boolean crashData) {
        callbackHostData(audienceStatefulListInfo, crashData);
        NotifyApplyMicStateInfo dialogInfo = DialogQueueUtils.getInstance().getmNotifyApplyMicStateInfo();
        if (DialogQueueUtils.getInstance().getDialogQueue().getmDialogQueue().isEmpty() && uidList.size() > 0 && dialogInfo == null) {
            //如果弹窗列表数据为空，但是申请连麦的数据不为空就需要弹窗
            for (OnstageListInfo.ReqListDTO info : uidList) {
                callbackMicStateNotify(Constants.OPERATION_ACTION_APPLY, gainMicStateInfo(info));
            }
        } else if (dialogInfo != null) {
            if (uidList.size() > 0) {
                //弹窗数据不为空，申请连麦的数据也不为空。说明有多个连麦数据
                if (!hasDialog(dialogInfo)) {
                    //当前弹出的弹窗没有在申请连麦列表理说明用户已经取消连麦
                    callbackMicStateNotify(Constants.OPERATION_ACTION_CANCEL, null);
                } else {
                    //说明有其他的弹窗存在，需要判断该弹窗是否被观众取消
                    for (NotifyApplyMicStateInfo stateInfo : DialogQueueUtils.getInstance().getDialogQueue().getmDialogQueue()) {
                        if (!hasDialog(stateInfo)) {
                            DialogQueueUtils.getInstance().remove(stateInfo);
                        }
                    }
                }

            } else {
                DialogQueueUtils.getInstance().getDialogQueue().clear();
                //弹窗数据不为空，申请连麦的数据为空。说明有用户取消连麦。需要取消弹窗并toast提示 观众已取消连麦申请
                callbackMicStateNotify(Constants.OPERATION_ACTION_CANCEL, null);
            }

        }

    }

    /**
     * 观众端数据回调
     *
     * @param audienceStatefulListInfo
     * @param micType
     * @param crashData
     */
    private void callbackAudienceData(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo, int micType, boolean crashData) {
        if (mCallback != null) {
            mCallback.audienceData(audienceStatefulListInfo, micType, crashData);
        }
    }


    /**
     * 房主端数据回调
     *
     * @param audienceStatefulListInfo
     * @param crashData
     */
    private void callbackHostData(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo, boolean crashData) {
        if (mCallback != null) {
            mCallback.hostData(audienceStatefulListInfo, crashData);
        }
    }

    /**
     * 连麦的弹窗需要处理的回调
     * @param action
     * @param notifyInfo
     */
    private void callbackMicStateNotify(int action, NotifyApplyMicStateInfo notifyInfo) {
        if (mCallback != null) {
            mCallback.applyMicStateNotify(action, notifyInfo);
        }
    }

    /**
     * 申请连麦列表是否有自己
     *
     * @return
     */
    private boolean uidListIncludeMe() {
        for (OnstageListInfo.ReqListDTO info : uidList) {
            if (UserManager.getInstance().getUserInfo().getUid() == info.getUid()) {
                return true;
            }
        }
        return false;
    }

    /**
     * 已经上麦列表是否包含本人
     *
     * @param audienceStatefulListInfo
     * @return
     */
    private boolean audienceListIncludeMe(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo) {
        for (StatefulListInfo.AttendeeListDTO info : audienceStatefulListInfo) {
            if (UserManager.getInstance().getUserInfo().getUid() == info.getUid()) {
                return true;
            }
        }
        return false;
    }

    /**
     * 返回本人的数据
     * @param audienceStatefulListInfo
     * @return
     */
    private StatefulListInfo.AttendeeListDTO returnAudienceListIncludeMe(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo) {
        for (StatefulListInfo.AttendeeListDTO info : audienceStatefulListInfo) {
            if (UserManager.getInstance().getUserInfo().getUid() == info.getUid()) {
                return info;
            }
        }
        return null;
    }

    /**
     * 申请连麦列表是否有对应的弹窗
     *
     * @param stateInfo
     * @return
     */
    private boolean hasDialog(NotifyApplyMicStateInfo stateInfo) {
        for (OnstageListInfo.ReqListDTO info : uidList) {
            if (info.getUid() == stateInfo.getUid()) {
                return true;
            }
        }
        return false;
    }

    /**
     * 数据的转换
     *
     * @param info
     * @return
     */
    private NotifyApplyMicStateInfo gainMicStateInfo(OnstageListInfo.ReqListDTO info) {
        NotifyApplyMicStateInfo notifyInfo = new NotifyApplyMicStateInfo();
        notifyInfo.setUid(info.getUid());
        notifyInfo.setNickName(info.getNickName());
        return notifyInfo;
    }

    /**
     * 注册网络状态的回调
     * @param context
     */
    public void registerNetworkCallback(Context context) {
        networkCallback = new NetworkCallbackImpl();
        NetworkRequest.Builder builder = new NetworkRequest.Builder();
        NetworkRequest request = builder.build();
        connMgr = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        if (connMgr != null) {
            connMgr.registerNetworkCallback(request, networkCallback);
        }
    }

    private void unregisterNetworkCallback() {
        if (connMgr != null && networkCallback != null) {
            connMgr.unregisterNetworkCallback(networkCallback);
        }
    }

    public void setCallBack(INetworkOperationCallback callBack) {
        this.mCallback = callBack;
    }

    public void setStartFirst(boolean flag) {
        this.startFirst.set(flag);
    }

    public void setPersonalStateSeq(long personalStateSeq) {
        this.personalStateSeq.set(personalStateSeq);
    }

    public void clearData() {
        setStartFirst(true);
        setPersonalStateSeq(0);
        setCallBack(null);
        unregisterNetworkCallback();
    }
}
