package im.zego.liveshow.manager;

import android.content.Context;
import android.view.Gravity;
import android.view.View;
import android.view.Window;

import androidx.annotation.NonNull;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;

import im.zego.liveshow.R;
import im.zego.liveshow.liveroom.view.ResolutionView;
import im.zego.liveshow.sdk.ZegoSDKManager;

import static im.zego.liveshow.Constants.*;

public class ResolutionManager {

    private int mResolutionType = 1;//分辨率类型，从0开始

    private static volatile ResolutionManager singleton = null;

    private ResolutionManager() {
    }

    public static ResolutionManager getInstance() {
        if (singleton == null) {
            synchronized (RoomManager.class) {
                if (singleton == null) {
                    singleton = new ResolutionManager();
                }
            }
        }
        return singleton;
    }

    public void reset(int roomRole){
        if(roomRole == ROOM_ROLE_HOST){
            mResolutionType = 1;
            ZegoSDKManager.getInstance().getDeviceService().setVideoConfig(720, 1280, 1500 * 1000, 15);
        }else {
            mResolutionType = 3;
            ZegoSDKManager.getInstance().getDeviceService().setVideoConfig(360, 640, 600 * 1000, 15);
        }
    }


    public void showBottomSheetDialog(Context activity) {
        View view = View.inflate(activity, R.layout.liveshow_dialog_resolution, null);
        final ResolutionView resolutionView1080 = view.findViewById(R.id.resolution_1080);
        final ResolutionView resolutionView720 = view.findViewById(R.id.resolution_720);
        final ResolutionView resolutionView540 = view.findViewById(R.id.resolution_540);
        final ResolutionView resolutionView360 = view.findViewById(R.id.resolution_360);
        final ResolutionView resolutionView270 = view.findViewById(R.id.resolution_270);
        final ResolutionView resolutionView180 = view.findViewById(R.id.resolution_180);

        ResolutionView[] list = new ResolutionView[]{resolutionView1080, resolutionView720, resolutionView540,
                resolutionView360, resolutionView270, resolutionView180};

        list[mResolutionType].setCheck(true);

        View.OnClickListener clickListener = new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                resolutionView1080.setCheck(false);
                resolutionView720.setCheck(false);
                resolutionView540.setCheck(false);
                resolutionView360.setCheck(false);
                resolutionView270.setCheck(false);
                resolutionView180.setCheck(false);

                if (v instanceof ResolutionView) {
                    ResolutionView resolutionView = ((ResolutionView) v);
                    resolutionView.setCheck(true);
                    for (int i = 0; i < list.length; i++) {
                        if (list[i] == v) {
                            mResolutionType = i;
                        }
                    }
                    ZegoSDKManager.getInstance().getDeviceService().setVideoConfig(resolutionView.getResolutionWidth(),
                            resolutionView.getResolutionHeight(), resolutionView.getResolutionBitrate() * 1000, resolutionView.getResolutionFPS());
                }
            }
        };

        resolutionView1080.setOnClickListener(clickListener);
        resolutionView720.setOnClickListener(clickListener);
        resolutionView540.setOnClickListener(clickListener);
        resolutionView360.setOnClickListener(clickListener);
        resolutionView270.setOnClickListener(clickListener);
        resolutionView180.setOnClickListener(clickListener);

        BottomSheetDialog mBottomSheetDialog = new BottomSheetDialog(activity, R.style.BottomSheetDialog);
        mBottomSheetDialog.setContentView(view);
        final BottomSheetBehavior mDialogBehavior = BottomSheetBehavior.from((View) view.getParent());
        mDialogBehavior.addBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                    mBottomSheetDialog.dismiss();
                    mDialogBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {
            }
        });
        Window localWindow = mBottomSheetDialog.getWindow();
        localWindow.setGravity(Gravity.BOTTOM);
        localWindow.setWindowAnimations(R.style.LiveShowBottomSheetDialogAnim);

        mBottomSheetDialog.show();
    }




}
