package im.zego.liveshow.manager;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.TextureView;


import androidx.annotation.NonNull;

import com.google.gson.Gson;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import im.zego.liveshow.Constants;
import im.zego.liveshow.ErrorcodeConstants;
import im.zego.liveshow.R;
import im.zego.liveshow.effects.EffectsBeautyManager;
import im.zego.liveshow.liveroom.callbacks.IAnchorInfoCallback;
import im.zego.liveshow.manager.callbacks.ICreateRoomFailTimeoutCallback;
import im.zego.liveshow.liveroom.callbacks.IRoomQualityInfoCallback;
import im.zego.liveshow.manager.callbacks.IRoomConnectionCallback;
import im.zego.liveshow.manager.callbacks.IRoomDisconnectCallback;
import im.zego.liveshow.manager.callbacks.ISDKRoomStateCallback;
import im.zego.liveshow.manager.callbacks.ISwitchRoomStateCallback;
import im.zego.liveshow.model.OnlineInfo;
import im.zego.liveshow.model.RoomInfo;
import im.zego.liveshow.model.RoomListInfo;
import im.zego.liveshow.model.StatefulListInfo;
import im.zego.liveshow.model.UserInfo;
import im.zego.liveshow.model.rtc.NotifyApplyMicStateInfo;
import im.zego.liveshow.model.rtc.NotifyRefuseApplyMicInfo;
import im.zego.liveshow.model.rtc.NotifyUserListUpdateInfo;
import im.zego.liveshow.model.rtc.NotifyUserStateUpdateInfo;
import im.zego.liveshow.protocol.IApplyMicStateNotifyCallback;
import im.zego.liveshow.protocol.ICommonResponseCallback;
import im.zego.liveshow.protocol.ICreateRoomCallback;
import im.zego.liveshow.protocol.IExecuteCallback;
import im.zego.liveshow.protocol.IGetHostAudiencetCallback;
import im.zego.liveshow.protocol.IGetOnlineInfoCallback;
import im.zego.liveshow.protocol.IGetRoomListCallback;
import im.zego.liveshow.protocol.ILoginRoomCallback;
import im.zego.liveshow.manager.callbacks.IStreamCallback;
import im.zego.liveshow.protocol.RoomAPI;
import im.zego.liveshow.sdk.ZegoSDKManager;
import im.zego.liveshow.sdk.callbacks.IPublisherQualityUpdateCallback;
import im.zego.liveshow.sdk.callbacks.IRTCCommonCallback;
import im.zego.liveshow.sdk.callbacks.IRTCEventCallback;
import im.zego.liveshow.sdk.callbacks.IRoomMessageCallback;
import im.zego.liveshow.sdk.callbacks.IRoomStateCallback;
import im.zego.liveshow.sdk.callbacks.IStreamActiveCallback;
import im.zego.liveshow.sdk.callbacks.IStreamCountListener;
import im.zego.liveshow.sdk.callbacks.IZegoRoomConnectionStateListener;
import im.zego.liveshow.sdk.model.ZegoPlayStream;
import im.zego.liveshow.sdk.model.ZegoPublishStream;
import im.zego.liveshow.sdk.model.ZegoStream;
import im.zego.liveshow.utils.NetworkSpeedUtil;
import im.zego.liveshow.utils.ToastUtil;

import static im.zego.liveshow.Constants.MIC_CAMERA_CLOSE;
import static im.zego.liveshow.Constants.MIC_CAMERA_OPEN;
import static im.zego.liveshow.Constants.OPERATION_ACTION_APPLY;
import static im.zego.liveshow.Constants.OPERATION_TYPE_CAMERA;
import static im.zego.liveshow.Constants.OPERATION_TYPE_MIC;
import static im.zego.liveshow.Constants.OPERATION_TYPE_ONSTAGE_FULL;
import static im.zego.liveshow.Constants.OPERATION_TYPE_STAGE_STATE;
import static im.zego.liveshow.Constants.ROOM_ROLE_AUDIENCE;
import static im.zego.liveshow.Constants.ROOM_ROLE_HOST;

public class RoomManager {

    private static final String TAG = "RoomManager";
    private static final int networkError = 888;
    private static volatile RoomManager singleton = null;

    private Gson gson = new Gson();
    /**
     * 连麦人员列表(观众列表不包含主持人)
     */
    private List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo = new ArrayList<>();
    private StatefulListInfo.AttendeeListDTO hostInfo; //主持人数据
    private boolean mFront = true;//是否使用前置摄像头
    private boolean hasConMic = false; //本人是否有申请连麦、成功上麦后要设置为false
    private boolean micIsOffFromHost = false; //主播是否禁言

    private StatefulListInfo.AttendeeListDTO dialogIsShowInfo = null; //主播禁言弹窗是否弹出数据

    private IAnchorInfoCallback mAnchorInfoCallback = null;
    private IRoomQualityInfoCallback mRoomQualityInfoCallback = null;
    private ICreateRoomFailTimeoutCallback mCreateRoomFailTimeoutCallback = null;
    private IRoomDisconnectCallback mRoomDisconnectCallback;
    private IRoomConnectionCallback mRoomConnectionCallback;
    private ISDKRoomStateCallback mSDKRoomStateCallback;

    private HashMap<String, ISwitchRoomStateCallback> roomMap = new HashMap<>();


    private List<ZegoStream> mStreamList = new ArrayList<ZegoStream>();

    private Handler handler = new Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(@NonNull Message msg) {
            if (msg.what == networkError) {
                handler.removeMessages(networkError);
                if (mCreateRoomFailTimeoutCallback != null) {
                    mCreateRoomFailTimeoutCallback.onCreateRoomFailTimeout();
                }
            }
        }
    };

    private RoomManager() {
    }

    public static RoomManager getInstance() {
        if (singleton == null) {
            synchronized (RoomManager.class) {
                if (singleton == null) {
                    singleton = new RoomManager();
                }
            }
        }
        return singleton;
    }


    private RoomInfo mRoomInfo;
    private String mRoomID;

    public RoomInfo getRoomInfo() {
        return mRoomInfo;
    }

    public String getRoomID() {
        return mRoomID;
    }

    public void setRoomID(String roomID) {
        mRoomID = roomID;
    }

    public void setRoomInfo(RoomInfo roomInfo) {
        this.mRoomInfo = roomInfo;

        mRoomID = roomInfo.getRoomInfo().getRoomId();
        ZegoSDKManager.getInstance().setRoomMessageCallback(new IRoomMessageCallback() {

            @Override
            public void onReceiveBroadcastMessage(String userID, String userName, String content, String roomID) {
                final RoomInfo roomInfo = getRoomInfo();
                if (roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)) {
                    return;
                }


                try {
                    JSONObject jsonObject = new JSONObject(content);
                    int cmd = jsonObject.getInt("cmd");
                    String data = jsonObject.getString("data");
                    Log.d(TAG, "cmd:" + cmd);
                    Log.d(TAG, "data:" + data);
                    if (cmd == 6002) {  // 成员状态变更通知
                        if (mRTCEventCallback != null) {
                            mRTCEventCallback.onUserStateUpdateNotify(gson.fromJson(data, NotifyUserStateUpdateInfo.class));
                        }
                    } else if (cmd == 6003) { // 成员列表变更通知
                        if (mRTCEventCallback != null) {
                            mRTCEventCallback.onUserListUpdateNotify(gson.fromJson(data, NotifyUserListUpdateInfo.class));
                        }

                    } else if (cmd == 6006) { // 房间关闭
                        if (mRTCEventCallback != null) {
                            mRTCEventCallback.onRoomCloseNotify();
                        }
                    }
                } catch (JSONException jsonException) {
                    jsonException.printStackTrace();
                }
            }

            @Override
            public void onReceiveCustomCommand(String userID, String userName, String content, String roomID) {
                final RoomInfo roomInfo = getRoomInfo();
                if (roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)) {
                    return;
                }


                try {
                    JSONObject jsonObject = new JSONObject(content);
                    int cmd = jsonObject.getInt("cmd");
                    String data = jsonObject.getString("data");
                    Log.d(TAG, "cmd--:" + cmd);
                    Log.d(TAG, "data--:" + data);
                    if (cmd == 12006) { // 连麦申请
                        if (mRTCEventCallback != null) {
                            mRTCEventCallback.onApplyMicStateNotify(gson.fromJson(data, NotifyApplyMicStateInfo.class));
                        }
                    } else if (cmd == 12007) { // 取消连麦申请
                        if (mRTCEventCallback != null) {
                            mRTCEventCallback.onCancelApplyMicStateNotify(gson.fromJson(data, NotifyApplyMicStateInfo.class));
                        }
                    } else if (cmd == 12008) { // 拒绝申请
                        if (mRTCEventCallback != null) {
                            mRTCEventCallback.onRefuseApplyMicNotify(gson.fromJson(data, NotifyRefuseApplyMicInfo.class));
                        }
                    }
                } catch (JSONException jsonException) {
                    jsonException.printStackTrace();
                }
            }
        });
        ZegoSDKManager.getInstance().setStreamCountListener(mIStreamCountListener);
        ZegoSDKManager.getInstance().setRoomConnectionStateListener(new IZegoRoomConnectionStateListener() {
            @Override
            public void onConnected(int errorCode, String roomID) {

                if (mRoomConnectionCallback != null) {
                    mRoomConnectionCallback.onConnected();
                }
            }

            @Override
            public void onDisconnect(int errorCode, String roomID) {
                if (mRoomDisconnectCallback != null) {
                    mRoomDisconnectCallback.onDisconnect();
                }
            }

            @Override
            public void connecting(int errorCode, String roomID) {
                if (mRoomConnectionCallback != null) {
                    mRoomConnectionCallback.connecting();
                }
            }
        });
    }

    private IRTCEventCallback mRTCEventCallback = new IRTCEventCallback() {
        @Override
        public void onUserStateUpdateNotify(@NonNull NotifyUserStateUpdateInfo notifyInfo) {

            if (notifyInfo != null && notifyInfo.getUsers().size() > 0) {
                //1.mic变化 2.cameara 变化 3.连麦状态变化  4. 被邀请连麦 5. 取消邀请连麦 6. 拒绝邀请连麦 7. 因为台上人数满上台失败
                if (notifyInfo.getType() == OPERATION_TYPE_STAGE_STATE) {
                    List<NotifyUserStateUpdateInfo.UsersDTO> stateUpdateInfo = notifyInfo.getUsers();
                    for (int i = 0; i < stateUpdateInfo.size(); i++) {
                        NotifyUserStateUpdateInfo.UsersDTO usersDTO = stateUpdateInfo.get(i);

                        if (iApplyMicStateNotifyCallback != null) {
                            iApplyMicStateNotifyCallback.onMyStageNotify(usersDTO);
                        }

                        long userInfoID = mRoomInfo.getUserInfo().getUid();
                        if (userInfoID != usersDTO.getUid()) {
                            //房主本人即提示 已经结束连麦
                            if (mRoomInfo.getRoomInfo().getHostId() == userInfoID && usersDTO.getOnstageState() == 1) {
                                ToastUtil.showToast(notifyInfo.getOperatorName() + "已结束连麦");
                            }
                        } else {
                            //说明本人下麦（1.主动下麦不需要提示 2.被主播移下麦 要提示 你已经被主播移下麦）
                            if (notifyInfo.getOperatorUid() == mRoomInfo.getRoomInfo().getHostId() && usersDTO.getOnstageState() == 1) {
                                ToastUtil.showToast("你已被主播移下麦");
                            }
                        }
                    }

                    //被邀请连麦的逻辑（同意连麦-上台）
                    if (iApplyMicStateNotifyCallback != null) {
                        iApplyMicStateNotifyCallback.onInviteMicNotify(notifyInfo);
                    }

                } else {

                    if (notifyInfo.getType() == OPERATION_TYPE_MIC || notifyInfo.getType() == OPERATION_TYPE_CAMERA) {
                        if (iApplyMicStateNotifyCallback != null) {
                            iApplyMicStateNotifyCallback.onAudienceListNotify(notifyInfo);
                        }
                    } else {
                        //被邀请连麦的逻辑（被邀请/拒绝）
                        if (iApplyMicStateNotifyCallback != null) {
                            iApplyMicStateNotifyCallback.onInviteMicNotify(notifyInfo);
                        }

                    }

                }
            }

        }

        @Override
        public void onUserListUpdateNotify(@NonNull NotifyUserListUpdateInfo notifyInfo) {

            if (iApplyMicStateNotifyCallback != null) {
                //成员列表变更通知
                iApplyMicStateNotifyCallback.onUserListUpdateNotify(notifyInfo);
            }

        }

        @Override
        public void onApplyMicStateNotify(@NonNull NotifyApplyMicStateInfo notifyInfo) {
            if (iApplyMicStateNotifyCallback != null) {
                //1是申请连麦
                iApplyMicStateNotifyCallback.onApplyMicStateNotify(Constants.OPERATION_ACTION_APPLY, notifyInfo);
            }
        }

        @Override
        public void onCancelApplyMicStateNotify(@NonNull NotifyApplyMicStateInfo notifyInfo) {
            if (iApplyMicStateNotifyCallback != null) {
                //2是取消申请连麦
                iApplyMicStateNotifyCallback.onApplyMicStateNotify(Constants.OPERATION_ACTION_CANCEL, notifyInfo);
            }
        }

        @Override
        public void onRefuseApplyMicNotify(@NonNull NotifyRefuseApplyMicInfo notifyInfo) {
            if (notifyInfo.getReason() == 1) {
                ToastUtil.showToast(R.string.liveshow_connect_mic_refuse);
            } else if (notifyInfo.getReason() == 2) {
                ToastUtil.showToast(R.string.liveshow_connect_mic_full);
            }
            if (iApplyMicStateNotifyCallback != null) {
                //2是取消申请连麦
                iApplyMicStateNotifyCallback.onRefuseApplyMicNotify(notifyInfo);
            }
        }

        @Override
        public void onRoomCloseNotify() {
            //房间已经关闭
            if (iApplyMicStateNotifyCallback != null) {
                //2是取消申请连麦
                iApplyMicStateNotifyCallback.onRoomCloseNotify();
            }
            ZegoSDKManager.getInstance().getRoomService().exitRoom();
        }
    };


    private IStreamCountListener mIStreamCountListener = new IStreamCountListener() {
        @Override
        public void onStreamAdd(ZegoStream zegostream) {
            if (iStreamCallback != null) {
                iStreamCallback.onStreamAdd();
            }
        }

        @Override
        public void onStreamRemove(ZegoStream zegostream) {

        }
    };

    /**
     * 登录房间
     *
     * @param roomId
     */
    public void loginRoom(final String roomId, IRTCCommonCallback commonCallback) {
        Log.i(TAG, "loginRoom:roomId:" + roomId + ":time:" + System.currentTimeMillis());
        UserInfo userInfoTemp = UserManager.getInstance().getUserInfo();
        RoomAPI.loginRoom(roomId, userInfoTemp.getNickName(), userInfoTemp.getAvatar(), new ILoginRoomCallback() {
            @Override
            public void onLoginRoom(int errorCode, RoomInfo roomInfo) {
                if (errorCode == 0) {
                    setRoomInfo(roomInfo);
                    UserInfo userInfo = roomInfo.getUserInfo();
                    UserManager.getInstance().setUserInfo(userInfo);
                    setRoomStateCallback();
                    ZegoSDKManager.getInstance().getRoomService().loginRoom(String.valueOf(userInfo.getUid()), userInfo.getNickName(), roomInfo.getRoomInfo().getRoomId(), new IRTCCommonCallback() {
                        @Override
                        public void onRTCCallback(int errorCode) {
                            Log.e("ZegoRoomService", "errorCode:" + errorCode);

                            if (commonCallback != null) {
                                commonCallback.onRTCCallback(errorCode);
                            }
                        }
                    });
                } else {
                    if (commonCallback != null) {
                        commonCallback.onRTCCallback(errorCode);
                    }
                }
            }
        });
    }

    public void createRoom(String roomName, IRTCCommonCallback commonCallback) {
        UserInfo temp = UserManager.getInstance().getUserInfo();
        String name = temp.getNickName();
        String coverImg = UserManager.getInstance().getAvatar(name);
        String avatar = UserManager.getInstance().getAvatar(name);
        RoomAPI.createRoom(roomName, coverImg, name, avatar, new ICreateRoomCallback() {
            @Override
            public void onCreateRoom(int retcode, RoomInfo createdRoomInfo) {
                if (retcode == 0) {
                    handler.removeMessages(networkError);
                    UserInfo userInfo = createdRoomInfo.getUserInfo();
                    UserManager.getInstance().setUserInfo(userInfo);
                    setRoomInfo(createdRoomInfo);
                    setRoomStateCallback();
                    ZegoSDKManager.getInstance().getRoomService().loginRoom(String.valueOf(userInfo.getUid()),
                            userInfo.getNickName(),
                            createdRoomInfo.getRoomInfo().getRoomId(),
                            new IRTCCommonCallback() {
                                @Override
                                public void onRTCCallback(int errorCode) {
                                    if (commonCallback != null) {
                                        commonCallback.onRTCCallback(errorCode);
                                    }
                                }
                            });
                } else {

                    if (commonCallback != null) {
                        commonCallback.onRTCCallback(retcode);
                    }

                    if (retcode == ErrorcodeConstants.ErrorFailNetwork) {
                        handler.sendEmptyMessageDelayed(networkError, 60 * 1000);
                    }
                    CommonErrorCodeToast(retcode);
                }

            }
        });
    }


    public void switchRoom(String lastRoom, String newRoom, IRTCCommonCallback commonCallback) {

//        if( !TextUtils.isEmpty(newRoom) && callback != null){
//            roomMap.put(newRoom,callback);
//        }

//        ISwitchRoomStateCallback lastCallback = roomMap.remove(lastRoom);
//        if(lastCallback != null){
//            lastCallback.roomLogout();
//        }


        RoomAPI.switchRoom(UserManager.getInstance().getUserId(), lastRoom, newRoom, UserManager.getInstance().getUserInfo().getNickName(),
                UserManager.getInstance().getUserInfo().getAvatar(), new ILoginRoomCallback() {
                    @Override
                    public void onLoginRoom(int errorCode, RoomInfo roomInfo) {
                        if (errorCode == 0) {

                            clearRoomData();
                            clearOtherData();
                            clearCallback();

                            setRoomInfo(roomInfo);
                            UserInfo userInfo = roomInfo.getUserInfo();
                            UserManager.getInstance().setUserInfo(userInfo);
                            setRoomStateCallback();
                            ZegoSDKManager.getInstance().getRoomService().switchRoom(String.valueOf(UserManager.getInstance().getUserId()),
                                    UserManager.getInstance().getUserInfo().getNickName(), lastRoom, newRoom, new IRTCCommonCallback() {
                                        @Override
                                        public void onRTCCallback(int errorCode) {
                                            if (commonCallback != null) {
                                                commonCallback.onRTCCallback(errorCode);
                                            }
                                        }
                                    });
                        } else {
                            if (commonCallback != null) {
                                commonCallback.onRTCCallback(errorCode);
                            }
                        }
                    }
                });

//        if(callback != null){
//            callback.roomLogin();
//        }


    }

    /**
     * 关闭房间
     *
     * @param callback
     */
    public void closeRoom(final IExecuteCallback callback) {

        RoomAPI.closeRoom(UserManager.getInstance().getUserId(), mRoomID, new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if (errorCode == 0) {
                    if (callback != null) {
                        callback.onSuccess();
                    }
                } else {
                    if (callback != null) {
                        callback.onFailed(errorCode);
                    }
                    CommonErrorCodeToast(errorCode);
                }
            }
        });
    }


    /**
     * 拉取连麦人员列表
     *
     * @param roomID
     * @param uid
     */
    public void getStatefulList(String roomID, long uid, IGetHostAudiencetCallback callback) {
        RoomAPI.getStatefulList(roomID, uid, (errorCode, statefulListInfo) -> {
            if (errorCode == 0) {

                // 先清除之前的数据
                audienceStatefulListInfo.clear();
                hostInfo = null;

                List<StatefulListInfo.AttendeeListDTO> attendeeList = statefulListInfo.getAttendeeList();
                for (int i = 0; i < attendeeList.size(); i++) {
                    StatefulListInfo.AttendeeListDTO attendeeListDTO = attendeeList.get(i);
                    if (attendeeListDTO.getRole() == ROOM_ROLE_HOST) { //主持人
                        this.hostInfo = attendeeListDTO;

                        if (mAnchorInfoCallback != null) {
                            mAnchorInfoCallback.onAnchorInfoUpdate(hostInfo.getUserInfo());
                        }

                    } else if (attendeeListDTO.getRole() == ROOM_ROLE_AUDIENCE) { //观众
                        if (audienceStatefulListInfo != null) {
                            this.audienceStatefulListInfo.add(attendeeListDTO);
                        }
                    }
                }

                if (callback != null) {
                    callback.onGetStatefulData(errorCode, hostInfo, audienceStatefulListInfo, statefulListInfo, statefulListInfo.getAttendeeListSeq(), statefulListInfo.getGbPersonalStateSeq());
                }
            }
        });
    }

    public void exit() {
        // 保底处理
        ZegoSDKManager.getInstance().getRoomService().exitRoom();
        long uid = UserManager.getInstance().getUserId();
        RoomAPI.quitRoom(uid, mRoomID, new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {

            }
        });
        clearRoomData();
        setFront(true);
        setHasConMic(false);
        UserManager.getInstance().reset();
        setRoomQualityInfoCallback(null);
        setCreateRoomFailTimeoutCallback(null);
        setConnectionMicListener(null);
        setStreamStatusListener(null);
        setRoomDisconnectCallback(null);
        setRoomConnectionCallback(null);
        clearAllMessage();
    }

    public void clearRoomData() {

        mRoomInfo = null;
        mRoomID = null;
        hostInfo = null;
        audienceStatefulListInfo.clear();
        setDialogIsShowInfo(null);
        //美颜美型的数据要重置
        EffectsBeautyManager.getInstance().setBeautyInitData();
    }

    public void clearCallback() {
        setRoomQualityInfoCallback(null);
        setCreateRoomFailTimeoutCallback(null);
        setConnectionMicListener(null);
        setStreamStatusListener(null);
        setRoomDisconnectCallback(null);
        setRoomConnectionCallback(null);
    }

    public void clearOtherData() {
        setFront(true);
        setHasConMic(false);
        clearAllMessage();
    }

    public void logoutRoom() {
        // 保底处理
        ZegoSDKManager.getInstance().getRoomService().exitRoom();
        long uid = UserManager.getInstance().getUserId();
        RoomAPI.quitRoom(uid, mRoomID, new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {

            }
        });
        clearRoomData();
        clearOtherData();
        clearCallback();
        UserManager.getInstance().reset();

    }

    public void clearAllMessage() {
        handler.removeMessages(networkError);
    }


    /**
     * 主播推流
     *
     * @param view
     * @param commonCallback
     */
    public void startHostPublishStream(TextureView view, IRTCCommonCallback commonCallback) {
        String streamId = ZegoSDKManager.getInstance().getStreamService().generatePublishStreamID(String.valueOf(UserManager.getInstance().getUserId()));
        RoomAPI.setRoomStream(UserManager.getInstance().getUserId(), getRoomID(), streamId, new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if (errorCode == 0) {
                    startPublishStream(view, commonCallback);
                } else {
                    CommonErrorCodeToast(errorCode);
                }
            }
        });
    }


    /**
     * 推流
     *
     * @param view
     * @param commonCallback
     */
    public void startPublishStream(TextureView view, IRTCCommonCallback commonCallback) {

        String streamId = ZegoSDKManager.getInstance().getStreamService().generatePublishStreamID(String.valueOf(UserManager.getInstance().getUserId()));
        ZegoPublishStream zegoPublishStream = new ZegoPublishStream(String.valueOf(UserManager.getInstance().getUserId()), UserManager.getInstance().getUserInfo().getNickName(), streamId);
        zegoPublishStream.activeStream(view, new IStreamActiveCallback() {
            @Override
            public void onStreamActive(int errorCode, String streamID) {
                if (commonCallback != null) {
                    commonCallback.onRTCCallback(errorCode);
                }
            }
        });

        ZegoSDKManager.getInstance().setPublisherQualityUpdateCallback(new IPublisherQualityUpdateCallback() {
            @Override
            public void onPublisherQualityUpdate(double kbps, double fps, int rtt, double packetLostRate) {
                if (mRoomQualityInfoCallback != null) {
                    mRoomQualityInfoCallback.onRoomQualityInfoUpdate(kbps, fps, rtt, packetLostRate, NetworkSpeedUtil.getRxSpeed(), NetworkSpeedUtil.getTxSpeed());
                }
            }
        });
    }

    /**
     * 拉流显示页面
     *
     * @param attendeeListDTO
     * @param view
     * @param commonCallback
     */
    public void startPlayStream(StatefulListInfo.AttendeeListDTO attendeeListDTO, TextureView view, IRTCCommonCallback commonCallback) {
        ZegoStream zegoStream = ZegoSDKManager.getInstance().getStreamService().getStreamByUser(String.valueOf(attendeeListDTO.getUid()));

        if (zegoStream == null) {
            return;
        }
        String streamId = zegoStream.mStreamID;
        ZegoPlayStream playStream = new ZegoPlayStream(String.valueOf(attendeeListDTO.getUid()), attendeeListDTO.getNickName(), streamId);
        playStream.activeStream(view, new IStreamActiveCallback() {
            @Override
            public void onStreamActive(int errorCode, String streamID) {
                if (commonCallback != null) {
                    commonCallback.onRTCCallback(errorCode);
                }
            }
        });
    }


    /**
     * 直接拉流
     *
     * @param streamId
     * @param view
     * @param commonCallback
     */
    public void startPlayStream(String streamId, TextureView view, IRTCCommonCallback commonCallback) {

        if (streamId == null) {
            return;
        }

        ZegoPlayStream playStream = new ZegoPlayStream("", "", streamId);
        addStream(playStream);
        playStream.activeStream(view, new IStreamActiveCallback() {
            @Override
            public void onStreamActive(int errorCode, String streamID) {
                if (errorCode != 0) {
                    removeStream(playStream);
                }

                if (commonCallback != null) {
                    commonCallback.onRTCCallback(errorCode);
                }
            }
        });
    }

    /**
     * 停止拉流
     *
     * @param streamId
     */
    public void stopStream(String streamId) {


        if (streamId == null) {
            return;
        }

        ZegoPlayStream playStream = (ZegoPlayStream) getStream(streamId);
        if (playStream != null) {
            removeStream(playStream);
        }
        ZegoSDKManager.getInstance().getStreamService().stopPlayStream(streamId);

    }


    /**
     * 停止预览/推流
     */
    public void stopPreviewAndPublishStream() {
        ZegoSDKManager.getInstance().getDeviceService().stopPreview();
        ZegoSDKManager.getInstance().getStreamService().stopPublishStream();
    }

    /**
     * 停止拉流
     *
     * @param attendeeListDTO
     */
    public void stopPlayStream(StatefulListInfo.AttendeeListDTO attendeeListDTO) {
        ZegoStream zegoStream = ZegoSDKManager.getInstance().getStreamService().getStreamByUser(String.valueOf(attendeeListDTO.getUid()));
        if (zegoStream == null) {
            return;
        }
        ZegoSDKManager.getInstance().getStreamService().stopPlayStream(zegoStream.mStreamID);
    }


    /**
     * 获取观众连麦列表
     *
     * @return
     */
    public List<StatefulListInfo.AttendeeListDTO> getAudienceStatefulListInfo() {
        return audienceStatefulListInfo;
    }

    /**
     * 获取主持人的连麦信息
     *
     * @return
     */
    public StatefulListInfo.AttendeeListDTO getHostInfo() {
        return hostInfo;
    }

    private IApplyMicStateNotifyCallback iApplyMicStateNotifyCallback;

    public void setConnectionMicListener(IApplyMicStateNotifyCallback iApplyMicStateNotifyCallback) {
        this.iApplyMicStateNotifyCallback = iApplyMicStateNotifyCallback;
    }

    private IStreamCallback iStreamCallback;

    public void setStreamStatusListener(IStreamCallback iStreamCallback) {
        this.iStreamCallback = iStreamCallback;
    }

    public void CommonErrorCodeToast(int errorCode) {
        if (errorCode == ErrorcodeConstants.ERROR_ROOM_FULL) {
            ToastUtil.showToast(R.string.room_list_join_fail);
        } else if (errorCode == ErrorcodeConstants.ERROR_ALREADY_IN_ROOM) {
            ToastUtil.showToast(R.string.room_list_join_fail_common);
        } else if (errorCode == ErrorcodeConstants.ERROR_USER_OFF_LINE || errorCode == ErrorcodeConstants.ERROR_USER_NOT_IN_ROOM) {
            ToastUtil.showToast(R.string.room_list_join_fail_common);
        } else if (errorCode == ErrorcodeConstants.ErrorFailNetwork) {
            ToastUtil.showToast(R.string.fail_network);
        }else if (errorCode == ErrorcodeConstants.ERROR_SPEAKING_COUNT_TOUCH_LIMIT) {
            ToastUtil.showToast(R.string.liveshow_connect_mic_full);
        } else {
            ToastUtil.showToast(R.string.room_list_join_fail_common);
        }
    }

    public void setRoomQualityInfoCallback(IRoomQualityInfoCallback roomQualityInfoCallback) {
        this.mRoomQualityInfoCallback = roomQualityInfoCallback;
    }


    /**
     * 主播对观众连麦申请的操作（接受/拒绝）
     *
     * @param action
     * @param notifyInfo
     */
    public void applyMicStateNotifyRequest(int action, NotifyApplyMicStateInfo notifyInfo, IExecuteCallback callback) {
        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        RoomAPI.setUserInfo(userInfo.getUid(), userInfo.getNickName(), notifyInfo.getUid(), roomInfo.getRoomId(), action, errorCode -> {
            if (errorCode == 0) {//请求成功
                if (callback != null) {
                    callback.onSuccess();
                }
            } else {
                if (callback != null) {
                    callback.onFailed(errorCode);
                }
            }
        });
    }

    public void setAnchorInfoCallback(IAnchorInfoCallback anchorInfoCallback) {
        this.mAnchorInfoCallback = anchorInfoCallback;
    }

    private void setRoomStateCallback() {
        ZegoSDKManager.getInstance().setRoomStateCallback(new IRoomStateCallback() {
            @Override
            public void onPendingEnter() {
                if (mSDKRoomStateCallback != null) {
                    mSDKRoomStateCallback.loginRoom();
                }
            }

            @Override
            public void onEntered() {

            }

            @Override
            public void onExitRoom() {
                if (mSDKRoomStateCallback != null) {
                    mSDKRoomStateCallback.exitRoom();
                }
            }
        });
    }

    /**
     * 设置状态
     *
     * @param mic    1：关闭  2:打开
     * @param camera 1：关闭  2:打开
     * @param type   1.mic操作  2. camera操作 3. 关闭连麦
     */
    public void setStatus(boolean mic, boolean camera, int type, IExecuteCallback callback) {
        int mIsMicMuted = mic ? MIC_CAMERA_OPEN : MIC_CAMERA_CLOSE;
        int mIsCameraMuted = camera ? MIC_CAMERA_OPEN : MIC_CAMERA_CLOSE;
        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        RoomAPI.setStatus(userInfo.getUid(), roomInfo.getRoomId(), userInfo.getNickName(), userInfo.getUid(), mIsMicMuted, mIsCameraMuted, type, errorCode -> {
            if (errorCode == 0) {
                if (type == OPERATION_TYPE_MIC) {
                    //麦克风
                    ZegoSDKManager.getInstance().getDeviceService().enableMic(mic);
                } else if (type == OPERATION_TYPE_CAMERA) {
                    //摄像头
                    ZegoSDKManager.getInstance().getDeviceService().enableCamera(camera);
                } else if (type == Constants.OPERATION_TYPE_STAGE_STATE) {
                    //下麦
                    ZegoSDKManager.getInstance().getDeviceService().setRoomData(false);
                }
                if (callback != null) {
                    callback.onSuccess();
                }
            } else {
                ToastUtil.showToast(R.string.liveshow_operation_fail);
                if (callback != null) {
                    callback.onFailed(errorCode);
                }
            }
        });
    }

    /**
     * 连麦的请求  action 1.申请 2.取消申请 3. 接受申请 4. 拒绝申请
     *
     * @param action
     */
    public void setUserInfo(int action, IExecuteCallback callback) {
        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        RoomAPI.setUserInfo(userInfo.getUid(), userInfo.getNickName(), roomInfo.getHostId(), roomInfo.getRoomId(), action, errorCode -> {
            if (errorCode == 0) {
                if (callback != null) {
                    callback.onSuccess();
                }
            } else {
                if (callback != null) {
                    callback.onFailed(errorCode);
                }
            }
        });
    }

    public void setCreateRoomFailTimeoutCallback(ICreateRoomFailTimeoutCallback createRoomFailTimeoutCallback) {
        this.mCreateRoomFailTimeoutCallback = createRoomFailTimeoutCallback;
    }

    public void setRoomDisconnectCallback(IRoomDisconnectCallback roomDisconnectCallback) {
        this.mRoomDisconnectCallback = roomDisconnectCallback;
    }

    public boolean isFront() {
        return mFront;
    }

    public void setFront(boolean front) {
        this.mFront = front;
        ZegoSDKManager.getInstance().getDeviceService().setFrontCamera(mFront);
    }

    public void setSDKRoomStateCallback(ISDKRoomStateCallback SDKRoomStateCallback) {
        this.mSDKRoomStateCallback = SDKRoomStateCallback;
    }

    public void setRoomConnectionCallback(IRoomConnectionCallback roomConnectionCallback) {
        this.mRoomConnectionCallback = roomConnectionCallback;
    }

    public boolean isHasConMic() {
        return hasConMic;
    }

    public void setHasConMic(boolean hasConMic) {
        this.hasConMic = hasConMic;
    }

    public void getRoomList(IGetRoomListCallback callback) {
        RoomAPI.getRoomList(1, 1000, 0, null, new IGetRoomListCallback() {
            @Override
            public void onGetRoomList(int code, RoomListInfo roomInfoList) {
                callback.onGetRoomList(code, roomInfoList);
            }
        });
    }

    public void addStream(ZegoStream streamInfo) {
        if (!mStreamList.contains(streamInfo)) {
            mStreamList.add(streamInfo);
        }
    }

    public void removeStream(ZegoStream stream) {
        mStreamList.remove(stream);
    }

    public ZegoStream getStream(String streamID) {
        for (ZegoStream stream : mStreamList) {
            if (stream.mStreamID.equals(streamID)) {
                return stream;
            }
        }

        return null;
    }


    /**
     * 设置状态
     *
     * @param type 1.mic操作  2. camera操作 3. 关闭连麦
     */
    public void setMicStatusStatus(int mIsMicMuted, int mIsCameraMuted, int type, long target_uid, IExecuteCallback callback) {
        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        RoomAPI.setStatus(userInfo.getUid(), roomInfo.getRoomId(), userInfo.getNickName(), target_uid, mIsMicMuted, mIsCameraMuted, type, errorCode -> {
            if (errorCode == 0) {
                if (callback != null) {
                    callback.onSuccess();
                }
            } else {
                ToastUtil.showToast(R.string.liveshow_operation_fail);
                if (callback != null) {
                    callback.onFailed(errorCode);
                }
            }

        });
    }

    /**
     * 观众对主播邀请连麦的操作（接受/拒绝）
     *
     * @param action
     */
    public void inviteMicStateNotifyRequest(int action,String extra, IExecuteCallback callback) {
        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        RoomAPI.setOnstageInvite(userInfo.getUid(), userInfo.getNickName(), roomInfo.getHostId(), roomInfo.getRoomId(), action,extra, errorCode -> {
            if (errorCode == 0) {//请求成功
                if (callback != null) {
                    callback.onSuccess();
                }
            } else {
                if (callback != null) {
                    callback.onFailed(errorCode);
                }
            }
        });

    }

    public boolean isMicIsOffFromHost() {
        return micIsOffFromHost;
    }

    public void setMicIsOffFromHost(boolean micIsOffFromHost) {
        this.micIsOffFromHost = micIsOffFromHost;
    }

    public StatefulListInfo.AttendeeListDTO getDialogIsShowInfo() {
        return dialogIsShowInfo;
    }

    public void setDialogIsShowInfo(StatefulListInfo.AttendeeListDTO dialogIsShowInfo) {
        this.dialogIsShowInfo = dialogIsShowInfo;
    }

    /**
     * 获取房间人数信息
     */
    public void getOnlineInfo(IGetOnlineInfoCallback callback){
        RoomAPI.getOnlineInfo(RoomManager
                .getInstance().getRoomID(), UserManager.getInstance().getUserId(), new IGetOnlineInfoCallback() {
            @Override
            public void onGetOnlineInfo(int errorCode, OnlineInfo onlineInfo) {
                if(callback != null){
                    callback.onGetOnlineInfo(errorCode,onlineInfo);
                }
            }
        });
    }

}
