package im.zego.liveshow.protocol;

import android.text.TextUtils;
import android.util.Log;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.manager.UserManager;
import im.zego.liveshow.model.HeartbeatRespInfo;
import im.zego.liveshow.model.NoInfo;
import im.zego.liveshow.model.OnlineInfo;
import im.zego.liveshow.model.OnstageListInfo;
import im.zego.liveshow.model.RoomInfo;
import im.zego.liveshow.model.RoomListInfo;
import im.zego.liveshow.model.StatefulListInfo;
import im.zego.liveshow.sdk.ZegoSDKManager;
import im.zego.liveshow.sdk.callbacks.IRTCCommonCallback;

public class RoomAPI {

    private static final String TAG = "RoomAPI";

    /**
     * 拉取房间列表
     *
     * @param uid
     * @param count
     * @param from
     * @param callback
     */
    public static void getRoomList(long uid, int count, String from, @NotNull final IGetRoomListCallback callback) {

       getRoomList(uid,count,0,from,callback);
    }

    /**
     * 拉取房间列表
     *
     * @param uid
     * @param count 拉多少
     * @param direction 方向，0：从新到旧，1:从旧到新
     * @param from cursor，拉取起始点
     * @param callback
     */
    public static void getRoomList(long uid, int count,int direction, String from, @NotNull final IGetRoomListCallback callback) {

        String reqJson = "";

        if(TextUtils.isEmpty(from))
        {
            reqJson = APIBase.builder()
                    .addParams("uid", uid)
                    .addParams("count", count)
                    .addParams("direct", direction)
                    .build();
        }else {
            reqJson = APIBase.builder()
                    .addParams("uid", uid)
                    .addParams("count", count)
                    .addParams("direct", direction)
                    .addParams("from", from)
                    .build();
        }



        APIBase.asyncPost("/live_show/get_room_list", reqJson, RoomListInfo.class, new IAsyncPostCallback<RoomListInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, RoomListInfo responseJsonBean) {
                if (callback != null) {
                    callback.onGetRoomList(retcode, responseJsonBean);
                }
            }
        });
    }


    /***
     * 创建房间
     *
     * @param subject
     * @param callback
     */
    public static void createRoom(String subject, String coverImg,String nickName,String avatar,@NotNull final ICreateRoomCallback callback)
    {
        String reqJson;
        if(UserManager.getInstance().getUid() != 0){
            reqJson = APIBase.builder()
                    .addParams("subject", subject)
                    .addParams("cover_img", coverImg)
                    .addParams("nick_name", nickName)
                    .addParams("avatar", avatar)
                    .addParams("uid", UserManager.getInstance().getUid())
                    .build();
        }else {

            reqJson = APIBase.builder()
                    .addParams("subject", subject)
                    .addParams("cover_img", coverImg)
                    .addParams("nick_name", nickName)
                    .addParams("avatar", avatar)
                    .build();
        }

        APIBase.asyncPost("/live_show/create_room",reqJson, RoomInfo.class, (errorCode, message, roomInfo) -> {
            callback.onCreateRoom(errorCode, roomInfo);
        });
    }

    /**
     * 登录房间
     *
     * @param roomID
     * @param callback
     */
    public static void loginRoom( String roomID, String nickName,String avatar,@NotNull final ILoginRoomCallback callback) {
        String reqJson;

        if(UserManager.getInstance().getUid() != 0) {
            reqJson = APIBase.builder()
                    .addParams("room_id", roomID)
                    .addParams("nick_name", nickName)
                    .addParams("avatar", avatar)
                    .addParams("uid", UserManager.getInstance().getUid())
                    .build();

        }else {

            reqJson = APIBase.builder()
                    .addParams("room_id", roomID)
                    .addParams("nick_name", nickName)
                    .addParams("avatar", avatar)
                    .build();
        }

        APIBase.asyncPost("/live_show/login_room", reqJson, RoomInfo.class, new IAsyncPostCallback<RoomInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, RoomInfo roomInfo) {

                callback.onLoginRoom(errorCode, roomInfo);
            }
        });
    }

    /**
     *  切换房间
     * @param uid
     * @param fromRoomID
     * @param toRoomID
     * @param nickName
     * @param avatar
     * @param callback
     */
    public static void switchRoom(long uid,String fromRoomID,String toRoomID,String nickName,String avatar,@NotNull final ILoginRoomCallback callback){
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("from", fromRoomID)
                .addParams("room_id", toRoomID)
                .addParams("nick_name", nickName)
                .addParams("avatar", avatar)
                .build();
        APIBase.asyncPost("/live_show/switch_room", reqJson, RoomInfo.class, new IAsyncPostCallback<RoomInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, RoomInfo roomInfo) {

                callback.onLoginRoom(errorCode, roomInfo);
            }
        });

    }

    /**
     * 设置房间主播流
     * @param uid
     * @param roomID
     * @param streamId
     * @param callback
     */
    public static void setRoomStream(long uid,String roomID,String streamId,@NotNull final ICommonResponseCallback callback){
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", roomID)
                .addParams("stream_id", streamId)
                .build();
        APIBase.asyncPost("/live_show/set_room_stream", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, NoInfo noInfo) {

                callback.onResponse(errorCode);
            }
        });

    }

    /**
     * 拉取连麦人员列表
     *
     * @param uid
     * @param callback
     */
    public static void getStatefulList(String roomID,long uid, @NotNull final IGetStatefulListCallback callback) {


        String reqJson = APIBase.builder()
                .addParams("room_id", roomID)
                .addParams("uid", uid)
                .build();

        APIBase.asyncPost("/live_show/get_stateful_list", reqJson, StatefulListInfo.class, new IAsyncPostCallback<StatefulListInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, StatefulListInfo responseJsonBean) {
                final RoomInfo roomInfo = RoomManager.getInstance().getRoomInfo();
                if(roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)){
                    return;
                }

                if (callback != null) {
                    callback.onGetStatefulList(retcode, responseJsonBean);
                }
            }
        });
    }

    /**
     * 获取房间在线人数信息
     *
     * @param uid
     * @param callback
     */
    public static void getOnlineInfo(String roomID,long uid, @NotNull final IGetOnlineInfoCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("room_id", roomID)
                .addParams("uid", uid)
                .build();

        APIBase.asyncPost("/live_show/get_online_info", reqJson, OnlineInfo.class, new IAsyncPostCallback<OnlineInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, OnlineInfo responseJsonBean) {
                final RoomInfo roomInfo = RoomManager.getInstance().getRoomInfo();
                if(roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)){
                    return;
                }

                if (callback != null) {
                    callback.onGetOnlineInfo(retcode, responseJsonBean);
                }
            }
        });
    }

    /***
     * 离开房间
     *
     * @param uid
     * @param callback
     */
    public static void quitRoom(long uid,String roomID, @Nullable final ICommonResponseCallback callback)
    {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id",roomID)
                .build();

        APIBase.asyncPost("/live_show/quit_room", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 关闭房间
     *
     * @param uid
     * @param callback
     */
    public static void closeRoom(long uid,String roomID, @NotNull final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", roomID)
                .build();

        APIBase.asyncPost("/live_show/close_room", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 设置状态
     *
     * @param uid
     * @param callback
     */
    public static void setStatus(long uid,String roomID,String  nickName,long targetUid,int mic,int camera,int type, @NotNull final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", roomID)
                .addParams("nick_name", nickName)
                .addParams("target_uid",targetUid)
                .addParams("mic",mic)
                .addParams("camera",camera)
                .addParams("type",type)
                .build();

        APIBase.asyncPost("/live_show/set_status", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {

                final RoomInfo roomInfo = RoomManager.getInstance().getRoomInfo();
                if(roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)){
                    return;
                }

                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    /**
     * 心跳
     * @param uid
     * @param roomID
     * @param callback
     */
    public static void heartBeat(long uid, String roomID,@NotNull IHeartBeatCallback callback) {

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", roomID)
                .build();

        APIBase.asyncPost("/live_show/heartbeat", reqJson, HeartbeatRespInfo.class, new IAsyncPostCallback<HeartbeatRespInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, HeartbeatRespInfo responseJsonBean) {
                final RoomInfo roomInfo = RoomManager.getInstance().getRoomInfo();
                if(roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)){
                    return;
                }
                if (callback != null) {
                    callback.onHeartBeat(errorCode, responseJsonBean);
                }
            }
        });
    }


    /**
     * 连麦申请/取消/接受/拒绝
     * @param uid
     * @param nickName
     * @param targetUid
     * @param roomID
     * @param action // 1.申请 2.取消申请 3. 接受申请 4. 拒绝申请
     * @param callback
     */
    public static void setUserInfo(long uid,String nickName, long targetUid,String roomID ,int action,@NotNull final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("nick_name", nickName)
                .addParams("target_uid", targetUid)
                .addParams("room_id", roomID)
                .addParams("action", action)
                .build();

        APIBase.asyncPost("/live_show/onstage_request_action", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                final RoomInfo roomInfo = RoomManager.getInstance().getRoomInfo();
                if(roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)){
                    return;
                }
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


    /**
     * 获取连麦申请列表
     * @param uid
     * @param roomID
     * @param callback
     */
    public static void getOnstageList(long uid,String roomID ,@NotNull final IGetConMicListCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", roomID)
                .build();

        APIBase.asyncPost("/live_show/get_onstage_req_list", reqJson, OnstageListInfo.class, new IAsyncPostCallback<OnstageListInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, OnstageListInfo responseJsonBean) {
                final RoomInfo roomInfo = RoomManager.getInstance().getRoomInfo();
                if(roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)){
                    return;
                }
                if (callback != null) {
                    callback.getOnstageList(retcode,responseJsonBean);
                }
            }
        });
    }


    /**
     * 拉取获取成员列表
     *
     * @param uid
     * @param callback
     */
    public static void getAttendeeList(String roomID,long uid, @NotNull final IGetStatefulListCallback callback) {


        String reqJson = APIBase.builder()
                .addParams("room_id", roomID)
                .addParams("uid", uid)
                .build();

        APIBase.asyncPost("/live_show/get_attendee_list", reqJson, StatefulListInfo.class, new IAsyncPostCallback<StatefulListInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, StatefulListInfo responseJsonBean) {
                final RoomInfo roomInfo = RoomManager.getInstance().getRoomInfo();
                if(roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)){
                    return;
                }

                if (callback != null) {
                    callback.onGetStatefulList(retcode, responseJsonBean);
                }
            }
        });
    }


    public static void setOnstageInvite(long uid,String nickName, long targetUid,String roomID ,int action,String extra,@NotNull final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("nick_name", nickName)
                .addParams("target_uid", targetUid)
                .addParams("room_id", roomID)
                .addParams("action", action)
                .addParams("extra", extra)
                .build();

        APIBase.asyncPost("/live_show/onstage_invite_action", reqJson, NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                final RoomInfo roomInfo = RoomManager.getInstance().getRoomInfo();
                if(roomInfo == null || !roomInfo.getRoomInfo().getRoomId().equals(roomID)){
                    return;
                }
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }


}
