package im.zego.liveshow.roomlist;

import static im.zego.liveshow.ErrorcodeConstants.ErrorFailNetwork;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.ViewGroup;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.scwang.smart.refresh.layout.SmartRefreshLayout;
import com.scwang.smart.refresh.layout.api.RefreshLayout;
import com.scwang.smart.refresh.layout.listener.DefaultRefreshInitializer;

import java.util.ArrayList;

import im.zego.common.util.MiscUtils;
import im.zego.common.util.ScreenAdaptationUtil;
import im.zego.liveshow.R;
import im.zego.liveshow.effects.EffectsBeautyManager;
import im.zego.liveshow.effects.SoundEffectsManager;
import im.zego.liveshow.liveroom.LiveRoomActivity;
import im.zego.liveshow.liveroom.LiveRoomActivity2;
import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.manager.UserManager;
import im.zego.liveshow.manager.callbacks.ISDKRoomStateCallback;
import im.zego.liveshow.model.RoomListInfo;
import im.zego.liveshow.protocol.IGetRoomListCallback;
import im.zego.liveshow.protocol.RoomAPI;
import im.zego.liveshow.roomlist.adapter.EmptyAndLoadMoreAdapterWrapper;
import im.zego.liveshow.roomlist.adapter.RoomListAdapter;
import im.zego.liveshow.roomlist.view.GridLayoutItemDecoration;
import im.zego.liveshow.roomlist.view.LoadMoreView;
import im.zego.liveshow.sdk.ZegoSDKManager;
import im.zego.liveshow.sdk.callbacks.IRTCCommonCallback;
import im.zego.liveshow.utils.CheckDoubleClick;
import im.zego.liveshow.utils.NetworkSpeedUtil;
import im.zego.liveshow.utils.PermissionHelper;
import im.zego.liveshow.utils.SharedPreferencesUtil;
import im.zego.liveshow.utils.ToastUtil;
import im.zego.liveshow.view.BaseInquiryDialog;
import im.zego.liveshow.view.ProcessView;

public class RoomListActivity extends AppCompatActivity {

    private static final int STATE_NORMAL = 0;
    private static final int STATE_REFRESH = 1;
    private static final int STATE_LOADING = 2;
    private static final int STATE_FINISH = 3;
    private static final int STATE_ERROR = 4;

    private int mState;

    private View mWindowBar;
    private View mBack;
    private RecyclerView mRoomList;
    private SwipeRefreshLayout mRefreshRoomList;
    private RoomListAdapter mAdapter;
    private EmptyAndLoadMoreAdapterWrapper mAdapterWrapper;

    private LoadMoreView mLoadMoreView;
    private View mCreateLive;
    private static final int PAGE_NUM = 10;

    private String mRoomID;

    private ProcessView processView;

    //是否已经进房
    private boolean mSDKRoomState = false;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        SharedPreferencesUtil.init(this.getApplicationContext());
//        ScreenAdaptationUtil.init(this.getApplicationContext(), 375);
//        ScreenAdaptationUtil.setCustomDensity(this);
        EffectsBeautyManager.getInstance().initEffectsSDK(getApplication());
        ZegoSDKManager.getInstance().initSDK(getApplication());
        NetworkSpeedUtil.init(RoomListActivity.this);
        setContentView(R.layout.liveshow_activity_room_list);
        initView();
        initWindowBar();
        initRecycleView();
        initListener();
        initData();


    }


    private void initView() {
        mWindowBar = findViewById(R.id.window_bar);
        mBack = findViewById(R.id.back);
        mRoomList = findViewById(R.id.room_list);
        mRefreshRoomList = findViewById(R.id.refresh_room_list);
        mCreateLive = findViewById(R.id.create_live);
        processView = new ProcessView(RoomListActivity.this);
    }

    private void initWindowBar() {
        setViewHeight(mWindowBar, MiscUtils.getStatusBarHeight(this));
        WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.flags = lp.flags | WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS;
        getWindow().setAttributes(lp);
    }

    private void initRecycleView() {
        GridLayoutManager layoutManager = new GridLayoutManager(this, 2);
        layoutManager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {

            @Override
            public int getSpanSize(int position) {
                if ((position == 0 && layoutManager.getItemCount() == 1) || layoutManager.getItemCount() - 1 == position) {
                    return layoutManager.getSpanCount();
                } else {
                    return 1;
                }
            }
        });
        mRoomList.setLayoutManager(layoutManager);

        mAdapter = new RoomListAdapter();

        View emptyView = LayoutInflater.from(this).inflate(R.layout.liveshow_view_empty_list, mRefreshRoomList, false);

        mLoadMoreView = new LoadMoreView(this);
        mAdapterWrapper = new EmptyAndLoadMoreAdapterWrapper(mAdapter);
        mAdapterWrapper.setEmptyView(emptyView);
        mAdapterWrapper.setLoadMoreView(mLoadMoreView);
        mRoomList.addItemDecoration(new GridLayoutItemDecoration(this, MiscUtils.dp2px(this, 16),
                MiscUtils.dp2px(this, 18), true, false));
        mRoomList.setAdapter(mAdapterWrapper);

        mAdapter.setOnClickListener(new RoomListAdapter.OnClickListener() {
            @Override
            public void onClick(View v, int position, RoomListInfo.RoomListDTO data) {

                if(CheckDoubleClick.isFastDoubleClick(2 * ViewConfiguration.getTapTimeout())){
                    return;
                }


                if(mSDKRoomState){
                    return;
                }

                processView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mRoomList.getRootView());

                RoomManager.getInstance().loginRoom(data.getRoomId(), new IRTCCommonCallback() {
                    @Override
                    public void onRTCCallback(int errorCode) {
                        processView.dismiss();

                        if(errorCode == 0) {
                            Intent intent = new Intent(RoomListActivity.this, LiveRoomActivity2.class);
                            intent.putExtra(LiveRoomActivity2.LOGIN_OR_CREATE_ROOM, LiveRoomActivity2.LOGIN_ROOM);
                            intent.putExtra(LiveRoomActivity2.SELECTION_ROOM_ID, data.getRoomId());
                            startActivity(intent);
                        }else {
                            RoomManager.getInstance().CommonErrorCodeToast(errorCode);
                        }
                    }
                });
            }
        });
    }

    private void initListener() {
        mBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onBackPressed();
            }
        });

        mCreateLive.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                PermissionHelper.onCameraAndAudioPermissionGranted(RoomListActivity.this, new PermissionHelper.GrantResult() {
                    @Override
                    public void onGrantResult(boolean allGranted) {
                        if (allGranted) {
                            Intent intent = new Intent(RoomListActivity.this, LiveRoomActivity2.class);
                            intent.putExtra(LiveRoomActivity.LOGIN_OR_CREATE_ROOM, LiveRoomActivity.CREATE_ROOM);
                            startActivity(intent);
                        } else {
                            BaseInquiryDialog baseInquiryDialog = new BaseInquiryDialog(RoomListActivity.this);
                            baseInquiryDialog.setMsgTitle("提示");
                            baseInquiryDialog.setMsgContent("需要获取摄像头麦克风权限");
                            baseInquiryDialog.setLeftButtonContent("取消");
                            baseInquiryDialog.setRightButtonContent("设置");
                            baseInquiryDialog.setSureListener(v -> {
                                baseInquiryDialog.dismiss();
                                Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
                                        .setData(Uri.fromParts("package", getPackageName(), null));
                                startActivityForResult(intent, 666);
                            });
                            baseInquiryDialog.setCancelListener(v -> baseInquiryDialog.dismiss());
                        }
                    }
                });
            }
        });

        mRefreshRoomList.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                refresh();
            }
        });

        mAdapterWrapper.setOnLoadMoreListener(new EmptyAndLoadMoreAdapterWrapper.OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                loadMore();
            }
        });
        RoomManager.getInstance().setSDKRoomStateCallback(new ISDKRoomStateCallback() {
            @Override
            public void loginRoom() {
                mSDKRoomState = true;
            }

            @Override
            public void exitRoom() {
                mSDKRoomState = false;
                processView.dismiss();
            }
        });
    }

    private void initData() {
        UserManager.getInstance().reset();

        SmartRefreshLayout.setDefaultRefreshInitializer(new DefaultRefreshInitializer() {
            @Override
            public void initialize(@NonNull Context context, @NonNull RefreshLayout layout) {
                layout.getLayout().setTag("close egg");
            }
        });
    }

    private void refresh() {

        mState = STATE_REFRESH;
        mRoomID = null;
        RoomAPI.getRoomList(1, PAGE_NUM, null, new IGetRoomListCallback() {
            @SuppressLint("NotifyDataSetChanged")
            @Override
            public void onGetRoomList(int retcode, RoomListInfo roomInfoList) {
                mRefreshRoomList.setRefreshing(false);
                if (retcode == 0) {
                    if (roomInfoList != null && roomInfoList.getRoomList() != null) {
                        mAdapter.refreshData(roomInfoList.getRoomList());

                        final int size = roomInfoList.getRoomList().size();
                        if (size > 0) {
                            mRoomID = roomInfoList.getRoomList().get(size - 1).getRoomId();
                        }
                        if (size < PAGE_NUM) {
                            mState = STATE_FINISH;
                            mLoadMoreView.showFinish();
                        } else {
                            mState = STATE_NORMAL;
                            mLoadMoreView.showLoading();
                        }

                        mAdapterWrapper.notifyDataSetChanged();


                    } else {
                        mState = STATE_NORMAL;
                        mAdapter.refreshData(new ArrayList<RoomListInfo.RoomListDTO>());
                        mAdapterWrapper.notifyDataSetChanged();
                    }
                } else {
                    if (retcode == ErrorFailNetwork) {
                        ToastUtil.showToast(R.string.fail_network);
                    }
                    mState = STATE_ERROR;
                }
            }
        });
    }

    private void loadMore() {
        if (mState == STATE_REFRESH) {
            return;
        }

        if (mState == STATE_NORMAL) {
            mState = STATE_LOADING;
            mLoadMoreView.showLoading();

            RoomAPI.getRoomList(1, PAGE_NUM, mRoomID, new IGetRoomListCallback() {
                @Override
                public void onGetRoomList(int retcode, RoomListInfo roomInfoList) {
                    if (retcode == 0) {
                        if (roomInfoList != null && roomInfoList.getRoomList() != null) {

                            mAdapter.addData(roomInfoList.getRoomList());

                            final int size = roomInfoList.getRoomList().size();
                            if (size > 0) {
                                mRoomID = roomInfoList.getRoomList().get(size - 1).getRoomId();
                            }

                            if (size < PAGE_NUM) {
                                mState = STATE_FINISH;
                                mLoadMoreView.showFinish();
                            } else {
                                mState = STATE_NORMAL;
                                mLoadMoreView.showLoading();
                            }

                            mAdapterWrapper.notifyDataSetChanged();


                        } else {
                            mState = STATE_FINISH;
                            mLoadMoreView.showFinish();
                        }
                    } else {
                        if (retcode == ErrorFailNetwork) {
                            ToastUtil.showToast(R.string.fail_network);
                        }

                        mState = STATE_ERROR;
                        mLoadMoreView.showFinish();
                    }
                }

            });
        }
    }


    @Override
    protected void onResume() {
        super.onResume();
        if(mSDKRoomState){
            processView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mRoomList.getRootView());
        }
        mRefreshRoomList.setRefreshing(true);
        refresh();
    }

    /**
     * 重设 view 的宽高
     */
    private void setViewHeight(View view, int nHeight) {
        ViewGroup.LayoutParams lp = view.getLayoutParams();
        if (lp.height != nHeight) {
            lp.height = nHeight;
            view.setLayoutParams(lp);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
//        ScreenAdaptationUtil.unInit(this.getApplicationContext());
        EffectsBeautyManager.getInstance().clearEffectData();
        SoundEffectsManager.getInstance().destroyAudioEffect();
        ZegoSDKManager.getInstance().uninitSDK();
        NetworkSpeedUtil.uninit();

    }

}
