package im.zego.liveshow.roomlist.adapter;

import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.List;

import im.zego.liveshow.R;
import im.zego.liveshow.manager.UserManager;
import im.zego.liveshow.model.RoomListInfo;


public class RoomListAdapter extends RecyclerView.Adapter<RoomListAdapter.RoomListViewHolder> {

    private List<RoomListInfo.RoomListDTO> mData = new ArrayList<>();

    public RoomListAdapter() {
    }

    @NonNull
    @Override
    public RoomListViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.liveshow_item_room_list, parent, false);
        return new RoomListViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final @NonNull RoomListViewHolder holder, final int position) {

        final int pos = holder.getAdapterPosition();
        RoomListInfo.RoomListDTO info = mData.get(pos);

        try {
            holder.background.setBackgroundResource(UserManager.coverList[Integer.parseInt(info.getCoverImg())-1]);
        }catch (Exception e){
            holder.background.setBackgroundResource(UserManager.coverList[Integer.parseInt(UserManager.getInstance().getAvatar(info.getCoverImg()))-1]);
        }

        holder.peopleNum.setText(String.valueOf(info.getOnline()));
//        holder.peopleNum.setText(String.valueOf(info.getRoomId()));
        holder.subject.setText(String.valueOf(info.getSubject()));

        holder.background.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(mOnClickListener != null)
                {
                    mOnClickListener.onClick(v,pos,info);
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        if(mData == null)
        {
            return 0;
        }
        return mData.size();
    }

    class RoomListViewHolder extends RecyclerView.ViewHolder {

        View background;
        TextView peopleNum;
        TextView subject;
        private RoomListViewHolder(View itemView) {
            super(itemView);
            background = itemView.findViewById(R.id.background);
            peopleNum = itemView.findViewById(R.id.people_num);
            subject = itemView.findViewById(R.id.subject);

        }
    }

    public void refreshData(List<RoomListInfo.RoomListDTO> newData)
    {
        if(newData == null)
        {
            mData = new ArrayList<>();
        }
        mData = newData;

    }


    public void addData(List<RoomListInfo.RoomListDTO> newData) {
        if(newData == null)
        {
            return;
        }
        int size = newData.size();
        first:for(int i= 0; i < size;i++)
        {
            for(int j= 0 ; j < mData.size();j++)
            {
                if(mData.get(j).getRoomId().equals(newData.get(i).getRoomId()))
                {
                    continue first;
                }
            }
            mData.add(newData.get(i));
        }
//        mData.addAll(newData);

    }

    private OnClickListener mOnClickListener;
    public interface OnClickListener {
        void onClick(View v, int position,RoomListInfo.RoomListDTO data);
    }

    public void setOnClickListener(OnClickListener onClickListener)
    {
        mOnClickListener = onClickListener;
    }

    private String getSumString(int sum)
    {
        if(sum < 1000)
        {
            return String.valueOf(sum);
        }else {
            return ((sum/100)/10.0) + "k";
        }
    }

}
