package im.zego.liveshow.sdk;

import android.app.Application;
import android.text.TextUtils;
import android.util.Log;
import android.view.TextureView;

import androidx.annotation.NonNull;

import org.json.JSONObject;
import org.w3c.dom.Text;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import im.zego.common.util.T;
import im.zego.effects.sdkmanager.EffectsSDKManager;
import im.zego.liveshow.sdk.callbacks.IBroadcastMessageListener;
import im.zego.liveshow.sdk.callbacks.ICustomCommandListener;
import im.zego.liveshow.sdk.callbacks.IPublisherQualityUpdateCallback;
import im.zego.liveshow.sdk.callbacks.IRTCCommonCallback;
import im.zego.liveshow.sdk.callbacks.IRemoteDeviceStateListener;
import im.zego.liveshow.sdk.callbacks.IRoomExtraInfoUpdateListener;
import im.zego.liveshow.sdk.callbacks.IRoomLoginCallback;
import im.zego.liveshow.sdk.callbacks.IRoomUserListener;
import im.zego.liveshow.sdk.callbacks.IStreamCountListener;
import im.zego.liveshow.sdk.callbacks.IZegoRoomConnectionStateListener;
import im.zego.liveshow.sdk.model.ZegoPlayStream;
import im.zego.liveshow.sdk.model.ZegoRoomUser;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.callback.IZegoCustomVideoProcessHandler;
import im.zego.zegoexpress.callback.IZegoEventHandler;
import im.zego.zegoexpress.constants.ZegoAECMode;
import im.zego.zegoexpress.constants.ZegoANSMode;
import im.zego.zegoexpress.constants.ZegoOrientation;
import im.zego.zegoexpress.constants.ZegoPlayerState;
import im.zego.zegoexpress.constants.ZegoPublishChannel;
import im.zego.zegoexpress.constants.ZegoPublisherState;
import im.zego.zegoexpress.constants.ZegoRemoteDeviceState;
import im.zego.zegoexpress.constants.ZegoRoomState;
import im.zego.zegoexpress.constants.ZegoScenario;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.constants.ZegoVideoBufferType;
import im.zego.zegoexpress.constants.ZegoViewMode;
import im.zego.zegoexpress.entity.ZegoBroadcastMessageInfo;
import im.zego.zegoexpress.entity.ZegoCanvas;
import im.zego.zegoexpress.entity.ZegoCustomVideoProcessConfig;
import im.zego.zegoexpress.entity.ZegoEngineConfig;
import im.zego.zegoexpress.entity.ZegoEngineProfile;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoRoomConfig;
import im.zego.zegoexpress.entity.ZegoRoomExtraInfo;
import im.zego.zegoexpress.entity.ZegoStream;
import im.zego.zegoexpress.entity.ZegoUser;
import im.zego.zegoexpress.entity.ZegoVideoConfig;

import static im.zego.zegoexpress.constants.ZegoRoomState.CONNECTED;
import static im.zego.zegoexpress.constants.ZegoUpdateType.ADD;
import static im.zego.zegoexpress.constants.ZegoUpdateType.DELETE;


public class ZegoExpressWrapper implements IZegoVideoSDKProxy{
    private String TAG = "ZegoExpressWrapper";

    public static final int MAX_USER_COUNT = 10000000;

    private ZegoExpressEngine expressEngine;

    private String mLoginRoomID = "";
    private Map<String,IRTCCommonCallback> publishResultMap = new HashMap<>();
    private Map<String,IRTCCommonCallback> playResultMap = new HashMap<>();
    private IRoomLoginCallback mLoginCallback = null;
    private IRoomUserListener mUserListener = null;
    private IRemoteDeviceStateListener mRemoteDeviceStateListener = null;
    private ICustomCommandListener mCustomCommandListener = null;
    private IBroadcastMessageListener mBroadcastMessageListener = null;
    private IRoomExtraInfoUpdateListener mRoomExtraInfoUpdateListener = null;
    private IZegoRoomConnectionStateListener mZegoRoomConnectionStateListener = null;
    private IStreamCountListener mStreamCountListener = null;
    private IPublisherQualityUpdateCallback mPublisherQualityUpdateCallback = null;

    @Override
    public void initSDK(Application application, long appID, String appSign) {
        Log.d(TAG, "init ZegoExpressEngine ,version:${ZegoExpressEngine.getVersion()}");
        ZegoEngineConfig config =new ZegoEngineConfig();

        config.advancedConfig.put("allow_verbose_print_high_frequency_content","true");
        config.advancedConfig.put("enable_callback_verbose", "true");
        config.advancedConfig.put("use_data_record","true");
        ZegoExpressEngine.setEngineConfig(config);

//        ZegoExpressEngine engine = ZegoExpressEngine.createEngine(
//                appID, appSign, false,
//                ZegoScenario.COMMUNICATION, application, mEventHandler
//        );

        ZegoEngineProfile profile = new ZegoEngineProfile();
        profile.appID = appID;
        profile.appSign = appSign;
        profile.scenario = ZegoScenario.GENERAL;
        profile.application = (Application) application.getApplicationContext();

        ZegoExpressEngine engine = ZegoExpressEngine.createEngine(profile, mEventHandler);

//        if (engine == null) {
//            initCallback.initResult(false)
//            return
//        }
//
        expressEngine = engine;

        enableAudio3A(true);
//        expressEngine.startPerformanceMonitor(3000)

        setEffects();

        if(expressEngine == null){
            T.show("引擎创建失败，请检查appID ，appSign是否存在问题");
        }
    }

    @Override
    public void unInitSDK() {
        ZegoExpressEngine.destroyEngine(null);
        expressEngine = null;
    }

    @Override
    public boolean isInit() {
        return expressEngine != null;
    }

    @Override
    public void startPublishing(String streamID, String userName,@NonNull IRTCCommonCallback callback) {
        if(expressEngine == null){
            return;
        }

        expressEngine.startPublishingStream(streamID);
        publishResultMap.put(streamID,callback);
    }

    @Override
    public void muteVideoPublish(boolean mute) {

        if(expressEngine == null){
            return;
        }

        expressEngine.mutePublishStreamVideo(mute);
    }

    @Override
    public void muteAudioPublish(boolean mute) {

        if(expressEngine == null){
            return;
        }

        expressEngine.mutePublishStreamAudio(mute);
    }

    @Override
    public void mutePlayStreamAudio(String streamID, boolean mute) {

        if(expressEngine == null){
            return;
        }

        expressEngine.mutePlayStreamAudio(streamID,mute);
    }

    @Override
    public void stopPublishing() {
        if(expressEngine == null){
            return;
        }

        expressEngine.stopPublishingStream();
    }

    @Override
    public void activateVideoPlayStream(String streamID, boolean enable) {

        if(expressEngine == null){
            return;
        }

        expressEngine.mutePlayStreamVideo(streamID, !enable);
    }

    @Override
    public void activateAudioPlayStream(String streamID, boolean enable) {

        if(expressEngine == null){
            return;
        }

        expressEngine.mutePlayStreamAudio(streamID, !enable);
    }

    @Override
    public void startPlayingStream(String streamID, TextureView view,@NonNull IRTCCommonCallback callback) {
        if(expressEngine == null){
            return;
        }

        ZegoCanvas zegoCanvas = new ZegoCanvas(view);
        zegoCanvas.viewMode = ZegoViewMode.ASPECT_FILL;

        expressEngine.startPlayingStream(streamID, zegoCanvas);
        playResultMap.put(streamID,callback);
    }

    @Override
    public void updatePlayView(String streamID, TextureView view) {

        if(expressEngine == null){
            return;
        }

        expressEngine.startPlayingStream(streamID,new ZegoCanvas(view));
    }

    @Override
    public void stopPlayingStream(String streamID) {
        if(expressEngine == null){
            return;
        }

        expressEngine.stopPlayingStream(streamID);
    }

    @Override
    public void enableCamera(boolean enable) {
        if(expressEngine == null){
            return;
        }
        expressEngine.enableCamera(enable);
    }

    @Override
    public void setFrontCam(boolean front) {
        if(expressEngine == null){
            return;
        }
        expressEngine.useFrontCamera(front);
    }

    @Override
    public void enableMic(boolean enable) {
        if(expressEngine == null){
            return;
        }
        expressEngine.muteMicrophone(!enable);
    }

    @Override
    public void enableSpeaker(boolean enable) {
        if(expressEngine == null){
            return;
        }
        expressEngine.muteSpeaker(!enable);
    }

    @Override
    public void setAppOrientation(int rotation) {
        if(expressEngine == null){
            return;
        }
        expressEngine.setAppOrientation(ZegoOrientation.getZegoOrientation(rotation));
    }

    @Override
    public void startPreview(TextureView view) {
        if(expressEngine == null){
            return;
        }
        ZegoCanvas zegoCanvas = new ZegoCanvas(view);
        zegoCanvas.viewMode = ZegoViewMode.ASPECT_FILL;
        expressEngine.startPreview(zegoCanvas);
    }

    @Override
    public void stopPreview() {
        if(expressEngine == null){
            return;
        }
        expressEngine.stopPreview();
    }

    @Override
    public void setVideoConfig(int width, int height, int bitrate, int fps) {
        if(expressEngine == null){
            return;
        }

        ZegoVideoConfig config = new ZegoVideoConfig();
        config.setCaptureResolution(width, height);
        config.setEncodeResolution(width, height);
        config.setVideoBitrate(bitrate / 1000);
        config.setVideoFPS(fps);
        expressEngine.setVideoConfig(config);
    }

    @Override
    public void loginRoom(String userID, String userName, String roomID, IRoomLoginCallback callback) {
        if(expressEngine == null){
            return;
        }
        ZegoUser user = new ZegoUser(userID, userName);
        ZegoRoomConfig roomConfig = new ZegoRoomConfig();
        roomConfig.maxMemberCount = MAX_USER_COUNT;
        roomConfig.isUserStatusNotify = true;


        mIsLoginRoom = true;
        mLoginRoomID = roomID;
        mLoginCallback = callback;
        expressEngine.loginRoom(roomID, user, roomConfig);
    }

    @Override
    public void logoutRoom(String roomID) {


        mLoginRoomID = "";

        mIsLoginRoom = false;

        if(expressEngine == null){
            return;
        }
        expressEngine.logoutRoom(roomID);
    }

    @Override
    public void switchRoom(String userID, String userName,String fromRoomID, String toRoomID, IRoomLoginCallback callback){

        if(expressEngine == null){
            return;
        }

        ZegoUser user = new ZegoUser(userID, userName);

        ZegoRoomConfig roomConfig = new ZegoRoomConfig();
        roomConfig.maxMemberCount = MAX_USER_COUNT;
        roomConfig.isUserStatusNotify = true;
        mIsLoginRoom = true;
        mLoginRoomID = toRoomID;
        mLoginCallback = callback;


        if(!TextUtils.isEmpty(fromRoomID)) {
            expressEngine.switchRoom(fromRoomID, toRoomID, roomConfig);
        }else {
            expressEngine.logoutRoom();
            expressEngine.loginRoom(toRoomID, user, roomConfig);
        }
    }

    private void enableAudio3A(boolean enable) {
        if(expressEngine == null){
            return;
        }

        expressEngine.enableAEC(enable);
        expressEngine.enableAGC(enable);
        expressEngine.enableANS(enable);
        expressEngine.setAECMode(ZegoAECMode.MEDIUM);
        expressEngine.setANSMode(ZegoANSMode.MEDIUM);
    }

    @Override
    public void requireHardwareDecoder(boolean require) {

        if(expressEngine == null){
            return;
        }
        expressEngine.enableHardwareDecoder(require);
    }

    @Override
    public void requireHardwareEncoder(boolean require) {

        if(expressEngine == null){
            return;
        }

        expressEngine.enableHardwareEncoder(require);
    }

    @Override
    public String getVersion() {
        return ZegoExpressEngine.getVersion();
    }

    @Override
    public void uploadLog() {
        if(expressEngine == null){
            return;
        }

        expressEngine.uploadLog();
    }

    @Override
    public void setRoomUserListener(IRoomUserListener userListener) {
        mUserListener = userListener;
    }

    @Override
    public void setRemoteDeviceEventCallback(IRemoteDeviceStateListener remoteDeviceStateListener) {
        mRemoteDeviceStateListener = remoteDeviceStateListener;
    }

    @Override
    public void setCustomCommandListener(ICustomCommandListener listener) {
        mCustomCommandListener = listener;
    }

    @Override
    public void setBroadcastMessageListener(IBroadcastMessageListener listener) {
        mBroadcastMessageListener = listener;
    }

    @Override
    public void setRoomExtraInfoUpdateListener(IRoomExtraInfoUpdateListener listener) {
        mRoomExtraInfoUpdateListener = listener;
    }

    @Override
    public void setStreamCountListener(IStreamCountListener streamCountListener) {
        this.mStreamCountListener = streamCountListener;
    }


    @Override
    public void setZegoRoomConnectionStateListener(IZegoRoomConnectionStateListener zegoRoomStateListener) {
        this.mZegoRoomConnectionStateListener = zegoRoomStateListener;
    }

    @Override
    public void setPublisherQualityUpdateCallback(IPublisherQualityUpdateCallback publisherQualityUpdateCallback) {
        this.mPublisherQualityUpdateCallback = publisherQualityUpdateCallback;
    }


    private String mRoomID = null;
    private boolean mIsLoginRoom = false;
    private final IZegoEventHandler mEventHandler = new IZegoEventHandler() {
        @Override
        public void onDebugError(int errorCode, String funcName, String info) {
            super.onDebugError(errorCode, funcName, info);
            Log.e("onDebugError","errorCode:" + errorCode + ":funcName:" + funcName + ":info:" + info);
        }

        @Override
        public void onRoomUserUpdate(
                String roomID,ZegoUpdateType updateType, ArrayList<ZegoUser> userList
        ) {

            if (updateType == ZegoUpdateType.ADD) {
                for (ZegoUser user : userList) {
                    ZegoRoomUser roomUser = new ZegoRoomUser(user.userID, user.userName);
                    if(mUserListener != null)
                    {
                        mUserListener.onUserAdd(roomUser);
                    }

                }
            } else {
                for (ZegoUser user : userList) {
                    ZegoRoomUser roomUser = new ZegoRoomUser(user.userID, user.userName);
                    if(mUserListener != null)
                    {
                        mUserListener.onUserRemove(roomUser);
                    }

                }
            }
        }

        @Override
        public void onPublisherStateUpdate(String streamID, ZegoPublisherState state, int errorCode, JSONObject extendedData) {
            if (state == ZegoPublisherState.PUBLISHING) {

                IRTCCommonCallback streamPublishCallback = publishResultMap.remove(streamID);
                if (streamPublishCallback != null) {
                    streamPublishCallback.onRTCCallback(errorCode);
                }
            }

        }

        @Override
        public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData) {

            if (expressEngine == null) {
                // ERROR_RTC_SDK_NOT_INITIALIZED
                return;
            }

            for (ZegoStream stream : streamList) {
                if (updateType == ADD) {
                    ZegoPlayStream playStream =  new ZegoPlayStream(stream.user.userID, stream.user.userName, stream.streamID);
                    if(mStreamCountListener != null){
                        mStreamCountListener.onStreamAdd(playStream);
                    }

                } else if (updateType == DELETE) {
                    ZegoPlayStream playStream =  new ZegoPlayStream(stream.user.userID, stream.user.userName, stream.streamID);
                    if(mStreamCountListener != null){
                        mStreamCountListener.onStreamRemove(playStream);
                    }

                }
            }
        }

        @Override
        public void onPlayerStateUpdate(String streamID, ZegoPlayerState state,
                                        int errorCode, JSONObject extendedData){

            if (state == ZegoPlayerState.PLAYING) {
                IRTCCommonCallback streamPlayCallback = playResultMap.remove(streamID);
                if(streamPlayCallback != null) {
                    streamPlayCallback.onRTCCallback(errorCode);
                }
            }
        }

        @Override
        public void onPublisherQualityUpdate(String streamID, ZegoPublishStreamQuality quality) {
            super.onPublisherQualityUpdate(streamID, quality);

            if(mPublisherQualityUpdateCallback != null){
                mPublisherQualityUpdateCallback.onPublisherQualityUpdate(quality.videoKBPS,quality.videoEncodeFPS,quality.rtt,quality.packetLostRate);
            }

        }

        @Override
        public void onPlayerQualityUpdate(String streamID, ZegoPlayStreamQuality quality) {
            super.onPlayerQualityUpdate(streamID, quality);

        }

        @Override
        public void onRoomExtraInfoUpdate(String roomID, ArrayList<ZegoRoomExtraInfo> roomExtraInfoList) {

            if (roomExtraInfoList == null) {
                return;
            }
            for (ZegoRoomExtraInfo extraInfo : roomExtraInfoList){
                mRoomExtraInfoUpdateListener.onRoomExtraInfoUpdate(roomID,extraInfo);
            }

        }

        @Override
        public void onIMRecvCustomCommand(String roomID, ZegoUser fromUser, String command) {

            if (command == null || command.length() == 0) {
                return;
            }

            if(mCustomCommandListener != null){
                mCustomCommandListener.onRecvCustomCommand(
                        fromUser.userID, fromUser.userName,
                        command, roomID);
            }

        }

        @Override
        public void onIMRecvBroadcastMessage(String roomID, ArrayList<ZegoBroadcastMessageInfo> messageList) {

            if(messageList == null)
            {
                return;
            }

            for(ZegoBroadcastMessageInfo info:messageList)
            {
                if(mBroadcastMessageListener != null){
                    mBroadcastMessageListener.onRecvBroadcastMessage(
                            info.fromUser.userID, info.fromUser.userName,
                            info.message, roomID);
                }
            }
        }



        @Override
        public void onRoomStateUpdate(String roomID, ZegoRoomState state, int errorCode, JSONObject extendedData) {
            super.onRoomStateUpdate(roomID, state, errorCode, extendedData);

            if(state == ZegoRoomState.DISCONNECTED)
            {
                if (mIsLoginRoom && !TextUtils.isEmpty(mLoginRoomID) && mLoginRoomID.equals(roomID)) {
                    if(mLoginCallback != null)
                    {
                        mLoginCallback.onLoginCallback(roomID,errorCode);
                        mLoginCallback = null;
                    }
                    mIsLoginRoom = false;
                } else {

                    if(mZegoRoomConnectionStateListener != null) {
                        mZegoRoomConnectionStateListener.onDisconnect(errorCode, roomID);
                    }

                }
            }else if (state == CONNECTED) {
                if (mIsLoginRoom && !TextUtils.isEmpty(mLoginRoomID) && mLoginRoomID.equals(roomID)) {
                    if(mLoginCallback != null)
                    {
                        mLoginCallback.onLoginCallback(roomID,errorCode);
                        mLoginCallback = null;
                    }
                    mIsLoginRoom = false;
                } else {
                    if(mZegoRoomConnectionStateListener != null) {
                        mZegoRoomConnectionStateListener.onConnected(errorCode, roomID);
                    }
                }

            }else if(state == ZegoRoomState.CONNECTING) {
                if(mZegoRoomConnectionStateListener != null) {
                    mZegoRoomConnectionStateListener.connecting(errorCode, roomID);
                }
            }
        }

        @Override
        public void onRemoteCameraStateUpdate(String streamID , ZegoRemoteDeviceState state) {
            int status;
            if (state == ZegoRemoteDeviceState.OPEN) {
                status = DEVICE_STATUS_OPEN;
            }
            else {
                status = DEVICE_STATUS_CLOSE;
            }

            if(mRemoteDeviceStateListener != null){
                mRemoteDeviceStateListener.onRemoteCameraStatusUpdate(streamID,status);
            }

        }

        @Override
        public void onRemoteMicStateUpdate(String streamID,ZegoRemoteDeviceState state) {
            int status ;
            if (state == ZegoRemoteDeviceState.OPEN){
                status = DEVICE_STATUS_OPEN;
            }
            else {
                status = DEVICE_STATUS_CLOSE;
            }
            if(mRemoteDeviceStateListener != null){
                mRemoteDeviceStateListener.onRemoteMicStatusUpdate(streamID,status);
            }
        }

        @Override
        public void onPlayerRecvAudioFirstFrame(String streamID) {
            super.onPlayerRecvAudioFirstFrame(streamID);
        }

        @Override
        public void onPlayerRenderVideoFirstFrame(String streamID) {
            super.onPlayerRenderVideoFirstFrame(streamID);
        }
    };

    private void setEffects(){
        if(expressEngine == null){
            return;
        }

        ZegoCustomVideoProcessConfig configVideo = new ZegoCustomVideoProcessConfig();
        configVideo.bufferType = ZegoVideoBufferType.GL_TEXTURE_2D;
        expressEngine.enableCustomVideoProcessing(true, configVideo, ZegoPublishChannel.MAIN);
        expressEngine.setCustomVideoProcessHandler(new IZegoCustomVideoProcessHandler() {
            @Override
            public void onStart(ZegoPublishChannel channel) {
                EffectsSDKManager.getInstance().onStart();
            }

            @Override
            public void onStop(ZegoPublishChannel channel) {
                EffectsSDKManager.getInstance().onStop();
            }

            @Override
            public void onCapturedUnprocessedTextureData(int textureID, int width, int height, long referenceTimeMillisecond, ZegoPublishChannel channel) {
                int processedTextureID = EffectsSDKManager.getInstance().gainProcessedTextureID(textureID,width,height);
                // Send processed texture to ZegoExpressEngine
                if(expressEngine == null){
                    return;
                }

                expressEngine.sendCustomVideoProcessedTextureData(processedTextureID, width, height, referenceTimeMillisecond);
            }
        });
    }

}
