package im.zego.liveshow.sdk;

import android.util.Log;

import com.google.gson.Gson;

import org.json.JSONException;
import org.json.JSONObject;

import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.model.rtc.NotifyApplyMicStateInfo;
import im.zego.liveshow.model.rtc.NotifyRefuseApplyMicInfo;
import im.zego.liveshow.model.rtc.NotifyUserListUpdateInfo;
import im.zego.liveshow.model.rtc.NotifyUserStateUpdateInfo;
import im.zego.liveshow.sdk.callbacks.IBroadcastMessageListener;
import im.zego.liveshow.sdk.callbacks.ICustomCommandListener;
import im.zego.liveshow.sdk.callbacks.IRTCCommonCallback;
import im.zego.liveshow.sdk.callbacks.IRTCEventCallback;
import im.zego.liveshow.sdk.callbacks.IRoomLoginCallback;
import im.zego.liveshow.sdk.callbacks.IRoomMessageCallback;
import im.zego.liveshow.sdk.callbacks.IRoomStateCallback;
import im.zego.liveshow.sdk.callbacks.IZegoRoomConnectionStateListener;

public class ZegoRoomService {

    public static final int OUTSIDE = 0;
    public static final int PENDING_ENTER = 1;
    public static final int ENTERED = 2;

    private static final String TAG = "ZegoRoomService";

    private int mState = OUTSIDE;
    private String mRoomID = "";

    private IZegoVideoSDKProxy mSDKProxy = null;
    private IZegoRoomConnectionStateListener mZegoRoomConnectionStateListener = null;
    private IRoomMessageCallback mRoomMessageCallback = null;
    private IRoomStateCallback mRoomStateCallback = null;
    public ZegoRoomService(IZegoVideoSDKProxy sdkProxy) {
        mSDKProxy = sdkProxy;
    }

    private void registerCallback() {

        mSDKProxy.setBroadcastMessageListener(new IBroadcastMessageListener() {
            @Override
            public void onRecvBroadcastMessage(String userID, String userName, String content, String roomID) {

                if(mRoomMessageCallback != null){
                    mRoomMessageCallback.onReceiveBroadcastMessage(userID,userName,content,roomID);
                }


            }
        });

        mSDKProxy.setCustomCommandListener(new ICustomCommandListener() {
            @Override
            public void onRecvCustomCommand(String userID, String userName, String content, String roomID) {

                if(mRoomMessageCallback != null){
                    mRoomMessageCallback.onReceiveCustomCommand(userID,userName,content,roomID);
                }


            }
        });

        mSDKProxy.setZegoRoomConnectionStateListener(new IZegoRoomConnectionStateListener() {
            @Override
            public void onConnected(int errorCode, String roomID) {
                Log.i(TAG,"onConnected:errorCode:" + errorCode + ":roomID:" + roomID);
                if (mZegoRoomConnectionStateListener != null) {
                    mZegoRoomConnectionStateListener.onConnected(errorCode, roomID);
                }
            }

            @Override
            public void onDisconnect(int errorCode, String roomID) {
                Log.i(TAG,"onDisconnect:errorCode:" + errorCode+ ":roomID:" + roomID);
                if (mZegoRoomConnectionStateListener != null) {
                    mZegoRoomConnectionStateListener.onDisconnect(errorCode, roomID);
                }
            }

            @Override
            public void connecting(int errorCode, String roomID) {
                Log.i(TAG,"connecting:errorCode:"+ errorCode+ ":roomID:" + roomID);
                if (mZegoRoomConnectionStateListener != null) {
                    mZegoRoomConnectionStateListener.connecting(errorCode, roomID);
                }
            }
        });


    }

    void unRegisterCallback() {
        mSDKProxy.setBroadcastMessageListener(null);
        mSDKProxy.setCustomCommandListener(null);
        mSDKProxy.setZegoRoomConnectionStateListener(null);

    }

    public void clearAll() {
        unRegisterCallback();
    }


    public void loginRoom(
            String userID,
            String userName,
            String roomID,
            final IRTCCommonCallback callback
    ) {
        Log.i(TAG,"loginRoom:userID:" + userID + ":userName:" + userName + ":roomID:" + roomID);
        if (mState != OUTSIDE) {
            return;
        }
        registerCallback();
        ZegoSDKManager.getInstance().getDeviceService().registerCallback();
        ZegoSDKManager.getInstance().getStreamService().registerCallback();

        mState = PENDING_ENTER;
        if(mRoomStateCallback != null)
        {
            mRoomStateCallback.onPendingEnter();
        }

        mSDKProxy.loginRoom(userID, userName, roomID, new IRoomLoginCallback() {

            @Override
            public void onLoginCallback(String roomID,int errorCode) {

                if (errorCode == 0) {
                    mRoomID = roomID;
                    mState = ENTERED;
                    if(mRoomStateCallback != null)
                    {
                        mRoomStateCallback.onEntered();
                    }
                    onRoomEntered();
                } else {
                    mState = OUTSIDE;
                    exitRoom();
                }

                if(callback != null) {
                    callback.onRTCCallback(errorCode);
                }
            }

        });
    }

    private void onRoomEntered() {
        Log.i(TAG,"onRoomEntered");
        ZegoDeviceService deviceService = ZegoSDKManager.getInstance().getDeviceService();
        deviceService.setFrontCamera(RoomManager.getInstance().isFront());
        mSDKProxy.requireHardwareDecoder(true);
        mSDKProxy.requireHardwareEncoder(true);
        deviceService.enableSpeaker(true);
        deviceService.enableMic(true);
        deviceService.enableCamera(true);
    }

    public void switchRoom(String userID, String userName,String fromRoomID,String toRoomID,IRTCCommonCallback callback){
        Log.i(TAG,"switchRoom");

        ZegoSDKManager.getInstance().getStreamService().clearAll();
        ZegoSDKManager.getInstance().getDeviceService().clearAll();

        mState = OUTSIDE;
        if(mRoomStateCallback != null)
        {
            mRoomStateCallback.onExitRoom();
        }
        mZegoRoomConnectionStateListener = null;
        mRoomID = "";
        clearAll();


        if (mState != OUTSIDE) {
            return;
        }

        registerCallback();
        ZegoSDKManager.getInstance().getDeviceService().registerCallback();
        ZegoSDKManager.getInstance().getStreamService().registerCallback();

        mState = PENDING_ENTER;
        if(mRoomStateCallback != null)
        {
            mRoomStateCallback.onPendingEnter();
        }


        mSDKProxy.switchRoom( userID,  userName,fromRoomID,toRoomID, new IRoomLoginCallback() {

            @Override
            public void onLoginCallback(String roomID,int errorCode) {

                if (errorCode == 0) {
                    mRoomID = roomID;
                    mState = ENTERED;
                    if(mRoomStateCallback != null)
                    {
                        mRoomStateCallback.onEntered();
                    }
                    onRoomEntered();
                } else {
                    mState = OUTSIDE;
                    exitRoom();
                }

                if(callback != null) {
                    callback.onRTCCallback(errorCode);
                }
            }

        });
    }


    public void exitRoom() {
        Log.i(TAG,"exitRoom");
        ZegoSDKManager.getInstance().getStreamService().clearAll();
        ZegoSDKManager.getInstance().getDeviceService().clearAll();
        mSDKProxy.logoutRoom(mRoomID);
        mState = OUTSIDE;
        if(mRoomStateCallback != null)
        {
            mRoomStateCallback.onExitRoom();
        }
        mRoomStateCallback = null;
        mZegoRoomConnectionStateListener = null;
        mRoomID = "";
        clearAll();
    }

    boolean isInRoom() {
        return mState == ENTERED;
    }



    public void setZegoRoomConnectionStateListener(IZegoRoomConnectionStateListener zegoRoomConnectionStateListener) {
        this.mZegoRoomConnectionStateListener = zegoRoomConnectionStateListener;
    }


    public void setRoomMessageCallback(IRoomMessageCallback roomMessageCallback){
        this.mRoomMessageCallback = roomMessageCallback;
    }




    public void setRoomStateCallback(IRoomStateCallback roomStateCallback) {
        this.mRoomStateCallback = roomStateCallback;
    }
}
