package im.zego.liveshow.utils;


import im.zego.liveshow.model.rtc.NotifyApplyMicStateInfo;

/**
 * 保存弹窗的消息对象队列
 */
public class DialogQueueUtils {

    private DialogQueue mDialogQueue;
    private DialogListener dialogListener;

    private static volatile DialogQueueUtils singleton = null;

    private NotifyApplyMicStateInfo mNotifyApplyMicStateInfo = null;

    private DialogQueueUtils() {
        mDialogQueue = new DialogQueue();
    }

    public static DialogQueueUtils getInstance() {
        if (singleton == null) {
            synchronized (DialogQueueUtils.class) {
                if (singleton == null) {
                    singleton = new DialogQueueUtils();
                }
            }
        }
        return singleton;
    }

    /**
     * 对象入队
     *
     * @param dialog
     */
    public void addData(NotifyApplyMicStateInfo dialog) {
        if (dialog != null) {
            mDialogQueue.offer(dialog);
        }
    }

    /**
     * 处理弹窗是第一次过来的时候就需要调用 show()
     *
     * @return
     */
    public boolean canShow() {
        return mDialogQueue.canShow();
    }

    public void setInfoNull(){
        mNotifyApplyMicStateInfo = null;
    }

    /**
     * 移除指定对象
     * @param info
     */
    public void remove(NotifyApplyMicStateInfo info){
        mDialogQueue.remove(info);
    }

    public void clear() {
        mNotifyApplyMicStateInfo = null;
        setDialogListener(null);
        mDialogQueue.clear();
    }

    public NotifyApplyMicStateInfo getmNotifyApplyMicStateInfo() {
        return mNotifyApplyMicStateInfo;
    }

    public DialogQueue getDialogQueue() {
        return mDialogQueue;
    }

    /**
     * 弹窗显示
     */
    public void showDialog() {
        if (mNotifyApplyMicStateInfo == null) {
            //从队列中拿出一个NotifyApplyMicStateInfo实例,并从列表中移除
            mNotifyApplyMicStateInfo = mDialogQueue.poll();
            //当队列为空的时候拿出来的会是null
            if (mNotifyApplyMicStateInfo != null) {
                if (dialogListener != null) {
                    dialogListener.onData(mNotifyApplyMicStateInfo);
                }
            }
        }
    }

    public void setDialogListener(DialogListener dialogListener) {
        this.dialogListener = dialogListener;
    }

    public interface DialogListener {
        void onData(NotifyApplyMicStateInfo mNotifyApplyMicStateInfo);
    }

}

