package im.zego.liveshow.utils;

import android.content.Context;
import android.widget.ImageView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.request.RequestOptions;

import im.zego.liveshow.R;
import im.zego.liveshow.manager.UserManager;
import jp.wasabeef.glide.transformations.BlurTransformation;

public class GlideLoader {

    /**
     * 加载圆图
     *
     * @param resourceId
     * @param imageView
     */
    public static void displayCircleImage(int resourceId, ImageView imageView) {
        Glide.with(imageView.getContext())
                .load(resourceId)
                .dontAnimate()
                .circleCrop()
                .diskCacheStrategy(DiskCacheStrategy.ALL) // 设置缓存的策略
                .into(imageView);
    }

    /**
     * 加载原图带边框
     *
     * @param resourceId
     * @param imageView
     */
    public static void displayCircleImageByBorder(int resourceId, ImageView imageView, Context mContext) {
        Glide.with(imageView.getContext())
                .load(resourceId)
                .transform(new GlideCircleTransform(mContext, 2, mContext.getResources().getColor(R.color.liveshow_ffa653ff)))
                .diskCacheStrategy(DiskCacheStrategy.ALL)
                .into(imageView);
    }

    /**
     * 加载高斯模糊的背景图
     *
     * @param resourceId
     * @param imageView
     */
    public static void displayRoundImage(int resourceId, ImageView imageView) {
        Glide.with(imageView.getContext())
                .load(resourceId)
                .apply(RequestOptions.bitmapTransform(new BlurTransformation(15, 2)))
                .into(imageView);
    }


    /**
     * 获取头像显示-根据下标获取对应的图片资源
     *
     * @param avatar
     * @param imageView
     */
    public static void displayCircleImageByAvatar(String avatar, ImageView imageView) {
        try {
            int a = Integer.parseInt(avatar);
            if (a > 0 && a <= UserManager.getInstance().headPortraits.length) {
                displayCircleImage(UserManager.getInstance().headPortraits[a - 1], imageView);
            } else {
                displayCircleImage(UserManager.getInstance().headPortraits[0], imageView);
            }
        } catch (Exception e) {
            displayCircleImage(UserManager.getInstance().headPortraits[0], imageView);
        }
    }

    public static void displayRoundImageByAvatar(String avatar, ImageView imageView) {
        try {
            int a = Integer.parseInt(avatar);
            if (a > 0 && a <= UserManager.getInstance().backgroundList.length) {
                displayRoundImage(UserManager.getInstance().backgroundList[a - 1], imageView);
            } else {
                displayRoundImage(UserManager.getInstance().backgroundList[0], imageView);
            }
        } catch (Exception e) {
            displayRoundImage(UserManager.getInstance().backgroundList[0], imageView);
        }
    }


}
