package im.zego.liveshow.utils;

import static android.net.TrafficStats.UNSUPPORTED;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.net.TrafficStats;
import android.os.Handler;
import android.os.HandlerThread;
import android.util.Log;

import java.text.NumberFormat;

public class NetworkSpeedUtil {



    private static long rxBytes = 0;//下载总流量
    private static long txBytes = 0;//上传总流量
    private static long lastRxBytes = 0;//下载总流量
    private static long lastTxBytes = 0;//上传总流量
    private static long time = 0;
    private static long lastTime = 0;
    private static int uid = -1;

    private static HandlerThread mThread;
    private static Handler networkSpeedHandler = null;


    private static String rxSpeed;
    private static String txSpeed;

    private static Runnable runnable =new Runnable(){

        @Override
        public void run() {
            lastRxBytes = rxBytes;
            lastTxBytes = txBytes;
            rxBytes = TrafficStats.getUidRxBytes(uid);
            txBytes = TrafficStats.getUidTxBytes(uid);

            lastTime = time;
            time = System.currentTimeMillis();

            if(rxBytes == UNSUPPORTED || txBytes == UNSUPPORTED)
            {
                rxSpeed = "UNSUPPORTED";
                txSpeed = "UNSUPPORTED";
            }else {
                rxSpeed = getRate(lastRxBytes,rxBytes,lastTime,time);
                txSpeed = getRate(lastTxBytes,txBytes,lastTime,time);
            }


            try {
                if(networkSpeedHandler != null) {
                    networkSpeedHandler.postDelayed(runnable, 1000);
                }
            }catch (Exception e){

            }
        }
    };

    /**
     * 获取当前应用uid
     * @return
     */
    private static int getUid(Context context) {
        try {
            PackageManager pm = context.getPackageManager();
            @SuppressLint("WrongConstant")
            ApplicationInfo ai = pm.getApplicationInfo(context.getPackageName(), PackageManager.GET_ACTIVITIES);
            return ai.uid;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return -1;
    }

    public static void init(Context context){
        uid = getUid(context);
        rxBytes = TrafficStats.getUidRxBytes(uid);
        txBytes = TrafficStats.getUidTxBytes(uid);
        lastRxBytes = rxBytes;
        lastTxBytes = txBytes;
        lastTime = time;
        time = System.currentTimeMillis();
        mThread = new HandlerThread("NetworkSpeedUtil");
        mThread.start();

        networkSpeedHandler = new Handler(mThread.getLooper());
        networkSpeedHandler.post(runnable);
    }

    public static void uninit(){
        mThread.quit();
        networkSpeedHandler = null;
        uid = -1;
        rxBytes = 0;
        txBytes = 0;
        lastRxBytes = 0;
        lastTxBytes = 0;



    }

    private static String getRate(long lastBytes,long bytes,long lastTime,long time) {

        long bytesSpeed = (long) ((bytes - lastBytes)/((time - lastTime)/1000.0));

        // 对上传速率进行换算
        if (bytesSpeed >= 1000000) {
            // 字节换算成M,设置精确到小数点后1位
            NumberFormat numberFormat = NumberFormat.getInstance();

            numberFormat.setMaximumFractionDigits(2);

            return numberFormat.format(((float) bytesSpeed) / 1000000) + " MB/s";

        } else if (bytesSpeed >= 1000) {
            // 字节换算成K,设置精确到小数点后1位
            NumberFormat numberFormat = NumberFormat.getInstance();

            numberFormat.setMaximumFractionDigits(0);

            return numberFormat.format(((float) bytesSpeed) / 1000) + " KB/s";
        } else {
            // 直接显示字节
            return bytesSpeed + " B/s";
        }


    }


    public static String getRxSpeed() {
        return rxSpeed;
    }

    public static String getTxSpeed() {
        return txSpeed;
    }
}
