package im.zego.liveshow.view;

import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.TypedValue;
import android.view.View;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.annotation.RequiresApi;

import im.zego.liveshow.Constants;
import im.zego.liveshow.R;
import im.zego.liveshow.effects.EffectsBeautyManager;

/**
 * 美颜 美型弹窗
 */
public class EffectsBeautyDialog extends BaseBottomSheetDialogFragment implements View.OnClickListener {

    private TextView mTvBeauty;   //美颜按钮
    private TextView mTvBeautyType;  //美型按钮
    private ImageView mResetting;
    private EffectsBeautyView mEffectsBeautyView;  //美颜view
    private EffectsBeautyTypeView mEffectsBeautyTypeView; //美型view
    private SeekBarWithNumber seekBarWithNumber;

    private String selectType = "";
    private int selectIndex = 0;
    private int beautyPosition = -1;
    private int beautyTypePosition = -1;

    private int min = 0;

    @Override
    public int getLayoutResId() {
        return R.layout.liveshow_dialog_effect;
    }

    @Override
    public void initView(Bundle savedInstanceState) {

        mTvBeauty = rootView.findViewById(R.id.tv_beauty);
        mTvBeautyType = rootView.findViewById(R.id.tv_beauty_type);
        mResetting = rootView.findViewById(R.id.resetting);
        mEffectsBeautyView = rootView.findViewById(R.id.effectsBeautyView);
        mEffectsBeautyTypeView = rootView.findViewById(R.id.effectsBeautyTypeView);
        seekBarWithNumber = rootView.findViewById(R.id.seekBarWithNumber);

        mTvBeauty.setOnClickListener(this);
        mTvBeautyType.setOnClickListener(this);
        mResetting.setOnClickListener(this);

        selectView(mTvBeauty);
        unSelectView(mTvBeautyType);
        mEffectsBeautyView.setVisibility(View.VISIBLE);
        mEffectsBeautyTypeView.setVisibility(View.GONE);

        mEffectsBeautyView.setCallBack((type, minProgress, position) -> {
            selectType = type;
            beautyPosition = position;
            setSeekBarProgress(type, 0, minProgress);
        });
        mEffectsBeautyTypeView.setCallBack((type, minProgress, position) -> {
            selectType = type;
            setSeekBarProgress(type, 1, minProgress);
            beautyTypePosition = position;
        });

        setSeekBar();

    }

    /**
     * 选中效果
     * @param view
     */
    private void selectView(TextView view) {
        view.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 16);
        view.setTextColor(getContext().getResources().getColor(R.color.white_color));
    }

    /**
     * 未选中效果
     * @param view
     */
    private void unSelectView(TextView view) {
        view.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 14);
        view.setTextColor(getContext().getResources().getColor(R.color.white_color_30alpha));
    }

    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.tv_beauty) {  //美颜
            selectIndex = 0;
            selectView(mTvBeauty);
            unSelectView(mTvBeautyType);
            mEffectsBeautyTypeView.notifyData();
            mEffectsBeautyView.setVisibility(View.VISIBLE);
            mEffectsBeautyTypeView.setVisibility(View.GONE);
            seekBarWithNumber.setVisibility(View.INVISIBLE);
            mEffectsBeautyView.setAdaptet();

        } else if (id == R.id.tv_beauty_type) { //美型
            selectIndex = 1;
            selectView(mTvBeautyType);
            unSelectView(mTvBeauty);
            mEffectsBeautyView.notifyData();
            mEffectsBeautyView.setVisibility(View.GONE);
            mEffectsBeautyTypeView.setVisibility(View.VISIBLE);
            seekBarWithNumber.setVisibility(View.INVISIBLE);
            mEffectsBeautyTypeView.setAdapter();

        } else if (id == R.id.resetting) {  //重置
            seekBarWithNumber.setVisibility(View.INVISIBLE);
            if (selectIndex == 0) {
                EffectsBeautyManager.getInstance().setBeautyMap();
                mEffectsBeautyView.notifyPosition(beautyPosition);
                beautyPosition = -1;
                EffectsBeautyManager.getInstance().initBeautyData();
            } else {
                EffectsBeautyManager.getInstance().setBeautyTypeMap();
                mEffectsBeautyTypeView.notifyPosition(beautyTypePosition);
                beautyTypePosition = -1;
                EffectsBeautyManager.getInstance().initBeautyTypeData();
            }
        }
    }

    /**
     * 设置 进度条数据
     * @param type
     * @param index
     * @param minProgress
     */
    private void setSeekBarProgress(String type, int index, int minProgress) {
        int progress;
        if (index == 0) {
            progress = EffectsBeautyManager.getInstance().getBeautyMap().get(type);
        } else {
            progress = EffectsBeautyManager.getInstance().getBeautyTypeMap().get(type);
        }

        if (seekBarWithNumber.getVisibility() != View.VISIBLE) {
            seekBarWithNumber.setVisibility(View.VISIBLE);
        }

        min = minProgress;
        seekBarWithNumber.setOffsetValue(minProgress);
        seekBarWithNumber.setMax(100 - minProgress);
        seekBarWithNumber.setProgress(progress);
    }

    private void setSeekBar() {
        seekBarWithNumber.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {

            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                int currentProgress = (progress + min);
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @RequiresApi(api = Build.VERSION_CODES.N)
            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {
                int currentProgress = seekBar.getProgress();
                if (!TextUtils.isEmpty(selectType)) {
                    if (selectIndex == 0) {
                        EffectsBeautyManager.getInstance().getBeautyMap().replace(selectType, currentProgress);
                    } else {
                        EffectsBeautyManager.getInstance().getBeautyTypeMap().replace(selectType, currentProgress);
                    }

                    switch (selectType) {
                        case Constants.EFFECT_WHITE: //美白
                            EffectsBeautyManager.getInstance().setWhitenParam(currentProgress);
                            break;
                        case Constants.EFFECT_SMOOTH: //磨皮
                            EffectsBeautyManager.getInstance().setSmoothParam(currentProgress);
                            break;
                        case Constants.EFFECT_SHARPEN: //锐化
                            EffectsBeautyManager.getInstance().setSharpenParam(currentProgress);
                            break;
                        case Constants.EFFECT_ROSY: //红润
                            EffectsBeautyManager.getInstance().setRosyParam(currentProgress);
                            break;
                        case Constants.EFFECT_BIG_EYE: //大眼
                            EffectsBeautyManager.getInstance().setBigEyeParam(currentProgress);
                            break;
                        case Constants.EFFECT_FACE_LIFTING: //瘦脸
                            EffectsBeautyManager.getInstance().setFaceLiftingParam(currentProgress);
                            break;
                        case Constants.EFFECT_SMALL_MOUTH: //小嘴
                            EffectsBeautyManager.getInstance().setSmallMouthParam(currentProgress);
                            break;
                        case Constants.EFFECT_EYES_BRIGHTENING: //亮眼
                            EffectsBeautyManager.getInstance().setEyesBrighteningParam(currentProgress);
                            break;
                        case Constants.EFFECT_NOSE_NARROWING: //瘦鼻
                            EffectsBeautyManager.getInstance().setNoseNarrowingParam(currentProgress);
                            break;
                        case Constants.EFFECT_TEETH_WHITENING: // 白牙
                            EffectsBeautyManager.getInstance().setTeethWhiteningParam(currentProgress);
                            break;
                        case Constants.EFFECT_LONG_CHIN: //长下巴
                            EffectsBeautyManager.getInstance().setLongChinParam(currentProgress);
                            break;
                    }
                }
            }
        });
    }


}
