package im.zego.liveshow.view;

import android.content.Context;
import android.util.AttributeSet;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.chad.library.adapter.base.BaseQuickAdapter;

import java.util.ArrayList;
import java.util.List;

import im.zego.liveshow.Constants;
import im.zego.liveshow.R;
import im.zego.liveshow.effects.adapter.EffectsBeautyAdapter;
import im.zego.liveshow.model.BeautyInfo;

/**
 * 美颜
 */
public class EffectsBeautyView extends ConstraintLayout {

    private IBeautyCallBack callBack;

    private RecyclerView mRvBeauty;
    private EffectsBeautyAdapter effectsBeautyAdapter;

    public EffectsBeautyView(@NonNull Context context) {
        this(context, null);
    }

    public EffectsBeautyView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public EffectsBeautyView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        View view = inflate(context, R.layout.liveshow_view_beauty, this);
        mRvBeauty = view.findViewById(R.id.rv_beauty);

        setAdaptet();

    }

    public void setAdaptet() {
        this.mRvBeauty.setLayoutManager(new LinearLayoutManager(getContext(), LinearLayoutManager.HORIZONTAL, false));
        this.effectsBeautyAdapter = new EffectsBeautyAdapter(createBeautyList());
        this.mRvBeauty.setAdapter(this.effectsBeautyAdapter);
        this.effectsBeautyAdapter.setOnItemClickListener((BaseQuickAdapter adapter, View viewItem, int position) -> {
            effectsBeautyAdapter.notifySelect(position, false);
            returnData(position);
        });
    }

    private void returnData(int position) {
        switch (position) {
            case 0:
                if (callBack != null) {
                    callBack.onClickBeauty(Constants.EFFECT_WHITE, 0,position);
                }
                break;
            case 1:
                if (callBack != null) {
                    callBack.onClickBeauty(Constants.EFFECT_SMOOTH, 0,position);
                }
                break;
            case 2:
                if (callBack != null) {
                    callBack.onClickBeauty(Constants.EFFECT_SHARPEN, 0,position);
                }
                break;
            case 3:
                if (callBack != null) {
                    callBack.onClickBeauty(Constants.EFFECT_ROSY, 0,position);
                }
                break;
        }
    }

    public void notifyData() {
        effectsBeautyAdapter = null;
    }

    public void notifyPosition(int position) {
       if(position != -1 && effectsBeautyAdapter != null){
           effectsBeautyAdapter.notifySelect(position,true);
       }
    }

    /**
     * 美颜数据创建
     *
     * @return
     */
    private List<BeautyInfo> createBeautyList() {
        List<BeautyInfo> list = new ArrayList<>();
        String[] beautyList = getResources().getStringArray(R.array.beauty_list);
        for (int i = 0; i < beautyList.length; ++i) {
            String beautyName = beautyList[i];
            int beautyIconUnselect = BeautyInfo.beautyUnselect[i];
            int beautyIconSelect = BeautyInfo.beautySelect[i];
            list.add(new BeautyInfo().setBeautyName(beautyName).setBeautyIconUnSelect(beautyIconUnselect).setBeautyIconSelect(beautyIconSelect));
        }
        return list;
    }

    public void setCallBack(IBeautyCallBack callBack) {
        this.callBack = callBack;
    }

    public interface IBeautyCallBack {
        void onClickBeauty(String type, int minProgress,int position);
    }

}
