package im.zego.liveshow.view;

import android.content.Context;
import android.graphics.Color;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.constraintlayout.widget.ConstraintLayout;

import im.zego.liveshow.R;

/**
 * 实时数据（码率：676 kbs
 * 延迟：460 ms
 * 上行网络：89 KB/s 等数据）
 */
public class LiveshowRealtimeLayout extends ConstraintLayout {

    private TextView mTvKbs; //码率
    private TextView mTvDelay; //延迟
    private TextView mTvUplinkNetwork; //上行网络
    private TextView mTvFps; //帧率
    private TextView mTvPacketLoss; //丢包率
    private TextView mTvDownlinkNetwork; //下行网络
    private ImageView mIvClose;

    private Context context;
    private CloseListener closeListener;

    public LiveshowRealtimeLayout(Context context) {
        super(context);
        init(context);
    }

    public LiveshowRealtimeLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(context);
    }

    private void init(Context context) {
        LayoutInflater.from(getContext()).inflate(R.layout.liveshow_realtime_data, this, true);
        this.context = context;
        mTvKbs = findViewById(R.id.tv_kbs);
        mTvDelay = findViewById(R.id.tv_delay);
        mTvUplinkNetwork = findViewById(R.id.tv_uplink_network);
        mTvFps = findViewById(R.id.tv_fps);
        mTvPacketLoss = findViewById(R.id.tv_packet_loss);
        mTvDownlinkNetwork = findViewById(R.id.tv_downlink_network);
        mIvClose = findViewById(R.id.iv_close);
        mIvClose.setOnClickListener(v -> {
            if (closeListener != null) {
                closeListener.close();
            }
        });

    }

    /**
     * 设置全部数据
     *
     * @param kbs             码率
     * @param delayRate       延迟
     * @param uplinkNetwork   上行网络
     * @param fbs             帧率
     * @param packetLossRate  丢包率
     * @param downlinkNetwork 下行网络
     */
    public void setAllData(int kbs, int delayRate, String uplinkNetwork, int fbs, int packetLossRate, String downlinkNetwork) {
        mTvKbs.setText(String.format(context.getString(R.string.liveshow_realtime_kbs), kbs));
        mTvDelay.setText(String.format(context.getString(R.string.liveshow_realtime_delay), delayRate));
        mTvUplinkNetwork.setText(String.format(context.getString(R.string.liveshow_realtime_uplink_network), uplinkNetwork));
        mTvFps.setText(String.format(context.getString(R.string.liveshow_realtime_fps), fbs));
        mTvPacketLoss.setText(String.format(context.getString(R.string.liveshow_realtime_packet_loss), packetLossRate));
        mTvDownlinkNetwork.setText(String.format(context.getString(R.string.liveshow_realtime_down_network), downlinkNetwork));
    }

    /**
     * 码率
     *
     * @param kbs
     */
    public void setKbs(int kbs) {
        mTvKbs.setText(String.format(context.getString(R.string.liveshow_realtime_kbs), kbs));
    }

    /**
     * 延迟
     *
     * @param delayRate
     */
    public void setDelay(int delayRate) {
        mTvDelay.setText(String.format(context.getString(R.string.liveshow_realtime_delay), delayRate));
    }

    /**
     * 上行网络
     *
     * @param uplinkNetwork
     */
    public void seUplinkNetwork(String uplinkNetwork) {
        mTvUplinkNetwork.setText(String.format(context.getString(R.string.liveshow_realtime_uplink_network), uplinkNetwork));
    }

    /**
     * 帧率
     *
     * @param fbs
     */
    public void setFps(int fbs) {
        mTvFps.setText(String.format(context.getString(R.string.liveshow_realtime_fps), fbs));
    }

    /**
     * 丢包率
     *
     * @param packetLossRate
     */
    public void setPacketLoss(int packetLossRate) {
        mTvPacketLoss.setText(String.format(context.getString(R.string.liveshow_realtime_packet_loss), packetLossRate));
    }

    /**
     * 下行网络
     *
     * @param downlinkNetwork
     */
    public void setDownlinkNetwork(String downlinkNetwork) {
        mTvDownlinkNetwork.setText(String.format(context.getString(R.string.liveshow_realtime_down_network), downlinkNetwork));
    }


    public LiveshowRealtimeLayout setCloseListener(CloseListener closeListener) {
        this.closeListener = closeListener;
        return this;
    }

    public interface CloseListener {
        void close();
    }


}
