package im.zego.liveshow.view;

import android.content.DialogInterface;
import android.os.Bundle;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import im.zego.liveshow.Constants;
import im.zego.liveshow.R;
import im.zego.liveshow.liveroom.adapter.OnlineAdapter;
import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.model.RoomInfo;
import im.zego.liveshow.model.StatefulListInfo;
import im.zego.liveshow.model.UserInfo;
import im.zego.liveshow.model.rtc.NotifyUserStateUpdateInfo;
import im.zego.liveshow.protocol.RoomAPI;
import im.zego.liveshow.utils.ListSortUtils;
import im.zego.liveshow.utils.ToastUtil;

import static im.zego.liveshow.Constants.OPERATION_ACTION_APPLY;
import static im.zego.liveshow.Constants.ROOM_ROLE_AUDIENCE;
import static im.zego.liveshow.Constants.ROOM_ROLE_HOST;

/**
 * 在线人数（主播/观众 端）
 */
public class OnlineDialog extends BaseBottomSheetDialogFragment {

    private TextView mTvOnlineNum;
    private RecyclerView mRvOnline;
    private OnlineAdapter onlineAdapter;

    private List<StatefulListInfo.AttendeeListDTO> attendeeListAll = new ArrayList<>();
    private List<StatefulListInfo.AttendeeListDTO> attendeeListMIc = new ArrayList<>();
    private List<StatefulListInfo.AttendeeListDTO> attendeeListInfo = new ArrayList<>();

    private StatefulListInfo.AttendeeListDTO attendeeListHost = null;  //主播信息
    private StatefulListInfo.AttendeeListDTO attendeeListMine = null;  //个人信息


    @Override
    public int getLayoutResId() {
        return R.layout.liveshow_dialog_online;
    }

    @Override
    public void initView(Bundle savedInstanceState) {

        setShowHeight(true); //设置弹窗最大高度

        mTvOnlineNum = rootView.findViewById(R.id.tv_online_num);
        mRvOnline = rootView.findViewById(R.id.rv_online);

        onlineAdapter = new OnlineAdapter(mContext);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(mContext);
        mRvOnline.setLayoutManager(linearLayoutManager);
        mRvOnline.setAdapter(onlineAdapter);
        //解决刷新闪烁问题
        ((SimpleItemAnimator) mRvOnline.getItemAnimator()).setSupportsChangeAnimations(false);

        onlineAdapter.setInviteStageListener(userInfo -> {
            //邀请上麦
            setOnstageInvite(userInfo);

        });

        getAttendeeList();
    }

    /**
     * 拉取获取成员列表
     */
    public void getAttendeeList() {

        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();

        RoomAPI.getAttendeeList(roomInfo.getRoomId(), userInfo.getUid(), (errorCode, statefulListInfo) -> {
            if (errorCode == 0) {

                // 先清除之前的数据
                attendeeListHost = null;
                attendeeListMine = null;
                attendeeListAll.clear();
                attendeeListInfo.clear();
                attendeeListMIc.clear();

                List<StatefulListInfo.AttendeeListDTO> attendeeList = statefulListInfo.getAttendeeList();

                mTvOnlineNum.setText(String.format(mContext.getString(R.string.liveshow_online_num), attendeeList.size()));

                for (int i = 0; i < attendeeList.size(); i++) {
                    StatefulListInfo.AttendeeListDTO attendeeListDTO = attendeeList.get(i);
                    if (attendeeListDTO.getRole() == ROOM_ROLE_HOST) { //主持人
                        attendeeListHost = attendeeListDTO;
                    } else if (attendeeListDTO.getRole() == ROOM_ROLE_AUDIENCE) { //观众
                        if (userInfo.getUid() == attendeeListDTO.getUid()) { //本人
                            attendeeListMine = attendeeListDTO;
                        } else {
                            if (attendeeListDTO.getOnstageState() == Constants.STATE_ON_STAGE) { //在台上
                                attendeeListMIc.add(attendeeListDTO);
                            } else {
                                attendeeListInfo.add(attendeeListDTO);
                            }
                        }
                    }
                }
                if (attendeeListHost != null) {
                    attendeeListAll.add(attendeeListHost);
                }

                if (attendeeListMine != null) {
                    attendeeListAll.add(attendeeListMine);
                }

                attendeeListAll.addAll(ListSortUtils.listSorting(attendeeListMIc));
                attendeeListAll.addAll(attendeeListInfo);

                onlineAdapter.addData(attendeeListAll);
            } else {
                ToastUtil.showToast(R.string.liveshow_operation_fail);
            }
        });
    }

    /**
     * 邀请连麦
     */
    public void setOnstageInvite(StatefulListInfo.AttendeeListDTO attendeeListDTO) {
        RoomInfo.RoomInfoDTO roomInfo = RoomManager.getInstance().getRoomInfo().getRoomInfo();
        UserInfo userInfo = RoomManager.getInstance().getRoomInfo().getUserInfo();
        RoomAPI.setOnstageInvite(userInfo.getUid(), userInfo.getNickName(), attendeeListDTO.getUid(), roomInfo.getRoomId(), OPERATION_ACTION_APPLY, "", errorCode -> {
            if (errorCode == 0) {
                //数据变更为被邀请
                ToastUtil.showToast("你已发送邀请");
                attendeeListDTO.setOnstageState(3);
                onlineAdapter.updateOnstageData(attendeeListDTO);
            } else if (errorCode == 80005) {
                ToastUtil.showToast(R.string.liveshow_connect_mic_full);
            }
        });
    }

    /**
     * 数据变更
     *
     * @param notifyInfo
     */
    public void onInviteMicNotify(NotifyUserStateUpdateInfo notifyInfo) {
        if (notifyInfo.getType() == Constants.OPERATION_TYPE_INVITE || notifyInfo.getType() == Constants.OPERATION_TYPE_INVITE_REFUSE
                || notifyInfo.getType() == Constants.OPERATION_TYPE_STAGE_STATE) {
            //被邀请上麦 \ 拒绝 \ 同意
            List<NotifyUserStateUpdateInfo.UsersDTO> stateUpdateInfo = notifyInfo.getUsers();
            for (int i = 0; i < stateUpdateInfo.size(); i++) {
                NotifyUserStateUpdateInfo.UsersDTO UsersDTO = stateUpdateInfo.get(i);
                StatefulListInfo.AttendeeListDTO userInfo = new StatefulListInfo.AttendeeListDTO();
                userInfo.setUid(UsersDTO.getUid());
                userInfo.setOnstageState(UsersDTO.getOnstageState());
                onlineAdapter.updateOnstageData(userInfo);
            }
        }
    }


    /**
     * 主播网络重连后收不到观众拒绝连麦的提示
     */
    public void refuseToast(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo) {
        for (StatefulListInfo.AttendeeListDTO info : attendeeListAll) {
            if(info.getOnstageState() == Constants.STATE_INVITE_CON_MIC){
                if(!audienceListIncludeMe(audienceStatefulListInfo,info.getUid())){
                    ToastUtil.showToast(info.getNickName()+"已拒绝连麦邀请");
                }
            }
        }
        getAttendeeList();
    }

    /**
     * 是否包含数据
     *
     * @param audienceStatefulListInfo
     * @return
     */
    private boolean audienceListIncludeMe(List<StatefulListInfo.AttendeeListDTO> audienceStatefulListInfo,long inviteUid) {
        for (StatefulListInfo.AttendeeListDTO info : audienceStatefulListInfo) {
            if (inviteUid == info.getUid()) {
                return true;
            }
        }
        return false;
    }


    @Override
    public void onDismiss(@NonNull @NotNull DialogInterface dialog) {
        super.onDismiss(dialog);
        // 先清除之前的数据
        attendeeListHost = null;
        attendeeListMine = null;
        attendeeListAll.clear();
        attendeeListInfo.clear();
        attendeeListMIc.clear();

    }
}
