package im.zego.liveshow.view;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.provider.Settings;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.FragmentActivity;

import im.zego.liveshow.Constants;
import im.zego.liveshow.R;
import im.zego.liveshow.manager.RoomManager;
import im.zego.liveshow.protocol.IExecuteCallback;
import im.zego.liveshow.sdk.ZegoSDKManager;
import im.zego.liveshow.utils.CheckDoubleClick;
import im.zego.liveshow.utils.PermissionHelper;
import im.zego.liveshow.utils.ToastUtil;

import static im.zego.liveshow.Constants.ROOM_ROLE_AUDIENCE;

public class RoomBottomLayerLayout extends ConstraintLayout {

    public static final int CONNECTION_MIC_PREPARE = 1; //连麦准备（还没开始连麦）
    public static final int CONNECTION_MIC_GO_ON = 2; //连麦中（按钮变成取消连麦-可以取消申请连麦）
    public static final int CONNECTION_MIC_SUCCESS = 3; //成功连麦（按钮变成下麦）

    private ConstraintLayout mClOverturn; //翻转
    private ConstraintLayout mClCamera; //摄像头
    private ImageView mIvCamera;
    private ConstraintLayout mClMic; //麦克风
    private ImageView mIvMic;
    private ConstraintLayout mClConnectionMic; //连麦
    private ImageView mIvConnectionMic; //连麦图标
    private TextView mTvConnectionMic;
    private TextView mTvCancleConnectionMic; //取消连麦

    private CameraMicListener cameraMicListener; //摄像头/麦克风监听

    private int connectionMicType = 1;
    private boolean mIsMicMuted = true; //麦克风是否开启
    private boolean mIsCameraMuted = true; //摄像头是否开启
    private boolean mIsFrontCam = true; //摄像头是否翻转 true 表示使用前置摄像头；false 表示使用后置摄像头

    private Context mContext;

    public RoomBottomLayerLayout(Context context) {
        super(context);
        init();
    }

    public RoomBottomLayerLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
        this.mContext = context;
        init();
        initListener();
    }


    private void init() {
        LayoutInflater.from(getContext()).inflate(R.layout.liveshow_more_select, this, true);

        mClOverturn = findViewById(R.id.cl_overturn);
        mClCamera = findViewById(R.id.cl_camera);
        mClMic = findViewById(R.id.cl_mic);
        mIvCamera = findViewById(R.id.iv_camera);
        mIvMic = findViewById(R.id.iv_mic);
        mClConnectionMic = findViewById(R.id.cl_connection_mic);
        mIvConnectionMic = findViewById(R.id.iv_connection_mic);
        mTvConnectionMic = findViewById(R.id.tv_connection_mic);
        mTvCancleConnectionMic = findViewById(R.id.tv_cancle_connection_mic);

        bottomConnectionMicView(CONNECTION_MIC_PREPARE);
        RoomManager.getInstance().setHasConMic(false);
    }

    private void initListener() {
        //翻转
        mClOverturn.setOnClickListener(v -> {
            //防快速点击
            if (CheckDoubleClick.isFastDoubleClick()) {
                return;
            }
            mIsFrontCam = !mIsFrontCam;
            //摄像头翻转
            ZegoSDKManager.getInstance().getDeviceService().setFrontCamera(mIsFrontCam);
        });

        //摄像头
        mClCamera.setOnClickListener(v -> {
            if (cameraMicListener != null) {
                //防快速点击
                if (CheckDoubleClick.isFastDoubleClick()) {
                    return;
                }
                switchCamera(mIsCameraMuted);
                //摄像头开启/关闭(需要请求服务器 服务器成功回掉后才设置sdk的摄像头状态)
                //"type"  :   1.mic操作  2. camera操作 3. 关闭连麦
                cameraMicListener.cameraOrMic(mIsMicMuted, mIsCameraMuted, Constants.OPERATION_TYPE_CAMERA, ROOM_ROLE_AUDIENCE);
            }
        });

        //麦克风
        mClMic.setOnClickListener(v -> {
            if (cameraMicListener != null) {
                //防快速点击
                if (CheckDoubleClick.isFastDoubleClick()) {
                    return;
                }

                //已经被主播禁麦是不可以自动恢复解禁的
                if(RoomManager.getInstance().isMicIsOffFromHost()){
                    ToastUtil.showToast("你已被主播禁言");
                }else {
                    switchMic(mIsMicMuted);
                    //麦克风开启/关闭(需要请求服务器 服务器成功回掉后才设置sdk的麦克风状态)
                    cameraMicListener.cameraOrMic(mIsMicMuted, mIsCameraMuted, Constants.OPERATION_TYPE_MIC, ROOM_ROLE_AUDIENCE);
                }

            }
        });

        //上下麦的操作
        mClConnectionMic.setOnClickListener(v -> {
            if (CheckDoubleClick.isFastDoubleClick()) {
                return;
            }
            //连麦
            if (connectionMicType == RoomBottomLayerLayout.CONNECTION_MIC_PREPARE) {
                //申请连麦
                connectionMicPermission();
            } else if (connectionMicType == RoomBottomLayerLayout.CONNECTION_MIC_GO_ON) {
                //取消连麦
                connectionMicHttpRequest(Constants.OPERATION_ACTION_CANCEL);
                bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
            } else if (connectionMicType == RoomBottomLayerLayout.CONNECTION_MIC_SUCCESS) {
                //下麦
                showFinishConnectionMicDialog();
            }
        });
    }

    /**
     * 翻转、摄像头、连麦按钮是否显示（只有观众成功上麦后才能显示）
     *
     * @param isVisible
     */
    public void bottomLayerVisible(boolean isVisible) {
        mClOverturn.setVisibility(isVisible ? VISIBLE : GONE);
        mClCamera.setVisibility(isVisible ? VISIBLE : GONE);
        mClMic.setVisibility(isVisible ? VISIBLE : GONE);
    }

    /**
     * 连麦按钮的各种状态
     *
     * @param connectionMicType
     */
    public void bottomConnectionMicView(int connectionMicType) {

        this.connectionMicType = connectionMicType;
        if (CONNECTION_MIC_PREPARE == connectionMicType) { //连麦的前期准备（按钮显示连麦）
            mClConnectionMic.setBackgroundResource(R.drawable.liveshow_round_gray_bg);
            mTvConnectionMic.setText(getContext().getString(R.string.liveshow_connect_mic));
            mIvConnectionMic.setVisibility(VISIBLE);
            mTvConnectionMic.setVisibility(VISIBLE);
            mTvCancleConnectionMic.setVisibility(GONE);
        } else if (CONNECTION_MIC_GO_ON == connectionMicType) { //连麦中（按钮显示-取消申请）
            mClConnectionMic.setBackgroundResource(R.drawable.liveshow_round_gray_bg);
            mTvCancleConnectionMic.setVisibility(VISIBLE);
            mIvConnectionMic.setVisibility(GONE);
            mTvConnectionMic.setVisibility(GONE);
        } else if (CONNECTION_MIC_SUCCESS == connectionMicType) {  //连麦成功连（按钮显示下麦）
            mClConnectionMic.setBackgroundResource(R.drawable.liveshow_round_red_bg);
            mTvConnectionMic.setText(getContext().getString(R.string.liveshow_mic_off));
            mIvConnectionMic.setVisibility(VISIBLE);
            mTvConnectionMic.setVisibility(VISIBLE);
            mTvCancleConnectionMic.setVisibility(GONE);
        }
    }

    /**
     * 麦克风状态
     *
     * @param isMuted
     */
    public void switchMic(boolean isMuted) {
        if (isMuted) {
            mIvMic.setImageResource(R.drawable.liveshow_mic_off);
        } else {
            mIvMic.setImageResource(R.drawable.liveshow_mic_on);
        }
        mIsMicMuted = !isMuted;
    }

    /**
     * @param isOpen
     */
    public void switchCamera(boolean isOpen) {
        if (isOpen) {
            mIvCamera.setImageResource(R.drawable.liveshow_cam_off);
        } else {
            mIvCamera.setImageResource(R.drawable.liveshow_cam_on);
        }
        mIsCameraMuted = !isOpen;
    }

    public boolean ismIsMicMuted() {
        return mIsMicMuted;
    }

    public boolean ismIsCameraMuted() {
        return mIsCameraMuted;
    }

    public RoomBottomLayerLayout setCameraMicListener(CameraMicListener cameraMicListener) {
        this.cameraMicListener = cameraMicListener;
        return this;
    }

    public interface CameraMicListener {
        void cameraOrMic(boolean mIsMicMuted, boolean mIsCameraMuted, int type, int operator);
    }

    /**
     * 连麦申请
     */
    private void connectionMicPermission() {
        PermissionHelper.onCameraAndAudioPermissionGranted((FragmentActivity) mContext, new PermissionHelper.GrantResult() {
            @Override
            public void onGrantResult(boolean allGranted) {
                if (allGranted) {
                    bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_GO_ON);
                    connectionMicHttpRequest(Constants.OPERATION_ACTION_APPLY);
                    ToastUtil.showToast(R.string.liveshow_connect_mic_gon_on);
                } else {
                    BaseInquiryDialog baseInquiryDialog = new BaseInquiryDialog(mContext);
                    baseInquiryDialog.setMsgTitle(mContext.getString(R.string.liveshow_tips));
                    baseInquiryDialog.setMsgContent(mContext.getString(R.string.liveshow_need_pemission_content));
                    baseInquiryDialog.setLeftButtonContent(mContext.getString(R.string.liveshow_btn_cancle));
                    baseInquiryDialog.setRightButtonContent(mContext.getString(R.string.liveshow_go_setting));
                    baseInquiryDialog.setSureListener(v -> {
                        baseInquiryDialog.dismiss();
                        Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
                                .setData(Uri.fromParts("package", mContext.getPackageName(), null));
                        ((Activity) mContext).startActivityForResult(intent, 666);
                    });
                    baseInquiryDialog.setCancelListener(v -> {
                        baseInquiryDialog.dismiss();
                    });
                }
            }
        });
    }

    /**
     * 连麦的请求  action 1.申请 2.取消申请 3. 接受申请 4. 拒绝申请
     */
    private void connectionMicHttpRequest(int action) {
        RoomManager.getInstance().setUserInfo(action, new IExecuteCallback() {
            @Override
            public void onSuccess() {
                if (action == Constants.OPERATION_ACTION_APPLY) {
                    bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_GO_ON);
                    RoomManager.getInstance().setHasConMic(true);
                } else if (action == Constants.OPERATION_ACTION_CANCEL) {
                    bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
                    RoomManager.getInstance().setHasConMic(false);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                if (action == Constants.OPERATION_ACTION_APPLY) {
                    bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_PREPARE);
                } else if (action == Constants.OPERATION_ACTION_CANCEL) {
                    bottomConnectionMicView(RoomBottomLayerLayout.CONNECTION_MIC_GO_ON);
                }
            }
        });
    }

    /**
     * 结束连麦的询问弹窗
     */
    private void showFinishConnectionMicDialog() {
        if (!((Activity) mContext).isFinishing()) {
            BaseInquiryDialog baseInquiryDialog = new BaseInquiryDialog(mContext, R.layout.liveshow_finish_connection_mic_dialog);
            baseInquiryDialog.setMsgTitle(mContext.getString(R.string.liveshow_finish_connect_mic));
            baseInquiryDialog.setMsgContent(mContext.getString(R.string.liveshow_finish_connect_mic_content));
            baseInquiryDialog.setSureListener(v -> {
                baseInquiryDialog.dismiss();
                //下麦的接口请求
                if (cameraMicListener != null) {
                    cameraMicListener.cameraOrMic(false, false, Constants.OPERATION_TYPE_STAGE_STATE, ROOM_ROLE_AUDIENCE);
                }
            });
            baseInquiryDialog.setCancelListener(v -> baseInquiryDialog.dismiss());
        }
    }

}
