package im.zego.liveshow.view;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.SeekBar;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.chad.library.adapter.base.BaseQuickAdapter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import im.zego.liveshow.R;
import im.zego.liveshow.effects.SoundEffectsManager;
import im.zego.liveshow.effects.adapter.BackgroundSoundAdapter;
import im.zego.liveshow.effects.adapter.ReverbPresetAdapter;
import im.zego.liveshow.effects.adapter.VoiceChangeAdapter;
import im.zego.liveshow.model.AudioEffectInfo;
import im.zego.liveshow.model.ReverbPresetInfo;
import im.zego.liveshow.model.VoiceChangeInfo;
import im.zego.liveshow.utils.ToastUtil;

/**
 * 音效 dialog
 */
public class SoundEffectsDialog extends BaseBottomSheetDialogFragment {

    private RecyclerView mRvBgSound;  //背景音乐
    private SoundEffectsSeekBar mMusicSoundSeekbar; //音乐音量
    private SoundEffectsSeekBar mPeopleSoundSeekbar; //人声音量
    private RecyclerView mRvVoiceChange; //变声
    private RecyclerView mRvReverbPreset;  //混响

    private BackgroundSoundAdapter backgroundSoundAdapter;

    private VoiceChangeAdapter voiceChangeAdapter;
    private ReverbPresetAdapter reverbPresetAdapter;

    private SoundEffectsManager soundEffectsManager;

    @Override
    public int getLayoutResId() {
        return R.layout.liveshow_dialog_sound_effects;
    }

    @Override
    public void initView(Bundle savedInstanceState) {

        setShowHeight(true); //设置弹窗最大高度

        mRvBgSound = rootView.findViewById(R.id.rv_bg_sound);
        mMusicSoundSeekbar = rootView.findViewById(R.id.music_sound_seekbar);
        mPeopleSoundSeekbar = rootView.findViewById(R.id.people_sound_seekbar);
        mRvVoiceChange = rootView.findViewById(R.id.rv_voice_change);
        mRvReverbPreset = rootView.findViewById(R.id.rv_reverbpreset);

        soundEffectsManager = SoundEffectsManager.getInstance();

        setBackgroundSound();
        setVoiceChange();
        setReverbPreset();

        setVolume(); //音乐音量
        setCaptureVolume(); //人声音量

    }

    /**
     * 背景音乐
     */
    private void setBackgroundSound() {
        this.mRvBgSound.setLayoutManager(new LinearLayoutManager(getContext(), LinearLayoutManager.HORIZONTAL, false));
        this.backgroundSoundAdapter = new BackgroundSoundAdapter(createBackgroundSoundList());
        this.mRvBgSound.setAdapter(this.backgroundSoundAdapter);
        if (manager().getBackgroundSoundPosition() != -1) {
            backgroundSoundAdapter.notifySelect(manager().getBackgroundSoundPosition(), true);
        }
        this.backgroundSoundAdapter.setOnItemClickListener((BaseQuickAdapter adapter, View view, int position) -> {

            String songPath = SoundEffectsManager.getInstance().getmSongFilePaths().get(position);

            if (TextUtils.isEmpty(songPath)) {
                backgroundSoundAdapter.notifySelect(manager().getBackgroundSoundPosition(), true);
                //copy 音乐特效的资料
                SoundEffectsManager.getInstance().initLocalAudioEffectlist(mContext);
                ToastUtil.showToast("资源加载失败，请重试");
            }

            if (manager().getBackgroundSoundPosition() != -1) {
                if (manager().getBackgroundSoundPosition() == position) {
                    manager().audioEffectStop(position);
                    manager().setBackgroundSoundPosition(-1);
                } else {
                    manager().audioEffectStop(manager().getBackgroundSoundPosition());
                    manager().audioEffectPlayer(position, songPath);
                    manager().setBackgroundSoundPosition(position);
                }
            } else {
                manager().setBackgroundSoundPosition(position);
                manager().audioEffectPlayer(position, songPath);
            }
            backgroundSoundAdapter.notifySelect(position, true);
        });
    }

    /**
     * 变声
     */
    private void setVoiceChange() {
        this.mRvVoiceChange.setLayoutManager(new LinearLayoutManager(getContext(), LinearLayoutManager.HORIZONTAL, false));
        this.voiceChangeAdapter = new VoiceChangeAdapter(createVoiceChangeList());
        this.mRvVoiceChange.setAdapter(this.voiceChangeAdapter);
        this.voiceChangeAdapter.notifySelect(manager().getVoiceChangePosition(), false);
        this.voiceChangeAdapter.setOnItemClickListener((BaseQuickAdapter adapter, View view, int position) -> {
            voiceChangeAdapter.notifySelect(position, false);
            String[] options = getResources().getStringArray(R.array.voicePreset);
            manager().setVoiceChangerPreset(options[position]);
            manager().setVoiceChangePosition(position);
        });
    }

    /**
     * 混响
     */
    private void setReverbPreset() {
        this.mRvReverbPreset.setLayoutManager(new LinearLayoutManager(getContext(), LinearLayoutManager.HORIZONTAL, false));
        this.reverbPresetAdapter = new ReverbPresetAdapter(createReverbPresetList(), mContext);
        this.mRvReverbPreset.setAdapter(this.reverbPresetAdapter);
        this.reverbPresetAdapter.notifySelect(manager().getReverbPresetPosition(), false);
        this.reverbPresetAdapter.setOnItemClickListener((BaseQuickAdapter adapter, View view, int position) -> {
            reverbPresetAdapter.notifySelect(position, false);
            String[] options = getResources().getStringArray(R.array.reverbPreset);
            manager().setReverbPreset(options[position]);
            manager().setReverbPresetPosition(position);
        });
    }

    /**
     * 音乐音量
     */
    private void setVolume() {
        mMusicSoundSeekbar.getSeekBar().setProgress(manager().getMusicVolum());
        mMusicSoundSeekbar.setProgress(manager().getMusicVolum());
        mMusicSoundSeekbar.getSeekBar().setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                mMusicSoundSeekbar.setProgress(progress);
                manager().setVolume(progress);
                manager().setMusicVolum(progress);
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

    }

    /**
     * 人声音量
     */
    private void setCaptureVolume() {
        mPeopleSoundSeekbar.getSeekBar().setProgress(manager().getMusicVolum());
        mPeopleSoundSeekbar.setProgress(manager().getVocalVolume());
        mPeopleSoundSeekbar.getSeekBar().setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                mPeopleSoundSeekbar.setProgress(progress);
                manager().setCaptureVolume(progress);
                manager().setVocalVolume(progress);
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });
    }

    /**
     * 背景音数据创建
     *
     * @return
     */
    private List<String> createBackgroundSoundList() {
        List<String> list = new ArrayList<>();
        String[] languageList = getResources().getStringArray(R.array.background_sound_list);
        List stringList = Arrays.asList(languageList);
        list.addAll(stringList);
        return list;
    }

    /**
     * 变声数据创建
     *
     * @return
     */
    private List<VoiceChangeInfo> createVoiceChangeList() {
        List<VoiceChangeInfo> list = new ArrayList<>();
        String[] voiceList = getResources().getStringArray(R.array.voice_change_list);
        for (int i = 0; i < voiceList.length; ++i) {
            String voiceName = voiceList[i];
            int voiceIconUnselect = VoiceChangeInfo.voiceChangeUnselect[i];
            int voiceIconSelect = VoiceChangeInfo.voiceChangeSelect[i];
            list.add(new VoiceChangeInfo().setVoiceName(voiceName).setVoiceIconUnSelect(voiceIconUnselect).setVoiceIconSelect(voiceIconSelect));
        }
        return list;
    }

    /**
     * 混响数据创建
     *
     * @return
     */
    private List<ReverbPresetInfo> createReverbPresetList() {
        List<ReverbPresetInfo> list = new ArrayList<>();
        String[] reverbPresetList = getResources().getStringArray(R.array.reverbPreset_list);
        for (int i = 0; i < reverbPresetList.length; ++i) {
            String rpName = reverbPresetList[i];
            int rpIcon = ReverbPresetInfo.reverbPresetIcon[i];
            list.add(new ReverbPresetInfo().setRpName(rpName).setRpIcon(rpIcon));
        }
        return list;
    }


    private SoundEffectsManager manager() {
        if (soundEffectsManager != null) {
            return soundEffectsManager;
        } else {
            return SoundEffectsManager.getInstance();
        }
    }

}
