package im.zego.goenjoy;

import androidx.appcompat.app.AppCompatActivity;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;

import im.zego.common.model.LoginInfo;
import im.zego.common.util.ScreenAdaptationUtil;
import im.zego.ktv.chorus.login.LoginInfoManager;
import im.zego.ktv.chorus.protocol.ILoginCallback;

import im.zego.gochat.login.LoginActivity;
import im.zego.ktv.chorus.roomlist.activity.RoomListActivity;
import im.zego.ktv.utils.ToastUtil;

import static im.zego.gochat.ErrorcodeConstants.ERROR_NICK_NAME_ALREADY_IN_USE;

public class MainActivity extends AppCompatActivity {

    public final static int PERMISSION_REQUEST_CODE = 0x1000;
    public static final int GoChatErrorFailNetwork = -2;

    private static final String TAG = "MainActivity";
//    private ActivityMainBinding mBinding;
    private final static String[] PERMISSIONS_REQUEST = {Manifest.permission.RECORD_AUDIO};

    private boolean logging = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if((getIntent().getFlags() & Intent.FLAG_ACTIVITY_BROUGHT_TO_FRONT) != 0){
            finish();
            return;
        }

        requestWindowFeature(Window.FEATURE_NO_TITLE);

        ScreenAdaptationUtil.init(this.getApplicationContext(), 375);
        ScreenAdaptationUtil.setCustomDensity(this);

        setContentView(R.layout.activity_main_entrance);
        initWindowBar();

        checkOrRequestPermission(PERMISSION_REQUEST_CODE);
    }

    private void initWindowBar() {
        WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.flags = lp.flags | WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS;
        getWindow().setAttributes(lp);
    }

    /**
     * 多人连麦直播
     * @param view
     */
    public void enterLiveShow(View view){
        Intent intent = new Intent();
        intent.setClass(MainActivity.this, im.zego.liveshow.roomlist.RoomListActivity.class);
        startActivity(intent);
    }

    /**
     * 登陆go语聊
     * @param view
     */
    public void enterGoChatFunc(View view){
        Intent intent = new Intent();
        intent.setClass(this, LoginActivity.class);
        startActivity(intent);
    }

    /**
     * KTV合唱
     * @param view
     */
    public void enterKTVFunc(View view){
        loginServer();
    }
    private void loginServer() {
        if(logging){
            return;
        }
        logging = true;
//        String myNickName = ZegoApplication.getKTVRandomUserName();
//        String avatar = ZegoApplication.getAvatarByUserName(myNickName);
        LoginInfoManager.getInstance().loginUser("", "", new ILoginCallback<LoginInfo>() {
            @Override
            public void onLogin(int errorCode, LoginInfo loginInfo) {
                logging = false;
                if (errorCode == 0) {
//                    loginInfo.setNickName(myNickName);
//                    loginInfo.setAvatar(avatar);
                    LoginInfoManager.getInstance().setLoginInfo(loginInfo);
//                    RTCSDKManager.getInstance().setUserID(loginInfo.getUid(), loginInfo.getNickName());
//                    RTCSDKMusicManager.getInstance().setUserID(loginInfo.getUid(), loginInfo.getNickName());

                    Intent intent = new Intent();
                    intent.setClass(MainActivity.this, RoomListActivity.class);
                    startActivity(intent);
                }else if(errorCode == ERROR_NICK_NAME_ALREADY_IN_USE){
                    loginServer();
                }else if(errorCode == GoChatErrorFailNetwork)
                {
                    ToastUtil.showWarnToast(getResources().getString(R.string.login_tips_fail_network));
                } else {
                    ToastUtil.showWarnToast(getResources().getString(R.string.login_tips_fail));
                }
            }
        });
    }

    public void ViewPrivacyProtocol(View view){
        Uri uri = Uri.parse(getString(R.string.privacy_policy_url));
        Intent intent = new Intent(Intent.ACTION_VIEW, uri);
        startActivity(intent);
    }

    public void ViewUserProtocol(View view){
        Uri uri = Uri.parse(getString(R.string.user_protocol_url));
        Intent intent = new Intent(Intent.ACTION_VIEW, uri);
        startActivity(intent);
    }

    public void MainSetting(View view){
        startActivity(new Intent(this, MainSettingActivity.class));
    }

    /**
     * 检查并申请权限
     *
     * @param requestCode requestCode
     * @return 权限是否已经允许
     */
    protected boolean checkOrRequestPermission(int requestCode) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (checkSelfPermission(Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED) {
                this.requestPermissions(PERMISSIONS_REQUEST, requestCode);
                return false;
            }
        }
        return true;
    }

    @Override
    protected void onDestroy() {
        ScreenAdaptationUtil.unInit(this.getApplicationContext());
        super.onDestroy();
    }
}