package im.zego.goenjoy;

import android.app.Activity;
import android.app.Application;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.webkit.JavascriptInterface;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ImageView;

import androidx.appcompat.app.AppCompatActivity;

import im.zego.common.application.ZegoApplication;
import im.zego.common.util.DeviceInfoManager;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.constants.ZegoScenario;
import im.zego.zegoexpress.entity.ZegoEngineProfile;

public class MainFeedBackActivity extends AppCompatActivity {
    private WebView mWebView;
    private ImageView back_img;
    private String feedbackUrl;
    private String TAG = "FeedbackLog";
    private ZegoExpressEngine mZegoExpressEngine;

    public final static String FEEDBACK_API_URL = "https://demo-operation.zego.im";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_feedback);

        ZegoEngineProfile profile = new ZegoEngineProfile();
        profile.appID = im.zego.gochat.AuthConstants.APP_ID;
        profile.appSign = im.zego.gochat.AuthConstants.APP_SIGN;
        profile.scenario = ZegoScenario.GENERAL;
        profile.application = (Application) ZegoApplication.getApplication();

        mZegoExpressEngine = ZegoExpressEngine.createEngine(profile, null);
//        mZegoExpressEngine = ZegoExpressEngine.createEngine(AuthConstants.APP_ID, AuthConstants.APP_SIGN, false, ZegoScenario.GENERAL, (Application) ZegoApplication.getApplication(), null);
        mWebView = findViewById(R.id.tv_feedbook_webview);
        StringBuilder sb = new StringBuilder();
        sb.append(FEEDBACK_API_URL)
                .append("/feedback/goenjoy")
                .append("/index.html?platform=8")
                .append("&system_version=android_")
                .append(android.os.Build.VERSION.RELEASE)
                .append("&app_version=")
                .append(BuildConfig.VERSION_NAME)
                .append("&sdk_version=")
                .append(ZegoExpressEngine.getVersion())
                .append("&device_id=")
                .append(DeviceInfoManager.generateDeviceId(this))
                .append("&client=")
                .append(DeviceInfoManager.getProductName());
        feedbackUrl = sb.toString();
        loadFeedbackWeb(feedbackUrl);
    }

    public static void actionActivity(Activity activity) {
        Intent intent = new Intent(activity, MainFeedBackActivity.class);
        activity.startActivity(intent);
    }

    private void loadFeedbackWeb(String url) {
        mWebView.loadUrl(url);
        mWebView.getSettings().setJavaScriptEnabled(true);
        mWebView.getSettings().setDomStorageEnabled(true);
        mWebView.getSettings().setUseWideViewPort(true);
        mWebView.getSettings().setLoadWithOverviewMode(true);
        mWebView.getSettings().setSupportZoom(false);
        mWebView.setWebViewClient(new WebViewClient() {
            @Override
            public boolean shouldOverrideUrlLoading(WebView view, String url) {
                view.loadUrl(url);
                return true;
            }
        });
        mWebView.addJavascriptInterface(new CallAndroid(), "feedback");
    }

    class CallAndroid {
        @JavascriptInterface
        public void callback(int result) {
            Log.i(TAG, "提交反馈结果：" + result);
        }

        @JavascriptInterface
        public void uploadLog() {
            Log.i(TAG, "开始上传日志");
            mZegoExpressEngine.uploadLog();
        }

    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mZegoExpressEngine != null) {
            ZegoExpressEngine.destroyEngine(null);
        }
    }
}
