package im.zego.goenjoy;

import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;

import im.zego.common.proxy.LoginManagerProxy;
import im.zego.common.util.MiscUtils;
import im.zego.common.util.ZegoUtil;
import im.zego.zegoexpress.ZegoExpressEngine;


public class MainSettingActivity extends AppCompatActivity {
    private static final String TAG = "MainSettingActivity";

    private TextView mTitleText;
    private TextView appVersion;
    private TextView sdkVersion;

    private ImageView leftIcon;
    private ConstraintLayout mClSettingActivity;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.main_setting_activity);
        initWindowBar();
        mTitleText = findViewById(R.id.title_text);
        appVersion = findViewById(R.id.app_version);
        sdkVersion = findViewById(R.id.SDK_version);
        leftIcon = findViewById(R.id.left_icon);
        mClSettingActivity = findViewById(R.id.cl_setting_activity);

        ImageView imageView = LoginManagerProxy.getInstance().getAdvertiseImageView(MainSettingActivity.this);
        ConstraintLayout.LayoutParams layoutParams = new ConstraintLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        int margin = MiscUtils.dp2px(this, 15);
        layoutParams.setMargins(margin,margin,margin,margin);
        layoutParams.leftToLeft = ConstraintLayout.LayoutParams.PARENT_ID;
        layoutParams.rightToRight = ConstraintLayout.LayoutParams.PARENT_ID;
        layoutParams.topToBottom = R.id.logout_account;
        mClSettingActivity.addView(imageView,layoutParams);
        imageView.setVisibility(View.INVISIBLE);

        try {
            appVersion.setText(String.format("APP 版本：%s", getPackageManager().getPackageInfo(getPackageName(), 0).versionName));
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        sdkVersion.setText(String.format("SDK 版本：%s", ZegoExpressEngine.getVersion()));
        leftIcon.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });
        mTitleText.setText("设置");
        mTitleText.setVisibility(View.VISIBLE);
    }

    public void UploadLog(View view) {
        MainFeedBackActivity.actionActivity(MainSettingActivity.this);
    }

    public void ShareLog(View view) {
        ZegoUtil.sendLogs(this);
    }

    /**
     * 关于我们
     * @param view
     */
    public void AboutUs(View view) {
        //更改为内嵌的webView访问
        LoginManagerProxy.getInstance().startWebViewActivity(MainSettingActivity.this,getString(R.string.about_us_url),"关于我们");
    }

    /**
     * 增添注销和退出，展示一个Dialog以示缓冲操作
     * 采用LoginLib中通用的Dialog
     */
    public void CancelAccount(View view) {
        showCancelAccountDialog();
    }

    private void showCancelAccountDialog() {
        LoginManagerProxy.getInstance().showCancelAccountDialog(MainSettingActivity.this);
    }

    //退出登录
    public void LogoutAccount(View view) {
        showLogoutAccountDialog();
    }

    private void showLogoutAccountDialog() {
        LoginManagerProxy.getInstance().showLogoutAccountDialog(MainSettingActivity.this);
    }


    /**
     * 隐私政策
     * @param view
     */
    public void ViewPrivacyProtocol(View view) {
        //更改为内嵌的webView访问
        LoginManagerProxy.getInstance().startWebViewActivity(MainSettingActivity.this,getString(R.string.privacy_policy_url),"隐私政策");
    }

    /**
     * 用户协议
     * @param view
     */
    public void ViewUserProtocol(View view) {
        //更改为内嵌的webView访问
        LoginManagerProxy.getInstance().startWebViewActivity(MainSettingActivity.this,getString(R.string.user_protocol_url),"用户协议");
    }

    private void initWindowBar() {
        WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.flags = lp.flags | WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS;
        getWindow().setAttributes(lp);
    }

}