package im.zego.common.application;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.util.Locale;

import im.zego.common.proxy.LoginManagerProxy;
import im.zego.common.util.BulyConfig;
import im.zego.common.util.BulyInit;
import im.zego.common.util.SharedPreferencesUtil;
import im.zego.common.ytee.logutil.LogcatFileManager;

/**
 * Created by zego on 2018/10/16.
 */

public class ZegoApplication extends Application {

    private static final String BUGLY_APP_ID = "";

    private static int activityCount = 0;
    private static ZegoApplication mZegoApplication;
    private static Context appContext = null;

    public static ZegoApplication getApplication() {
        return mZegoApplication;
    }

    public static Context getAppContext() {
        return appContext;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        mZegoApplication = this;
        appContext = getApplicationContext();
        init();
        initBugly();

        try {
            Class<?> mainClass = Class.forName("im.zego.goenjoy.MainActivity");
            Intent mainIntent = new Intent(this, mainClass);

            LoginManagerProxy.getInstance().initIntent(mainIntent).setLoginConfig(this);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    private void init() {
        LogcatFileManager.getInstance().startLogcatManager(this);
        SharedPreferencesUtil.init(ZegoApplication.getAppContext());

        registerActivityLifecycleCallbacks(new ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(@NonNull Activity activity, @Nullable Bundle savedInstanceState) {

            }

            @Override
            public void onActivityStarted(@NonNull Activity activity) {
                activityCount++;
            }

            @Override
            public void onActivityResumed(@NonNull Activity activity) {

            }

            @Override
            public void onActivityPaused(@NonNull Activity activity) {

            }

            @Override
            public void onActivityStopped(@NonNull Activity activity) {
                activityCount--;
            }

            @Override
            public void onActivitySaveInstanceState(@NonNull Activity activity, @NonNull Bundle outState) {

            }

            @Override
            public void onActivityDestroyed(@NonNull Activity activity) {

            }
        });
    }

    public static boolean isOnBackground() {
        return activityCount == 0;
    }



    /**
     * bugly 初始化
     */
    private void initBugly() {

        boolean debug = false;//BuildConfig.DEBUG;
        BulyConfig bulyConfig = new BulyConfig().setApplication(this)
                .setDebug(debug)
                .setAppId(BUGLY_APP_ID)
                .setOnlyMainProcess(true);
        BulyInit.init(bulyConfig);

    }

}
