package im.zego.common.util;

import android.app.Application;

public class BulyConfig {

    private Application application;
    /**
     * bugly中应用申请的app Id
     */
    private String appId;

    /**
     * SDK调试模式开关
     * <p>
     * 调试模式的行为特性如下：
     * <p>
     * 输出详细的Bugly SDK的Log；
     * 每一条Crash都会被立即上报；
     * 自定义日志将会在Logcat中输出
     * <p>
     * 建议在测试阶段建议设置成true，发布时设置为false
     */
    private boolean debug;

    /**
     * 是否只在主进程初始化
     * <p>
     * 如果App使用了多进程且各个进程都会初始化Bugly（例如在Application类onCreate()中初始化Bugly）
     * ，那么每个进程下的Bugly都会进行数据上报，造成不必要的资源浪费。
     * 因此，为了节省流量、内存等资源，建议初始化的时候对上报进程进行控制，只在主进程下上报数据：
     * 判断是否是主进程（通过进程名是否为包名来判断），并在初始化Bugly时增加一个上报进程的策略配置。
     */
    private boolean onlyMainProcess;


    public Application getApplication() {
        return application;
    }

    public BulyConfig setApplication(Application application) {
        this.application = application;
        return this;
    }

    public String getAppId() {
        return appId;
    }

    public BulyConfig setAppId(String appId) {
        this.appId = appId;
        return this;
    }

    public boolean isDebug() {
        return debug;
    }

    public BulyConfig setDebug(boolean debug) {
        this.debug = debug;
        return this;
    }

    public boolean isOnlyMainProcess() {
        return onlyMainProcess;
    }

    public BulyConfig setOnlyMainProcess(boolean onlyMainProcess) {
        this.onlyMainProcess = onlyMainProcess;
        return this;
    }

}
