package im.zego.common.util;

import android.content.Context;
import android.text.TextUtils;

import com.tencent.bugly.crashreport.CrashReport;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

/**
 * * 参考文档：
 *  * 1. Bugly符号表插件使用指南: https://bugly.qq.com/docs/utility-tools/plugin-gradle-bugly/?v=20200622202242
 *  * 2. Bugly Android SDK 使用指南 :https://bugly.qq.com/docs/user-guide/instruction-manual-android/?v=20200622202242
 */
public class BulyInit {

    /**
     * 开始初始化
     *
     * @param config
     */
    public static void init(BulyConfig config) {
        Context context = config.getApplication();
        String appId = config.getAppId();
        boolean debug = config.isDebug();
        CrashReport.UserStrategy strategy = config.isOnlyMainProcess() ? Utils.createMainProcessStrategy(context) : null;
        CrashReport.initCrashReport(context, appId, debug, strategy);
    }

    private final static class Utils {
        /**
         * 创建主进程上报的策略
         *
         * @param c
         * @return
         */
        private static CrashReport.UserStrategy createMainProcessStrategy(Context c) {
            Context context = c.getApplicationContext();
            // 获取当前包名
            String packageName = context.getPackageName();
            // 获取当前进程名
            String processName = getProcessName(android.os.Process.myPid());
            // 设置是否为上报进程
            CrashReport.UserStrategy strategy = new CrashReport.UserStrategy(context);
            strategy.setUploadProcess(processName == null || processName.equals(packageName));
            return strategy;
        }

        /**
         * 获取进程号对应的进程名
         *
         * @param pid 进程号
         * @return 进程名
         */
        private static String getProcessName(int pid) {
            BufferedReader reader = null;
            try {
                reader = new BufferedReader(new FileReader("/proc/" + pid + "/cmdline"));
                String processName = reader.readLine();
                if (!TextUtils.isEmpty(processName)) {
                    processName = processName.trim();
                }
                return processName;
            } catch (Throwable throwable) {
                throwable.printStackTrace();
            } finally {
                try {
                    if (reader != null) {
                        reader.close();
                    }
                } catch (IOException exception) {
                    exception.printStackTrace();
                }
            }
            return null;
        }
    }


}
