package im.zego.common.util;

import android.content.Context;
import android.content.SharedPreferences;

import im.zego.common.model.LoginInfo;

public class SharedPreferencesUtil {

    public static final String USER_ID = "uid";
    public static final String NICK_NAME = "nickName";
    public static final String AVATAR = "avatar";

    private static final String DEFAULT_SP = "im.zego.GoEnjoy";
    public static final String KEY_SHOW_PRIVACY_PAGE = "GOENJOY_PRIVACY";


    private static Context mApplication;

    public static void init(Context context)
    {
        mApplication = context;
    }


    private static SharedPreferences getSharedPreferences(String spFileName) {
        return mApplication.getSharedPreferences(spFileName, Context.MODE_PRIVATE);
    }

    public static void cacheLoginInfo(LoginInfo loginInfo) {
        if (loginInfo == null) {
            return;
        }
        SharedPreferences sharedPreferences = getSharedPreferences(DEFAULT_SP);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putLong(USER_ID, loginInfo.getUid());
        editor.putString(NICK_NAME, loginInfo.getNickName());
        editor.putString(AVATAR, loginInfo.getAvatar());
        editor.apply();
    }

    public static LoginInfo getLoginInfo() {
        SharedPreferences sharedPreferences = getSharedPreferences(DEFAULT_SP);

        long defUserId = 0L;
        long userId = sharedPreferences.getLong(USER_ID, defUserId);
        String nickName = sharedPreferences.getString(NICK_NAME, "");
        String avatar = sharedPreferences.getString(AVATAR, "");
        LoginInfo loginInfo = new LoginInfo();
        loginInfo.setUid(userId);
        loginInfo.setNickName(nickName);
        loginInfo.setAvatar(avatar);
        return loginInfo;
    }

    public static void clearLoginInfo()
    {
        SharedPreferences sharedPreferences = getSharedPreferences(DEFAULT_SP);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.remove(USER_ID);
        editor.remove(NICK_NAME);
        editor.remove(AVATAR);
        editor.apply();
    }

    public static void setFirstShowAudioPermission(boolean isFirst) {
        SharedPreferences sharedPreferences = getSharedPreferences(DEFAULT_SP);
        sharedPreferences.edit().putBoolean("first_show_audio_permission", isFirst).apply();
    }

    public static boolean isFirstShowAudioPermission() {
        SharedPreferences sharedPreferences = getSharedPreferences(DEFAULT_SP);
        return sharedPreferences.getBoolean("first_show_audio_permission", true);
    }

    public static Boolean getBooleanValue(String keyShowPrivacyPage, boolean b) {
        SharedPreferences sharedPreferences = getSharedPreferences(DEFAULT_SP);
        return sharedPreferences.getBoolean(keyShowPrivacyPage, b);
    }

    public static void setBooleanValue(String keyShowPrivacyPage, boolean b) {
        SharedPreferences sharedPreferences = getSharedPreferences(DEFAULT_SP);
        SharedPreferences.Editor  editor = sharedPreferences.edit();
        editor.putBoolean(keyShowPrivacyPage, b);
        editor.commit();
    }


}
