package im.zego.common.util;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.text.TextUtils;
import android.util.Log;


import androidx.core.content.FileProvider;

import java.io.File;
import java.io.FilenameFilter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Pattern;

import im.zego.common.application.ZegoApplication;
import im.zego.zegoexpress.utils.ZegoLogUtil;


/**
 * Created by zego on 2019/4/16.
 */

public class ZegoUtil {

    public static void sendLogs(Activity activity) {
        String rootPath = ZegoLogUtil.getLogPath(activity);
        File rootDir = new File(rootPath + File.separator + "zegologs");
        File[] logFiles = rootDir.listFiles(new FilenameFilter() {
            @Override
            public boolean accept(File dir, String name) {
                return !TextUtils.isEmpty(name) && ((name.startsWith("zegoavlog") && name.endsWith(".txt"))
                        || (name.startsWith("logcat") && name.endsWith(".log")));
            }
        });

        if (logFiles.length <= 0) {
            Log.w("ZegoUtil", "not found any log files.");
            return;
        }
        File cacheDir = ZegoApplication.getAppContext().getExternalCacheDir();
        if (cacheDir == null || !cacheDir.canWrite()) {
            cacheDir = ZegoApplication.getAppContext().getCacheDir();
        }

        File[] oldLogCaches = cacheDir.listFiles(new FilenameFilter() {
            @Override
            public boolean accept(File dir, String name) {
                return name.startsWith("zegoavlog") && name.endsWith(".zip");
            }
        });

        for (File cache : oldLogCaches) {
            cache.delete();
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd-HHmmss");
        String zipFileName = String.format("zegoavlog_%s.zip", sdf.format(new Date()));
        File zipFile = new File(cacheDir, zipFileName);

        try {
            ZipUtil.zipFiles(logFiles, zipFile, "goenjoy 日志信息");

            Intent shareIntent = new Intent(Intent.ACTION_SEND);
            Uri uri;

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                uri = FileProvider.getUriForFile(ZegoApplication.getAppContext(), "im.zego.goenjoy.fileProvider", zipFile);
            } else {
                uri = Uri.fromFile(zipFile);
            }

            shareIntent.putExtra(Intent.EXTRA_STREAM, uri);
            shareIntent.setType("application/zip");

            shareIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK
                    | Intent.FLAG_ACTIVITY_NEW_TASK
                    | Intent.FLAG_GRANT_READ_URI_PERMISSION
            );
            activity.startActivity(shareIntent);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
