package im.zego.gochat.chatroom;

import android.content.DialogInterface;
import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Point;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;
import androidx.recyclerview.widget.RecyclerView;

import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.List;

import im.zego.common.proxy.QualityManagerProxy;
import im.zego.common.util.SharedPreferencesUtil;
import im.zego.gochat.MainActivity;
import im.zego.gochat.R;
import im.zego.gochat.login.LoginInfoManager;
import im.zego.gochat.base.ZGBaseActivity;
import im.zego.gochat.chatroom.adapter.ChatRoomAudienceAdapter;
import im.zego.gochat.chatroom.adapter.ChatRoomStageGuestAdapter;
import im.zego.gochat.chatroom.adapter.IAudienceDataChangeListener;
import im.zego.gochat.chatroom.adapter.IAvatarClickListener;
import im.zego.gochat.custom.ProcessView;
import im.zego.gochat.custom.RaiseHandListPopWindow;
import im.zego.gochat.custom.RoomBottomView;
import im.zego.gochat.custom.ScrollGridLayoutManager;
import im.zego.gochat.custom.ZegoPopupWindow;
import im.zego.gochat.model.NotifyInvitedInfo;
import im.zego.gochat.model.NotifyRefuseInviteInfo;
import im.zego.gochat.model.NotifyUserListUpdateInfo;
import im.zego.gochat.model.NotifyUserStateUpdateInfo;
import im.zego.gochat.model.RoomInfo;
import im.zego.gochat.model.UserInfo;
import im.zego.gochat.protocol.ILoginRoomCallback;
import im.zego.gochat.protocol.RoomAPI;
import im.zego.gochat.rtc.RTCSDKManager;
import im.zego.gochat.permission.AppSettingsDialogHolderActivity;
import im.zego.gochat.permission.PermissionUtils;
import im.zego.gochat.rtc.callbacks.IRTCCommonCallback;
import im.zego.gochat.rtc.callbacks.IRTCEventCallback;
import im.zego.gochat.utils.ToastUtil;
import im.zego.gochat.utils.MiscUtils;

import static im.zego.gochat.ErrorcodeConstants.ERROR_ALREADY_IN_ROOM;
import static im.zego.gochat.ErrorcodeConstants.ERROR_ROOM_NOT_EXIST;
import static im.zego.gochat.ErrorcodeConstants.ERROR_USER_OFF_LINE;

/**
 * 语聊房间activity
 */
public class ChatRoomActivity extends ZGBaseActivity implements View.OnClickListener, RoomBottomView.IRoomBottomViewCallBack, IAvatarClickListener, IRTCEventCallback, IAudienceDataChangeListener, IRaiseHandNumChangeNotify {
    private final static String TAG = "ChatRoomActivity";

    private ImageView rightIcon;
    private TextView centerText;
    private RecyclerView guestRecyclerView;
    private RecyclerView audienceRecyclerView;
    private ChatRoomStageGuestAdapter stateGuestAdapter;
    private ChatRoomAudienceAdapter audienceAdapter;
    private RaiseHandListPopWindow raiseHandListPopWindow;   //举手列表
    private RoomBottomView bottomView;                       //底部栏
    private TextView roomBottomTitle;
    private ProcessView processView;
    private long myUid = LoginInfoManager.getInstance().getLoginInfo().getUid();
    private final static int PERMISSION_REQUEST_AUDIO_COMMON = 0X100;
    private final static int PERMISSION_REQUEST_AUDIO_RAISE_HAND = 0X101;
    private final static int REQUEST_CODE_FOR_MIC = 0X200;
    private View mPopupWindowShadow;               //popwindow遮罩
    private ZegoPopupWindow onstagePopwindow;

    //邀请弹框
    private AlertDialog.Builder inviteDialogBuilder;
    private AlertDialog inviteDialog;


    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR);
        }

        setContentView(R.layout.activity_chat_room);

        if (!LoginInfoManager.getInstance().isLogin()) {
            MainActivity.restartLogin();
        } else {
            myUid = SharedPreferencesUtil.getLoginInfo().getUid();
        }

        //房间信息缺少时，销毁页面
        if (ChatRoomManager.getInstance().getRoomInfo() == null) {
            finish();
            return;
        }

        initView();
        setStatusBar();
        pullUserList();

        startChatRoomService();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        stopChatRoomService();
        RTCSDKManager.getInstance().removeListener(this);
        ChatRoomManager.getInstance().setOnRaiseHandNumChangeNotify(null);
    }

    /**
     * 设置状态栏颜色
     */
    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    private void setStatusBar() {
        Window window = getWindow();
        //取消设置透明状态栏,使 ContentView 内容不再覆盖状态栏
        window.clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
        //需要设置这个 flag 才能调用 setStatusBarColor 来设置状态栏颜色
        window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
        window.setStatusBarColor(getResources().getColor(R.color.room_bg));
    }

    private void initView() {
        rightIcon = findViewById(R.id.right_icon);
        centerText = findViewById(R.id.title_text);
        if (ChatRoomManager.getInstance().getRoomInfo() != null) {
            centerText.setText(ChatRoomManager.getInstance().getRoomInfo().getSubject());
        } else {
            centerText.setText(R.string.go_chat_app_name);
        }
        rightIcon.setImageResource(R.mipmap.nav_end);
        rightIcon.setOnClickListener(this);
        guestRecyclerView = findViewById(R.id.rv_guest_list);
        mPopupWindowShadow = findViewById(R.id.popup_window_shadow);
        stateGuestAdapter = new ChatRoomStageGuestAdapter(this);
        ScrollGridLayoutManager manager = new ScrollGridLayoutManager(this, 3);
        manager.setScrollEnabled(false);
        guestRecyclerView.setLayoutManager(manager);
        guestRecyclerView.setAdapter(stateGuestAdapter);
        audienceRecyclerView = findViewById(R.id.rv_audience_list);
        audienceAdapter = new ChatRoomAudienceAdapter(this);
        ScrollGridLayoutManager audienceManager = new ScrollGridLayoutManager(this, 4);
        audienceManager.setScrollEnabled(false);
        audienceRecyclerView.setLayoutManager(audienceManager);
        audienceRecyclerView.setAdapter(audienceAdapter);
        raiseHandListPopWindow = new RaiseHandListPopWindow(this);
        bottomView = findViewById(R.id.bottom_toolbar);
        bottomView.setCallBack(this);
        processView = new ProcessView(ChatRoomActivity.this);
        roomBottomTitle = findViewById(R.id.room_bottom_title);
        audienceAdapter.setIAudienceDataChangeListener(this);
    }

    /**
     * 拉取成员列表
     */
    private void pullUserList() {
        ChatRoomManager.getInstance().getUserList(myUid, new IExecuteCallback() {
            @Override
            public void onSuccess() {
                //拉取成功，可以通过ChatManager拿到所有数据
                onRoomInitSuccess();
            }

            @Override
            public void onFailed(int errorCode) {

            }
        });
    }

    /**
     * 房间初始化成功，加载成员列表成功，可以通过ChatRoomManager拿到用户数据
     */
    private void onRoomInitSuccess() {
        audienceAdapter.initUserList();
        stateGuestAdapter.initUserList();
        if (ChatRoomManager.getInstance().getMyUserInfo().isHost()) {
            rightIcon.setVisibility(View.VISIBLE);
        }

        UserInfo userInfo = ChatRoomManager.getInstance().getMyUserInfo();
        if (userInfo != null) {
            myUid = userInfo.getUserID();
            bottomView.setBottomView(userInfo.getRole(), userInfo.isMicOpen());
        }

        if (userInfo.isHost() || userInfo.isStageGuest()) {
            if (PermissionUtils.hasPermission(this, Manifest.permission.RECORD_AUDIO)) {
                RTCSDKManager.getInstance().startPublishAudio(new IRTCCommonCallback() {
                    @Override
                    public void onRTCCallback(int errorCode) {
                        Log.d(TAG, "rtc startPublishAudio errorCode:" + errorCode);
                    }
                });
            } else {
                requestAudioPermission(PERMISSION_REQUEST_AUDIO_COMMON);
            }
        }

        if (userInfo.isHost()) {
            ChatRoomManager.getInstance().setOnRaiseHandNumChangeNotify(this);
        }

        RTCSDKManager.getInstance().addListener(this);
    }

    /**
     * 拉取举手成员列表
     */
    private void pullRaiseHandList() {
        ChatRoomManager.getInstance().getRaiseHandList(myUid, 1, 5000, new IExecuteCallback() {
            @Override
            public void onSuccess() {
                //拉取成功，可以通过ChatManager拿到所有数据
                List<UserInfo> list = ChatRoomManager.getInstance().getRaiseHandList();
                if (list != null) {
                    raiseHandListPopWindow.initRaiseHandList();
                }
            }

            @Override
            public void onFailed(int errorCode) {

            }
        });
    }

    private void requestAudioPermission(int code) {
        if (SharedPreferencesUtil.isFirstShowAudioPermission()) {
            SharedPreferencesUtil.setFirstShowAudioPermission(false);
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.RECORD_AUDIO},
                    code);
        } else {
            startSettingMic();
        }
    }

    private void startSettingMic() {
        AppSettingsDialogHolderActivity.startMeetingStyleSettingDialog(this,
                this.getString(R.string.microphone_privacy_title),
                this.getString(R.string.microphone_privacy_msg),
                this.getString(R.string.setting),
                this.getString(R.string.cancel), REQUEST_CODE_FOR_MIC);
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        UserInfo myUserInfo = ChatRoomManager.getInstance().getMyUserInfo();
        if (requestCode == PERMISSION_REQUEST_AUDIO_COMMON) {
            //根据用户状态变更来推流
            ChatRoomManager.getInstance().setUserInfo(myUid, myUid, grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED, myUserInfo.getOnstageState(), 1, new IExecuteCallback() {
                @Override
                public void onSuccess() {

                }

                @Override
                public void onFailed(int errorCode) {

                }
            });
            bottomView.setBottomView(myUserInfo.getRole(), grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED);
            myUserInfo.setMicOpen(grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED);
            stateGuestAdapter.onUserMicStatusChanged(myUserInfo);
        } else if (requestCode == PERMISSION_REQUEST_AUDIO_RAISE_HAND) {
            if ((grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED)) {
                raiseHand();
            } else {
                ToastUtil.showWarnToast(getString(R.string.raise_hand_tip_fail));
            }
        }
    }

    @Override
    public void onClick(View v) {
        if (v.getId() == R.id.right_icon) {//点击结束房间
            AlertDialog.Builder builder = new AlertDialog.Builder(ChatRoomActivity.this)
                    .setTitle(R.string.end_title)
                    .setMessage(R.string.end_host_msg)
                    .setNeutralButton(R.string.cancel, new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {

                        }
                    })
                    .setPositiveButton(R.string.confirm, new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            processView.setIsShowShade(false).setCancelable(false).show((ViewGroup) rightIcon.getRootView());
                            ChatRoomManager.getInstance().closeRoom(myUid, new IExecuteCallback() {
                                @Override
                                public void onSuccess() {
                                    processView.dismiss();
                                    ChatRoomManager.getInstance().logoutRoom();
                                    startActivity(QualityManagerProxy.getQualityIntent(ChatRoomActivity.this));
                                    finish();
                                }

                                @Override
                                public void onFailed(int errorCode) {
                                    processView.dismiss();
                                }
                            });
                        }
                    });
            builder.show();
        }
    }

    /**
     * 离开房间
     */
    @Override
    public void onClickLeaveRoom() {
        UserInfo userInfo = ChatRoomManager.getInstance().getMyUserInfo();
        if (userInfo == null) {
            ChatRoomManager.getInstance().logoutRoom();
            startActivity(QualityManagerProxy.getQualityIntent(ChatRoomActivity.this));
            finish();
        } else if (userInfo.isAudience()) {
            // 听众
            ChatRoomManager.getInstance().logoutRoom();
            startActivity(QualityManagerProxy.getQualityIntent(ChatRoomActivity.this));
            finish();
        } else if (userInfo.isStageGuest()) {
            // 台上嘉宾
            ChatRoomManager.getInstance().logoutRoom();
            startActivity(QualityManagerProxy.getQualityIntent(ChatRoomActivity.this));
            finish();
        } else if (userInfo.isHost()) {

            if (stateGuestAdapter.getItemCount() <= 1) {
                AlertDialog.Builder builder = new AlertDialog.Builder(ChatRoomActivity.this)
                        .setTitle(R.string.leave_title)
                        .setMessage(R.string.leave_end_msg)
                        .setNeutralButton(R.string.cancel, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                Log.d(TAG, "you have cancelled leave room, because room will be closed.");
                            }
                        })
                        .setPositiveButton(R.string.confirm, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                Log.d(TAG, "you have agreed leave room, and room will be closed.");
                                ChatRoomManager.getInstance().logoutRoom();
                                startActivity(QualityManagerProxy.getQualityIntent(ChatRoomActivity.this));
                                finish();
                            }
                        });
                builder.show();
            } else {
                AlertDialog.Builder builder = new AlertDialog.Builder(ChatRoomActivity.this)
                        .setTitle(R.string.leave_title)
                        .setMessage(R.string.leave_designate_msg)
                        .setNeutralButton(R.string.cancel, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                Log.d(TAG, "you have cancelled leave room, because host will be changed.");
                            }
                        })
                        .setPositiveButton(R.string.confirm, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                Log.d(TAG, "you have agreed leave room, and host will be changed.");
                                ChatRoomManager.getInstance().logoutRoom();
                                startActivity(QualityManagerProxy.getQualityIntent(ChatRoomActivity.this));
                                finish();
                            }
                        });
                builder.show();
            }
        } else {
            ChatRoomManager.getInstance().logoutRoom();
            startActivity(QualityManagerProxy.getQualityIntent(ChatRoomActivity.this));
            finish();
        }
    }

    @Override
    public void onClickRaiseHandList() {
        pullRaiseHandList();
        raiseHandListPopWindow.showAtLocation(bottomView, Gravity.BOTTOM | Gravity.CENTER_HORIZONTAL, 0, 0);
    }

    @Override
    public void onClickMic() {
        UserInfo myUserInfo = ChatRoomManager.getInstance().getMyUserInfo();
        if (PermissionUtils.hasPermission(this, Manifest.permission.RECORD_AUDIO)) {
            ChatRoomManager.getInstance().setUserInfo(myUid, myUid, !myUserInfo.isMicOpen(), myUserInfo.getOnstageState(), 1, new IExecuteCallback() {
                @Override
                public void onSuccess() {

                }

                @Override
                public void onFailed(int errorCode) {

                }
            });
        } else {
            requestAudioPermission(PERMISSION_REQUEST_AUDIO_COMMON);
        }

    }

    @Override
    public void onClickRaiseHand() {
        //举手
        if (PermissionUtils.hasPermission(this, Manifest.permission.RECORD_AUDIO)) {
            //有音频权限
            raiseHand();
        } else {
            requestAudioPermission(PERMISSION_REQUEST_AUDIO_RAISE_HAND);
        }
    }

    /**
     * 举手
     */
    private void raiseHand() {
        UserInfo myUserInfo = ChatRoomManager.getInstance().getMyUserInfo();
        Log.d(TAG, "raiseHand:myUserInfo.getRaiseHand() :" + myUserInfo.isRaisedHand());
        processView.setIsShowShade(false).setCancelable(false).show((ViewGroup) rightIcon.getRootView());
        ChatRoomManager.getInstance().raiseHand(myUid, !myUserInfo.isRaisedHand(), new IExecuteCallback() {
            @Override
            public void onSuccess() {
                //成功则修改本地的举手状态
                processView.dismiss();
                bottomView.updateRaiseHandStatus(!myUserInfo.isRaisedHand());
                ChatRoomManager.getInstance().getMyUserInfo().setRaiseHandStatus(!myUserInfo.isRaisedHand());
            }

            @Override
            public void onFailed(int errorCode) {
                processView.dismiss();
            }
        });
    }

    private void startChatRoomService() {
        Intent intent = new Intent(this, ChatRoomService.class);
        if (Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            startForegroundService(intent);
        } else {
            startService(intent);
        }
    }

    private void stopChatRoomService() {
        Intent intent = new Intent(this, ChatRoomService.class);
        stopService(intent);
    }

    private void showRoomCloseAlertDialog() {
        if (!isFinishing()) {
            AlertDialog.Builder builder = new AlertDialog.Builder(ChatRoomActivity.this)
                    .setTitle(R.string.end_title)
                    .setMessage(R.string.end_audience_msg)
                    .setNeutralButton(R.string.confirm, new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            ChatRoomManager.getInstance().logoutRoom();
                            startActivity(QualityManagerProxy.getQualityIntent(ChatRoomActivity.this));
                            finish();
                        }
                    });
            builder.show().setCanceledOnTouchOutside(false);
        }
    }

    @Override
    public void onBackPressed() {
        onClickLeaveRoom();
    }

    @Override
    public void clickAudience(UserInfo targetUserInfo) {
        Point point = new Point();
        getWindowManager().getDefaultDisplay().getSize(point);
        int width = point.x - MiscUtils.dp2px(this, 28);
        onstagePopwindow = new ZegoPopupWindow(this, R.layout.popup_menu_layout,
                width,
                WindowManager.LayoutParams.WRAP_CONTENT,
                MiscUtils.dp2px(this, 49));

        // 邀请上台
        onstagePopwindow.addCommand(getResources().getString(R.string.raise_hand_invite_button), v -> {
            //点击邀请听众上台
            ChatRoomManager.getInstance().inviteOnstage(myUid, targetUserInfo.getUserID(), new IExecuteCallback() {
                @Override
                public void onSuccess() {
                    ToastUtil.showNormalToast(R.string.invite_host_tip_invited);
                }

                @Override
                public void onFailed(int errorCode) {

                }
            });
        });

        mPopupWindowShadow.setVisibility(View.VISIBLE);
        onstagePopwindow.setOnDismissListener(() -> mPopupWindowShadow.setVisibility(View.GONE));
        onstagePopwindow.showAtLocation(bottomView, Gravity.BOTTOM, 0, MiscUtils.dp2px(this, 30));
    }

    @Override
    public void clickStageGuest(UserInfo targetUserInfo) {
        Point point = new Point();
        getWindowManager().getDefaultDisplay().getSize(point);
        int width = point.x - MiscUtils.dp2px(this, 28);
        onstagePopwindow = new ZegoPopupWindow(this, R.layout.popup_menu_layout,
                width,
                WindowManager.LayoutParams.WRAP_CONTENT,
                MiscUtils.dp2px(this, 49));

        // 设置为听众
        onstagePopwindow.addCommand(getResources().getString(R.string.manage_set_as_audience), v -> {
            ChatRoomManager.getInstance().setUserInfo(myUid, targetUserInfo.getUserID(),
                    false, 1, 2, new IExecuteCallback() {
                        @Override
                        public void onSuccess() {

                        }

                        @Override
                        public void onFailed(int errorCode) {

                        }
                    });
        });
        onstagePopwindow.addDivider();
        // 关闭麦克风
        onstagePopwindow.addCommand(getString(R.string.manage_turn_off_microphone), v -> {
            //关闭上台人员麦克风
            ChatRoomManager.getInstance().setUserInfo(myUid, targetUserInfo.getUserID(), false, targetUserInfo.getOnstageState(), 1, new IExecuteCallback() {
                @Override
                public void onSuccess() {

                }

                @Override
                public void onFailed(int errorCode) {

                }
            });
        });

        mPopupWindowShadow.setVisibility(View.VISIBLE);
        onstagePopwindow.setOnDismissListener(() -> mPopupWindowShadow.setVisibility(View.GONE));
        onstagePopwindow.showAtLocation(bottomView, Gravity.BOTTOM, 0, MiscUtils.dp2px(this, 30));
    }

    @Override
    public void clickSelf(UserInfo userInfo) {
        if (userInfo.isAudience()) {
            return;
        }
        Point point = new Point();
        getWindowManager().getDefaultDisplay().getSize(point);
        int width = point.x - MiscUtils.dp2px(this, 28);
        onstagePopwindow = new ZegoPopupWindow(this, R.layout.popup_menu_layout,
                width,
                WindowManager.LayoutParams.WRAP_CONTENT,
                MiscUtils.dp2px(this, 49));
        // 下台
        onstagePopwindow.addCommand(getString(R.string.down_stage_button), new ZegoPopupWindow.ICommandListener() {
            @Override
            public void onClickListener(View v) {
                //自己主动下台
                //关闭上台人员麦克风
                ChatRoomManager.getInstance().setUserInfo(myUid, userInfo.getUserID(), false, 1, 2, new IExecuteCallback() {
                    @Override
                    public void onSuccess() {
                        ToastUtil.showNormalToast(R.string.down_stage_tip);
                    }

                    @Override
                    public void onFailed(int errorCode) {

                    }
                });

            }
        });
        mPopupWindowShadow.setVisibility(View.VISIBLE);
        onstagePopwindow.setOnDismissListener(() -> mPopupWindowShadow.setVisibility(View.GONE));
        onstagePopwindow.showAtLocation(bottomView, Gravity.BOTTOM, 0, MiscUtils.dp2px(this, 30));
    }

    @Override
    public void onUserListUpdateNotify(@NotNull NotifyUserListUpdateInfo notifyInfo) {
        for (NotifyUserListUpdateInfo.UserUpdateInfo updateInfo : notifyInfo.users) {
            if (updateInfo.delta == 1) {
                audienceAdapter.onUserIn(updateInfo.toUserInfo(updateInfo));
            } else if (updateInfo.delta == -1) {
                if (updateInfo.isAudience()) {
                    audienceAdapter.onUserOut(updateInfo.uid);
                } else if (updateInfo.isHost() || updateInfo.isStageGuest()) {
                    stateGuestAdapter.onUserOffStage(updateInfo.uid);
                }
            }
        }
    }

    @Override
    public void onUserStateUpdateNotify(@NotNull NotifyUserStateUpdateInfo notifyInfo) {
        if (notifyInfo.type == 1) {
            Log.d(TAG, "举手变更通知");
            // TODO: 2/16/21 举手操作，更新举手人数
//            if (notifyInfo.users != null) {
//                int size = notifyInfo.users.size();
//                for (int i = 0; i < size; i++) {
//                    if (notifyInfo.users.get(i).isRaisedHand()) {
//
//                    } else {
//
//                    }
//
//                }
//            }
            /*if (raiseHandCountTextView.getVisibility() != View.VISIBLE) {
                return;
            }
            // 举手操作，更新举手人数
            int alreadyRaiseCount = ChatRoomManager.getInstance().getRaiseHandList().size();
            int addedRaiseCount = notifyInfo.users.size();
            raiseHandCountTextView.setText(String.valueOf(alreadyRaiseCount + addedRaiseCount));*/
        } else if (notifyInfo.type == 2) {
            // 角色变更通知
            for (NotifyUserStateUpdateInfo.UserStateInfo userStateInfo : notifyInfo.users) {
                UserInfo notifyUserInfo = userStateInfo.toUserInfo();
                Log.d(TAG, String.format("notifyType:%d user:%d nickName:%s newRole:%d newMic:%b", notifyInfo.type, notifyUserInfo.getUserID(), notifyUserInfo.getNickName(), notifyUserInfo.getRole(), notifyUserInfo.isMicOpen()));

                UserInfo myUserInfo = ChatRoomManager.getInstance().getMyUserInfo();
                if (myUserInfo != null && myUserInfo.getUserID() == userStateInfo.getUserID()) {
                    // myUserInfo的role已有manager变更，这里只负责界面相关展示
                    bottomView.setBottomView(notifyUserInfo.getRole(), notifyUserInfo.isMicOpen());

                    if (!userStateInfo.isHost()) {
                        ChatRoomManager.getInstance().setOnRaiseHandNumChangeNotify(null);
                        rightIcon.setVisibility(View.INVISIBLE);
                    } else if (!isFinishing()) {
                        // 自己变成主持人
                        ChatRoomManager.getInstance().setOnRaiseHandNumChangeNotify(this);
                        rightIcon.setVisibility(View.VISIBLE);

                        AlertDialog.Builder builder = new AlertDialog.Builder(ChatRoomActivity.this)
                                .setTitle(R.string.designate_title)
                                .setMessage(R.string.designate_msg)
                                .setNeutralButton(R.string.confirm, null);
                        builder.show();
                    }
                }

                if (notifyUserInfo.isAudience()) {
                    // ->观众
                    audienceAdapter.onUserIn(notifyUserInfo);
                    // 可能不在里面，保底处理
                    stateGuestAdapter.onUserOffStage(notifyUserInfo.getUserID());


                    if (notifyInfo.operatorUid != myUid && myUid == notifyUserInfo.getUserID()) {
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                ToastUtil.showNormalToast(R.string.set_as_audience_tip);
                            }
                        });

                    }

                } else if (notifyUserInfo.isStageGuest()) {
                    // ->台上嘉宾
                    stateGuestAdapter.onUserOnStage(notifyUserInfo);
                    // 可能不在里面，保底处理
                    audienceAdapter.onUserOut(notifyUserInfo.getUserID());
                }
            }
        } else if (notifyInfo.type == 3) {
            // 麦克风变更通知
            for (NotifyUserStateUpdateInfo.UserStateInfo userStateInfo : notifyInfo.users) {
                UserInfo changedUserInfo = userStateInfo.toUserInfo();
                Log.d(TAG, String.format("notifyType:%d user:%d nickName:%s newRole:%d newMic:%b", notifyInfo.type, changedUserInfo.getUserID(), changedUserInfo.getNickName(), changedUserInfo.getRole(), changedUserInfo.isMicOpen()));

                if (changedUserInfo.isHost() || changedUserInfo.isStageGuest()) {
                    // 如果更新失败，会当新用户加入
                    stateGuestAdapter.onUserMicStatusChanged(changedUserInfo);
                    audienceAdapter.onUserOut(changedUserInfo.getUserID());
                } else {
                    // 台下没有麦克风变化
                    Log.w(TAG, String.format("notifyType:%d user:%d, audience user but receive mic change notify", notifyInfo.type, changedUserInfo.getUserID()));
                }

                UserInfo myUserInfo = ChatRoomManager.getInstance().getMyUserInfo();
                if (myUserInfo != null && myUserInfo.getUserID() == userStateInfo.getUserID()) {
                    bottomView.setBottomView(myUserInfo.getRole(), myUserInfo.isMicOpen());
                }
            }
        }
    }

    @Override
    public void onRoomCloseNotify() {
        Log.d(TAG, "房间关闭");

        UserInfo myUserInfo = ChatRoomManager.getInstance().getMyUserInfo();
        if (!myUserInfo.isHost()) {
            showRoomCloseAlertDialog();
        }
    }

    @Override
    public void onInvitedNotify(@NotNull NotifyInvitedInfo invitedInfo) {
        Log.d(TAG, "邀请：" + invitedInfo.getTargetUid());

        if (invitedInfo.getTargetUid() == myUid) {
            /**
             * 已弹出弹窗，不在弹窗
             */
            if (inviteDialog != null && inviteDialog.isShowing()) {
                return;
            }

            if (!isFinishing()) {
                inviteDialogBuilder = new AlertDialog.Builder(ChatRoomActivity.this)
                        .setTitle(R.string.invite_title)
                        .setMessage(R.string.invite_msg)
                        .setCancelable(false)
                        .setPositiveButton(R.string.invite_agree, null)
                        .setNeutralButton(R.string.invite_reject, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                ChatRoomManager.getInstance().responseRejectInviteOnstage(myUid, invitedInfo.getInviteToken(), new IExecuteCallback() {
                                    @Override
                                    public void onSuccess() {

                                    }

                                    @Override
                                    public void onFailed(int errorCode) {

                                    }
                                });
                            }
                        });
                inviteDialog = inviteDialogBuilder.show();
                inviteDialog.getButton(AlertDialog.BUTTON_POSITIVE).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        inviteDialog.dismiss();
                        if (!PermissionUtils.hasPermission(ChatRoomActivity.this, Manifest.permission.RECORD_AUDIO)) {
                            //没有音频权限
                            requestAudioPermission(PERMISSION_REQUEST_AUDIO_COMMON);
                        }

                        ChatRoomManager.getInstance().responseAgreeInviteOnstage(myUid, invitedInfo.getInviteToken(), new IExecuteCallback() {
                            @Override
                            public void onSuccess() {
                                bottomView.updateRaiseHandStatus(false);
                            }

                            @Override
                            public void onFailed(int errorCode) {

                            }
                        });
                    }
                });
            }
        }
    }

    @Override
    public void onRefusedInviteNotify(@NotNull NotifyRefuseInviteInfo notifyRefuseInviteInfo) {
        Log.d(TAG, "拒绝邀请：" + notifyRefuseInviteInfo.getTargetUid());
        if (notifyRefuseInviteInfo.getTargetUid() == myUid) {
            ToastUtil.showWarnToast(notifyRefuseInviteInfo.getNickName() + getString(R.string.invite_host_tip_fail));
        }
    }

    @Override
    public void onRoomReconnected() {
        RoomInfo curRoomInfo = ChatRoomManager.getInstance().getRoomInfo();
        if (curRoomInfo != null) {
            RTCSDKManager.getInstance().initQualityManager(ChatRoomActivity.this);
            RoomAPI.loginRoom(myUid, curRoomInfo.getRoomID(), new ILoginRoomCallback() {
                @Override
                public void onLoginRoom(int errorCode, RoomInfo roomInfo) {
                    Log.d(TAG, "room relogin, errorCode:" + errorCode);
                    if (errorCode == ERROR_ROOM_NOT_EXIST) {
                        showRoomCloseAlertDialog();
                    } else if (errorCode == ERROR_ALREADY_IN_ROOM) {
                        // do nothing
                    } else if (errorCode == ERROR_USER_OFF_LINE) {

                    }
                }
            });
        }
    }

    @Override
    public void onUserVolumeUpdate(HashMap<Long, Float> userVolumeMap) {
        if (stateGuestAdapter != null) {
            stateGuestAdapter.onUserVolumeUpdated(userVolumeMap);
        }
    }

    @Override
    public void onDataChange(int size) {
        if (size == 0) {
            roomBottomTitle.setVisibility(View.GONE);
        } else {
            roomBottomTitle.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onRaiseHandNumChange(int raiseHandNum) {
        bottomView.post(new Runnable() {
            @Override
            public void run() {
                bottomView.setRaiseHandCount(raiseHandNum);
            }
        });
    }
}
