package im.zego.gochat.chatroom;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.IBinder;

import androidx.annotation.Nullable;

import im.zego.gochat.R;

import static android.content.pm.ServiceInfo.FOREGROUND_SERVICE_TYPE_MICROPHONE;

public class ChatRoomService extends Service {

    private static final String CHANNEL_ID = "channel_chat_id";
    private static final String CHANNEL_NAME = "channel_chat_room";
    private static final int NOTIFICATION_ID = 110;

    @Override
    public void onCreate() {
        super.onCreate();

        startForeground();

    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        stopForeground(true);
    }

    private void startForeground() {
        Intent notificationIntent = new Intent(this, ChatRoomActivity.class);
        PendingIntent pendingIntent =
                PendingIntent.getActivity(this, 0, notificationIntent, 0);
        Notification notification = null;

        NotificationManager manager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);

        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            NotificationChannel notificationChannel = new NotificationChannel(CHANNEL_ID,
                    CHANNEL_NAME, NotificationManager.IMPORTANCE_LOW);
            notificationChannel.setSound(null, null);
            notificationChannel.enableLights(false);
            notificationChannel.enableVibration(false);
            manager.createNotificationChannel(notificationChannel);
            notification = new Notification.Builder(this, CHANNEL_ID)
                    .setLargeIcon(BitmapFactory.decodeResource(this.getResources(), R.mipmap.app_icon))
                    .setContentTitle(getResources().getString(R.string.room_in_progress_title))
                    .setContentText(getResources().getString(R.string.room_in_progress_msg))
                    .setSmallIcon(R.mipmap.app_icon)
                    .setContentIntent(pendingIntent)
                    .setWhen(System.currentTimeMillis())
                    .build();


        } else {
            notification = new Notification.Builder(this)
                    .setLargeIcon(BitmapFactory.decodeResource(this.getResources(), R.mipmap.app_icon))
                    .setSmallIcon(R.mipmap.app_icon)
                    .setContentTitle(getResources().getString(R.string.room_in_progress_title))
                    .setContentText(getResources().getString(R.string.room_in_progress_msg))
                    .setContentIntent(pendingIntent)
                    .setWhen(System.currentTimeMillis())
                    .build();
        }


        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            startForeground(NOTIFICATION_ID, notification, FOREGROUND_SERVICE_TYPE_MICROPHONE);
        } else {
            startForeground(NOTIFICATION_ID, notification);
        }

    }


    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }
}
