package im.zego.gochat.chatroom.adapter;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import org.jetbrains.annotations.NotNull;

import java.util.Iterator;
import java.util.Map;

import im.zego.gochat.R;
import im.zego.gochat.chatroom.ChatRoomManager;
import im.zego.gochat.model.UserInfo;
import im.zego.gochat.utils.GlideImageLoader;

/**
 * 房间内听众
 */
public class ChatRoomAudienceAdapter extends RecyclerView.Adapter<ChatRoomAudienceAdapter.AudienceViewHolder> {
    private final UserInfoOrderedSet audienceList = new UserInfoOrderedSet(UserInfoOrderedSet.ORDER_BY_ENTERROOM);
    private IAvatarClickListener mListener;

    public ChatRoomAudienceAdapter(IAvatarClickListener listener) {
        this.mListener = listener;
    }

    @NotNull
    @Override
    public AudienceViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_audience_list, parent, false);
        return new AudienceViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull AudienceViewHolder holder, int position) {
        UserInfo audienceUserInfo = audienceList.get(position);
        GlideImageLoader.getInstance().displayImageByAvatar(audienceUserInfo.getAvatar(), holder.avatar);
        holder.userName.setText(audienceUserInfo.getNickName());

        holder.itemView.setOnClickListener(v -> {
            UserInfo myUserInfo = ChatRoomManager.getInstance().getMyUserInfo();
            UserInfo clickedUserInfo = audienceList.get(holder.getLayoutPosition());
            // 可以操作自己，但只有主持人才能操作别人
            if (myUserInfo.getUserID() == clickedUserInfo.getUserID()) {
                mListener.clickSelf(clickedUserInfo);
            } else if (myUserInfo.isHost()) {
                mListener.clickAudience(clickedUserInfo);
            }
        });
    }


    @Override
    public int getItemCount() {
        return audienceList.size();
    }

    public void initUserList() {
        audienceList.clear();
        Map<Long, UserInfo> userModelMap = ChatRoomManager.getInstance().getUserModelMap();
        Iterator iter = userModelMap.entrySet().iterator();
        while (iter.hasNext()) {
            Map.Entry entry = (Map.Entry) iter.next();
            UserInfo userModel = (UserInfo) entry.getValue();
            if (userModel.isAudience()) {
                audienceList.add(userModel);
            }
        }
        notifyData();
    }

    /**
     * 成员进入观众列表
     *
     * @param userInfo
     */
    public void onUserIn(UserInfo userInfo) {
        audienceList.add(userInfo);
        notifyData();
    }

    /**
     * 成员离开观众列表
     */
    public void onUserOut(long userId) {
        audienceList.remove(userId);
        notifyData();
    }


    static class AudienceViewHolder extends RecyclerView.ViewHolder {
        private ImageView avatar;
        private TextView userName;

        public AudienceViewHolder(View itemView) {
            super(itemView);
            avatar = itemView.findViewById(R.id.iv_avatar);
            userName = itemView.findViewById(R.id.tv_user_name);
        }
    }

    private void notifyData()
    {
        if(mIAudienceDataChangeListener != null)
        {
            mIAudienceDataChangeListener.onDataChange(getItemCount());
        }
        notifyDataSetChanged();
    }


    private IAudienceDataChangeListener mIAudienceDataChangeListener;

    public void setIAudienceDataChangeListener(IAudienceDataChangeListener listener)
    {
        mIAudienceDataChangeListener = listener;
    }
}
