package im.zego.gochat.chatroom.adapter;

import android.graphics.Color;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import im.zego.gochat.R;
import im.zego.gochat.chatroom.ChatRoomManager;
import im.zego.gochat.chatroom.IExecuteCallback;
import im.zego.gochat.utils.GlideImageLoader;

/**
 * 举手列表
 */
public class RaiseHandAdapter extends RecyclerView.Adapter<RaiseHandAdapter.GuestViewHodler> {
    private final UserInfoOrderedSet raiseHandList = new UserInfoOrderedSet(UserInfoOrderedSet.ORDER_BY_ONSTAGE);
    private final SparseArray<Object> arrayMap = new SparseArray<>();
    private static final Object selectObject = new Object();
    @NonNull
    @Override
    public GuestViewHodler onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_raise_hand_list, parent, false);
        return new GuestViewHodler(view);
    }

    @Override
    public void onBindViewHolder(@NonNull GuestViewHodler holder, int position) {
        GlideImageLoader.getInstance().displayImageByAvatar(raiseHandList.get(position).getAvatar(), holder.avatar);
        holder.userName.setText(raiseHandList.get(position).getNickName());

        if(arrayMap.get(position) == selectObject)
        {
            holder.button.setSelected(true);
            holder.button.setText(R.string.raise_hand_invited_button);
        }else {
            holder.button.setSelected(false);
            holder.button.setText(R.string.raise_hand_invite_button);
        }

        holder.button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                ChatRoomManager.getInstance().inviteOnstage(ChatRoomManager.getInstance().getMyUserInfo().getUserID(), raiseHandList.get(position).getUserID(),
                        new IExecuteCallback() {
                            @Override
                            public void onSuccess() {
                                holder.button.setSelected(true);
                                holder.button.setText(R.string.raise_hand_invited_button);
                                arrayMap.put(holder.getAdapterPosition(),selectObject);
                            }

                            @Override
                            public void onFailed(int errorCode) {

                            }
                        });
            }
        });
    }

    @Override
    public int getItemCount() {
        return raiseHandList.size();
    }


    public void initRaiseHandList() {
        raiseHandList.clear();
        arrayMap.clear();
        raiseHandList.addAll(ChatRoomManager.getInstance().getRaiseHandList());
        notifyDataSetChanged();
    }

    class GuestViewHodler extends RecyclerView.ViewHolder {
        private ImageView avatar;
        private TextView userName;
        private Button button;

        public GuestViewHodler(View itemView) {
            super(itemView);
            avatar = itemView.findViewById(R.id.iv_raise_hand_avatar);
            userName = itemView.findViewById(R.id.tv_user_name);
            button = itemView.findViewById(R.id.bt_raise_hand_invite_onstage);
        }
    }
}
