package im.zego.gochat.chatroom.adapter;

import java.util.ArrayList;
import java.util.List;

import im.zego.gochat.model.UserInfo;

/**
 * 伪装成set容器，封装去重及检查存在逻辑
 */
public class UserInfoOrderedSet {
    private final ArrayList<UserInfo> mUserInfoArray = new ArrayList<>();

    public static final int ORDER_BY_ONSTAGE = 1;
    public static final int ORDER_BY_ENTERROOM = 2;

    private int mOrderBy = ORDER_BY_ONSTAGE;

    public UserInfoOrderedSet() {
    }

    public UserInfoOrderedSet(int orderBy) {
        mOrderBy = orderBy;
    }

    public void add(UserInfo newItem) {
        if (mOrderBy == ORDER_BY_ONSTAGE) {
            addOrderByOnStage(newItem);
        } else if (mOrderBy == ORDER_BY_ENTERROOM) {
            addOrderByEnterRoom(newItem);
        }
    }

    private void addOrderByOnStage(UserInfo newItem) {
        // 上台，确保丢最后
        UserInfo existsItem = findAlreadyExistsUserInfo(newItem.getUserID());
        if (existsItem != null) {
            mUserInfoArray.remove(existsItem);
        }
        mUserInfoArray.add(newItem);
    }

    private void addOrderByEnterRoom(UserInfo newItem) {
        // 原本就是有序的，所以已有item不做变化，新item从后向前找一个合适的位置插入
        UserInfo existsItem = findAlreadyExistsUserInfo(newItem.getUserID());
        if (existsItem != null) {
            return;
        }

        int insertIdx = mUserInfoArray.size();
        for (int i = mUserInfoArray.size() - 1; i >= 0; i--) {
            UserInfo idxItem = mUserInfoArray.get(i);
            if (newItem.getLoginTimeStamp() >= idxItem.getLoginTimeStamp()) {
                insertIdx = i + 1;
                break;
            }
        }
        if (insertIdx < mUserInfoArray.size()) {
            mUserInfoArray.add(insertIdx, newItem);
        } else {
            mUserInfoArray.add(newItem);
        }
    }

    public void add(int idx, UserInfo newItem) {
        UserInfo existsItem = findAlreadyExistsUserInfo(newItem.getUserID());
        if (existsItem != null) {
            mUserInfoArray.remove(existsItem);
        }
        mUserInfoArray.add(idx, newItem);
    }

    public void addAll(List<UserInfo> userInfoList) {
        mUserInfoArray.addAll(userInfoList);
    }

    public void remove(long userID) {
        mUserInfoArray.remove(findAlreadyExistsUserInfo(userID));
    }

    private UserInfo findAlreadyExistsUserInfo(long uid) {
        UserInfo existsItem = null;
        for (UserInfo userInfo : mUserInfoArray) {
            if (userInfo.getUserID() == uid) {
                existsItem = userInfo;
                break;
            }
        }

        return existsItem;
    }

    public void clear() {
        mUserInfoArray.clear();
    }

    public int size() {
        return mUserInfoArray.size();
    }

    public UserInfo get(int idx) {
        return mUserInfoArray.get(idx);
    }

    public UserInfo getIndexByUID(long uid) {
        for (UserInfo userInfo : mUserInfoArray) {
            if (userInfo.getUserID() == uid) {
                return userInfo;
            }
        }
        return null;
    }

    public UserInfo update(UserInfo changedUserInfo) {
        UserInfo updatedUserInfo = null;
        for (UserInfo userInfo : mUserInfoArray) {
            if (userInfo.getUserID() == changedUserInfo.getUserID()) {
                userInfo.update(changedUserInfo);
                updatedUserInfo = userInfo;
                break;
            }
        }

        return updatedUserInfo;
    }

    public int indexOf(UserInfo userInfo) {
        return mUserInfoArray.indexOf(userInfo);
    }

    public int indexOf(long uid) {
        for (int i = 0; i < mUserInfoArray.size(); i++) {
            if (mUserInfoArray.get(i).getUserID() == uid) {
                return i;
            }
        }
        return -1;
    }
}
