package im.zego.gochat.custom;

import android.content.Context;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;

import im.zego.gochat.R;

/**
 * 房间底部栏
 */
public class RoomBottomView extends RelativeLayout implements View.OnClickListener {
    private ImageView leaveRoomImageView;
    private RelativeLayout raiseHandeList;
    private ImageView micImageView;
    private ImageView raiseHandImageView;
    private IRoomBottomViewCallBack callBack;
    private TextView tvRaiseHandNum;

    public RoomBottomView(Context context) {
        super(context);
        initView(context);
    }


    public RoomBottomView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        initView(context);
    }

    public RoomBottomView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initView(context);
    }


    private void initView(Context context) {
        LayoutInflater.from(context).inflate(R.layout.room_bottom_bar, this);
        leaveRoomImageView = findViewById(R.id.iv_leave_room);
        raiseHandeList = findViewById(R.id.rl_raise_hand_list);
        micImageView = findViewById(R.id.iv_toolbar_mic);
        raiseHandImageView = findViewById(R.id.iv_toolbar_raise_hand);
        tvRaiseHandNum = findViewById(R.id.tv_raise_hand_num);
        leaveRoomImageView.setOnClickListener(this);
        raiseHandeList.setOnClickListener(this);
        micImageView.setOnClickListener(this);
        raiseHandImageView.setOnClickListener(this);
    }

    public void setCallBack(IRoomBottomViewCallBack callBack) {
        this.callBack = callBack;
    }

    /**
     * 设置不同属性的成员UI
     *
     * @param role 1、观众  2、嘉宾 3、主持人
     */
    public void setBottomView(int role, boolean isMicOpen) {
        if (role == 1) {
            raiseHandImageView.setVisibility(VISIBLE);
            micImageView.setVisibility(GONE);
            raiseHandeList.setVisibility(GONE);
        } else if (role == 2) {
            raiseHandImageView.setVisibility(GONE);
            micImageView.setVisibility(VISIBLE);
            raiseHandeList.setVisibility(GONE);
        } else if (role == 3) {
            raiseHandImageView.setVisibility(GONE);
            micImageView.setVisibility(VISIBLE);
            raiseHandeList.setVisibility(VISIBLE);
        }

        if (isMicOpen) {
            micImageView.setImageDrawable(ContextCompat.getDrawable(getContext(), R.mipmap.toolbar_mic_on));
        } else {
            micImageView.setImageDrawable(ContextCompat.getDrawable(getContext(), R.mipmap.toolbar_mic_off));
        }
    }

    /**
     * 设置麦克风状态
     *
     * @param enable
     */
    public void setMicStatus(boolean enable) {

    }

    /**
     * 设置举手人数，适用于主持方
     *
     * @param count
     */
    public void setRaiseHandCount(int count) {
        if (raiseHandeList.getVisibility() == View.VISIBLE) {
            // 前提是其父控件是可见的
            if (count > 0) {
                tvRaiseHandNum.setVisibility(View.VISIBLE);
                tvRaiseHandNum.setText(String.valueOf(count));
            } else {
                tvRaiseHandNum.setVisibility(View.INVISIBLE);
            }
        }
    }

    /**
     * 更新举手按钮的状态，适用于听众
     */
    public void updateRaiseHandStatus(boolean isRaiseHand) {

        if (isRaiseHand) {
            raiseHandImageView.setImageDrawable(ContextCompat.getDrawable(getContext(), R.mipmap.toolbar_raisehand_applying));
        } else {
            raiseHandImageView.setImageDrawable(ContextCompat.getDrawable(getContext(), R.mipmap.toolbar_raise_hand));
        }

    }

    @Override
    public void onClick(View v) {
        if (callBack == null)
            return;
        int id = v.getId();
        if (id == R.id.iv_leave_room) {
            callBack.onClickLeaveRoom();
        } else if (id == R.id.rl_raise_hand_list) {
            callBack.onClickRaiseHandList();
        } else if (id == R.id.iv_toolbar_mic) {
            callBack.onClickMic();
        } else if (id == R.id.iv_toolbar_raise_hand) {
            callBack.onClickRaiseHand();
        }
    }

    public interface IRoomBottomViewCallBack {
        void onClickLeaveRoom();

        void onClickRaiseHandList();

        void onClickMic();

        void onClickRaiseHand();
    }
}
