package im.zego.gochat.custom;

import android.graphics.Color;
import android.graphics.drawable.GradientDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.fragment.app.DialogFragment;

import im.zego.gochat.R;
import im.zego.gochat.utils.MiscUtils;


/**
 * 会议间对话框样式title（标题）、content（正文）、leftBtn（左按钮/灰色）、rightBtn（右按钮/蓝色）、checkItem（勾选项）、closeBtn（关闭按钮）
 * 1、title、content、leftBtn、rightBtn
 * 2、title、content、singleBtn
 * 3、title、content、checkItem、leftBtn、rightBtn
 * 4、title、content、leftBtn、rightBtn、closeBtn
 * 5、
 */
public class ZegoMeetingDialog extends ZegoBaseDialogFragment {

    private static final String TAG = "ZegoMeetingDialog";

    private static final int default_left_btn_color = Color.parseColor("#666666");
    private static final int default_right_btn_color = Color.parseColor("#1a7bff");

    private TextView mTitle;
    private TextView mContent;
    private TextView mLeft;
    private TextView mRight;
    private ImageView mClose;

    private boolean isCloseVisible = false;
    private boolean isLeftVisible = true;
    private boolean isRightVisible = true;
    private String leftBtnString;
    private String rightBtnString;
    private int leftBgColor = default_left_btn_color;
    private int rightBgColor = default_right_btn_color;
    private int radiusDp = 4;

    private DialogClickListener mDialogClickListener;
    private DialogClickListenerWithParam mDialogClickListenerWithParam;
    private ShowListener mShowListener;
    private ConfirmListener mConfirmListener;
    private CancelListener mCancelListener;

    private CloseListener mCloseListener;

    public interface CancelListener {
        void onCancel();
    }

    public interface ConfirmListener {
        void onConfirm();
    }

    public interface ShowListener {
        void onShow();
    }

    public interface CloseListener {
        void onClose();
    }

    public void setShowListener(ShowListener mShowListener) {
        this.mShowListener = mShowListener;
    }

    public void setConfirmListener(ConfirmListener listener) {
        mConfirmListener = listener;
    }

    public void setDialogClickListener(DialogClickListener dialogClickListener) {
        mDialogClickListener = dialogClickListener;
    }

    public void setDialogClickListenerWithParam(DialogClickListenerWithParam mDialogClickListenerWithParam) {
        this.mDialogClickListenerWithParam = mDialogClickListenerWithParam;
    }

    public interface DialogClickListener {
        void onCancel();

        void onConfirm();
    }

    public interface DialogClickListenerWithParam {
        void onCancel(DialogFragment dialog);

        void onConfirm(DialogFragment dialog);
    }

    public static ZegoMeetingDialog newInstance(String title, String content) {
        ZegoMeetingDialog zegoMeetingDialog = new ZegoMeetingDialog();
        Bundle args = new Bundle();
        args.putString("title", title);
        args.putString("content", content);
        zegoMeetingDialog.setArguments(args);
        return zegoMeetingDialog;
    }

    public void setLeftBgColor(int color) {
        leftBgColor = color;
    }

    public void setRightColor(int color) {
        rightBgColor = color;
    }

    public void setLeftBtnVisible(boolean visible) {
        isLeftVisible = visible;
    }

    public void setRightBtnVisible(boolean visible) {
        isRightVisible = visible;
    }

    public void setCloseVisible(boolean visible) {
        isCloseVisible = visible;
    }

    public void setLeftBtnString(String leftBtnString) {
        this.leftBtnString = leftBtnString;
    }

    public void setRightBtnString(String rightBtnString) {
        this.rightBtnString = rightBtnString;
    }

    @Override
    protected View createView(LayoutInflater inflater, ViewGroup container) {
        ViewGroup viewGroup = (ViewGroup) inflater.inflate(R.layout.layout_zego_meeting_dialog, null);
        mTitle = viewGroup.findViewById(R.id.title);
        mContent = viewGroup.findViewById(R.id.content);
        mLeft = viewGroup.findViewById(R.id.left_btn);
        mRight = viewGroup.findViewById(R.id.right_btn);
        mClose = viewGroup.findViewById(R.id.close);
        Bundle arguments = getArguments();
        if (arguments != null) {
            String content = arguments.getString("content");
            String title = arguments.getString("title");
            mTitle.setText(title);
            mContent.setText(content);
        }

        if (expanded) {
            int width = MiscUtils.dp2px(getContext(), 68);
            ViewGroup.LayoutParams layoutParams = mLeft.getLayoutParams();
            layoutParams.width = width;
            ViewGroup.LayoutParams layoutParams1 = mRight.getLayoutParams();
            layoutParams1.width = width;
            mLeft.setLayoutParams(layoutParams);
            mRight.setLayoutParams(layoutParams1);
        }

        initBtn();
        mLeft.setOnClickListener(v -> onCancel());
        mRight.setOnClickListener(v -> onConfirm());
        mClose.setOnClickListener(v -> onClose());
        if (mShowListener != null) {
            mShowListener.onShow();
        }
        return viewGroup;
    }

    boolean expanded = false;

    /**
     * 当显示四个字的时候，按钮要设置宽度为136 x 48
     */
    public void expandWidth() {
        expanded = true;
    }

    public void updateRightView() {
        if (mRight != null) {
            mRight.setText(rightBtnString);
        }
    }

    private void initBtn() {
        if (isLeftVisible) {
            GradientDrawable leftDrawable = new GradientDrawable();
            leftDrawable.setColor(leftBgColor);
            leftDrawable.setCornerRadius(MiscUtils.dp2px(getContext(), radiusDp));
            mLeft.setBackground(leftDrawable);
            mLeft.setVisibility(View.VISIBLE);
        } else {
            mLeft.setVisibility(View.GONE);
        }

        if (isRightVisible) {
            GradientDrawable rightDrawable = new GradientDrawable();
            rightDrawable.setColor(rightBgColor);
            rightDrawable.setCornerRadius(MiscUtils.dp2px(getContext(), radiusDp));
            mRight.setBackground(rightDrawable);
            mRight.setVisibility(View.VISIBLE);
        } else {
            mRight.setVisibility(View.GONE);
        }

        if (isCloseVisible) {
            mClose.setVisibility(View.VISIBLE);
        } else {
            mClose.setVisibility(View.GONE);
        }
        mLeft.setText(leftBtnString);
        mRight.setText(rightBtnString);
    }


    @Override
    protected int getDialogStyle() {
        return super.getDialogStyle();
    }

    /**
     * note that:mDialogClickListener.onConfirm()的回调需要自己处理dismiss
     */
    private void onConfirm() {
        if (mConfirmListener != null) {
            dismiss();
            mConfirmListener.onConfirm();
        }
        if (mDialogClickListener != null) {
            mDialogClickListener.onConfirm();
        }
        if (mDialogClickListenerWithParam != null) {
            mDialogClickListenerWithParam.onConfirm(this);
        }
    }

    private void onCancel() {
        dismiss();
        if (mDialogClickListener != null) {
            mDialogClickListener.onCancel();
        }
        if (mDialogClickListenerWithParam != null) {
            mDialogClickListenerWithParam.onCancel(this);
        }
    }

    private void onClose() {
        dismiss();
    }

    @Override
    protected boolean canceledOnTouchOutside() {
        return false;
    }

    @Override
    protected boolean cancelable() {
        return false;
    }

    @Override
    protected float backgroundDimAmount() {
        return 0.5f;
    }
}
