package im.zego.gochat.custom;

import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.BitmapDrawable;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.PopupWindow;
import android.widget.TextView;

import im.zego.gochat.R;


public class ZegoPopupWindow {
    private Context mContext;
    private PopupWindow mPopupWindow;
    private int mItemHeight;
    private int mWidth;
    private int mHeight;
    private ViewGroup mContainer;
    private PopupWindow.OnDismissListener mOnDismissListener;
    public interface ICommandListener {
        void onClickListener(View v);
    }

    public ZegoPopupWindow(Context context, int ResourceId, int width, int height, int itemHeight) {
        mContext = context;
        mContainer = (ViewGroup) LayoutInflater.from(context).inflate(ResourceId, null);
        mWidth = width;
        mHeight = height;
        mItemHeight = itemHeight;
    }

    public ZegoPopupWindow setItemHeight(int itemHeight) {
        mItemHeight = itemHeight;
        return this;
    }

    public ZegoPopupWindow addCommand(String command, ICommandListener listener) {
        TextView textView = new TextView(mContext);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, mItemHeight);
        layoutParams.gravity = Gravity.CENTER;
        textView.setGravity(Gravity.CENTER);
        textView.setText(command);
        textView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 14f);
        textView.setTextColor(Color.parseColor("#1f1f1f"));
        textView.setOnClickListener(v -> {
            listener.onClickListener(v);
            if (mPopupWindow != null) {
                mPopupWindow.dismiss();
            }
        });
        mContainer.addView(textView, layoutParams);
        return this;
    }

    public void addDivider() {
        View view = new View(mContext);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 1);
        view.setBackgroundColor(Color.parseColor("#d9d9d9"));
        mContainer.addView(view, layoutParams);
    }

    public ZegoPopupWindow addSpace(int height) {
        TextView textView = new TextView(mContext);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, height);
        mContainer.addView(textView, layoutParams);
        return this;
    }

    public ZegoPopupWindow setOnDismissListener(PopupWindow.OnDismissListener listener) {
        mOnDismissListener = listener;
        return this;
    }

    public void showAtLocation(View v, int gravity, int xoffset, int yoffset) {
        mPopupWindow = new PopupWindow(mContainer, mWidth, mHeight);
        mPopupWindow.setOutsideTouchable(true);
        mPopupWindow.setBackgroundDrawable(new BitmapDrawable());
        mPopupWindow.setOnDismissListener(mOnDismissListener);
        mPopupWindow.setAnimationStyle(R.style.BottomDialogAnimation);
        mPopupWindow.showAtLocation(v, gravity, xoffset, yoffset);
    }

    public void dismiss() {
        if (mPopupWindow != null) {
            mPopupWindow.dismiss();
        }
    }
}
