package im.zego.gochat.login;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;

import androidx.annotation.Nullable;

import im.zego.common.application.ZegoApplication;
import im.zego.common.model.LoginInfo;
import im.zego.gochat.R;
import im.zego.gochat.base.ZGBaseActivity;
import im.zego.gochat.custom.ProcessView;
import im.zego.gochat.protocol.ILoginCallback;
import im.zego.gochat.roomlist.activity.RoomListActivity;
import im.zego.gochat.utils.GlideImageLoader;
import im.zego.gochat.utils.ScreenAdaptationUtil;
import im.zego.gochat.utils.ToastUtil;

import static im.zego.gochat.ErrorcodeConstants.ERROR_NICK_NAME_ALREADY_IN_USE;
import static im.zego.gochat.ErrorcodeConstants.GoChatErrorFailNetwork;

public class LoginActivity extends ZGBaseActivity {

    public static final String NICK_NAME = "nick_name";

    private EditText mEtLoginName;
    private Button mBtLogin;
    private ProcessView mProcessView;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);
//        ScreenAdaptationUtil.init(ZegoApplication.getAppContext(),375);
        initView();
        initListener();
        initData();
    }

    private void initView()
    {
        mEtLoginName = findViewById(R.id.et_login_name);
        mBtLogin = findViewById(R.id.bt_login);
        mProcessView = new ProcessView(LoginActivity.this);
    }

    private void initListener()
    {
        mEtLoginName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                if("".equals(s.toString().trim()))
                {
                    mBtLogin.setEnabled(false);
                }else {
                    mBtLogin.setEnabled(true);
                }
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });


        mBtLogin.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                    mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mBtLogin.getRootView());
                    String nickname = mEtLoginName.getText().toString().trim();
                    String avatar = GlideImageLoader.getInstance().getAvatarByNickname(nickname);
                    LoginInfoManager.getInstance().loginUser(nickname,avatar, new ILoginCallback<LoginInfo>() {
                        @Override
                        public void onLogin(int retcode, LoginInfo loginInfo) {
                            mProcessView.dismiss();
                            if(retcode == 0)
                            {
                                ToastUtil.showNormalToast(getResources().getString(R.string.login_tips_success));
                                LoginInfoManager.getInstance().setLoginInfo(loginInfo);
                                startActivity(new Intent(LoginActivity.this, RoomListActivity.class));
                                finish();
                            }else if(retcode == ERROR_NICK_NAME_ALREADY_IN_USE){
                                ToastUtil.showWarnToast(getResources().getString(R.string.login_tips_fail_nickname));
                                mEtLoginName.setText("");
                            }else if(retcode == GoChatErrorFailNetwork)
                            {
                                ToastUtil.showWarnToast(getResources().getString(R.string.login_tips_fail_network));
                            } else {
                                ToastUtil.showWarnToast(getResources().getString(R.string.login_tips_fail));
                            }
                        }
                    });
            }
        });
    }


    private void initData()
    {
        String nickName = getIntent().getStringExtra(NICK_NAME);
        if(!TextUtils.isEmpty(nickName))
        {
            mEtLoginName.setText(nickName);
        }

        final LoginInfo loginInfo = LoginInfoManager.getInstance().getLoginInfo();
        if(loginInfo != null && loginInfo.getUid() != 0 && !TextUtils.isEmpty(loginInfo.getNickName()))
        {
            mEtLoginName.setText(loginInfo.getNickName());
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    //    private void showToast(TopToastView.ToastMessageType type, String message)
//    {
//        mTtvLoginToast.showToastMessage(type,message);
//    }

}
