package im.zego.gochat.login;

import android.os.Handler;
import android.os.Looper;

import org.jetbrains.annotations.NotNull;

import java.util.Timer;
import java.util.TimerTask;

import im.zego.common.model.LoginInfo;
import im.zego.common.util.SharedPreferencesUtil;
import im.zego.gochat.model.HeartbeatRespInfo;
import im.zego.gochat.protocol.ICommonResponseCallback;
import im.zego.gochat.protocol.IHeartBeatCallback;
import im.zego.gochat.protocol.ILoginCallback;
import im.zego.gochat.protocol.LoginAPI;

public class LoginInfoManager {

    private static volatile LoginInfoManager singleton = null;

    public static LoginInfoManager getInstance() {
        if (singleton == null) {
            synchronized (LoginInfoManager.class) {
                if (singleton == null) {
                    singleton = new LoginInfoManager();
                    sMainHandler = new Handler(Looper.getMainLooper());
                }
            }
        }
        return singleton;
    }

    private static TimerTask sHeartBeatTimerTask;
    private long heartInterval = 15;
    private Timer mHeartTimer;
    private static Handler sMainHandler;

    public void setLoginInfo(LoginInfo loginInfo)
    {
        SharedPreferencesUtil.cacheLoginInfo(loginInfo);
    }

    public LoginInfo getLoginInfo()
    {
        return SharedPreferencesUtil.getLoginInfo();
    }

    public boolean isLogin()
    {
        return SharedPreferencesUtil.getLoginInfo().getUid() != 0;
    }

    public void clearLoginInfo()
    {
        SharedPreferencesUtil.clearLoginInfo();
    }

    public void loginUser(@NotNull String nickName, @NotNull String avatar, @NotNull final ILoginCallback<LoginInfo> callback) {
        LoginAPI.login(nickName, avatar, new ILoginCallback<LoginInfo>() {
            @Override
            public void onLogin(int errorCode, LoginInfo loginInfo) {
                if (errorCode == 0) {
                    if (mHeartTimer == null) {
                        mHeartTimer = new Timer();
                        sHeartBeatTimerTask = new HeartBeatTimerTask();
                        LoginAPI.heartBeat(loginInfo.getUid(), new IHeartBeatCallback() {
                            @Override
                            public void onHeartBeat(int errorCode, HeartbeatRespInfo heartbeatRespInfo) {
                                if (errorCode == 0) {
                                    heartInterval = heartbeatRespInfo.getInterval();
                                    mHeartTimer.schedule(sHeartBeatTimerTask, heartInterval*1000, heartInterval * 1000);
                                }
                            }
                        });
                    }
                }
                callback.onLogin(errorCode, loginInfo);
            }
        });
    }

    public void logoutUser(ICommonResponseCallback callback) {
        LoginAPI.logout(getLoginInfo().getUid(), new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                callback.onResponse(errorCode);

                if (mHeartTimer != null) {
                    sHeartBeatTimerTask.cancel();
                    mHeartTimer.cancel();
                    mHeartTimer = null;
                }


            }
        });
    }

    private static class HeartBeatTimerTask extends TimerTask
    {
        @Override
        public void run() {

            sMainHandler.post(() -> {
                LoginAPI.heartBeat(LoginInfoManager.getInstance().getLoginInfo().getUid(), new IHeartBeatCallback() {
                    @Override
                    public void onHeartBeat(int errorCode, HeartbeatRespInfo heartbeatRespInfo) {
                    }
                });
            });

        }
    }
}
