package im.zego.gochat.permission;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.text.TextUtils;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;

import im.zego.gochat.R;
import im.zego.gochat.custom.ZegoBaseDialogFragment;
import im.zego.gochat.custom.ZegoMeetingDialog;
import im.zego.gochat.utils.MiscUtils;

public class AppSettingsDialogHolderActivity extends AppCompatActivity {
    private static final int APP_SETTINGS_RC = 7534;
    public static final int DEFAULT_SETTINGS_REQ_CODE = 16061;

    private static final String TITLE_EXTRA = "title";
    private static final String RATIONALE_EXTRA = "rationale";
    private static final String POSITIVE_EXTRA = "positiveText";
    private static final String NEGATIVE_EXTRA = "negativeText";
    private static final String IS_MEETING_STYLE_EXTRA = "is_meeting_style";

    private ZegoBaseDialogFragment mDialog;

    // 这里有一个坑
    // startActivityForResult 对于singleTask singleTop等launchMode无效，并且会立即返回onActivityResult
    // 所以在start前做一个trick，判断栈顶如果是当前Activity就直接return
    private static void startSettingActivity(Object activityOrFragment,
                                             String title,
                                             String rationale, String positiveText, String negativeText,
                                             boolean isMeetingStyle, int requestCode) {
        Activity currentActivity = MiscUtils.getCurrentActivity();
        if (currentActivity != null && currentActivity instanceof AppSettingsDialogHolderActivity) {
            return;
        }

        Context context;
        if (activityOrFragment instanceof Activity) {
            context = (Activity) activityOrFragment;
        } else if (activityOrFragment instanceof Fragment) {
            context = ((Fragment) activityOrFragment).getContext();
        } else {
            throw new IllegalStateException("Unknown object: " + activityOrFragment);
        }

        Intent intent = new Intent(context, AppSettingsDialogHolderActivity.class);
        intent.putExtra(TITLE_EXTRA, title);
        intent.putExtra(RATIONALE_EXTRA, rationale);
        intent.putExtra(POSITIVE_EXTRA, positiveText);
        intent.putExtra(NEGATIVE_EXTRA, negativeText);
        intent.putExtra(IS_MEETING_STYLE_EXTRA, isMeetingStyle);
        startForResult(activityOrFragment, intent, requestCode);
    }

    public static void startSettingDialog(Object activityOrFragment,
                                          String title,
                                          String rationale, String positiveText, String negativeText) {
        startSettingActivity(activityOrFragment, title, rationale, positiveText, negativeText,
                false, DEFAULT_SETTINGS_REQ_CODE);
    }

    public static void startMeetingStyleSettingDialog(Object activityOrFragment,
                                                      String title,
                                                      String rationale, String positiveText, String negativeText,
                                                      int requestCode) {
        startSettingActivity(activityOrFragment, title, rationale, positiveText, negativeText,
                true, requestCode);
    }

    private static void startForResult(Object activityOrFragment, Intent intent, int requestCode) {
        if (activityOrFragment instanceof Activity) {
            ((Activity) activityOrFragment).startActivityForResult(intent, requestCode);
        } else if (activityOrFragment instanceof Fragment) {
            ((Fragment) activityOrFragment).startActivityForResult(intent, requestCode);
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        String title = getIntent().getStringExtra(TITLE_EXTRA);
        String rationale = getIntent().getStringExtra(RATIONALE_EXTRA);
        String positiveButtonText = getIntent().getStringExtra(POSITIVE_EXTRA);
        String negativeButtonText = getIntent().getStringExtra(NEGATIVE_EXTRA);

        //默认不展示标题
        title = TextUtils.isEmpty(title) ? "" : title;
        rationale = TextUtils.isEmpty(rationale) ? "" : rationale;
        positiveButtonText = TextUtils.isEmpty(positiveButtonText) ?
                getString(R.string.setting) : positiveButtonText;
        negativeButtonText = TextUtils.isEmpty(negativeButtonText) ?
                getString(android.R.string.cancel) : negativeButtonText;


        //会议室内部的样式是黑色的，这个要特殊处理
        mDialog = AppSettingDialog.showMeetingDialog(getSupportFragmentManager(),
                title, rationale, positiveButtonText, negativeButtonText,
                new ZegoMeetingDialog.DialogClickListener() {
                    @Override
                    public void onCancel() {
                        onClick(Dialog.BUTTON_NEGATIVE);
                    }

                    @Override
                    public void onConfirm() {
                        onClick(Dialog.BUTTON_POSITIVE);
                    }
                });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mDialog != null && !mDialog.isDetached()) {
            mDialog.dismiss();
        }
    }

    public void onClick(int which) {
        if (which == Dialog.BUTTON_POSITIVE) {
            Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
                    .setData(Uri.fromParts("package", getPackageName(), null));
            startActivityForResult(intent, APP_SETTINGS_RC);
        } else if (which == Dialog.BUTTON_NEGATIVE) {
            setResult(Activity.RESULT_CANCELED);
            finish();
        } else {
            throw new IllegalStateException("Unknown button type: " + which);
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        //如果不是dialog取消返回的，默认resultCode为OK
        setResult(Activity.RESULT_OK, data);
        finish();
    }
}
