package im.zego.gochat.protocol;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.google.gson.Gson;

import org.jetbrains.annotations.NotNull;
import org.json.JSONObject;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import im.zego.gochat.BackendApiConstants;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

import static im.zego.gochat.ErrorcodeConstants.GoChatErrorFailNetwork;
import static im.zego.gochat.ErrorcodeConstants.GoChatErrorJSONFormatInvalid;

public class APIBase {
    private static final String TAG = "APIBase";

    private static Gson mGson = new Gson();
    private static final Handler okHandler = new Handler(Looper.getMainLooper());

    public static JsonBuilder builder() {
        return new JsonBuilder();
    }

    public static Gson getGson() {
        return mGson;
    }

    public static class OkHttpInstance{

        private volatile static OkHttpInstance instance;
        private OkHttpClient mOkHttpClient;

        private OkHttpInstance() {
            OkHttpClient.Builder builder = new OkHttpClient.Builder()
                    .connectTimeout(15, TimeUnit.SECONDS)
                    .writeTimeout(20, TimeUnit.SECONDS)
                    .readTimeout(20, TimeUnit.SECONDS);
            mOkHttpClient = builder.build();
        }

        public static OkHttpClient getInstance()
        {
            if (instance == null) {
                synchronized (OkHttpInstance.class) {
                    if (instance == null) {
                        instance = new OkHttpInstance();
                    }
                }
            }
            return instance.mOkHttpClient;
        }

    }

    public static <T> void asyncPost(@NotNull String api, @NotNull final String reqJson, final Class<T> classType, final IAsyncPostCallback<T> reqCallback) {
        MediaType mediaType = MediaType.parse("application/json; charset=utf-8");
        RequestBody stringBody = RequestBody.Companion.create(reqJson, mediaType);
        Request request = new Request.Builder()
                .url(BackendApiConstants.BACKEND_API_URL + api)
                .post(stringBody)
                .build();

        OkHttpInstance.getInstance().newCall(request).enqueue(new Callback() {
            @Override
            public void onResponse(@NotNull Call call, @NotNull final Response response) {
                String str = "";
                try {
                    str = response.body().string();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                final String finalStr = str;
                Log.d(TAG, "onResponse: api: " + call.request().url().toString());
                Log.d(TAG, "onResponse: reqJson: " + reqJson);
                Log.d(TAG, "onResponse: respJson: " + finalStr);

                try {
                    JSONObject jsonObject = new JSONObject(finalStr);
                    final JSONObject retJsonObj = jsonObject.getJSONObject("ret");
                    final int retcode = retJsonObj.getInt("code");
                    final String retmsg = retJsonObj.getString("message");
                    String dataJson = "";
                    try {
                        dataJson = jsonObject.getJSONObject("data").toString();
                    }catch (Exception jsonException)
                    {
                        dataJson = "";
                    }

                    if (reqCallback != null) {
                        final T bean = mGson.fromJson(dataJson, classType);

                            okHandler.post(new Runnable() {
                                @Override
                                public void run() {
                                    reqCallback.onResponse(retcode, retmsg, bean);
                                }
                            });

                    }
                } catch (Exception jsonException) {
                    jsonException.printStackTrace();
                    if (reqCallback != null) {
                        okHandler.post(new Runnable() {
                            @Override
                            public void run() {
                                reqCallback.onResponse(GoChatErrorJSONFormatInvalid, "", null);
                            }
                        });
                    }
                }


            }

            @Override
            public void onFailure(@NotNull Call call, @NotNull IOException e) {
                Log.d(TAG, "onFailure: " + e.getMessage());

                if (reqCallback != null) {
                    okHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            reqCallback.onResponse(GoChatErrorFailNetwork, "", null);
                        }
                    });
                }

            }
        });

    }

}
