package im.zego.gochat.protocol;

import org.jetbrains.annotations.NotNull;

import im.zego.common.model.LoginInfo;
import im.zego.gochat.chatroom.ChatRoomManager;
import im.zego.gochat.model.HeartbeatRespInfo;
import im.zego.gochat.model.NoInfo;
import im.zego.gochat.rtc.RTCSDKManager;

public class LoginAPI {
    public static void login(@NotNull String nickName, @NotNull String avatar, @NotNull final ILoginCallback<LoginInfo> callback) {
        String reqJson = APIBase.builder()
                .addParams("nick_name", nickName)
                .addParams("avatar", avatar)
                .build();

        APIBase.asyncPost("/chat_room/login", reqJson,LoginInfo.class, new IAsyncPostCallback<LoginInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, LoginInfo responseJsonBean) {
                if (errorCode == 0) {
                    RTCSDKManager.getInstance().setUserID(responseJsonBean.getUid(), responseJsonBean.getNickName());
                }

                if (callback != null) {
                    callback.onLogin(errorCode, responseJsonBean);
                }
            }
        });
    }

    public static void logout(Long uid, final ICommonResponseCallback callback) {
        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id","")
                .build();

        APIBase.asyncPost("/chat_room/logout", reqJson,NoInfo.class, new IAsyncPostCallback<NoInfo>() {
            @Override
            public void onResponse(int retcode, @NotNull String message, NoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onResponse(retcode);
                }
            }
        });
    }

    public static void heartBeat(long uid, @NotNull IHeartBeatCallback callback) {
        String curRoomID = "";
        if (ChatRoomManager.getInstance().getRoomInfo() != null) {
            curRoomID = ChatRoomManager.getInstance().getRoomInfo().getRoomID();
        }

        String reqJson = APIBase.builder()
                .addParams("uid", uid)
                .addParams("room_id", curRoomID)
                .build();

        APIBase.asyncPost("/chat_room/heartbeat", reqJson, HeartbeatRespInfo.class, new IAsyncPostCallback<HeartbeatRespInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, HeartbeatRespInfo responseJsonBean) {
                if (callback != null) {
                    callback.onHeartBeat(errorCode, responseJsonBean);
                }
            }
        });
    }
}
