package im.zego.gochat.roomlist.activity;


import android.content.DialogInterface;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.animation.AlphaAnimation;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.PopupWindow;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;

import im.zego.common.application.ZegoApplication;
import im.zego.gochat.MainActivity;
import im.zego.gochat.R;
import im.zego.gochat.login.LoginInfoManager;
import im.zego.gochat.base.ZGBaseActivity;
import im.zego.gochat.chatroom.ChatRoomActivity;
import im.zego.gochat.chatroom.ChatRoomManager;
import im.zego.gochat.login.LoginActivity;
import im.zego.gochat.model.RoomInfo;
import im.zego.gochat.protocol.ICommonResponseCallback;
import im.zego.gochat.protocol.ICreateRoomCallback;
import im.zego.gochat.protocol.RoomAPI;
import im.zego.gochat.rtc.RTCSDKManager;
import im.zego.gochat.utils.GlideImageLoader;
import im.zego.gochat.utils.MiscUtils;
import im.zego.gochat.custom.ProcessView;
import im.zego.gochat.utils.ToastUtil;

import static im.zego.gochat.ErrorcodeConstants.ERROR_USER_OFF_LINE;
import static im.zego.gochat.ErrorcodeConstants.GoChatErrorFailNetwork;

public class RoomListActivity extends ZGBaseActivity  {
    private FrameLayout mFlRoomList;
    private ImageView mIvLogout;
    private FrameLayout mFlRoomListFragment;

    private ConstraintLayout mClAddRoom;

    private ProcessView mProcessView;

    private PopupWindow mPopupWindow;

    private BottomSheetDialog mBottomSheetDialog;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            getWindow().getDecorView().setSystemUiVisibility( View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR);
        }

        setContentView(R.layout.activity_room_list);

        if (!LoginInfoManager.getInstance().isLogin()) {
            MainActivity.restartLogin();
        }
        RTCSDKManager.getInstance().init(ZegoApplication.getAppContext());

        initView();
        initListener();
        initFragment();
    }

    private void initView() {
        mFlRoomList = findViewById(R.id.fl_room_list);
        mIvLogout = findViewById(R.id.iv_logout);
        mFlRoomListFragment = findViewById(R.id.fl_room_list_fragment);
        mClAddRoom = findViewById(R.id.cl_add_room);
        mProcessView = new ProcessView(RoomListActivity.this);

        GlideImageLoader.getInstance().displayImageByAvatar(LoginInfoManager.getInstance().getLoginInfo().getAvatar(),mIvLogout);
    }

    private void initListener() {
        mIvLogout.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mPopupWindow == null || !mPopupWindow.isShowing()) {
                    showPopupWindow();
                } else {
                    mPopupWindow.dismiss();
                }
            }
        });

        mClAddRoom.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showBottomSheetDialog();
                AlphaAnimation alphaAnimation = new AlphaAnimation(1.0f, 0f);
                alphaAnimation.setDuration(300);
                mClAddRoom.setAnimation(alphaAnimation);
                mClAddRoom.setVisibility(View.GONE);
            }
        });

    }

    private void initFragment() {

        Fragment fragment = getSupportFragmentManager().findFragmentById(R.id.fl_room_list_fragment);
        if(fragment == null) {
            FragmentTransaction beginTransaction = getSupportFragmentManager().beginTransaction();
            RoomListFragment roomListFragment = new RoomListFragment();
            beginTransaction.add(R.id.fl_room_list_fragment, roomListFragment);
            beginTransaction.commit();
        }
    }

    private void showPopupWindow() {
        LayoutInflater layoutInflater = LayoutInflater.from(this);
        View popupWindow = layoutInflater.inflate(R.layout.view_logout, mFlRoomList, false);
        popupWindow.findViewById(R.id.bt_logout).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mPopupWindow != null && mPopupWindow.isShowing()) {
                    mPopupWindow.dismiss();
                    mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mFlRoomList.getRootView());
                    LoginInfoManager.getInstance().logoutUser(new ICommonResponseCallback() {
                        @Override
                        public void onResponse(int errorCode) {
                            mProcessView.dismiss();
                            if(errorCode == 0)
                            {
                                String nickName = LoginInfoManager.getInstance().getLoginInfo().getNickName();
                                LoginInfoManager.getInstance().clearLoginInfo();
                                Intent intent = new Intent(RoomListActivity.this,LoginActivity.class);
                                intent.putExtra(LoginActivity.NICK_NAME,nickName);
                                startActivity(intent);
                                finish();
                            }else
                            {
                                ToastUtil.showWarnToast(R.string.fail_network);
                            }
                        }
                    });
                }
            }
        });

        mPopupWindow = new PopupWindow(popupWindow, ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        mPopupWindow.setOutsideTouchable(false);
        mPopupWindow.showAtLocation(mFlRoomList, Gravity.RIGHT | Gravity.TOP, MiscUtils.dp2px(RoomListActivity.this, 16), MiscUtils.dp2px(RoomListActivity.this, 76));

    }

    private void showBottomSheetDialog() {
        View view = View.inflate(RoomListActivity.this, R.layout.dialog_add_room, null);

        final EditText mEtRoomName = view.findViewById(R.id.et_room_name);
        final Button mBtCreateRoom = view.findViewById(R.id.bt_create_room);

        view.findViewById(R.id.v_top_line).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mBottomSheetDialog.dismiss();
            }
        });

        mEtRoomName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                if ("".equals(s.toString().trim())) {
                    mBtCreateRoom.setEnabled(false);
                } else {
                    mBtCreateRoom.setEnabled(true);
                }
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        mBtCreateRoom.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mBottomSheetDialog.dismiss();
                mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mFlRoomList.getRootView());

                RTCSDKManager.getInstance().initQualityManager(RoomListActivity.this);
                RoomAPI.createRoom(LoginInfoManager.getInstance().getLoginInfo().getUid(), mEtRoomName.getText().toString().trim(), new ICreateRoomCallback() {
                    @Override
                    public void onCreateRoom(int retcode, RoomInfo createdRoomInfo) {
                        if (retcode == 0) {
                            mProcessView.dismiss();
                            ChatRoomManager.getInstance().setRoomInfo(createdRoomInfo);
                            startActivity(new Intent(RoomListActivity.this, ChatRoomActivity.class));
                        }else if(retcode == GoChatErrorFailNetwork)
                        {
                            mProcessView.dismiss();
                            ToastUtil.showWarnToast(R.string.room_list_join_fail_network);
                        }else if(retcode == ERROR_USER_OFF_LINE){
                            MainActivity.restartLogin();
                        } else {
                            mProcessView.dismiss();
                            ToastUtil.showWarnToast(R.string.room_list_start_fail);
                        }
                    }
                });

            }
        });

        mBottomSheetDialog = new BottomSheetDialog(RoomListActivity.this, R.style.BottomSheetDialog);
        mBottomSheetDialog.setContentView(view);
        final BottomSheetBehavior mDialogBehavior = BottomSheetBehavior.from((View) view.getParent());
        mDialogBehavior.addBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                    mBottomSheetDialog.dismiss();
                    mDialogBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {
            }
        });
        Window localWindow = mBottomSheetDialog.getWindow();
        localWindow.setGravity(Gravity.BOTTOM);
        localWindow.setWindowAnimations(R.style.BottomSheetDialogAnim);

        mBottomSheetDialog.show();
        mBottomSheetDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
            @Override
            public void onDismiss(DialogInterface dialog) {
                AlphaAnimation alphaAnimation = new AlphaAnimation(0.0f, 1f);
                alphaAnimation.setDuration(300);
                mClAddRoom.setAnimation(alphaAnimation);
                mClAddRoom.setVisibility(View.VISIBLE);
            }
        });
    }


    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            moveTaskToBack(false);
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        RTCSDKManager.getInstance().uninit();
    }
}
