package im.zego.gochat.roomlist.activity;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import java.util.ArrayList;
import java.util.List;

import im.zego.gochat.MainActivity;
import im.zego.gochat.R;
import im.zego.gochat.login.LoginInfoManager;
import im.zego.gochat.chatroom.ChatRoomManager;
import im.zego.gochat.model.RoomListInfo;
import im.zego.gochat.protocol.IGetRoomListCallback;
import im.zego.gochat.protocol.RoomAPI;
import im.zego.gochat.roomlist.adapter.EmptyAndLoadMoreAdapterWrapper;
import im.zego.gochat.roomlist.adapter.RoomListAdapter;
import im.zego.gochat.roomlist.view.LoadMoreView;
import im.zego.gochat.roomlist.view.RoomListItemDecoration;
import im.zego.gochat.rtc.RTCSDKManager;
import im.zego.gochat.utils.ToastUtil;

import static im.zego.gochat.ErrorcodeConstants.GoChatErrorFailNetwork;

public class RoomListFragment extends Fragment {

    private static final int STATE_NORMAL = 0;
    private static final int STATE_REFRESH = 1;
    private static final int STATE_LOADING = 2;
    private static final int STATE_FINISH = 3;
    private static final int STATE_ERROR = 4;

    private static final int BEGIN_TIME = 0;

    private static final int COUNT_SIZE = 10;
    private long mTimestamp = BEGIN_TIME;

    private SwipeRefreshLayout mSrlRoomList;
    private RecyclerView mRvRoomList;

    private RoomListAdapter mAdapter;
    private EmptyAndLoadMoreAdapterWrapper emptyAndLoadMoreAdapterWrapper;

    private LoadMoreView loadMoreView;

    private int mState;

    private View view;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        if(view == null) {
            view = inflater.inflate(R.layout.fragment_room_list, null, false);
            initView(view);
            initRecycleView();
            initListener();
            initLoadData();
        }
        return view;
    }

    private void initView(View view)
    {
        mSrlRoomList = view.findViewById(R.id.srl_room_list);
        mRvRoomList = view.findViewById(R.id.rv_room_list);
    }

    private void initRecycleView()
    {
        LinearLayoutManager layoutManager = new LinearLayoutManager(getContext());
        layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        mRvRoomList.setLayoutManager(layoutManager);

        mAdapter = new RoomListAdapter();

        View emptyView = LayoutInflater.from(getContext()).inflate(R.layout.view_empty_list,mSrlRoomList,false);

        loadMoreView = new LoadMoreView(getContext());

        emptyAndLoadMoreAdapterWrapper = new EmptyAndLoadMoreAdapterWrapper(mAdapter);
        emptyAndLoadMoreAdapterWrapper.setEmptyView(emptyView);
        emptyAndLoadMoreAdapterWrapper.setLoadMoreView(loadMoreView);
        mRvRoomList.setAdapter(emptyAndLoadMoreAdapterWrapper);

        RoomListItemDecoration decoration = new RoomListItemDecoration( 15);
        mRvRoomList.addItemDecoration(decoration);
    }


    private void initListener()
    {
        mSrlRoomList.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                RoomListFragment.this.onRefresh();
            }
        });

        mAdapter.setOnClickListener(new RoomListAdapter.OnClickListener() {
            @Override
            public void onClick(View v, int position, RoomListInfo.RoomListDTO data) {
                if (!LoginInfoManager.getInstance().isLogin()) {
                    MainActivity.restartLogin();
                } else {
                    RTCSDKManager.getInstance().initQualityManager(getActivity());
                    ChatRoomManager.getInstance().loginRoom(getActivity(), LoginInfoManager.getInstance().getLoginInfo().getUid(), data.getRoomId());
                }
            }
        });

        mSrlRoomList.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                RoomListFragment.this.onRefresh();
            }
        });

        emptyAndLoadMoreAdapterWrapper.setOnLoadMoreListener(new EmptyAndLoadMoreAdapterWrapper.OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                RoomListFragment.this.onLoadMore();
            }
        });
    }

    private void initLoadData()
    {
        mSrlRoomList.setRefreshing(true);
        onRefresh();
    }

    private void onRefresh()
    {
        mState = STATE_REFRESH;

        mTimestamp = BEGIN_TIME;
        RoomAPI.getRoomList(LoginInfoManager.getInstance().getLoginInfo().getUid(), COUNT_SIZE, mTimestamp, new IGetRoomListCallback() {
            @Override
            public void onGetRoomList(int retcode, RoomListInfo roomInfoList) {

                mSrlRoomList.setRefreshing(false);
                if(retcode == 0)
                {
                    if(roomInfoList != null && roomInfoList.getRoomList() != null) {

                        List<RoomListInfo.RoomListDTO> list =roomInfoList.getRoomList();
                        if (list.size() < COUNT_SIZE) {
                            mState = STATE_FINISH;
                            loadMoreView.showFinish();
                        } else {
                            mState = STATE_NORMAL;
                        }

                        if(list.size() > 0)
                        {
                            mTimestamp = list.get(list.size()-1).getCreateTime();
                        }

                        mAdapter.refreshData(roomInfoList.getRoomList());
                        emptyAndLoadMoreAdapterWrapper.notifyDataSetChanged();
                    }else{
                        mState = STATE_NORMAL;
                        mAdapter.refreshData(new ArrayList<RoomListInfo.RoomListDTO>());
                        emptyAndLoadMoreAdapterWrapper.notifyDataSetChanged();
                    }
                }else {
                    if(retcode == GoChatErrorFailNetwork) {
                        ToastUtil.showWarnToast(R.string.fail_network);

                    }
                    mState = STATE_ERROR;
                }
            }
        });
    }

    private void onLoadMore()
    {
        if(mState == STATE_REFRESH) {
            return;
        }

        if(mState == STATE_NORMAL) {
            mState = STATE_LOADING;
            loadMoreView.showLoading();

            RoomAPI.getRoomList(LoginInfoManager.getInstance().getLoginInfo().getUid(), COUNT_SIZE, mTimestamp, new IGetRoomListCallback() {
                @Override
                public void onGetRoomList(int retcode, RoomListInfo roomInfoList) {
                    if(retcode == 0 )
                    {
                        if(roomInfoList != null && roomInfoList.getRoomList() != null) {

                            List<RoomListInfo.RoomListDTO> list =roomInfoList.getRoomList();
                            if (list.size() < COUNT_SIZE) {
                                mState = STATE_FINISH;
                                loadMoreView.showFinish();
                            } else {
                                mState = STATE_NORMAL;
                            }

                            if(list.size() > 0)
                            {
                                mTimestamp = list.get(list.size()-1).getCreateTime();
                            }

                            mAdapter.addData(roomInfoList.getRoomList());
                            emptyAndLoadMoreAdapterWrapper.notifyItemRangeInserted(mAdapter.getItemCount() - roomInfoList.getRoomList().size(), roomInfoList.getRoomList().size());
                        }else {
                            mState = STATE_ERROR;
                            loadMoreView.showFinish();
                        }
                    }else {
                        if(retcode == GoChatErrorFailNetwork) {
                            ToastUtil.showWarnToast(R.string.fail_network);
                        }

                        mState = STATE_ERROR;
                        loadMoreView.showFinish();
                    }
                }
            });


        }
    }

}
