package im.zego.gochat.roomlist.adapter;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.RecyclerView;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import im.zego.gochat.R;
import im.zego.gochat.model.RoomListInfo;


public class RoomListAdapter extends RecyclerView.Adapter<RoomListAdapter.RoomListViewHolder> {

    private List<RoomListInfo.RoomListDTO> mData = new ArrayList<>();

    public RoomListAdapter() {
    }

    @NonNull
    @Override
    public RoomListViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_room_list, parent, false);
        return new RoomListViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final @NonNull RoomListViewHolder holder, final int position) {

        RoomListInfo.RoomListDTO info = mData.get(holder.getAdapterPosition());
        holder.mTvRoomName.setText(info.getSubject());
        holder.mTvTotalPeople.setText(getSumString(info.getOnlineCount()));
        holder.mTvTotalGuest.setText(getSumString(info.getSpeakCount()));
        holder.mClRoomListItem.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(mOnClickListener != null)
                {
                    mOnClickListener.onClick(v,holder.getAdapterPosition(),mData.get(holder.getAdapterPosition()));
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        if(mData == null)
        {
            return 0;
        }
        return mData.size();
    }

    class RoomListViewHolder extends RecyclerView.ViewHolder {

        public ConstraintLayout mClRoomListItem;
        public TextView mTvRoomName;
        public TextView mTvTotalPeople;
        public TextView mTvTotalGuest;

        private RoomListViewHolder(View itemView) {
            super(itemView);
            mClRoomListItem = itemView.findViewById(R.id.cl_room_list_item);
            mTvRoomName = itemView.findViewById(R.id.tv_room_name);
            mTvTotalPeople = itemView.findViewById(R.id.tv_total_people);
            mTvTotalGuest = itemView.findViewById(R.id.tv_total_guest);

        }
    }

    public void refreshData(List<RoomListInfo.RoomListDTO> newData)
    {
        mData = newData;

    }


    public void addData(List<RoomListInfo.RoomListDTO> newData) {
        mData.addAll(newData);

    }

    private OnClickListener mOnClickListener;
    public interface OnClickListener {
        void onClick(View v, int position,RoomListInfo.RoomListDTO data);
    }

    public void setOnClickListener(OnClickListener onClickListener)
    {
        mOnClickListener = onClickListener;
    }

    private String getSumString(int sum)
    {
        if(sum < 1000)
        {
            return String.valueOf(sum);
        }else {
            return ((sum/100)/10.0) + "k";
        }
    }

}
