package im.zego.gochat.rtc;

import android.app.Application;
import android.content.Context;
import android.os.Environment;
import android.util.Log;

import org.jetbrains.annotations.NotNull;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import im.zego.common.proxy.QualityManagerProxy;
import im.zego.common.util.SystemUtil;
import im.zego.common.util.T;
import im.zego.gochat.AuthConstants;
import im.zego.gochat.login.LoginInfoManager;
import im.zego.gochat.model.NotifyInvitedInfo;
import im.zego.gochat.model.NotifyRefuseInviteInfo;
import im.zego.gochat.model.NotifyUserListUpdateInfo;
import im.zego.gochat.model.NotifyUserStateUpdateInfo;
import im.zego.gochat.protocol.APIBase;
import im.zego.gochat.rtc.callbacks.IRTCCommonCallback;
import im.zego.gochat.rtc.callbacks.IRTCEventCallback;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.callback.IZegoDestroyCompletionCallback;
import im.zego.zegoexpress.callback.IZegoEventHandler;
import im.zego.zegoexpress.constants.ZegoAudioCodecID;
import im.zego.zegoexpress.constants.ZegoLanguage;
import im.zego.zegoexpress.constants.ZegoPublisherState;
import im.zego.zegoexpress.constants.ZegoRoomState;
import im.zego.zegoexpress.constants.ZegoScenario;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoAudioConfig;
import im.zego.zegoexpress.entity.ZegoEngineConfig;
import im.zego.zegoexpress.entity.ZegoEngineProfile;
import im.zego.zegoexpress.entity.ZegoRoomExtraInfo;
import im.zego.zegoexpress.entity.ZegoStream;
import im.zego.zegoexpress.entity.ZegoUser;

import static im.zego.gochat.ErrorcodeConstants.ERROR_RTC_SDK_INVALID_USERID;
import static im.zego.gochat.ErrorcodeConstants.ERROR_RTC_SDK_NOT_INITIALIZED;
import static im.zego.gochat.ErrorcodeConstants.ERROR_RTC_SDK_NOT_IN_ROOM;
import static im.zego.zegoexpress.constants.ZegoRoomState.CONNECTED;
import static im.zego.zegoexpress.constants.ZegoUpdateType.ADD;
import static im.zego.zegoexpress.constants.ZegoUpdateType.DELETE;

public class RTCSDKManager {

    private static final String TAG = "RTCSDKManager";

    private RTCSDKManager() {
    }

    private static final class Holder {
        private static final RTCSDKManager INSTANCE = new RTCSDKManager();
    }

    public static RTCSDKManager getInstance() {
        return Holder.INSTANCE;
    }

    private ZegoExpressEngine mSDKEngine = null;
    private final ArrayList<WeakReference<IRTCEventCallback>> mEventCallbackList = new ArrayList<>();
    private ZegoUser mUserInfo = null;
    private String mRoomID = null;
    /**
     * [key] : uid
     * [value] : streamID
     */
    private final HashMap<Long, String> mUserStreamIDMap = new HashMap<>();

    private final HashMap<Long, Float> mUserVolumeMap = new HashMap<>();
    private boolean mHasFirstLoginRoom = false;

    public void addListener(IRTCEventCallback callback) {
        synchronized (RTCSDKManager.class) {
            if (findAlreayAddCallback(callback) == null) {
                WeakReference<IRTCEventCallback> weakCallback = new WeakReference<>(callback);
                mEventCallbackList.add(weakCallback);
            }
        }
    }

    public void addFirstListener(IRTCEventCallback callback) {
        synchronized (RTCSDKManager.class) {
            // 先删除后添加，确保第一响应
            WeakReference<IRTCEventCallback> findWeakCallback = findAlreayAddCallback(callback);
            if (findWeakCallback != null) {
                mEventCallbackList.remove(findWeakCallback);
            }

            WeakReference<IRTCEventCallback> weakCallback = new WeakReference<>(callback);
            mEventCallbackList.add(0, weakCallback);
        }
    }

    public void removeListener(IRTCEventCallback callback) {
        synchronized (RTCSDKManager.class) {
            WeakReference<IRTCEventCallback> weakCallback = findAlreayAddCallback(callback);
            if (weakCallback != null) {
                mEventCallbackList.remove(weakCallback);
            }
        }
    }

    public void clearEventCallbackList() {
        synchronized (RTCSDKManager.class) {
            mEventCallbackList.clear();
        }
    }

    public Object[] getEventCallbackList() {
        Object[] callbacks = null;
        synchronized (RTCSDKManager.class) {
            if (mEventCallbackList.size() > 0) {
                callbacks = mEventCallbackList.toArray();
            }
        }
        return callbacks;
    }

    private WeakReference<IRTCEventCallback> findAlreayAddCallback(IRTCEventCallback callback) {
        WeakReference<IRTCEventCallback> retWeakCallback = null;
        for (WeakReference<IRTCEventCallback> weakCallback : mEventCallbackList) {
            if (weakCallback.get() != null && weakCallback.get() == callback) {
                retWeakCallback = weakCallback;
                break;
            }
        }

        return retWeakCallback;
    }

    public void init(@NotNull Context application) {
        if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
            ZegoEngineConfig engineConfig = new ZegoEngineConfig();
            if (application.getExternalFilesDir(null) != null) {
                engineConfig.logConfig.logPath = application.getExternalFilesDir(null).getAbsolutePath() + File.separator + "zegologs";
            } else {
                engineConfig.logConfig.logPath = application.getFilesDir().getAbsolutePath() + File.separator + "zegologs";
            }
            engineConfig.advancedConfig.put("allow_verbose_print_high_frequency_content","true");
            engineConfig.advancedConfig.put("enable_callback_verbose","true");
            ZegoExpressEngine.setEngineConfig(engineConfig);
        }
        ZegoEngineProfile profile = new ZegoEngineProfile();
        profile.appID = AuthConstants.APP_ID;
        profile.appSign = AuthConstants.APP_SIGN;
        profile.scenario = ZegoScenario.GENERAL;
        profile.application = (Application) application.getApplicationContext();

        mSDKEngine = ZegoExpressEngine.createEngine(profile, mEventHandler);
//        mSDKEngine = ZegoExpressEngine.createEngine(AuthConstants.APP_ID, AuthConstants.APP_SIGN, false, ZegoScenario.GENERAL, (Application) application, mEventHandler);

        if(mSDKEngine == null){
            T.show("引擎创建失败，请检查appID ，appSign是否存在问题");
        }else {
            mSDKEngine.setDebugVerbose(true, ZegoLanguage.CHINESE);

            mSDKEngine.enableANS(true);
            ZegoAudioConfig audioConfig = new ZegoAudioConfig();
            audioConfig.bitrate = 24;
            audioConfig.codecID = ZegoAudioCodecID.LOW3;
            mSDKEngine.setAudioConfig(audioConfig);
            mSDKEngine.startPerformanceMonitor(3000);
        }
    }

    public void uninit() {
        clearEventCallbackList();
        ZegoExpressEngine.destroyEngine(new IZegoDestroyCompletionCallback() {
            @Override
            public void onDestroyCompletion() {

            }
        });

        mSDKEngine = null;
    }

    public void setUserID(long userID, @NotNull String userName) {
        mUserInfo = new ZegoUser(String.valueOf(userID), userName);
    }

    public void loginRoom(@NotNull String roomID, IRTCCommonCallback callback) {
        if (mSDKEngine == null) {
            callback.onRTCCallback(ERROR_RTC_SDK_NOT_INITIALIZED);
            return;
        } else if (mUserInfo == null) {
            callback.onRTCCallback(ERROR_RTC_SDK_INVALID_USERID);
            return;
        }

        mUserStreamIDMap.clear();
        mRoomID = roomID;
        QualityManagerProxy.getInstance().setRoomID(roomID);
        QualityManagerProxy.getInstance().setUserID(mUserInfo.userID);
        QualityManagerProxy.getInstance().setUserName(mUserInfo.userName);
        QualityManagerProxy.getInstance().setProductName("goenjoy");
        String language = SystemUtil.getSystemLanguage();
        Log.i(TAG,"系统语言："+language);
        if (language.equals("en")){
            QualityManagerProxy.getInstance().setLanguageType(false);
        }else {
            QualityManagerProxy.getInstance().setLanguageType(true);
        }
        QualityManagerProxy.getInstance().setLoginOnStart();
        Log.i(TAG,"goChat开始登陆");
        mSDKEngine.loginRoom(roomID, mUserInfo);
        callback.onRTCCallback(0);
        mSDKEngine.startSoundLevelMonitor();
    }

    public void logoutRoom() {
        if (mRoomID != null) {
            System.out.println("goChat登出");
            mSDKEngine.logoutRoom(mRoomID);
            mRoomID = null;
            mUserStreamIDMap.clear();
            mHasFirstLoginRoom = false;
        }
        if (mSDKEngine != null) {
            mSDKEngine.stopSoundLevelMonitor();
        }
    }

    public void startPublishAudio(@NotNull IRTCCommonCallback callback) {
        if (mSDKEngine == null) {
            callback.onRTCCallback(ERROR_RTC_SDK_NOT_INITIALIZED);
            return;
        } else if (mUserInfo == null) {
            callback.onRTCCallback(ERROR_RTC_SDK_INVALID_USERID);
            return;
        } else if (mRoomID == null) {
            callback.onRTCCallback(ERROR_RTC_SDK_NOT_IN_ROOM);
            return;
        }

        String publishAudioStreamID = String.format("a_gochat_main_%s_%s", mRoomID,mUserInfo.userID);
        QualityManagerProxy.getInstance().setPublishStreamID(publishAudioStreamID);
        mSDKEngine.enableCamera(false);
        mSDKEngine.startPublishingStream(publishAudioStreamID);
        if (callback != null) {
            callback.onRTCCallback(0);
        }
    }

    public void stopPublishAudio(IRTCCommonCallback callback) {
        if (mSDKEngine == null) {
            if (callback != null) {
                callback.onRTCCallback(ERROR_RTC_SDK_NOT_INITIALIZED);
            }
            return;
        }

        mSDKEngine.stopPublishingStream();
        if (callback != null) {
            callback.onRTCCallback(0);
        }
    }

    public void muteMicrophone(boolean isMute){
        mSDKEngine.muteMicrophone(isMute);
    }

    public void stopPlayingAudio(long userID) {
        String userStreamID = mUserStreamIDMap.get(userID);
        if (userStreamID != null) {
            mSDKEngine.stopPlayingStream(userStreamID);
            mUserStreamIDMap.remove(userID);
        }
    }

    private final IZegoEventHandler mEventHandler = new IZegoEventHandler() {
        @Override
        public void onDebugError(int errorCode, String funcName, String info) {
            super.onDebugError(errorCode, funcName, info);
        }

        @Override
        public void onPublisherStateUpdate(String streamID, ZegoPublisherState state, int errorCode, JSONObject extendedData) {
            super.onPublisherStateUpdate(streamID, state, errorCode, extendedData);
        }

        @Override
        public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData) {
            super.onRoomStreamUpdate(roomID, updateType, streamList, extendedData);

            if (mSDKEngine == null) {
                // ERROR_RTC_SDK_NOT_INITIALIZED
                return;
            } else if (mRoomID == null) {
                // ERROR_RTC_SDK_NOT_IN_ROOM
                return;
            }

            for (ZegoStream stream : streamList) {
                if (updateType == ADD) {
                    QualityManagerProxy.getInstance().setPlayerStreamOnStart(stream.streamID);
                    mSDKEngine.startPlayingStream(stream.streamID, null);
                    mUserStreamIDMap.put(Long.parseLong(stream.user.userID), stream.streamID);
                } else if (updateType == DELETE) {
                    mSDKEngine.stopPlayingStream(stream.streamID);
                    mUserStreamIDMap.remove(Long.parseLong(stream.user.userID));
                }
            }
        }

        @Override
        public void onRoomExtraInfoUpdate(String roomID, ArrayList<ZegoRoomExtraInfo> roomExtraInfoList) {
            super.onRoomExtraInfoUpdate(roomID, roomExtraInfoList);

            if (roomExtraInfoList == null) {
                return;
            }

            for (ZegoRoomExtraInfo extraInfo : roomExtraInfoList) {
                try {
                    JSONObject jsonObject = new JSONObject(extraInfo.value);
                    int cmd = jsonObject.getInt("cmd");
                    Log.d(TAG, String.format("cmd:%d loginUserID:%d", cmd, LoginInfoManager.getInstance().getLoginInfo().getUid()));
                    Log.d(TAG, "data:" + jsonObject.getString("data"));
                    if (cmd == 6002) {  // 成员状态变更通知
                        NotifyUserStateUpdateInfo notifyInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyUserStateUpdateInfo.class);
                        Log.d(TAG, "type:" + notifyInfo.type);  // 1、举手操作 2、角色变更 3、麦克风操作
//                        for (WeakReference<IRTCEventCallback> weakCallback : mEventCallbackList) {
//                            if (weakCallback.get() != null) {
//                                weakCallback.get().onUserStateUpdateNotify(notifyInfo);
//                            }
//                        }

                        Object[] callbacks = getEventCallbackList();
                        if (callbacks != null) {
                            for (int i = 0; i < callbacks.length; i++) {
                                if (((WeakReference<IRTCEventCallback>) callbacks[i]).get() != null) {
                                    ((WeakReference<IRTCEventCallback>) callbacks[i]).get().onUserStateUpdateNotify(notifyInfo);
                                }
                            }
                        }

                    } else if (cmd == 6003) { // 成员列表变更通知
                        NotifyUserListUpdateInfo notifyInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyUserListUpdateInfo.class);
//                        for (WeakReference<IRTCEventCallback> weakCallback : mEventCallbackList) {
//                            if (weakCallback.get() != null) {
//                                weakCallback.get().onUserListUpdateNotify(notifyInfo);
//                            }
//                        }

                        Object[] callbacks = getEventCallbackList();
                        if (callbacks != null) {
                            for (int i = 0; i < callbacks.length; i++) {
                                if (((WeakReference<IRTCEventCallback>) callbacks[i]).get() != null) {
                                    ((WeakReference<IRTCEventCallback>) callbacks[i]).get().onUserListUpdateNotify(notifyInfo);
                                }
                            }
                        }

                    } else if (cmd == 6006) {
//                        for (WeakReference<IRTCEventCallback> weakCallback : mEventCallbackList) {
//                            if (weakCallback.get() != null) {
//                                weakCallback.get().onRoomCloseNotify();
//                            }
//                        }

                        Object[] callbacks = getEventCallbackList();
                        if (callbacks != null) {
                            for (int i = 0; i < callbacks.length; i++) {
                                if (((WeakReference<IRTCEventCallback>) callbacks[i]).get() != null) {
                                    ((WeakReference<IRTCEventCallback>) callbacks[i]).get().onRoomCloseNotify();
                                }
                            }
                        }
                    }
                } catch (JSONException jsonException) {
                    jsonException.printStackTrace();
                }
            }

        }

        @Override
        public void onIMRecvCustomCommand(String roomID, ZegoUser fromUser, String command) {
            super.onIMRecvCustomCommand(roomID, fromUser, command);

            if (command == null || command.length() == 0) {
                return;
            }

            try {
                JSONObject jsonObject = new JSONObject(command);
                int cmd = jsonObject.getInt("cmd");
                Log.d(TAG, "cmd:" + cmd);
                Log.d(TAG, "data:" + jsonObject.getString("data"));
                if (cmd == 6004) {
                    NotifyInvitedInfo notifyInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyInvitedInfo.class);
//                    for (WeakReference<IRTCEventCallback> weakCallback : mEventCallbackList) {
//                        if (weakCallback.get() != null) {
//                            weakCallback.get().onInvitedNotify(notifyInfo);
//                        }
//                    }
                    Object[] callbacks = getEventCallbackList();
                    if (callbacks != null) {
                        for (int i = 0; i < callbacks.length; i++) {
                            if (((WeakReference<IRTCEventCallback>) callbacks[i]).get() != null) {
                                ((WeakReference<IRTCEventCallback>) callbacks[i]).get().onInvitedNotify(notifyInfo);
                            }
                        }
                    }
                } else if (cmd == 6005) {
                    NotifyRefuseInviteInfo notifyInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyRefuseInviteInfo.class);
//                    for (WeakReference<IRTCEventCallback> weakCallback : mEventCallbackList) {
//                        if (weakCallback.get() != null) {
//                            weakCallback.get().onRefusedInviteNotify(notifyInfo);
//                        }
//                    }
                    Object[] callbacks = getEventCallbackList();
                    if (callbacks != null) {
                        for (int i = 0; i < callbacks.length; i++) {
                            if (((WeakReference<IRTCEventCallback>) callbacks[i]).get() != null) {
                                ((WeakReference<IRTCEventCallback>) callbacks[i]).get().onRefusedInviteNotify(notifyInfo);
                            }
                        }
                    }
                }
            } catch (JSONException jsonException) {
                jsonException.printStackTrace();
            }
        }

        /**
         * 本地采集音频声浪回调
         *
         * 回调通知周期为 100 ms。
         * @param soundLevel 本地采集的声浪值，取值范围为 0.0 ~ 100.0
         */
        @Override
        public void onCapturedSoundLevelUpdate(float soundLevel) {
            super.onCapturedSoundLevelUpdate(soundLevel);
            mUserVolumeMap.clear();
            mUserVolumeMap.put(Long.valueOf(mUserInfo.userID),soundLevel);
            Object[] callbacks = getEventCallbackList();
            if (callbacks != null) {
                for (int i = 0; i < callbacks.length; i++) {
                    if (((WeakReference<IRTCEventCallback>) callbacks[i]).get() != null) {
                        ((WeakReference<IRTCEventCallback>) callbacks[i]).get().onUserVolumeUpdate(mUserVolumeMap);
                    }
                }
            }
        }

        /**
         * 远端拉流音频声浪回调
         *
         * 回调通知周期为 100 ms。
         * @param soundLevels 远端的声浪键值对，key 为流 ID，value 为对应的流的声浪值，value 取值范围为 0.0 ~ 100.0
         */
        @Override
        public void onRemoteSoundLevelUpdate(HashMap<String, Float> soundLevels) {
            super.onRemoteSoundLevelUpdate(soundLevels);
            mUserVolumeMap.clear();
            // 根据流id 更新对应的user的音量
            for (Map.Entry<Long, String> uidStreamPair : mUserStreamIDMap.entrySet()) {
                long uid = uidStreamPair.getKey();
                String streamID = uidStreamPair.getValue();
                for (Map.Entry<String, Float> pair : soundLevels.entrySet()) {
                    if(pair.getKey().equals(streamID)){
                        mUserVolumeMap.put(uid,pair.getValue());
                    }
                }
            }
            Object[] callbacks = getEventCallbackList();
            if (callbacks != null) {
                for (int i = 0; i < callbacks.length; i++) {
                    if (((WeakReference<IRTCEventCallback>) callbacks[i]).get() != null) {
                        ((WeakReference<IRTCEventCallback>) callbacks[i]).get().onUserVolumeUpdate(mUserVolumeMap);
                    }
                }
            }
        }

        @Override
        public void onRoomStateUpdate(String roomID, ZegoRoomState state, int errorCode, JSONObject extendedData) {
            super.onRoomStateUpdate(roomID, state, errorCode, extendedData);

            if (state == CONNECTED) {
                QualityManagerProxy.getInstance().setLoginOnFinish();
                boolean isReconnected = false;
                if (!mHasFirstLoginRoom) {
                    mHasFirstLoginRoom = true;
                } else {
                    isReconnected = true;
                }

                if (isReconnected) {
                    Log.d(TAG, "detect RTC Room reconnected");
//                    for (WeakReference<IRTCEventCallback> weakCallback : mEventCallbackList) {
//                        if (weakCallback.get() != null) {
//                            weakCallback.get().onRoomReconnected();
//                        }
//                    }

                    Object[] callbacks = getEventCallbackList();
                    if (callbacks != null) {
                        for (int i = 0; i < callbacks.length; i++) {
                            if (((WeakReference<IRTCEventCallback>) callbacks[i]).get() != null) {
                                ((WeakReference<IRTCEventCallback>) callbacks[i]).get().onRoomReconnected();
                            }
                        }
                    }
                }
            }
        }

        @Override
        public void onRecvExperimentalAPI(String content) {
            super.onRecvExperimentalAPI(content);
//            System.out.println("goChat："+content);
            QualityManagerProxy.getInstance().parsingLog(content);
        }

        @Override
        public void onPlayerRecvAudioFirstFrame(String streamID) {
            super.onPlayerRecvAudioFirstFrame(streamID);
            QualityManagerProxy.getInstance().setPlayerStreamOnFirstFrame(streamID);
        }

        @Override
        public void onPlayerRenderVideoFirstFrame(String streamID) {
            super.onPlayerRenderVideoFirstFrame(streamID);
            QualityManagerProxy.getInstance().setPlayerStreamOnFirstFrame(streamID);
        }
    };

    /**
     * 初始化质量报告本地插件
     */
    public void initQualityManager(Context context){
        QualityManagerProxy.getInstance().init(context.getApplicationContext());
        QualityManagerProxy.getInstance().setAppVersion(SystemUtil.getAppVersion(context.getApplicationContext()));
    }
}
