package im.zego.gochat.utils;

import android.app.Activity;
import android.app.Application;
import android.content.ComponentCallbacks;
import android.content.Context;
import android.content.res.Configuration;
import android.os.Bundle;
import android.util.DisplayMetrics;

/**
 * 屏幕适配工具类(适配宽度)
 */
public class ScreenAdaptationUtil {

    private static DisplayMetrics mAppDisplayMetrics;
    private static float mDensity;
    private static float mScaledDensity;
    private static float TARGET_WIDTH;

    public static void init(final Context application, float targetWidth)
    {

        TARGET_WIDTH = targetWidth;
        mAppDisplayMetrics = application.getResources().getDisplayMetrics();


        if(mDensity == 0) {
            mDensity = mAppDisplayMetrics.density;
            mScaledDensity = mAppDisplayMetrics.scaledDensity;

            application.registerComponentCallbacks(new ComponentCallbacks() {
                @Override
                public void onConfigurationChanged(Configuration newConfig) {
                    if (newConfig != null && newConfig.fontScale > 0) {
                        mScaledDensity = application.getResources().getDisplayMetrics().scaledDensity;
                    }
                }

                @Override
                public void onLowMemory() {
                }
            });

        }

        ((Application)application).registerActivityLifecycleCallbacks(new Application.ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
                setCustomDensity(activity);
            }
            @Override
            public void onActivityStarted(Activity activity) {
            }
            @Override
            public void onActivityResumed(Activity activity) {
            }
            @Override
            public void onActivityPaused(Activity activity) {
            }
            @Override
            public void onActivityStopped(Activity activity) {
            }
            @Override
            public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
            }
            @Override
            public void onActivityDestroyed(Activity activity) {

            }
        });
    }


    public static void setCustomDensity(final Activity activity )
    {

        float targetDensity = mAppDisplayMetrics.widthPixels / TARGET_WIDTH;
        float targetScaleDensity = targetDensity * (mScaledDensity / mDensity);
        int targetDensityDpi = (int) (160 * targetDensity);


        DisplayMetrics activityDisplayMetrics = activity.getResources().getDisplayMetrics();
        activityDisplayMetrics.density = targetDensity;
        activityDisplayMetrics.scaledDensity = targetScaleDensity;
        activityDisplayMetrics.densityDpi = targetDensityDpi;



    }


}
