package im.zego.ktv.chorus.custom;

import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.SeekBar;
import android.widget.Switch;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;

import im.zego.ktv.R;
import im.zego.ktv.custom.BaseBottomSheetDialogFragment;
import im.zego.ktv.custom.DefaultSeekBar;
import im.zego.ktv.chorus.ktvroom.adapter.EffectsAdjustAdapter;
import im.zego.ktv.chorus.ktvroom.adapter.VolumeAdjustAdapter;
import im.zego.ktv.chorus.model.EffectsItemInfo;
import im.zego.ktv.chorus.model.VolumeAdjustInfo;
import im.zego.ktv.chorus.rtc.RTCSDKManager;
import im.zego.ktv.chorus.rtc.ZGKTVPlayerManager;
import im.zego.zegoexpress.constants.ZegoReverbPreset;

/**
 * 调音弹窗
 */
public class AdjustVolumeDialog extends BaseBottomSheetDialogFragment implements EffectsAdjustAdapter.OnClickListener, VolumeAdjustAdapter.onSeekBarChange,
        CompoundButton.OnCheckedChangeListener {

    private EffectsAdjustAdapter mEffectsAdjustAdapter;
    ArrayList<EffectsItemInfo> mEffectsItemInfos = new ArrayList<>();
    private RecyclerView mEffectsList;

    private VolumeAdjustAdapter mVolumeAdjustAdapter;
    private ArrayList<VolumeAdjustInfo> mVolumeAdjustInfos = new ArrayList<>();
    private RecyclerView mVolumeList;

    private TextView reset;

    private TextView mLoopBack;
    private TextView mLoopbackDesc;
    private TextView mTvAuricularGyrus;
    private DefaultSeekBar mSbAuricularGyrus;
    private Switch mLoopbackSwitch;
    private View mViewAuricularGyrus;

    @Override
    public int getLayoutResId() {
        return R.layout.ktv_dialog_adjust_volume;
    }

    @Override
    public void initView(Bundle savedInstanceState) {
        setHeight(327);
        setShowHeight(true); //设置弹窗最大高度


        mEffectsList = rootView.findViewById(R.id.effects_list);
        mVolumeList = rootView.findViewById(R.id.volume_list);
        reset = rootView.findViewById(R.id.reset);
        mLoopBack = rootView.findViewById(R.id.loop_back);
        mLoopbackDesc = rootView.findViewById(R.id.loopback_desc);
        mTvAuricularGyrus = rootView.findViewById(R.id.tv_auricular_gyrus);
        mSbAuricularGyrus = rootView.findViewById(R.id.sb_auricular_gyrus);
        mLoopbackSwitch = rootView.findViewById(R.id.loopback_switch);
        mViewAuricularGyrus = rootView.findViewById(R.id.view_auricular_gyrus);

        mSbAuricularGyrus.setProgress(RTCSDKManager.getInstance().getHeadphoneMonitorVolumeNum());

        initVolumeData();
        initEffectData();
        initListener();

        mSbAuricularGyrus.setTouch(false);
        mLoopbackSwitch.setChecked(RTCSDKManager.getInstance().isLoopbackSwitch());

    }

    private void initListener() {
        reset.setOnClickListener(v -> {

            RTCSDKManager.getInstance().initAudioProcessParams();

            mLoopbackSwitch.setChecked(false);
            mVolumeAdjustInfos.clear();
            mVolumeAdjustInfos.add(new VolumeAdjustInfo(getContext().getString(R.string.ktv_mv_volume), 50, 100));
            mVolumeAdjustInfos.add(new VolumeAdjustInfo(getContext().getString(R.string.ktv_cap_volume), 100, 150));
            mVolumeAdjustAdapter.initAdjustItems(mVolumeAdjustInfos);
            resetVolume();
            closeHeadphoneMonitor();
            mSbAuricularGyrus.setProgress(50);
            if (mEffectsAdjustAdapter != null) {
                for (EffectsItemInfo info : mEffectsItemInfos) {
                    if (info.getIsSelected()) {
                        info.setIsSelected(false);
                        break;
                    }
                }
                selectItem(3);
                mEffectsAdjustAdapter.initEffectItems(mEffectsItemInfos);
            }

        });


        mSbAuricularGyrus.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {

            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {
                //耳返音量
                RTCSDKManager.getInstance().setLoopBackVolume(seekBar.getProgress());
            }
        });

        mLoopbackSwitch.setSaveEnabled(false);
        mLoopbackSwitch.setOnCheckedChangeListener(this);
    }


    /**
     * 伴奏、人声 音量设置
     */
    private void initVolumeData() {

        mVolumeAdjustInfos.clear();
        mVolumeAdjustInfos.add(new VolumeAdjustInfo(getContext().getString(R.string.ktv_mv_volume), ZGKTVPlayerManager.getInstance().getVolumeNum(), 100));
        mVolumeAdjustInfos.add(new VolumeAdjustInfo(getContext().getString(R.string.ktv_cap_volume), RTCSDKManager.getInstance().getCaptureVolumeNum(), 150));

        mVolumeAdjustAdapter = new VolumeAdjustAdapter();
        mVolumeAdjustAdapter.setListener(this);
        mVolumeAdjustAdapter.initAdjustItems(mVolumeAdjustInfos);
        mVolumeList.setLayoutManager(new LinearLayoutManager(this.getContext()));
        mVolumeList.setAdapter(mVolumeAdjustAdapter);
    }


    /**
     * 混响
     */
    private void initEffectData() {

        mEffectsItemInfos.clear();
        mEffectsItemInfos.add(new EffectsItemInfo(getContext().getString(R.string.ktv_origin_effect), false, R.mipmap.ktv_origin_effect));
        mEffectsItemInfos.add(new EffectsItemInfo(getContext().getString(R.string.ktv_pop_effect), false, R.mipmap.ktv_pop_effect));
        mEffectsItemInfos.add(new EffectsItemInfo(getContext().getString(R.string.ktv_rock_effect), false, R.mipmap.ktv_rock_effect));
        mEffectsItemInfos.add(new EffectsItemInfo(getContext().getString(R.string.ktv_record_effect), false, R.mipmap.ktv_record_effect));
        mEffectsItemInfos.add(new EffectsItemInfo(getContext().getString(R.string.ktv_concert_effect), false, R.mipmap.ktv_concert_effect));
        mEffectsItemInfos.add(new EffectsItemInfo(getContext().getString(R.string.ktv_ktv_effect), false, R.mipmap.ktv_effect));
        mEffectsItemInfos.add(new EffectsItemInfo(getContext().getString(R.string.ktv_room_effct), false, R.mipmap.ktv_room_effect));
        mEffectsItemInfos.add(new EffectsItemInfo(getContext().getString(R.string.ktv_church_effect), false, R.mipmap.ktv_church_effect));
        mEffectsItemInfos.add(new EffectsItemInfo(getContext().getString(R.string.ktv_music_hall_effect), false, R.mipmap.ktv_music_hall_effect));
        selectItem(RTCSDKManager.getInstance().getVerbPresetPos());
        mEffectsAdjustAdapter = new EffectsAdjustAdapter();
        mEffectsAdjustAdapter.setListener(this);
        mEffectsAdjustAdapter.initEffectItems(mEffectsItemInfos);
        mEffectsList.setLayoutManager(new LinearLayoutManager(this.getContext(), RecyclerView.HORIZONTAL, false));
        mEffectsList.setAdapter(mEffectsAdjustAdapter);
    }

    private void selectItem(int itemPosition) {
        for (int index = 0; index < mEffectsItemInfos.size(); index++) {
            if (mEffectsItemInfos.get(index).getIsSelected()) {
                mEffectsItemInfos.get(index).setIsSelected(false);
                break;
            }
        }
        mEffectsItemInfos.get(itemPosition).setIsSelected(true);
    }

    @Override
    public void onClick(View v, int position) {
        selectItem(position);
        if (mEffectsItemInfos.get(position).getItemName().equals(getString(R.string.ktv_church_effect))) {
            RTCSDKManager.getInstance().setVerbPresetMode(ZegoReverbPreset.LARGE_ROOM,position);
        } else if (mEffectsItemInfos.get(position).getItemName().equals(getString(R.string.ktv_origin_effect))) {
            RTCSDKManager.getInstance().setVerbPresetMode(ZegoReverbPreset.NONE,position);
        } else if (mEffectsItemInfos.get(position).getItemName().equals(getString(R.string.ktv_pop_effect))) {
            RTCSDKManager.getInstance().setVerbPresetMode(ZegoReverbPreset.POPULAR,position);
        } else if (mEffectsItemInfos.get(position).getItemName().equals(getString(R.string.ktv_rock_effect))) {
            RTCSDKManager.getInstance().setVerbPresetMode(ZegoReverbPreset.ROCK,position);
        } else if (mEffectsItemInfos.get(position).getItemName().equals(getString(R.string.ktv_record_effect))) {
            RTCSDKManager.getInstance().setVerbPresetMode(ZegoReverbPreset.RECORDING_STUDIO,position);
        } else if (mEffectsItemInfos.get(position).getItemName().equals(getString(R.string.ktv_concert_effect))) {
            RTCSDKManager.getInstance().setVerbPresetMode(ZegoReverbPreset.VOCAL_CONCERT,position);
        } else if (mEffectsItemInfos.get(position).getItemName().equals(getString(R.string.ktv_ktv_effect))) {
            RTCSDKManager.getInstance().setVerbPresetMode(ZegoReverbPreset.KTV,position);
        } else if (mEffectsItemInfos.get(position).getItemName().equals(getString(R.string.ktv_origin_effect))) {
            RTCSDKManager.getInstance().setVerbPresetMode(ZegoReverbPreset.SOFT_ROOM,position);
        } else if (mEffectsItemInfos.get(position).getItemName().equals(getString(R.string.ktv_music_hall_effect))) {
            RTCSDKManager.getInstance().setVerbPresetMode(ZegoReverbPreset.CONCERT_HALL,position);
        }

        if (mEffectsAdjustAdapter != null) {
            mEffectsAdjustAdapter.initEffectItems(mEffectsItemInfos);
        }
    }

    @Override
    public void onSeekBarChange(SeekBar seekBar, VolumeAdjustInfo info) {
        if (info.getItemName().equals(getContext().getString(R.string.ktv_mv_volume))) {
            //伴奏音量
            mVolumeAdjustInfos.get(0).setProgress(seekBar.getProgress());
            ZGKTVPlayerManager.getInstance().setVolume(seekBar.getProgress());
        } else if (info.getItemName().equals(getContext().getString(R.string.ktv_cap_volume))) {
            //人声音量
            mVolumeAdjustInfos.get(1).setProgress(seekBar.getProgress());
            RTCSDKManager.getInstance().setCaptureVolume(seekBar.getProgress());
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        //耳返开关
        RTCSDKManager.getInstance().enableHeadphoneMonitor(isChecked);

        if (isChecked) {
            openHeadphoneMonitor();
        } else {
            closeHeadphoneMonitor();
        }
    }

    /**
     * 耳返关闭
     */
    private void closeHeadphoneMonitor() {
        mViewAuricularGyrus.setVisibility(View.VISIBLE);
        mLoopBack.setTextColor(Color.parseColor("#80ffffff"));
        mLoopbackDesc.setTextColor(Color.parseColor("#80736588"));
        mTvAuricularGyrus.setTextColor(Color.parseColor("#80ffffff"));
        //默认不可滑动
        mSbAuricularGyrus.setTouch(false);
    }

    /**
     * 耳返开启
     */
    private void openHeadphoneMonitor() {
        mViewAuricularGyrus.setVisibility(View.GONE);
        mLoopBack.setTextColor(Color.parseColor("#ffffff"));
        mLoopbackDesc.setTextColor(Color.parseColor("#736588"));
        mTvAuricularGyrus.setTextColor(Color.parseColor("#ffffff"));
        //默认不可滑动
        mSbAuricularGyrus.setTouch(true);
    }

    public void resetVolume() {
        RTCSDKManager.getInstance().setLoopBackVolume(50);
        ZGKTVPlayerManager.getInstance().setVolume(50);
        RTCSDKManager.getInstance().setCaptureVolume(100);
    }


}
