package im.zego.ktv.chorus.custom;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import java.text.SimpleDateFormat;
import java.util.TimeZone;

import im.zego.ktv.R;
import im.zego.ktv.custom.SongSwitchView;
import im.zego.lyricview.ZegoLyricView;
import im.zego.lyricview.model.ZGKTVLyric;
import im.zego.pitchview.ZegoPitchView;

public class MusicPlayerMenu extends ConstraintLayout {


    public static final int OFF_STAGE = 1;
    public static final int ORDER_SONG = OFF_STAGE + 1;
    public static final int COMING_SOON = ORDER_SONG + 1;
    public static final int START_PLAYING = COMING_SOON + 1;

    private ConstraintLayout mLyricArea;
    private ZegoLyricView mZegoLyricView;
    private TextView mSongName;
    private TextView mPlayTime;
    private SongSwitchView mSongSwitch;
    private ConstraintLayout mNoSongArea;
    private ConstraintLayout mKtvClScore;
    private TextView mKtvTvScore;
    private TextView mSongTips;
    private TextView mSelectWhenNoSong;
    private ConstraintLayout mComingSong;
    private TextView mSongNameTips;

    private int state = OFF_STAGE;
    private String songName;
    private boolean hasAccompany;
    private boolean hasOriginal;
    private boolean showPitch;
    private Long playTime;
    private Long totalTime;
    private String songId;
    OnClickListener mOrderListener;
    OnClickListener mOffStageListener;

    private ZegoPitchView mPitchView;

    private int mScore = 0;

    public MusicPlayerMenu(@NonNull Context context) {
        this(context,null);
    }

    public MusicPlayerMenu(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public MusicPlayerMenu(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        View view = View.inflate(context, R.layout.ktv_music_player_menu,this);
        initView();
        initListener();
        updateData(state);
        switchState(state);
    }

    private void initView(){
        mLyricArea = findViewById(R.id.lyric_area);
        mZegoLyricView = findViewById(R.id.lyric_view);
        mSongName = findViewById(R.id.song_name);
        mPlayTime = findViewById(R.id.play_time);
        mSongSwitch = findViewById(R.id.song_switch);
        mNoSongArea = findViewById(R.id.no_song_area);
        mKtvClScore = findViewById(R.id.ktv_cl_score);
        mKtvTvScore = findViewById(R.id.ktv_tv_score);
        mSongTips = findViewById(R.id.song_tips);
        mSelectWhenNoSong = findViewById(R.id.select_when_no_song);
        mComingSong = findViewById(R.id.coming_song);
        mSongNameTips = findViewById(R.id.song_name_tips);
        mPitchView = findViewById(R.id.ktv_pitch);
    }

    private void initListener(){
        mSelectWhenNoSong.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if(state == OFF_STAGE){
                    if(mOffStageListener != null){
                        mOffStageListener.onClick(mSelectWhenNoSong);
                    }
                }else if(state == ORDER_SONG){
                    if(mOrderListener != null){
                        mOrderListener.onClick(mSelectWhenNoSong);
                    }
                }
            }
        });
    }

    public void setListener(OnClickListener orderListener,OnClickListener offStageListener){
        mOrderListener = orderListener;
        mOffStageListener = offStageListener;
    }


    private void switchState(int state){
        if(state == OFF_STAGE){
            mLyricArea.setVisibility(GONE);
            mZegoLyricView.setVisibility(GONE);
            mNoSongArea.setVisibility(VISIBLE);
            mComingSong.setVisibility(GONE);
        }else if(state == ORDER_SONG){
            mLyricArea.setVisibility(GONE);
            mZegoLyricView.setVisibility(GONE);
            mNoSongArea.setVisibility(VISIBLE);
            mComingSong.setVisibility(GONE);
        }else if(state == COMING_SOON){
            mLyricArea.setVisibility(VISIBLE);
            mZegoLyricView.setVisibility(GONE);
            mNoSongArea.setVisibility(GONE);
            mComingSong.setVisibility(VISIBLE);
            mPitchView.setVisibility(GONE);
            mKtvClScore.setVisibility(GONE);
        }else if(state == START_PLAYING){
            mLyricArea.setVisibility(VISIBLE);
            mZegoLyricView.setVisibility(VISIBLE);
            mNoSongArea.setVisibility(GONE);
            mComingSong.setVisibility(GONE);

            if(showPitch){
                mPitchView.setVisibility(VISIBLE);
                mKtvClScore.setVisibility(VISIBLE);
            }else {
                mPitchView.setVisibility(GONE);
                mKtvClScore.setVisibility(GONE);
            }
        }
    }

    private void updateData(int state){
        if(state == OFF_STAGE){
            mSongTips.setText("欢迎体验即构KTV，请上麦后点播歌曲");
            Drawable drawable = getResources().getDrawable(R.mipmap.ktv_onstage_icon);
            drawable.setBounds(0,0,drawable.getMinimumWidth(),drawable.getMinimumHeight());
            mSelectWhenNoSong.setCompoundDrawables(drawable,null,null,null);
            mSelectWhenNoSong.setText("上麦");
        }else if(state == ORDER_SONG){
            mSongTips.setText("立即点歌体验即构KTV实时合唱");
            Drawable drawable = getResources().getDrawable(R.mipmap.ktv_karaoka);
            drawable.setBounds(0,0,drawable.getMinimumWidth(),drawable.getMinimumHeight());
            mSelectWhenNoSong.setCompoundDrawables(drawable,null,null,null);
            mSelectWhenNoSong.setText("点歌");
        }else if(state == COMING_SOON){
            mZegoLyricView.reset("");
            this.songId = null;
            setTime(mPlayTime,playTime,totalTime);
            mSongName.setText(songName);
            mSongNameTips.setText(songName);
            mSongSwitch.setSongType(hasAccompany,hasOriginal);
        }else if(state == START_PLAYING){
            mSongName.setText(songName);
            setTime(mPlayTime,playTime,totalTime);
            mSongSwitch.setSongType(hasAccompany,hasOriginal);
        }
    }

    private static void setTime(TextView view, Long playTime, Long totalTime) {
        if (playTime == null || totalTime == null) {
            setText(view,"00:00 / 00:00");
        } else {
            String time = String.format("%s / %s", millisToString(playTime), millisToString(totalTime));
            setText(view,time);
        }

    }

    private static void setText(TextView view,CharSequence text){
        if(view == null){
            return;
        }

        if(view.getText().equals(text)){
            return;
        }

        view.setText(text);
    }


    private static String millisToString(long millis) {
        if (millis < 0) {
            millis = 0;
        }
        SimpleDateFormat formatter = new SimpleDateFormat("mm:ss");
        formatter.setTimeZone(TimeZone.getTimeZone("GMT+00:00"));
        return formatter.format(millis);
    }

    public void clearData(){
        songName = null;
        hasAccompany = true;
        hasOriginal = true;
        playTime = null;
        totalTime = null;
    }

    public void offState(){
        Log.e("APIBase","offState");
        state = OFF_STAGE;
        updateData(state);
        switchState(state);
        clearData();
    }

    public void orderSong(){
        Log.e("APIBase","orderSong");
        state = ORDER_SONG;
        updateData(state);
        switchState(state);
        clearData();
    }

    public void comingSoon(String songName,boolean hasAccompany,boolean hasOriginal){
        Log.e("APIBase","comingSoon");
        state = COMING_SOON;
        this.songName = songName;
        this.hasAccompany = hasAccompany;
        this.hasOriginal = hasOriginal;
        updateData(state);
        switchState(state);
    }

    public void startPlaying(String songName,boolean hasAccompany,boolean hasOriginal,boolean showPitch){
        Log.e("APIBase","startPlaying");
        state = START_PLAYING;
        this.songName = songName;
        this.hasAccompany = hasAccompany;
        this.hasOriginal = hasOriginal;
        this.showPitch = showPitch;
        updateData(state);
        switchState(state);
    }

    public void updateTime(Long playTime, Long totalTime){
        this.playTime = playTime;
        this.totalTime = totalTime;
        setTime(mPlayTime,this.playTime,this.totalTime);
    }

    public void setSongName(String songName){
        this.songName = songName;
        mSongName.setText(songName);
        mSongNameTips.setText(songName);
    }


    public SongSwitchView getSongSwitch(){
        return mSongSwitch;
    }

    public void setSongSwitchVisibility(boolean visibility){
        if(visibility) {
            mSongSwitch.setVisibility(VISIBLE);
        }else {
            mSongSwitch.setVisibility(GONE);
        }
    }

    public ZegoLyricView getLyricView(){
        return mZegoLyricView;
    }

    public ZegoPitchView getPitchView(){
        return mPitchView;
    }

    public void setLrc(String songId, ZGKTVLyric lrc) {
        if (this.songId != null && this.songId.equals(songId)) {
            return;
        }

        this.songId = songId;
        mZegoLyricView.setupLyric(lrc);
    }

    public int getState() {
        return state;
    }

    public void setScore(int score){
        this.mScore = score;
        setText(mKtvTvScore,String.valueOf(score));
    }
    public int getScore(){
       return mScore;
    }

}
