package im.zego.ktv.chorus.custom;

import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import androidx.constraintlayout.widget.ConstraintLayout;

import java.util.List;

import im.zego.ktv.R;
import im.zego.ktv.custom.BaseBottomSheetDialogFragment;
import im.zego.ktv.chorus.ktvroom.KTVRoomOrderSongManager;
import im.zego.ktv.chorus.ktvroom.callback.IOrderSongListUpdate;
import im.zego.ktv.chorus.model.OrderedSongInfo;


/**
 * 选择歌曲弹窗（点歌/已点）
 */
public class SongListDialog extends BaseBottomSheetDialogFragment implements View.OnClickListener {

    private SongListView mSongListView; //点歌
    private TextView mSongList;
    private WaitListView mWaitListView; //已点
    private TextView mWaitList;

    private ConstraintLayout.LayoutParams songListParams;
    private ConstraintLayout.LayoutParams waitListParams;

    private View mVSongList, mVWaitList;

    @Override
    public int getLayoutResId() {
        return R.layout.ktv_dialog_song;
    }

    @Override
    public void initView(Bundle savedInstanceState) {

        setHeight(450);
        setShowHeight(true); //设置弹窗最大高度

        mSongListView = rootView.findViewById(R.id.song_list_view);
        mWaitListView = rootView.findViewById(R.id.wait_list_view);
        mSongList = rootView.findViewById(R.id.song_list);
        mWaitList = rootView.findViewById(R.id.wait_list);
        mVSongList = rootView.findViewById(R.id.v_song_list);
        mVWaitList = rootView.findViewById(R.id.v_wait_list);

        songListParams = (ConstraintLayout.LayoutParams) mSongListView.getLayoutParams();
        waitListParams = (ConstraintLayout.LayoutParams) mWaitListView.getLayoutParams();

        mSongList.setOnClickListener(this);
        mWaitList.setOnClickListener(this);

        selectView(mSongList);
        unSelectView(mWaitList);

        mSongListView.setContext(getActivity());
        initData();
    }

    private void initData(){
        int orderSize = KTVRoomOrderSongManager.getInstance().getOrderList().size();

        String orderStr = "0";
        List<OrderedSongInfo.OrderedSongItemInfo> orderList = KTVRoomOrderSongManager.getInstance().getOrderList();
        if(orderList != null){
            if(orderList.size() > 99){
                orderStr = "99+";
            }else {
                orderStr = String.valueOf(orderList.size());
            }
        }
        mWaitList.setText("已点("+orderStr+")");

        KTVRoomOrderSongManager.getInstance().setOrderSongListUpdate(new IOrderSongListUpdate() {
            @Override
            public void OrderSongListUpdate(List<OrderedSongInfo.OrderedSongItemInfo> orderList) {
                String orderStr = "0";
                if(orderList != null){
                    if(orderList.size() > 99){
                        orderStr = "99+";
                    }else {
                        orderStr = String.valueOf(orderList.size());
                    }
                }
                mWaitList.setText("已点("+orderStr+")");
                mWaitListView.notifyData();

            }
        });
    }

    /**
     * 选中效果
     *
     * @param view
     */
    private void selectView(TextView view) {
        view.setTextColor(getContext().getResources().getColor(R.color.ktv_color_white));
    }

    /**
     * 未选中效果
     *
     * @param view
     */
    private void unSelectView(TextView view) {
        view.setTextColor(getContext().getResources().getColor(R.color.ktv_color_white_cc));
    }


    @Override
    public void onClick(View v) {
        int id = v.getId();

        if (id == R.id.song_list) {
            selectView(mSongList);
            unSelectView(mWaitList);
//                if (((ViewGroup) rootView).findViewById(R.id.song_list_view) == null) {
//                    ((ViewGroup) rootView).addView(mSongListView, songListParams);
//                }
            mSongListView.setVisibility(View.VISIBLE);
            mVSongList.setVisibility(View.VISIBLE);
            mWaitListView.setVisibility(View.GONE);
            mVWaitList.setVisibility(View.GONE);
//                ((ViewGroup) rootView).removeView(mWaitListView);
        } else if (id == R.id.wait_list) {
            selectView(mWaitList);
            unSelectView(mSongList);
//                if (((ViewGroup) rootView).findViewById(R.id.wait_list_view) == null) {
//                    ((ViewGroup) rootView).addView(mWaitListView, waitListParams);
//                }
            mSongListView.setVisibility(View.GONE);
            mWaitListView.setVisibility(View.VISIBLE);
            mVSongList.setVisibility(View.GONE);
            mVWaitList.setVisibility(View.VISIBLE);
//                ((ViewGroup) rootView).removeView(mSongListView);
        }
    }



}
