package im.zego.ktv.chorus.custom;


import android.content.Context;
import android.util.AttributeSet;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.view.ViewCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.viewpager2.adapter.FragmentStateAdapter;
import androidx.viewpager2.widget.ViewPager2;

import java.util.ArrayList;
import java.util.List;

import im.zego.ktv.R;
import im.zego.ktv.chorus.ktvroom.adapter.SongTypeAdapter;
import im.zego.ktv.chorus.ktvroom.fragment.HitSongFragment;
import im.zego.ktv.chorus.ktvroom.fragment.RecommendSongFragment;
import im.zego.ktv.chorus.model.SongCategoryBean;


/**
 * 点歌列表
 */
public class SongListView extends ConstraintLayout {

    private RecyclerView mSongType;  //歌曲类型列表
    private SongTypeAdapter mSongTypeAdapter;

    private ViewPager2 mViewPager;
    private SongPageAdapter mSongPageAdapter;

    private Context context;
    private FragmentActivity mFragmentActivity;

    public SongListView(@NonNull Context context) {
        this(context, null);
    }

    public SongListView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public SongListView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.context = context;
    }

    public void setContext(FragmentActivity mFragmentActivity) {
        this.mFragmentActivity = mFragmentActivity;
        initView();
    }

    private void initView() {
        View view = inflate(context, R.layout.ktv_view_song_list, this);

        mSongType = view.findViewById(R.id.song_type);
        mViewPager = view.findViewById(R.id.mViewPager);
        //点击
        view.findViewById(R.id.tv_search).setOnClickListener(v -> {
            SongSearchDialog mSongSearchDialog = new SongSearchDialog(mFragmentActivity, R.style.BaseBottomSheetDialog);
            mSongSearchDialog.show();
        });

        setSongType();

        initFragmentList();
        mViewPager.setOffscreenPageLimit(1);
        mViewPager.setAdapter(mSongPageAdapter);
        mViewPager.setCurrentItem(0, false);
        //禁止左右滑动
        mViewPager.setUserInputEnabled(false);

    }

    /**
     * 歌曲类别
     */
    private void setSongType() {
        this.mSongType.setLayoutManager(new LinearLayoutManager(getContext(), LinearLayoutManager.HORIZONTAL, false));
        this.mSongTypeAdapter = new SongTypeAdapter(getContext(), categoryList());
        this.mSongType.setAdapter(this.mSongTypeAdapter);

        //解决了 BottomSheetDialogFragment 在设置的时候不支持两个RecyclerView（第二个 RecyclerView不可滑动的问题）
        ViewCompat.setNestedScrollingEnabled(mSongType, false);

        this.mSongTypeAdapter.setOnItemListener((position, songCategoryBean) -> {
            this.mSongTypeAdapter.setSelectItem(position);
            if (mViewPager.getCurrentItem() != position) {
                mViewPager.setCurrentItem(position, false);
            }
        });

    }

    /**
     * 榜单歌曲类型
     *
     * @return
     */
    private List<SongCategoryBean> categoryList() {
        List<SongCategoryBean> list = new ArrayList<>();
        String[] categoryNameList = getResources().getStringArray(R.array.ktv_category_name);
        String[] categoryIdList = getResources().getStringArray(R.array.ktv_category_id);
        for (int i = 0; i < categoryNameList.length; ++i) {
            String categoryName = categoryNameList[i];
            String categoryId = categoryIdList[i];
            list.add(new SongCategoryBean().setCategoryName(categoryName).setCategoryId(categoryId));
        }
        return list;
    }

    private List<Fragment> dataList;

    private void initFragmentList() {
        dataList = new ArrayList<>();
        dataList.clear();
        for (int i = 0; i < 5; i++) {
            if(i == 0){
                RecommendSongFragment mRecommendSongFragment = new RecommendSongFragment();
                mRecommendSongFragment.setContext(getRootView(), i);
                dataList.add(mRecommendSongFragment);
            }else {
                HitSongFragment mHitSongFragment = new HitSongFragment();
                mHitSongFragment.setContext(getRootView(), i);
                dataList.add(mHitSongFragment);
            }
        }
        mSongPageAdapter = new SongPageAdapter(mFragmentActivity, dataList);
    }

    class SongPageAdapter extends FragmentStateAdapter {

        private List<Fragment> dataList;

        public SongPageAdapter(@NonNull FragmentActivity fragmentActivity, List<Fragment> data) {
            super(fragmentActivity);
            this.dataList = data;
        }

        @NonNull
        @Override
        public Fragment createFragment(int position) {
            return dataList.get(position);
        }

        @Override
        public int getItemCount() {
            return dataList.size();
        }

    }


}
