package im.zego.ktv.chorus.custom;

import static im.zego.ktv.ErrorCodeConstants.GoChatErrorFailNetwork;
import static im.zego.ktv.Constants.STATE_ERROR;
import static im.zego.ktv.Constants.STATE_FINISH;
import static im.zego.ktv.Constants.STATE_LOADING;
import static im.zego.ktv.Constants.STATE_NORMAL;

import android.app.Dialog;
import android.content.Context;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.FragmentActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import java.util.List;

import im.zego.common.util.SafetyHandler;
import im.zego.ktv.R;
import im.zego.ktv.custom.KTVLoadMoreView;
import im.zego.ktv.custom.ProcessView;
import im.zego.ktv.chorus.download.FileDownloadManager;
import im.zego.ktv.chorus.download.impl.FileDownloadStateListener;
import im.zego.ktv.chorus.ktvroom.adapter.SongListContentAdapter;
import im.zego.ktv.chorus.ktvroom.adapter.SongListTipsAdapter;
import im.zego.ktv.chorus.model.ktv.SearchTipsInfo;
import im.zego.ktv.chorus.model.ktv.TopSongInfo;
import im.zego.ktv.chorus.protocol.KTVRequestAPI;
import im.zego.ktv.chorus.protocol.ktv.ISearchTipsCallback;
import im.zego.ktv.chorus.protocol.ktv.ITopSongCallback;
import im.zego.ktv.chorus.rtc.ZGKTVSongLoadState;
import im.zego.ktv.utils.ToastUtil;
import im.zego.ktv.chorus.roomlist.adapter.EmptyAndLoadMoreAdapterWrapper;


/**
 * 歌曲搜索
 */
public class SongSearchDialog extends Dialog {

    private ImageView mIvCloseRight; //清空
    private TextView mCancelTv; //取消弹窗
    private EditText mEtSearch; //输入框
    private RecyclerView mSongListContent;
    private ConstraintLayout mCommonClEmptyView;
    private ImageView mCommonIvSearch;
    private TextView mTvSearchTips;

    private SongListContentAdapter mSongListContentAdapter;

    private KTVLoadMoreView mLoadMoreView;
    private EmptyAndLoadMoreAdapterWrapper mAdapterWrapper;
    private static final int PAGE_NUM = 10;
    private Context context;

    private int pageNum = 1;
    private int mState = STATE_NORMAL;
    private ProcessView mProcessView;

    protected String keyWord = "";

    //歌曲搜索提示词
    private RecyclerView mRvSongTips;
    private SongListTipsAdapter mSongListTipsAdapter;

    /**
     * 定时任务
     */
    private SafetyHandler safetyHandler;

    private boolean isOpenSoftKeyboard = true;

    public SongSearchDialog(@NonNull FragmentActivity mFragmentActivity, int themeResId) {
        super(mFragmentActivity, themeResId);
        init(mFragmentActivity);
    }

    private void init(FragmentActivity context) {
        this.context = context;

        safetyHandler = SafetyHandler.with(context);
        //设置dialog位置
        Window window = getWindow();
        if (window != null) {
            window.getDecorView().setPadding(0, 0, 0, 0);
            WindowManager.LayoutParams layoutParams = window.getAttributes();
            layoutParams.width = WindowManager.LayoutParams.MATCH_PARENT;
            layoutParams.height = WindowManager.LayoutParams.MATCH_PARENT;
            layoutParams.horizontalMargin = 0;
            layoutParams.gravity = Gravity.BOTTOM;
            window.setAttributes(layoutParams);
        }

        setContentView(R.layout.ktv_search_song_list);

        mIvCloseRight = findViewById(R.id.iv_close_right);
        mEtSearch = findViewById(R.id.et_search);
        mCancelTv = findViewById(R.id.cancelTv);
        mSongListContent = findViewById(R.id.mRecyclerView);
        mCommonClEmptyView = findViewById(R.id.common_cl_empty_view);
        mCommonIvSearch = findViewById(R.id.common_iv_search);
        mTvSearchTips = findViewById(R.id.tv_search_tips);
        mRvSongTips = findViewById(R.id.rv_song_tips);

        mCommonClEmptyView.setVisibility(View.GONE);

        mCancelTv.setOnClickListener(v -> dismiss());
        setSongListContent();
        mEtSearch.addTextChangedListener(watcher);
        mIvCloseRight.setOnClickListener(v -> {
            mEtSearch.setText("");
            keyWord = "";
            if (!isOpenSoftKeyboard) {
                showSoftInput();
            }
        });

        initEdit();
        setSearchTips();

    }

    private void initEdit() {

        //默认打开键盘
        mEtSearch.setFocusable(true);
        mEtSearch.setFocusableInTouchMode(true);
        mEtSearch.requestFocus();
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);

        mEtSearch.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                switch (actionId) {
                    //点击搜索的时候关闭键盘，并且执行搜索的动作
                    case EditorInfo.IME_ACTION_SEARCH:
                        closeSoftKeyboard();
                        if (!TextUtils.isEmpty(keyWord)) {
                            search();
                        }
                        break;

                    default:
                        break;
                }

                return false;
            }
        });

        findViewById(R.id.rootView).addOnLayoutChangeListener(new View.OnLayoutChangeListener() {
            @Override
            public void onLayoutChange(View v, int left, int top, int right, int bottom, int oldLeft, int oldTop, int oldRight, int oldBottom) {
                if (oldBottom != 0 && bottom != 0 && (oldBottom - bottom > 0)) {
                    //键盘弹起
                    mEtSearch.setCursorVisible(true);
                    isOpenSoftKeyboard = true;

                } else if (oldBottom != 0 && bottom != 0 && (bottom - oldBottom > 0)) {
                    //键盘收起
                    mEtSearch.setCursorVisible(false);
                    isOpenSoftKeyboard = false;
                }
            }
        });

    }

    /**
     * 歌曲列表内容
     */
    private void setSongListContent() {
        this.mSongListContentAdapter = new SongListContentAdapter(getContext());
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getContext());
        this.mSongListContent.setLayoutManager(linearLayoutManager);

        //解决刷新闪烁问题
        ((SimpleItemAnimator) this.mSongListContent.getItemAnimator()).setSupportsChangeAnimations(false);

        View emptyView = LayoutInflater.from(getContext()).inflate(R.layout.ktv_search_song_empty, mSongListContent, false);

        mLoadMoreView = new KTVLoadMoreView(getContext());
        mAdapterWrapper = new EmptyAndLoadMoreAdapterWrapper(mSongListContentAdapter);
        mAdapterWrapper.setEmptyView(emptyView);
        mAdapterWrapper.setLoadMoreView(mLoadMoreView);
        this.mSongListContent.setAdapter(this.mAdapterWrapper);

        mSongListContent.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if (event.getAction() == MotionEvent.ACTION_DOWN) {
                    closeSoftKeyboard();
                }
                return false;
            }
        });

        //加载更多
        mAdapterWrapper.setOnLoadMoreListener(new EmptyAndLoadMoreAdapterWrapper.OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                loadMore();
            }
        });

        mSongListContentAdapter.setOnSongItemClickListener(new SongListContentAdapter.OnSongItemClickListener() {
            @Override
            public void onClick(TopSongInfo.SongsDTO bean) {
                FileDownloadManager.getInstance().downloadFile(context, bean.getSongId());
                registerDownloadProgress(bean);
            }
        });

    }

    /**
     * 搜索提示词
     */
    private void setSearchTips() {

        this.mSongListTipsAdapter = new SongListTipsAdapter();
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getContext());
        this.mRvSongTips.setLayoutManager(linearLayoutManager);
        //解决刷新闪烁问题
        ((SimpleItemAnimator) this.mRvSongTips.getItemAnimator()).setSupportsChangeAnimations(false);
        this.mRvSongTips.setAdapter(this.mSongListTipsAdapter);

        mSongListTipsAdapter.setOnTipsItemClickListener(new SongListTipsAdapter.OnTipsItemClickListener() {
            @Override
            public void onClick(String searchTips) {
                closeSoftKeyboard();
                keyWord = searchTips;
                mEtSearch.setText(keyWord);
                if (!TextUtils.isEmpty(keyWord)) {
                    search();
                }
            }
        });

    }

    private void registerDownloadProgress(TopSongInfo.SongsDTO bean) {
        String songId = bean.getSongId();
        FileDownloadManager.getInstance().registerDownloadListener(context, songId, new FileDownloadStateListener() {
            @Override
            public void onDownloadState(ZGKTVSongLoadState state) {
                mSongListContentAdapter.updateDownloadState(songId, state, mSongListContent);
                if (state == ZGKTVSongLoadState.LOAD_COMPLETE) {
                    bean.setLoadState(ZGKTVSongLoadState.LOAD_COMPLETE.value());
                } else if (state == ZGKTVSongLoadState.LOADING) {
                    if (bean.getLoadState() != ZGKTVSongLoadState.LOADING.value()) {
                        bean.setLoadState(ZGKTVSongLoadState.LOADING.value());
                    }
                }
            }

            @Override
            public void onDownloadProgress(float progress) {
                mSongListContentAdapter.updateDownloadProgress(songId, progress, mSongListContent);
            }
        });
    }

    /**
     * 加载更多（通过sdk获取列表数据）
     */
    private void loadMore() {
        if (mState == STATE_FINISH) {
            mLoadMoreView.showFinish();
        } else {
            String content = mEtSearch.getText().toString();
            if (!TextUtils.isEmpty(content)) {
                pageNum++;
                mLoadMoreView.showLoading();
                searchSong(content, pageNum, PAGE_NUM);
            } else {
                mLoadMoreView.showFinish();
            }
        }
    }


    private TextWatcher watcher = new TextWatcher() {
        @Override
        public void beforeTextChanged(CharSequence s, int start, int count, int after) {

        }

        @Override
        public void onTextChanged(CharSequence s, int start, int before, int count) {

        }

        @Override
        public void afterTextChanged(Editable s) {
            if (SongSearchDialog.this.keyWord.equals(s.toString().trim())) {
                return;
            }

            keyWord = s.toString().trim();
            if (!TextUtils.isEmpty(keyWord)) {
                mIvCloseRight.setVisibility(View.VISIBLE);
            } else {
                mIvCloseRight.setVisibility(View.GONE);
                cleanEdit();
            }

            if (safetyHandler != null) {
                safetyHandler.removeCallbacks(delayRunnable);
                safetyHandler.postDelayed(delayRunnable, 300);
            }

        }
    };

    /**
     * 延时执行搜索
     */
    private Runnable delayRunnable = () -> {
        if (!TextUtils.isEmpty(keyWord)) {
            searchTips(keyWord);
        }
    };

    /**
     * 清除输入框后ui刷新
     */
    protected void cleanEdit() {
        mCommonClEmptyView.setVisibility(View.GONE);
        mSongListContentAdapter.clearData();
        mAdapterWrapper.notifyDataSetChanged();
        mSongListTipsAdapter.clearData();
    }

    /**
     * 输入文字/删除文字后停顿300ms 调用搜索接口搜索
     */
    private void search() {
        mRvSongTips.setVisibility(View.GONE);
        mSongListContent.setVisibility(View.VISIBLE);
        searchingData();
        mSongListContentAdapter.clearData();
        mAdapterWrapper.notifyDataSetChanged();
        pageNum = 1;
        searchSong(keyWord, pageNum, 10);
    }

    private void loading() {
        if (mProcessView == null) {
            mProcessView = new ProcessView(getContext());
        }
        mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mSongListContent.getRootView());
    }


    public void searchSong(String keyword, int page, int size) {

        if (pageNum == 1) {
            loading();
        }
        mState = STATE_LOADING;

        KTVRequestAPI.searchSong(keyword, page, size, new ITopSongCallback<TopSongInfo>() {
            @Override
            public void onTopSong(int errorCode, TopSongInfo searchSongInfo) {
                mCommonClEmptyView.setVisibility(View.GONE);
                if (mProcessView != null) {
                    mProcessView.dismiss();
                }
                if (errorCode == 0) {  //返回成功

                    if (TextUtils.isEmpty(SongSearchDialog.this.keyWord)) {
                        cleanEdit();
                        return;
                    }

                    if (!SongSearchDialog.this.keyWord.equals(keyword)) {
                        return;
                    }


                    if (searchSongInfo != null && searchSongInfo.getSongs() != null) {
                        List<TopSongInfo.SongsDTO> mData = searchSongInfo.getSongs();
                        //返回来的数据不固定，不受size 控制。只有返回的数据为0的时候才不加载下一页
                        mState = STATE_NORMAL;
                        if (mData.size() == 0) {
                            mLoadMoreView.showFinish();
                            if (page == 1) {
                                searchNoneData();
                            } else {
                                mState = STATE_FINISH;
                            }
                        }
                        if (page == 1) {
                            mSongListContentAdapter.clearAndAddData(searchSongInfo.getSongs());
                        } else {
                            mSongListContentAdapter.addData(searchSongInfo.getSongs());
                        }
                        mAdapterWrapper.notifyDataSetChanged();
                    } else {
                        mState = STATE_ERROR;
                        mLoadMoreView.showFinish();
                        if (page == 1) {
                            searchNoneData();
                        }
                    }
                } else {
                    if (errorCode == GoChatErrorFailNetwork) {
                        ToastUtil.showWarnToast(R.string.fail_network);
                    }
                    mState = STATE_ERROR;
                    mLoadMoreView.showFinish();
                    if (page == 1) {
                        searchNoneData();
                    }
                }
            }
        });
    }

    /**
     * 搜索提示
     */
    private void searchTips(String keyword) {

        mRvSongTips.setVisibility(View.VISIBLE);
        mSongListContent.setVisibility(View.GONE);
        mCommonClEmptyView.setVisibility(View.GONE);

        KTVRequestAPI.searchTips(keyword, new ISearchTipsCallback<SearchTipsInfo>() {
            @Override
            public void onSearchTips(int errorCode, SearchTipsInfo searchTipsInfo) {

                if (errorCode == 0) {
                    if (TextUtils.isEmpty(SongSearchDialog.this.keyWord)) {
                        cleanEdit();
                        return;
                    }

                    if (!SongSearchDialog.this.keyWord.equals(keyword)) {
                        return;
                    }

                    mSongListTipsAdapter.initTipsList(searchTipsInfo.getTips());
                } else {
                    if (errorCode == GoChatErrorFailNetwork) {
                        ToastUtil.showWarnToast(R.string.fail_network);
                    }
                }

            }
        });

    }

    private void searchNoneData() {
        mSongListContentAdapter.clearData();
        mAdapterWrapper.notifyDataSetChanged();
        mCommonClEmptyView.setVisibility(View.VISIBLE);
        mCommonIvSearch.setBackgroundResource(R.mipmap.ktv_search_none);
        mTvSearchTips.setText("查询不到这首歌");
    }

    private void searchingData() {
        mCommonClEmptyView.setVisibility(View.VISIBLE);
        mCommonIvSearch.setBackgroundResource(R.mipmap.ktv_search_ing);
        mTvSearchTips.setText("歌曲搜索中");
    }

    @Override
    public void dismiss() {
        closeSoftKeyboard();
        if (safetyHandler != null) {
            safetyHandler.onDestroy();
        }
        super.dismiss();
    }

    /**
     * 关闭键盘
     */
    private void closeSoftKeyboard() {
        if (getCurrentFocus() != null && getCurrentFocus().getWindowToken() != null) {
            InputMethodManager manager = (InputMethodManager) context.getSystemService(Context.INPUT_METHOD_SERVICE);
            manager.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
        }
    }

    /**
     * 弹起键盘
     */
    private void showSoftInput() {
        InputMethodManager imm = (InputMethodManager) context.getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.showSoftInput(mEtSearch, 0);
    }

}
