package im.zego.ktv.chorus.download.delivery;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import im.zego.ktv.chorus.download.impl.FileDownloadStateListener;

public class DefaultFileDownloadDelivery implements BaseFileDownloadDelivery {

    private ConcurrentHashMap<String, ArrayList<FileDownloadStateListener>> listenerHashMap = new ConcurrentHashMap<>();

    /**
     * 添加回调
     *
     * @param tag
     * @param fileDownloadStateListener
     */
    @Override
    public void addFileDownloadListener(String tag, FileDownloadStateListener fileDownloadStateListener) {
        ArrayList<FileDownloadStateListener> tags = listenerHashMap.get(tag);
        if (tags == null) {
            tags = new ArrayList<>();
            listenerHashMap.put(tag, tags);
        }
        synchronized (tags) {
            if (!tags.contains(fileDownloadStateListener)) {
                tags.add(fileDownloadStateListener);
            }
        }
    }

    /**
     * 移除回调
     *
     * @param tag
     */
    @Override
    public void removeFileDownloadListener(String tag) {
        ArrayList<FileDownloadStateListener> tags = listenerHashMap.get(tag);
        if (tags != null) {
            synchronized (tags) {
                tags.clear();
            }
            //tags可能出现并发添加，如完成回调时又有相同的任务添加了，如果到了这里，tags不为空，则不移除
            if (tags.isEmpty()) {
                listenerHashMap.remove(tag);
            }
        } else {
            listenerHashMap.remove(tag);
        }
    }

    @Override
    public void removeFileDownloadListener(FileDownloadStateListener listener) {
        Iterator<Map.Entry<String, ArrayList<FileDownloadStateListener>>> iterator = listenerHashMap.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, ArrayList<FileDownloadStateListener>> entry = iterator.next();
            ArrayList<FileDownloadStateListener> listeners = entry.getValue();
            if (listeners != null && listeners.contains(listener)) {
                synchronized (listeners) {
                    listeners.remove(listener);
                }
            }
        }
    }

    /**
     * 获得具体回调实例
     *
     * @param tag
     * @return
     */
    @Override
    public FileDownloadStateListener getFileDownloadListener(String tag) {
        return null;
    }

    @Override
    public ArrayList<FileDownloadStateListener> getFileDownloadListeners(String tag) {
        return listenerHashMap.get(tag);
    }

}
