package im.zego.ktv.chorus.ktvroom;

import static im.zego.ktv.ErrorCodeConstants.ERROR_ROOM_NOT_EXIST;
import static im.zego.ktv.ErrorCodeConstants.ERROR_USER_OFF_LINE;
import static im.zego.ktv.ErrorCodeConstants.GoChatErrorFailNetwork;

import android.app.Dialog;
import android.content.Intent;
import android.graphics.drawable.AnimationDrawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.Settings;
import android.text.TextUtils;
import android.util.ArrayMap;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.RequiresApi;
import androidx.lifecycle.ViewModelProviders;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.gson.Gson;

import org.jetbrains.annotations.NotNull;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import im.zego.common.model.LoginInfo;
import im.zego.common.proxy.QualityManagerProxy;
import im.zego.common.util.L;
import im.zego.common.util.SharedPreferencesUtil;
import im.zego.common.util.T;
import im.zego.ktv.R;
import im.zego.ktv.chorus.base.Constants;
import im.zego.ktv.chorus.base.NetworkQuality;
import im.zego.ktv.chorus.base.ZGBaseActivity;
import im.zego.ktv.chorus.custom.AdjustVolumeDialog;
import im.zego.ktv.custom.KTVScoreShowDialog;
import im.zego.ktv.custom.MessageDialog;
import im.zego.ktv.chorus.custom.MusicPlayerMenu;
import im.zego.ktv.chorus.custom.SongListDialog;
import im.zego.ktv.chorus.download.FileDownloadManager;
import im.zego.ktv.chorus.ktvroom.adapter.KTVRoomStageGuestAdapter;
import im.zego.ktv.chorus.ktvroom.callback.IExecuteCallback;
import im.zego.ktv.chorus.ktvroom.callback.INetworkKtvCallback;
import im.zego.ktv.chorus.ktvroom.callback.IOrderSongListUpdate;
import im.zego.ktv.chorus.ktvroom.callback.IPlayingSongUpdateCallback;
import im.zego.ktv.chorus.ktvroom.callback.IRoomSongStateUpdateListener;
import im.zego.ktv.chorus.ktvroom.callback.ISongInfoUpdateListener;
import im.zego.ktv.chorus.login.LoginInfoManager;
import im.zego.ktv.chorus.model.HostSEIInfo;
import im.zego.ktv.chorus.model.OrderedSongInfo;
import im.zego.ktv.chorus.model.RoomInfo;
import im.zego.ktv.chorus.model.UserDataMonitorInfo;
import im.zego.ktv.chorus.model.UserInfo;
import im.zego.ktv.chorus.model.notify.NotifyCustomBroadcastInfo;
import im.zego.ktv.chorus.model.notify.NotifyOrderListUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyPreOrderUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyRoomStateUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyUserListUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyUserStateUpdateInfo;
import im.zego.ktv.chorus.protocol.ICommonResponseCallback;
import im.zego.ktv.chorus.protocol.IGetOrderedSongListCallback;
import im.zego.ktv.chorus.protocol.IGetUserInfoCallback;
import im.zego.ktv.chorus.protocol.RoomAPI;
import im.zego.ktv.chorus.rtc.RTCSDKManager;
import im.zego.ktv.chorus.rtc.ZGKTVCopyrightedMusicManager;
import im.zego.ktv.chorus.rtc.ZGKTVCopyrightedSong;
import im.zego.ktv.chorus.rtc.ZGKTVLyricManager;
import im.zego.ktv.chorus.rtc.ZGKTVPlayerManager;
import im.zego.ktv.chorus.rtc.ZGKTVSongLoadState;
import im.zego.ktv.chorus.rtc.ZGKTVSongState;
import im.zego.ktv.chorus.rtc.callbacks.IRTCEventCallback;
import im.zego.ktv.chorus.rtc.callbacks.IZGKTVCopyrightedSongUpdateListener;
import im.zego.ktv.chorus.rtc.callbacks.IZGKTVPlayerUpdateListener;
import im.zego.ktv.utils.CheckDoubleClick;
import im.zego.ktv.utils.GlideImageLoader;
import im.zego.ktv.utils.ToastUtil;
import im.zego.ktv.chorus.viewmodel.KTVRoomViewModel;
import im.zego.ktv.utils.PermissionHelper;
import im.zego.lyricview.ZegoLyricView;
import im.zego.lyricview.listener.OnLyricLineFinishedListener;
import im.zego.lyricview.model.LineInfo;
import im.zego.lyricview.model.ZGKTVLyric;
import im.zego.pitchview.ZegoPitchViewHelper;
import im.zego.pitchview.model.ZGKTVPitch;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.ZegoMediaPlayer;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicGetStandardPitchCallback;
import im.zego.zegoexpress.constants.ZegoAudioRoute;
import im.zego.zegoexpress.constants.ZegoMediaPlayerState;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoNetworkSpeedTestQuality;
import im.zego.zegoexpress.entity.ZegoNetworkTimeInfo;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoStream;

/**
 * KTV房间activity
 */
public class KTVRoomActivity extends ZGBaseActivity implements IRTCEventCallback, IZGKTVCopyrightedSongUpdateListener {
    private final static String TAG = "KTVRoomActivity";

    private static final int DEF_SEI_COUNT = 7;
    private static final int MAX_SEI_COUNT = 8;
    private RecyclerView guestRecyclerView;
    private KTVRoomStageGuestAdapter stateGuestAdapter;
    private KTVRoomViewModel mViewModel;

    private Dialog mMicOperationDialog;
    private boolean mIsMicMuted = true;
    private UserInfo mMyInfo;
    private IExecuteCallback mIExecuteCallback;
    private AnimationDrawable mAnimationDrawable;
    private boolean mIsDataMonitorOpen = false;
    private long mZegoNetworkTime;
    private long mPlayProgress;
    private UserDataMonitorInfo mHostMonitorInfo;
    private UserDataMonitorInfo mMyMonitorInfo = new UserDataMonitorInfo();
    private HashMap<String, LinkedList<UserDataMonitorInfo>> mNetworkQualityMap;
    private LinkedList<UserDataMonitorInfo> myNetworkQualityList = new LinkedList<>();
    private NetworkQuality mMyNetworkQuality;
    private Timer mHostSyncProgressTimer;
    private long mCurrentSEIProgress = 0L;
    private long currentProgress = 0L;

    private KTVScoreShowDialog mScoreShowDialog;

    int progressInterval = 1;
    private long mPlayerCurrentTimeMillis = 0;//播放器进度回调时的时间，未播放状态需设为0
    private long mPlayerCurrentProgress = 0;//播放器进度，未播放状态需设为0

    private long lastSEIProgress = 0;
    private CustomProgressTimer mRefreshLyricProgressTimer = new CustomProgressTimer(new Runnable() {
        @Override
        public void run() {
            if (mZegoLyricView != null) {

                if (mMyInfo.isAudience()) {
                    // 如果是观众，需要基于sei做更新
                    if(mPlayerCurrentTimeMillis != 0 && mKState == 1) {
                        //计算出当前播放器进度 （当前进度 = 当前时间 - SEI接收时的时间 + SEI接收时的进度）
                        long customProgress = System.currentTimeMillis() - mPlayerCurrentTimeMillis + mPlayerCurrentProgress;
                        L.i(TAG,"SEI ProgressUpdate:" + customProgress);
                        currentProgress = customProgress;

                        //过滤短暂拉流卡顿导致的时间偏差
                        if(currentProgress - lastSEIProgress > 0){
                            mZegoLyricView.setCurrentTimeMillis(currentProgress);
                            lastSEIProgress = currentProgress;
                        }else if(currentProgress - lastSEIProgress < -3000){
                            //异常情况保证时间进度

                            lastSEIProgress = 0;
                        }


                    }else {
                        lastSEIProgress = 0;
                    }
                } else {
                    // 如果是麦上用户，算出当前播放器时间
                    if(mPlayerCurrentTimeMillis != 0 && ZGKTVPlayerManager.getInstance().isPlaying()){
                        //计算出当前播放器进度 （当前进度 = 当前时间 - 播放器回调时的时间 + 播放器进度）
                        long customProgress = System.currentTimeMillis() - mPlayerCurrentTimeMillis + mPlayerCurrentProgress;
                        L.i(TAG,"onPlaybackProgressUpdate:" + customProgress);
                        currentProgress = customProgress;
                        if (!RoomInfoHelp.isSongInfoExist()) {
                            L.e(TAG,"onPlaybackProgressUpdate isSongInfoExist" );
                            return;
                        }
                        String songId = RoomInfoHelp.getSongId();
                        ZGKTVLyric lrc = ZGKTVLyricManager.getInstance().getLyric(songId);
                        if (lrc != null) {
                            mZegoLyricView.setCurrentTimeMillis(customProgress);
                        }



                        progressInterval++;
                        if(progressInterval >= 2) {

                            ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
                            long duration = 0;
                            if (song != null) {
                                duration = song.getDuration();
                            }

                            if (duration == 0) {
                                duration = ZGKTVPlayerManager.getInstance().getTotalDuration();
                            }

                            mMusicPlayerMenu.updateTime(customProgress, duration);
                            mZegoNetworkTime = RTCSDKManager.getInstance().getNetworkTime();
                            mPlayProgress = customProgress;

                            startDataMonitorSync(customProgress, mZegoNetworkTime, duration);


                            progressInterval = 0;
                            mMusicPlayerMenu.getPitchView().setCurrentSongProgress(customProgress,mCurrentPitch );
                        }


                    }

                }
            }
        }
    });

    private TextView subject;
    private ImageView micTalking;
    private ZegoLyricView mZegoLyricView;
    private TextView mic;
    private TextView nextSong;
    private TextView adjust;

    private ImageView hostAvatar;
    private TextView hostNickName;
    private ImageView networkQuality;
    private TextView networkQualityContent;

    private TextView selectOnstage;

    private TextView selectSong;

    private MusicPlayerMenu mMusicPlayerMenu;
    private TextView dataMonitor;

    private MessageDialog messageDialog;
    private MessageDialog onlostDialog;
    private SongListDialog songListDialog; //点歌弹窗

    //当前播放的songId;
    private String songId;
    private TextView songNum;

    private boolean hostCloseRoom = false;

    private int mKState = 0;
    //计算 seek 回调次数，频繁的回调回导致卡顿的问题
    private int seiCount = DEF_SEI_COUNT;

    private int mCurrentPitch = 0;

    private Map<String,Boolean> pitchShowMap = new ArrayMap<>();
    private Map<String,Boolean> audioTrackCheckMap = new ArrayMap<>();

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        setContentView(R.layout.ktv_activity_ktv_room);

        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            try {
                Class<?> mainClass = Class.forName("im.zego.goenjoy.MainActivity");
                Intent intent = new Intent(KTVRoomActivity.this,mainClass);
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK);
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                startActivity(intent);
                finish();
            }catch (Exception e){

            }

            return;
        }

        mViewModel = ViewModelProviders.of(this).get(KTVRoomViewModel.class);
        KTVRoomManager.getInstance().setViewModel(mViewModel);
        bindView();



        subject.setText(KTVRoomManager.getInstance().getRoomInfo().getSubject());

        mAnimationDrawable = (AnimationDrawable) micTalking.getBackground();

        LoginInfo loginInfo = SharedPreferencesUtil.getLoginInfo();

        mMyInfo = new UserInfo(loginInfo.getUid(), loginInfo.getNickName(), 1);
        KTVRoomManager.getInstance().setMyLoginInfo(loginInfo);
        mViewModel.setMyInfo(mMyInfo);
        KTVRoomManager.getInstance().setMyUserInfo(mMyInfo);
        initView();
        initMediaPlayer();
        initServerCallback();
        initRTCSDKListener();
        pullUserList();
        initNetwork();

        if(!mMyInfo.isHost()){
            updateOrderList();
        }

        startKTVService();
    }

    private void bindView() {
        subject = findViewById(R.id.subject);
        micTalking = findViewById(R.id.mic_talking);

        mic = findViewById(R.id.mic);
        nextSong = findViewById(R.id.next_song);
        adjust = findViewById(R.id.adjust);

        hostAvatar = findViewById(R.id.host_avatar);
        hostNickName = findViewById(R.id.host_nick_name);
        networkQuality = findViewById(R.id.network_quality);
        networkQualityContent = findViewById(R.id.network_quality_content);


        selectOnstage = findViewById(R.id.select_onstage);

        selectSong = findViewById(R.id.select_song);

        mMusicPlayerMenu = findViewById(R.id.music_player_menu);
        mZegoLyricView = mMusicPlayerMenu.getLyricView();
        setNetworkQualityContent(networkQualityContent,mMyMonitorInfo.getNetworkQuality());
        dataMonitor = findViewById(R.id.data_monitor);
        songNum = findViewById(R.id.song_num);
    }

    private void initMediaPlayer() {

        mZegoLyricView.setOnLyricFinishLineListener(new OnLyricLineFinishedListener() {
            @Override
            public void onLyricLineFinished(int position, LineInfo lineInfo) {
                int lastScore = mMusicPlayerMenu.getScore();
                int totalScore = ZGKTVPlayerManager.getInstance().getTotalScore();
                if(totalScore >= 0 ) {
                    mMusicPlayerMenu.setScore(totalScore);
                }
                if(mMusicPlayerMenu.getPitchView().getPitchStartTime() < currentProgress) {

                    if(!mIsMicMuted) {
                        int score = ZGKTVPlayerManager.getInstance().getPreviousScore();
                        if(score <= 100) {
                            mMusicPlayerMenu.getPitchView().addScore(score);
                        }else {
                            mMusicPlayerMenu.getPitchView().addScore(totalScore - lastScore);
                        }
                    }else {
                        mMusicPlayerMenu.getPitchView().addScore(totalScore - lastScore);
                    }

                }

            }
        });
        /**
         * 上台和点歌点击事件
         */
        mMusicPlayerMenu.setListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                selectSong.performClick();
            }
        }, new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                selectOnstage.performClick();
            }
        });

        /**
         * 伴奏和原唱点击切换
         */
        mMusicPlayerMenu.getSongSwitch().setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                L.e(TAG,"buttonView:" + buttonView + ":isChecked:" + isChecked);




                if(!buttonView.isPressed()) {
                    return;
                }

                if(RoomInfoHelp.isSongInfoExist() ) {
                    String uniqueId  = RoomInfoHelp.getUniqueId();
                    if(!TextUtils.isEmpty(uniqueId)) {
                        audioTrackCheckMap.clear();
                        audioTrackCheckMap.put(uniqueId,isChecked);
                    }
                }

                if(RoomInfoHelp.isSongInfoExist() ) {

                    if(mMyInfo.isHost()) {
                        RoomInfo roomInfo = KTVRoomManager.getInstance().getRoomInfo();
                        RoomAPI.setSwitchAudioTrack(mMyInfo.getUserID(), roomInfo.getCreatorId(), roomInfo.getCreatorName(), !isChecked, roomInfo.getSongInfo().getSongId(), roomInfo.getSongInfo().getCreateTime(), new ICommonResponseCallback() {
                            @Override
                            public void onResponse(int errorCode) {
                                if(errorCode == 0){
                                    if(isChecked) {
                                        Toast.makeText(KTVRoomActivity.this, "切换伴奏成功", Toast.LENGTH_SHORT).show();
                                    }else {
                                        Toast.makeText(KTVRoomActivity.this, "切换原唱成功", Toast.LENGTH_SHORT).show();
                                    }

                                }else if(errorCode == GoChatErrorFailNetwork){
                                    ToastUtil.showWarnToast("网络异常，请检查网络后重试");
                                    audioTrackCheckMap.clear();
                                    mMusicPlayerMenu.getSongSwitch().setChecked(!isChecked);
                                }else {
                                    if(isChecked) {
                                        Toast.makeText(KTVRoomActivity.this, "切换伴奏失败", Toast.LENGTH_SHORT).show();
                                    }else {
                                        Toast.makeText(KTVRoomActivity.this, "切换原唱失败", Toast.LENGTH_SHORT).show();
                                    }
                                    audioTrackCheckMap.clear();
                                    mMusicPlayerMenu.getSongSwitch().setChecked(!isChecked);
                                }
                            }
                        });
                    }else if(mMyInfo.isStageGuest()){
                        ZGKTVPlayerManager.getInstance().setAudioOrigin(isChecked);
                    }
                }

            }
        });



        /**
         * 播放器状态回调
         */
        ZGKTVPlayerManager.getInstance().setOnIZGKTVPlayerUpdateListener(new IZGKTVPlayerUpdateListener() {

            @Override
            public void onPlaybackStateUpdate( ZegoMediaPlayer mediaPlayer, ZegoMediaPlayerState state, int errorCode) {

                if(errorCode == 1017013 || errorCode == 1017014 || errorCode == 1017015 || errorCode == 1017044 || errorCode == 1017045 || errorCode == 1017034){

                    if(RoomInfoHelp.getSongId() != null) {

                        FileDownloadManager.getInstance().downloadFile(RoomInfoHelp.getSongId());



                        if(mMyInfo.isHost()) {
                            long networkTime = RTCSDKManager.getInstance().getNetworkTime();
                            long musicPlayTime = networkTime + 3000;
                            KTVRoomManager.getInstance().syncTime(musicPlayTime, 0);
                        }

                    }else {
                        T.show("歌曲信息获取失败，请重试");
                    }
                }

                if(errorCode == 0 ){
                    if(state != ZegoMediaPlayerState.PLAYING) {
                        mPlayerCurrentTimeMillis = 0;
                        mPlayerCurrentProgress = 0;
                    }else {
                        mPlayerCurrentTimeMillis = System.currentTimeMillis();
                        mPlayerCurrentProgress = mediaPlayer.getCurrentProgress();
                        mRefreshLyricProgressTimer.scheduleAtFixedRate(0, 30);
                    }
                }


                if(errorCode == 0 || errorCode == ZGKTVPlayerManager.AUTO_PLAY_END_CODE) {
                    if (!RoomInfoHelp.isSongInfoExist()) {
                        return;
                    }
                    String songId = RoomInfoHelp.getSongId();
                    if (state == ZegoMediaPlayerState.PLAYING) {
                        mMusicPlayerMenu.setScore(0);

                        if(mIsMicMuted){
                            ZGKTVPlayerManager.getInstance().pauseScore();
                            mCurrentPitch = 0;
                        }else {
                            ZGKTVPlayerManager.getInstance().resumeScore();
                        }


                        switchMusicPlayerMenu();
                        ZGKTVLyric lrc = ZGKTVLyricManager.getInstance().getLyric(songId);
                        if (lrc != null) {
                            mMusicPlayerMenu.setLrc(songId, lrc);
                        }

//                    if(mMusicPlayerMenu.getState() != START_PLAYING){
                        switchMusicPlayerMenu();
//                    }


                        mMusicPlayerMenu.getPitchView().reset();
                        ZGKTVPlayerManager.getInstance().getStandardPitch(new IZegoCopyrightedMusicGetStandardPitchCallback() {
                            @Override
                            public void onGetStandardPitchCallback(int errorCode, String pitch) {


                                if (errorCode == 0) {
                                    Gson gson = new Gson();
                                    mMusicPlayerMenu.getPitchView().setStandardPitch(ZegoPitchViewHelper.parsePitch(gson.fromJson(pitch, ZGKTVPitch.class)));
                                }else{
                                    mMusicPlayerMenu.getPitchView().reset();
                                    T.show("获取音高线失败");
                                }
                            }
                        });


                    } else if (state == ZegoMediaPlayerState.PLAY_ENDED) {
                        if (mMyInfo.getUserID() == KTVRoomManager.getInstance().getHostInfo().getUserID() && errorCode == ZGKTVPlayerManager.AUTO_PLAY_END_CODE) {
                            KTVRoomManager.getInstance().switchSong(null);

                        }

                    }
                }



            }

            @Override
            public void onPlaybackProgressUpdate( ZegoMediaPlayer mediaPlayer, long progress) {

                mPlayerCurrentTimeMillis = System.currentTimeMillis();
                mPlayerCurrentProgress = mediaPlayer.getCurrentProgress();


            }

            @Override
            public void onCurrentPitchValueUpdate(String resourceID, int currentDuration, int pitchValue) {
//                int pitch = ZGKTVPlayerManager.getInstance().getCurrentPitch();
                if(mIsMicMuted){
                    return;
                }
                mCurrentPitch = pitchValue;
//                mMusicPlayerMenu.getPitchView().setCurrentSongTimeAndPitch(currentDuration,pitchValue);
            }
        });

        //房间信息重新拉取的时候 回调
        KTVRoomOrderSongManager.getInstance().setPlayingSongUpdateCallback(new IPlayingSongUpdateCallback() {
            @Override
            public void onPlayingSongUpdateCallback(RoomInfo roomInfo) {
                RoomInfoHelp.updateRoomInfo(roomInfo);
            }
        });
        /**
         * 已点列表获取歌曲具体信息回调
         */
        KTVRoomOrderSongManager.getInstance().setSongInfoUpdateListener(new ISongInfoUpdateListener() {
            @Override
            public void onSongInfoUpdate() {
                if(!RoomInfoHelp.isSongInfoExist()){
                    mMusicPlayerMenu.setSongName(null);
                    return;
                }
                String songId = RoomInfoHelp.getSongId();
                ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
                if(song != null) {
                    mMusicPlayerMenu.setSongName(song.getSongName());
                }
            }
        });
        setSongUpdateListener();
        /**
         * 房间歌曲信息更新
         */
        KTVRoomManager.getInstance().setRoomSongStateUpdateListener(new IRoomSongStateUpdateListener() {
            @Override
            public void onRoomSongStateUpdate() {
                if(!TextUtils.isEmpty(songId)){
                    ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
                    if(song != null){
                        song.setSongUpdateListener(null);
                    }
                }

                setSongUpdateListener();
            }
        });
        /**
         * 已点列表更新
         */
        KTVRoomOrderSongManager.getInstance().setKTVOrderSongListUpdate(new IOrderSongListUpdate() {
            @Override
            public void OrderSongListUpdate(List<OrderedSongInfo.OrderedSongItemInfo> orderList) {
                int size = 0;
                if(orderList != null){
                    size = orderList.size();
                }

                if(size == 0){
                    songNum.setVisibility(View.GONE);
                }else if(!mMyInfo.isAudience()){
                    songNum.setVisibility(View.VISIBLE);
                }

                if(size > 99)
                {
                    songNum.setText("99+");
                }else {
                    songNum.setText(String.valueOf(size));
                }
            }
        });
    }

    private void setSongUpdateListener(){
        songId = RoomInfoHelp.getSongId();
        if(!TextUtils.isEmpty(songId)){
            ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
            if(song == null){
                song = new ZGKTVCopyrightedSong();
                song.setSongID(songId);
                L.i("songId","putSong111 : " + " : : " + song.getLoadState());
                ZGKTVPlayerManager.getInstance().putSong(songId,song);
            }
            song.setSongUpdateListener(KTVRoomActivity.this);

        }
    }


    private void initServerCallback() {
        mIExecuteCallback = new IExecuteCallback() {
            @Override
            public void onSuccess(Constants.OperationType type) {
                switch (type) {
                    case OFF_STAGE:
                        L.e(TAG,"OFF_STAGE pitchShowMap clear: ");
                        pitchShowMap.clear();
                        audioTrackCheckMap.clear();
                        lastSEIProgress =0;
                        Toast.makeText(KTVRoomActivity.this, "下麦成功", Toast.LENGTH_SHORT).show();

                        //只有不是主播的时候才能有以下操作
                        if (mMyInfo.getUserID() != KTVRoomManager.getInstance().getHostInfo().getUserID()) {
                            //本人下麦后更新网络状态为 UNKNOWN
                            offStageAdapter();
                            L.i(TAG,"stopTimerTask  111");
                            ZGKTVPlayerManager.getInstance().stopTimerTask();
                            ZGKTVPlayerManager.getInstance().stop();
                            switchMusicPlayerMenu();
                        }

                        break;
                    case ON_STAGE:
                        audioTrackCheckMap.clear();
                        lastSEIProgress = 0;
//                        Toast.makeText(KTVRoomActivity.this, "上麦成功", Toast.LENGTH_SHORT).show();
                        if(seiCount < DEF_SEI_COUNT){
                            seiCount = DEF_SEI_COUNT;
                        }
                        updateOrderList();
                        switchMusicPlayerMenu();
                        break;
                    case NEXT_SONG:
                        Toast.makeText(KTVRoomActivity.this, "切歌成功", Toast.LENGTH_SHORT).show();
                        break;
                    case SELECT_SONG:
                        Toast.makeText(KTVRoomActivity.this, "点歌成功,已加入队列", Toast.LENGTH_SHORT).show();
                        break;
                    case GET_ONSTAGE_USERS: //获取麦上列表成功回调
                        mMyInfo = KTVRoomManager.getInstance().getmMyUserInfo();
                        onRoomInitSuccess();
                        break;
                    case MIC:
                        if (mIsMicMuted) {
                            mic.setCompoundDrawablesWithIntrinsicBounds(null, getDrawable(R.mipmap.ktv_mute_mic), null, null);
                            Toast.makeText(KTVRoomActivity.this, "麦克风已关闭", Toast.LENGTH_SHORT).show();
                            ZGKTVPlayerManager.getInstance().pauseScore();
                            mCurrentPitch = 0;
                        } else {
                            mic.setCompoundDrawablesWithIntrinsicBounds(null, getDrawable(R.mipmap.ktv_mic), null, null);
                            Toast.makeText(KTVRoomActivity.this, "麦克风已打开", Toast.LENGTH_SHORT).show();
                            ZGKTVPlayerManager.getInstance().resumeScore();
                        }
                        break;
                    case CLOSE_ROOM: //退出房间
                        finish();
                        startActivity(QualityManagerProxy.getQualityIntent(KTVRoomActivity.this));
                        break;
                    default:
                        break;
                }
            }

            @Override
            public void onFailed(int errorCode, Constants.OperationType type) {

                if (errorCode == ERROR_USER_OFF_LINE) {
                    showRoomCloseAlertDialog("网络异常", "网络无法连接，请离开房间后再试。");
                    return;
                }

                switch (type) {
                    case ON_STAGE:
                        if(errorCode == 80018){ //上麦人数达到上限
                            Toast.makeText(KTVRoomActivity.this, "上麦失败，麦位已满", Toast.LENGTH_SHORT).show();
                        }else {
                            if(mMyInfo.isOnstage()){
                                Toast.makeText(KTVRoomActivity.this, "切换麦位失败", Toast.LENGTH_SHORT).show();
                            }else {
                                Toast.makeText(KTVRoomActivity.this, "上麦失败", Toast.LENGTH_SHORT).show();
                            }
                        }

                        break;
                    case OFF_STAGE:
                        if (errorCode == ERROR_USER_OFF_LINE) {
//                            Toast.makeText(KTVRoomActivity.this, "用户不在线", Toast.LENGTH_SHORT).show();
//                            ToastUtil.showWarnToast(R.string.room_list_join_offline);
                            showRoomCloseAlertDialog("网络异常", "网络无法连接，请离开房间后再试。");
                        }
                        break;
                    default:
                        break;
                }
            }
        };
    }

    private void initRTCSDKListener() {
        RTCSDKManager.getInstance().addListener(this);
    }

    @Override
    protected void onDestroy() {
        mRefreshLyricProgressTimer.cancel();
        if (mAnimationDrawable != null) {
            mAnimationDrawable.stop();
        }
        if (mHostSyncProgressTimer != null) {
            mHostSyncProgressTimer.cancel();
            mHostSyncProgressTimer = null;
        }
//        ZGKTVMediaPlayer.getInstance().destroy();
        RTCSDKManager.getInstance().addListener(null);
        NetWorkKtvConManager.getInstance().clearData();
        stopKTVService();
        super.onDestroy();
    }


    private void initView() {

        //心跳
        LoginInfoManager.getInstance().heartBeat(mMyInfo.getUserID());

        //网络测速中
        setNetworkQualityContent(networkQualityContent, mMyMonitorInfo.getNetworkQuality());
        mZegoLyricView.setHighLightTextSizeSp(20);
        mZegoLyricView.setDefaultTextSizeSp(12);
        mZegoLyricView.setShaderTextSizeSp(12);
        mZegoLyricView.setLineSpaceDp(12);
        mZegoLyricView.setHintTextSizeSp(12);
        mZegoLyricView.reset(" ");
        guestRecyclerView = findViewById(R.id.rv_guest_list);


        mNetworkQualityMap = new HashMap<>();
        stateGuestAdapter = new KTVRoomStageGuestAdapter(mNetworkQualityMap);
        guestRecyclerView.setLayoutManager(new GridLayoutManager(this, 4));
        guestRecyclerView.setAdapter(stateGuestAdapter);
        ((SimpleItemAnimator) guestRecyclerView.getItemAnimator()).setSupportsChangeAnimations(false);

        //麦位的点击
        stateGuestAdapter.setOnclickItemListner(new KTVRoomStageGuestAdapter.OnItemClickListener() {
            @Override
            public void onItemClick(int position, UserInfo info) {
                if (info.getOnstageState() != 2) {
                    //房主不可以换麦位
                    if (!mMyInfo.isOnstage()) {
                        NetworkQuality networkQuality = mViewModel.getMyMonitorInfoMutableLiveData().getValue().getNetworkQuality();
                        if (NetworkQuality.BAD == networkQuality) {
                            Toast.makeText(KTVRoomActivity.this, "当前网络较差，无法支持KTV合唱", Toast.LENGTH_SHORT).show();
                        } else if (NetworkQuality.UNKNOWN == networkQuality) {
                            Toast.makeText(KTVRoomActivity.this, "请等待测速完成", Toast.LENGTH_SHORT).show();
                        } else {
                            KTVRoomManager.getInstance().onStageOperation(mMyInfo.getUserID(), mMyInfo.getUserID(), position, mIExecuteCallback);
                        }
                        return;
                    }
                    if (mMyInfo.getUserID() != KTVRoomManager.getInstance().getHostInfo().getUserID()) {
                        if(mMyInfo.isOnstage())
                        {
                            KTVRoomManager.getInstance().switchMicLocation(mMyInfo.getUserID(), mMyInfo.getUserID(), position, new IExecuteCallback() {
                                @Override
                                public void onSuccess(Constants.OperationType type) {

                                }

                                @Override
                                public void onFailed(int errorCode, Constants.OperationType type) {
                                    Toast.makeText(KTVRoomActivity.this, "切换麦位失败", Toast.LENGTH_SHORT).show();
                                }
                            });
                        }else {
                            KTVRoomManager.getInstance().onStageOperation(mMyInfo.getUserID(), mMyInfo.getUserID(), position, mIExecuteCallback);
                        }
                    } else {
                        Toast.makeText(KTVRoomActivity.this, "房主不可切换麦位", Toast.LENGTH_SHORT).show();
                    }
                } else if (info.getUserID() == mMyInfo.getUserID() || mMyInfo.getUserID() == KTVRoomManager.getInstance().getHostInfo().getUserID()) {
                    if (position == 0) {
                        return;
                    }

                    //点击自己的麦位 或者 主播点击已经上麦的人头像 弹出下麦dialog
                    if (mMicOperationDialog == null) {
                        mMicOperationDialog = new BottomSheetDialog(KTVRoomActivity.this, R.style.KTVBottomSheetDialog);
                        mMicOperationDialog.setContentView(R.layout.ktv_dialog_mic_operation);
                        mMicOperationDialog.findViewById(R.id.leave_stage).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                KTVRoomManager.getInstance().offStageOperation(mMyInfo.getUserID(), info.getUserID(), position, mIExecuteCallback);
                                mMicOperationDialog.dismiss();
                                mMicOperationDialog = null;
                            }
                        });
                        mMicOperationDialog.findViewById(R.id.cancel).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                mMicOperationDialog.dismiss();
                                mMicOperationDialog = null;
                            }
                        });
                    }
                    mMicOperationDialog.show();
                }
            }
        });

    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            return true;
        }
        return false;
    }

    /**
     * 拉取成员列表
     */
    private void pullUserList() {
        KTVRoomManager.getInstance().getUserList(mMyInfo.getUserID(), mIExecuteCallback);
    }

    /**
     * 房间初始化成功，加载成员列表成功，可以通过KTVRoomManager拿到用户数据
     */
    private void onRoomInitSuccess() {
        stateGuestAdapter.initUserList();
        UserInfo hostInfo = KTVRoomManager.getInstance().getHostInfo();
        if (hostInfo.getUserID() == mMyInfo.getUserID()) {
            //房主逻辑默认上台
            mMyInfo = hostInfo;
            mViewModel.setMyInfo(hostInfo);
        }
        mViewModel.setHostUser(hostInfo);
        GlideImageLoader.getInstance().displayImageByAvatar(hostInfo.getAvatar(), hostAvatar);
        hostNickName.setText(hostInfo.getNickName());

        Map<Long, UserInfo> userModelMap = KTVRoomManager.getInstance().getUserModelMap();
        if (!userModelMap.containsKey(mMyInfo.getUserID())) { //说明不在台上

            if(mMyInfo.isOnstage()){
                mMyInfo.setOnstageState(1);
            }
            //下麦过后的处理
            offStageAdapter();
            KTVRoomManager.getInstance().stageOperationSetRtc();
            L.e(TAG,"onRoomInitSuccess pitchShowMap clear: ");
            pitchShowMap.clear();
        }
        switchMusicPlayerMenu();

    }

//    public void pauseMV(View view) {
//        long networkTime = RTCSDKManager.getInstance().getNetworkTime();
//        long musicPauseTime = networkTime + Constants.COMMAND_TAKE_EFFECT_GAP;
//        if (mZgktvMediaPlayer.getCurrentState() == ZegoMediaPlayerState.PLAYING) {
//            KTVRoomManager.getInstance().syncTime(musicPauseTime, Constants.MediaPlayerBehavior.PAUSE.value());
//        }
//    }
//
//    public void resumeMV(View view) {
//        long networkTime = RTCSDKManager.getInstance().getNetworkTime();
//        long musicPauseTime = networkTime + Constants.COMMAND_TAKE_EFFECT_GAP;
//        if (mZgktvMediaPlayer.getCurrentState() == ZegoMediaPlayerState.PAUSING) {
//            KTVRoomManager.getInstance().syncTime(musicPauseTime, Constants.MediaPlayerBehavior.RESUME.value());
//        }
//    }

    private void showRoomCloseAlertDialog(String title, String content) {
        L.d(TAG, "room relogin, errorCode----w------:");
        //todo 房间已经被关闭 不管房主还是用户都要退出到房间列表
//        if (mMyInfo.getUserID() == KTVRoomManager.getInstance().getHostInfo().getUserID()) {
//            return;
//        }

        if (isFinishing() || messageDialog != null) {
            return;
        }

        messageDialog = new MessageDialog(this, R.layout.ktv_dialog_msg_single_button);
        messageDialog.setMsgTitle(title);
        messageDialog.setMsgContent(content);
        messageDialog.setSureListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                messageDialog.dismiss();
                messageDialog = null;
                KTVRoomManager.getInstance().logoutRoom(true);
                finish();
                startActivity(QualityManagerProxy.getQualityIntent(KTVRoomActivity.this));
            }
        });

    }

    /**
     * 成员列表变更通知
     * @param notifyInfo
     */
    @Override
    public void onUserListUpdateNotify(@NotNull NotifyUserListUpdateInfo notifyInfo) {
        for (NotifyUserListUpdateInfo.UserUpdateInfo updateInfo : notifyInfo.users) {
            if (updateInfo.delta == -1) {
                if (updateInfo.isStageGuest()) {
                    pullUserList();
                    break;
                }
            }
        }
    }

    /**
     *  成员状态变更通知
     * @param notifyInfo
     */
    @Override
    public void onUserStateUpdateNotify(@NotNull NotifyUserStateUpdateInfo notifyInfo) {
        if (notifyInfo.type == 2) {
            for (UserInfo info : notifyInfo.users) {
                if (info.getUserID() == mMyInfo.getUserID()) {
                    if (mMyInfo.getOnstageState() != info.getOnstageState()) {
                        if (info.getOnstageState() == 2) {
                            switchMic(true, null);
                            Toast.makeText(KTVRoomActivity.this, "上麦成功", Toast.LENGTH_SHORT).show();
                        } else if (info.getOnstageState() == 1 && notifyInfo.operatorUid == KTVRoomManager.getInstance().getHostInfo().getUserID()) {
                            Toast.makeText(KTVRoomActivity.this, "主播已请你下麦", Toast.LENGTH_SHORT).show();
                            L.i("主播已请你下麦","主播已请你下麦:" + info.isAudience());
                            offStageAdapter();
                            KTVRoomManager.getInstance().stageOperationSetRtc();
                            L.e(TAG,"主播已请你下麦 pitchShowMap clear: ");
                            pitchShowMap.clear();
                        }
                        mMyInfo = info;
                        mViewModel.setMyInfo(info);
                    } else if (mMyInfo.getMicIndex() != info.getMicIndex()) {
                        Toast.makeText(KTVRoomActivity.this, "已成功切换麦位", Toast.LENGTH_SHORT).show();
                        mMyInfo = info;
                    }
                    KTVRoomManager.getInstance().setMyUserInfo(mMyInfo);
                    break;
                }
            }
            pullUserList();
        } else {
            for (UserInfo userInfo : notifyInfo.users) {
                if (userInfo.getUserID() == KTVRoomManager.getInstance().getHostInfo().getUserID()) {
                    mViewModel.setHostUser(userInfo);
                    KTVRoomManager.getInstance().setHostUserInfo(userInfo);
//                    dealWithHostVolume(userInfo, null);
                } else {
//                    stateGuestAdapter.onUserStateChange(userInfo);
                }

                stateGuestAdapter.onUserStateChange(userInfo);
            }

        }
    }

    /**
     * 房间关闭
     */
    @Override
    public void onRoomCloseNotify() {
        L.d(TAG, "房间关闭");
        //房主主动退出房间的不需要提示 房间已被解散
        if(!hostCloseRoom){
            showRoomCloseAlertDialog("房间已被解散", "房主已退出歌房，房间将自动解散，点击确认退回房间列表。");
        }
    }

    /**
     *  房间状态变更
     */
    @Override
    public void onRoomStateNotify(NotifyRoomStateUpdateInfo info) {
        if (mMyInfo.isAudience()) {
            return;
        }
        L.e(TAG,"onRoomStateNotify");

        audioTrackCheckMap.clear();
        mMusicPlayerMenu.getSongSwitch().setChecked(info.getOriginPlay()==0);
        RoomInfoHelp.updateOriginPlay(info.getOriginPlay());
        ZGKTVPlayerManager.getInstance().setAudioOrigin(info.getOriginPlay()==0);
    }

    public static ZGKTVCopyrightedSong convertIdToSongIndex(String curSongId) {
        return ZGKTVPlayerManager.getInstance().getSong(curSongId);
    }


    @Override
    public void onRoomReconnected() {
        reLoadApi();
    }

    @Override
    public void onPlayerRecvSEI(String streamId, HostSEIInfo info) {

        if (mMyInfo.isAudience()) {

            if (!streamId.endsWith(Constants.MIX_STREAM_SURFIX)) {
                return;
            }

            if (info.getRole() != 3) {
                return;
            }

            if (TextUtils.isEmpty(info.getSongID())) {
                return;
            }

            if(!info.getSongID().equals(RoomInfoHelp.getSongId())){
                L.e("SEI","11 RoomInfoHelp.getSongId():" + RoomInfoHelp.getSongId() +" : info.getSongID() : " + info.getSongID());
                updateOrderList();
                return;
            }

            if(mKState != info.getState()){
                mKState = info.getState();
                switchMusicPlayerMenu();
            }

            String curSongId = info.getSongID();
            String songId = RoomInfoHelp.getSongId();


            mViewModel.setTotalTime(info.getTotalDuration());
            mViewModel.setPlayTime(info.getProcessMillis());
            ZGKTVLyric lrc = ZGKTVLyricManager.getInstance().getLyric(songId);
            if(lrc != null) {
                if(mKState == 1) {
                    mMusicPlayerMenu.setLrc(songId, lrc);
                }
            }
            mCurrentSEIProgress = info.getProcessMillis();

            if(mKState == 1) {
                mPlayerCurrentTimeMillis = System.currentTimeMillis();
                mPlayerCurrentProgress = mCurrentSEIProgress;
            }else{
                mPlayerCurrentTimeMillis = 0;
                mPlayerCurrentProgress = 0;
            }

            mRefreshLyricProgressTimer.scheduleAtFixedRate(0, 30);
            mMusicPlayerMenu.updateTime(info.getProcessMillis(),info.getTotalDuration());

        } else {

            if (streamId.endsWith(Constants.MIX_STREAM_SURFIX)) {
                return;
            }

            if (TextUtils.isEmpty(info.getSongID())) {
                return;
            }

            if (info.getSongID().equals(RoomInfoHelp.getSongId())) {
                countMsd(streamId, info,RoomInfoHelp.getSongId());
                if (info.getRole() != 3 || seiCount != 0) {
                    return;
                }

                // TODO: 10/21/21 SEI 同步播放
                ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(RoomInfoHelp.getSongId());

                ZGKTVLyric lrc = ZGKTVLyricManager.getInstance().getLyric(songId);
                if (lrc != null) {
                    mMusicPlayerMenu.setLrc(songId, lrc);
                } else {
                    ZGKTVCopyrightedMusicManager.getInstance().getKrcLyricForSongID(info.getSongID(), null);
                }


                if(song == null){
                    T.show("SEI 未获取到歌曲信息");
                }else {
                    if(song.getLoadState() == ZGKTVSongLoadState.NO_LOAD) {
                        String token = RoomInfoHelp.getToken();
                        L.e(TAG,"SEI 未获取到歌曲信息token");
                        FileDownloadManager.getInstance().downloadFileByToken(info.getSongID(),token);
                    }
                }

                if(song != null){
                    if( song.getLoadState() == ZGKTVSongLoadState.LOAD_COMPLETE && song.getLyric() != null && (song.getState().value() - 2 ) != info.getState()){
                        switch (ZegoMediaPlayerState.getZegoMediaPlayerState(info.getState())) {
                            case PLAYING:
                                RTCSDKManager.getInstance().restartMvStream();
                                if(RoomInfoHelp.isAccompany()){

                                    mMusicPlayerMenu.getSongSwitch().setChecked(true);
                                }else {

                                    mMusicPlayerMenu.getSongSwitch().setChecked(false);
                                }
                                ZGKTVPlayerManager.getInstance().startImmediateAfterLoad(song.getSongID(),RoomInfoHelp.isAccompany(), info.getPointTimeKey(), info.getProcessMillis());
                            break;
                        }
                    }
                }


//                int songIndex = convertIdToSongIndex(info.getSongID());
//                if (mZgktvMediaPlayer.getCurrentState() != ZegoMediaPlayerState.getZegoMediaPlayerState(info.getState())) {
//                    switch (ZegoMediaPlayerState.getZegoMediaPlayerState(info.getState())) {
//                        case PLAYING:
//                            RTCSDKManager.getInstance().restartMvStream();
//                            mZgktvMediaPlayer.startImmediateAfterLoad(mSongFilePaths.get(songIndex), info.getPointTimeKey(), info.getProcessMillis());
//                            break;
//                        case PAUSING:
//                            mZgktvMediaPlayer.pauseImmediateAfterLoad(mSongFilePaths.get(songIndex), info.getProcessMillis());
//                            break;
//                        default:
//                            break;
//                    }
//                }
            }else {
                if(info.getRole() == 3){
                    updateOrderList();
                }


                L.e("SEI","22 RoomInfoHelp.getSongId():" + RoomInfoHelp.getSongId() +" : info.getSongID() : " + info.getSongID());
            }
        }
    }

    private void countMsd(String streamID, HostSEIInfo info,String songId) {
        if (info.getRole() == 3) {
            seiCount++;
            if(seiCount < MAX_SEI_COUNT){
                return;
            }
            seiCount = 0;
            L.e("SEI", "countMsd:" + mZegoNetworkTime);
            L.e("SEI", "countMsd:" + mPlayProgress);
            L.e("SEI", "countMsd:" + info.getPointTimeKey());
            L.e("SEI", "countMsd:" + info.getProcessMillis());
        }

        if (info.getPointTimeKey() == 0 || mZegoNetworkTime == 0) {
            return;
        }



        final long musicDelay = (mZegoNetworkTime - mPlayProgress) - (info.getPointTimeKey() - info.getProcessMillis());
        if (info.getRole() == 3) {
            L.e("SEI", "countMsd:" + musicDelay);
        }

        if (info.getRole() == 3) {



            if (mHostMonitorInfo == null) {
                mHostMonitorInfo = new UserDataMonitorInfo();
            }
            mHostMonitorInfo.setMsd(musicDelay);

            if (Math.abs(musicDelay) > 30) {
                L.i("SEI","syncProgress:" + mZegoNetworkTime);
                L.i("SEI","syncProgress:" + mPlayProgress);
                L.i("SEI","syncProgress:" + info.getPointTimeKey());
                L.i("SEI","syncProgress:" + info.getProcessMillis());
                ZGKTVPlayerManager.getInstance().syncProgress(songId,musicDelay);
            }
        }

//        } else {
//            long uid = Long.parseLong(streamID);
//            stateGuestAdapter.updateGuestMsd(uid, musicDelay);
//        }

        long uid = Long.parseLong(streamID);
        stateGuestAdapter.updateGuestMsd(uid, musicDelay);
    }

    /**
     * 拉流质量更新
     * @param streamID
     * @param quality
     */
    @Override
    public void onPlayerQualityUpdate(String streamID, ZegoPlayStreamQuality quality) {

        //拉流质量回调
        // peerToPeerDelay ： （本端rtt + 对端rtt）/2 + delay
        // delay ： jitterbuffer上限
        // peerToPeerPacketLostRate ： 端到端丢包率
        //可以算出其他端rtt = (p2delay - 本端rtt/2 - delay)*2。这里要说明下质量回调里的rtt都是本端的并非对端的。也可以算出其他端plr = p2pplr - 本端plr。

        UserDataMonitorInfo userDataMonitorInfo = new UserDataMonitorInfo();
        userDataMonitorInfo.setRtt((quality.peerToPeerDelay - quality.rtt / 2 - quality.delay) * 2);
        userDataMonitorInfo.setPacketLoss(quality.peerToPeerPacketLostRate - quality.packetLostRate);
        userDataMonitorInfo.setPeerToPeerDelay(quality.peerToPeerDelay);
        if (!mMyInfo.isAudience()) {
            LinkedList<UserDataMonitorInfo> networkQualityBeans = addToNetworkList(streamID, userDataMonitorInfo);
            if (networkQualityBeans.size() >= Constants.SAMPLE_NUM_IN_GROUP) {
                userDataMonitorInfo.setNetworkQuality(checkNetworkQuality(networkQualityBeans, Constants.SAMPLE_NUM_IN_GROUP));
                if (streamID.startsWith(String.valueOf(KTVRoomManager.getInstance().getHostInfo().getUserID()))) {
                    if (mHostMonitorInfo != null) {
                        userDataMonitorInfo.setMsd(mHostMonitorInfo.getMsd());
                    }
                    mHostMonitorInfo = userDataMonitorInfo;
                    mViewModel.setHostMonitorInfoMutableLiveData(userDataMonitorInfo);
                } else {
//                    stateGuestAdapter.updateGuestPlayQuality(streamID, userDataMonitorInfo);
                }

//                stateGuestAdapter.updateGuestPlayQuality(streamID, userDataMonitorInfo);
            }
        } else {
            if (streamID.startsWith(String.valueOf(KTVRoomManager.getInstance().getHostInfo().getUserID()))) {
                mViewModel.setHostMonitorInfoMutableLiveData(userDataMonitorInfo);
            } else {
//                stateGuestAdapter.updateGuestPlayQuality(streamID, userDataMonitorInfo);
            }
//            stateGuestAdapter.updateGuestPlayQuality(streamID, userDataMonitorInfo);
        }

        stateGuestAdapter.updateGuestPlayQuality(streamID, userDataMonitorInfo);
    }

    /**
     * 房间流更新
     * @param roomID
     * @param updateType
     * @param streamList
     * @param extendedData
     */
    @Override
    public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData) {
        if (updateType == ZegoUpdateType.DELETE) {
            for (ZegoStream streamInfo : streamList) {
                mNetworkQualityMap.remove(streamInfo.streamID);
            }
        }
    }

    /**
     * 推流质量回调
     * @param streamID
     * @param quality
     * @param zegoNetworkTimeInfo
     */
    @Override
    public void onPublisherQualityUpdate(String streamID, ZegoPublishStreamQuality quality, ZegoNetworkTimeInfo zegoNetworkTimeInfo) {
        //推流质量回调
        if (streamID.contains("_")) {
            return;
        }
        UserDataMonitorInfo userDataMonitorInfo = new UserDataMonitorInfo();
        userDataMonitorInfo.setRtt(quality.rtt);
        userDataMonitorInfo.setNtpDeviation(zegoNetworkTimeInfo.maxDeviation);
        userDataMonitorInfo.setPacketLoss(quality.packetLostRate);
        mMyMonitorInfo.setRtt(quality.rtt);
        mMyMonitorInfo.setNtpDeviation(zegoNetworkTimeInfo.maxDeviation);
        mMyMonitorInfo.setPacketLoss(quality.packetLostRate);
        LinkedList<UserDataMonitorInfo> networkQualityBeans = addToNetworkList(streamID, userDataMonitorInfo);
        if (networkQualityBeans.size() >= Constants.SAMPLE_NUM_IN_GROUP) {
            NetworkQuality networkQualityBean = checkNetworkQuality(networkQualityBeans, Constants.SAMPLE_NUM_IN_GROUP);
            if (!NetworkQuality.BAD.equals(mMyNetworkQuality) && networkQuality.equals(NetworkQuality.BAD)) {
                Toast.makeText(KTVRoomActivity.this, "网络状态不佳，影响合唱效果，请切换网络", Toast.LENGTH_SHORT).show();
            }
            mMyNetworkQuality = networkQualityBean;
            userDataMonitorInfo.setNetworkQuality(networkQualityBean);
            mMyMonitorInfo.setNetworkQuality(networkQualityBean);
            if (mMyInfo.isHost()) {
                mViewModel.setHostMonitorInfoMutableLiveData(userDataMonitorInfo);
            } else {
//                stateGuestAdapter.updateGuestPlayQuality(streamID, userDataMonitorInfo);
            }
            stateGuestAdapter.updateGuestPlayQuality(streamID, userDataMonitorInfo);
            mViewModel.setMyMonitorInfoMutableLiveData(mMyMonitorInfo);
            setNetworkQuality(networkQuality, mMyMonitorInfo.getNetworkQuality());
            setNetworkQualityContent(networkQualityContent, mMyMonitorInfo.getNetworkQuality());
            networkQuality.setVisibility(mMyMonitorInfo.getNetworkQuality().equals(NetworkQuality.UNKNOWN) ? View.GONE : View.VISIBLE);


        }
    }

    /**
     * //点歌列表变更通知
     * @param notifyOrderListUpdateInfo
     */
    @Override
    public void onOrderListUpdate(NotifyOrderListUpdateInfo notifyOrderListUpdateInfo) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if(notifyOrderListUpdateInfo.getType() == 1){

                    KTVRoomOrderSongManager.getInstance().addOrderSongListForNotify(notifyOrderListUpdateInfo.getSongs());

                }else if(notifyOrderListUpdateInfo.getType() == 2){

                    KTVRoomOrderSongManager.getInstance().cancelOrderSongListForNotify(notifyOrderListUpdateInfo.getSongs());

                }else if(notifyOrderListUpdateInfo.getType() == 3){

                    KTVRoomOrderSongManager.getInstance().reorderSongListForNotify(notifyOrderListUpdateInfo.getSongs());

                }else if(notifyOrderListUpdateInfo.getType() == 4){



                    if(isSkipSong(notifyOrderListUpdateInfo)){

                        if(!mMyInfo.isAudience() && pitchShowMap.get(RoomInfoHelp.getUniqueId()) != null) {

                            int averageScore = ZGKTVPlayerManager.getInstance().getAverageScore();
                            int totalScore = ZGKTVPlayerManager.getInstance().getTotalScore();

                            if (!isFinishing() && !isDestroyed() && averageScore >=0 && totalScore >= 0) {

                                if (mScoreShowDialog != null) {
                                    mScoreShowDialog.dismiss();
                                    mScoreShowDialog = null;
                                }

                                mScoreShowDialog = new KTVScoreShowDialog(KTVRoomActivity.this);
                                mScoreShowDialog.show();
                                mScoreShowDialog.setScoreShow(averageScore,totalScore );

                                L.e(TAG,"show dialog averageScore:" + averageScore);
                                L.e(TAG,"show dialog totalScore:" + totalScore);
                            }else {
                                L.e(TAG,"show dialog error");
                            }
                        }
                    }
                    L.e(TAG,"onOrderListUpdate pitchShowMap clear: ");
                    pitchShowMap.clear();
                    //表示切歌了
                    if(notifyOrderListUpdateInfo.getSongs() == null || notifyOrderListUpdateInfo.getSongs().size() == 0){
                        if(RoomInfoHelp.isSongInfoExist()){

                            String songId = RoomInfoHelp.getSongId();
                            KTVRoomOrderSongManager.getInstance().skipSong(songId);
                            RoomInfoHelp.removeSongInfo();
                            L.i(TAG,"stopTimerTask 222");
                            ZGKTVPlayerManager.getInstance().stopTimerTask();
                            ZGKTVPlayerManager.getInstance().stop();
                            mMusicPlayerMenu.updateTime(null,null);
                        }

                    }

                    for(NotifyOrderListUpdateInfo.SongsDTO song : notifyOrderListUpdateInfo.getSongs()){
//                        if(song.getReadyStatus() == 2){
                            RoomInfoHelp.updateSongInfo(song);
//                        }

                        if(song.getReadyStatus() == 1){

                            String uniqueId = RoomInfoHelp.getUniqueId();
                            if(uniqueId != null && uniqueId.equals(song.getUniqueId())){

                                L.e(TAG, "信令接收顺序存在问题");

                            }else {
                                L.i(TAG, "stopTimerTask 444");
                                ZGKTVPlayerManager.getInstance().stopTimerTask();
                                ZGKTVPlayerManager.getInstance().stop();
                                mMusicPlayerMenu.updateTime(null, null);
                            }

                        }else if(song.getReadyStatus() == 2){
                            L.i(TAG,"stopTimerTask  555");
                            ZGKTVPlayerManager.getInstance().stopTimerTask();
                            ZGKTVPlayerManager.getInstance().stop();
                            mMusicPlayerMenu.updateTime(null,null);

                            if(mMyInfo.isHost()) {
                                long networkTime = RTCSDKManager.getInstance().getNetworkTime();
                                long musicPlayTime = networkTime + 3000;
                                KTVRoomManager.getInstance().syncTime(musicPlayTime, 0);
//                                ZGKTVPlayerManager.getInstance().play(song.getSongId());
                            }
                        }

                    }

                    switchMusicPlayerMenu();

                    KTVRoomOrderSongManager.getInstance().playOrderSongListForNotify(notifyOrderListUpdateInfo.getSongs());


                }
            }
        });
    }

    private boolean isSkipSong(NotifyOrderListUpdateInfo notifyOrderListUpdateInfo){
        List<OrderedSongInfo.OrderedSongItemInfo> datas = KTVRoomOrderSongManager.getInstance().getOrderList();

        if(datas != null && datas.size() > 0  ){


            if(notifyOrderListUpdateInfo.getSongs() == null || notifyOrderListUpdateInfo.getSongs().size() == 0)
            {
                L.e(TAG,"isSkipSong song is NULL");
                return true;
            }else {
                boolean isSameSong = false;
                for (NotifyOrderListUpdateInfo.SongsDTO song : notifyOrderListUpdateInfo.getSongs()) {
                    String uniqueId = RoomInfoHelp.getUniqueId();
                    if (uniqueId == null || song.getUniqueId().equals(uniqueId)) {
                        L.e(TAG,"isSkipSong uniqueId: " + uniqueId);
                        isSameSong = true;
                    }
                }
                return !isSameSong;
            }
        }
        return false;
    }

    /**
     * 预点歌通知回调
     * @param notifyPreOrderUpdateInfo
     */
    @Override
    public void onPreOrderUpdateInfo(NotifyPreOrderUpdateInfo notifyPreOrderUpdateInfo) {
        if(notifyPreOrderUpdateInfo == null){
            return;
        }


        if(!mMyInfo.isAudience()){
            KTVRoomOrderSongManager.getInstance().orderSongNotifyAck(mMyInfo.getUserID(), notifyPreOrderUpdateInfo.getSongId(), notifyPreOrderUpdateInfo.getUniqueId(), new ICommonResponseCallback() {
                @Override
                public void onResponse(int errorCode) {
                    if(errorCode ==0){
                        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(notifyPreOrderUpdateInfo.getSongId());
                        if(song !=null && song.getLoadState() == ZGKTVSongLoadState.LOAD_COMPLETE && song.getLyric() != null){
                            KTVRoomOrderSongManager.getInstance().songDownloadReport(mMyInfo.getUserID(),notifyPreOrderUpdateInfo.getSongId());
                        }
                    }
                }
            });
            FileDownloadManager.getInstance().downloadFileByToken(notifyPreOrderUpdateInfo.getSongId(),notifyPreOrderUpdateInfo.getShareToken());
        }


    }

    /**
     * //终端透传广播消息
     * @param notifyCustomBroadcastInfo
     */
    @Override
    public void onCustomBroadCastUpdate(NotifyCustomBroadcastInfo notifyCustomBroadcastInfo) {
        if (mMyInfo.isAudience()) {
            return;
        }
        long localNtpTime = RTCSDKManager.getInstance().getNetworkTime();
        if (localNtpTime > notifyCustomBroadcastInfo.getMillis()) {
            L.i(TAG, "onCustomBroadCastUpdate return ");
            return;
        }
        if(!RoomInfoHelp.isSongInfoExist()){
            return;
        }
        String songId = RoomInfoHelp.getSongId();
//        int songIndex = convertIdToSongIndex(mRoomState.getCurSong());
//        if (songIndex >= 0) {
            if (notifyCustomBroadcastInfo.getType() == Constants.MediaPlayerBehavior.PAUSE.value()) {
//                mZgktvMediaPlayer.pauseInFuture(customBroadcastInfo.getMillis());
            } else if (notifyCustomBroadcastInfo.getType() == Constants.MediaPlayerBehavior.PLAYING.value()) {
                RTCSDKManager.getInstance().restartMvStream();
                ZGKTVPlayerManager.getInstance().startInFuture(songId,RoomInfoHelp.isAccompany(),notifyCustomBroadcastInfo.getMillis(),mMyInfo.isHost());
                L.e(TAG,"onCustomBroadCastUpdate pitchShowMap clear: ");
                pitchShowMap.clear();
                if(RoomInfoHelp.getSongCreatorId() == mMyInfo.getUserID()) {
                    L.e(TAG,"onCustomBroadCastUpdate pitchShowMap put: ");
                    pitchShowMap.put(RoomInfoHelp.getUniqueId(), true);
                }
                mMusicPlayerMenu.post(runnable);
                mMusicPlayerMenu.postDelayed(runnable,500);
                mMusicPlayerMenu.postDelayed(runnable,1000);
                mMusicPlayerMenu.postDelayed(runnable,1500);
                mMusicPlayerMenu.postDelayed(runnable,2000);

            } else if (notifyCustomBroadcastInfo.getType() == Constants.MediaPlayerBehavior.RESUME.value()) {
//                RTCSDKManager.getInstance().restartMvStream();
//                mZgktvMediaPlayer.resumeInFuture(customBroadcastInfo.getMillis());
            }
//        }
    }

    private Runnable runnable = new Runnable() {
        @Override
        public void run() {

            mZegoNetworkTime = RTCSDKManager.getInstance().getNetworkTime();
            startDataMonitorSync(0, mZegoNetworkTime,0);


        }
    };

    /**
     * 声浪回调
     * @param userVolumeMap
     */
    @Override
    public void onUserVolumeUpdate(HashMap<String, Float> userVolumeMap) {
        if (stateGuestAdapter != null) {
            stateGuestAdapter.onUserVolumeUpdated(userVolumeMap);
        }
        UserInfo hostInfo = KTVRoomManager.getInstance().getHostInfo();
//        dealWithHostVolume(hostInfo, userVolumeMap);
    }

    /**
     * 音频设备变更通知（参见插拔耳机）
     * @param zegoAudioRoute
     */
    @Override
    public void onAudioRouteChange(ZegoAudioRoute zegoAudioRoute) {
        switch (zegoAudioRoute) {
            case HEADPHONE:
            case BLUETOOTH:
                RTCSDKManager.getInstance().enableAEC(false);
                break;
            default:
                RTCSDKManager.getInstance().enableAEC(true);
                break;
        }
    }

    /**
     * 网络测速质量回调通知
     * @param zegoNetworkSpeedTestQuality
     */
    @Override
    public void onNetworkSpeedTestQualityUpdate(ZegoNetworkSpeedTestQuality zegoNetworkSpeedTestQuality) {
        mMyMonitorInfo.setRtt(zegoNetworkSpeedTestQuality.rtt);
        mMyMonitorInfo.setPacketLoss(zegoNetworkSpeedTestQuality.packetLostRate);
        LinkedList<UserDataMonitorInfo> networkQualityBeans = addToNetworkList(String.valueOf(mMyInfo.getUserID()), mMyMonitorInfo);
        if (networkQualityBeans.size() >= Constants.SAMPLE_NUM_IN_GROUP_SPEED_TEST) {
            mMyMonitorInfo.setNetworkQuality(checkNetworkQuality(networkQualityBeans, Constants.SAMPLE_NUM_IN_GROUP_SPEED_TEST));
            mViewModel.setMyMonitorInfoMutableLiveData(mMyMonitorInfo);
            setNetworkQuality(networkQuality, mMyMonitorInfo.getNetworkQuality());
            setNetworkQualityContent(networkQualityContent, mMyMonitorInfo.getNetworkQuality());
            networkQuality.setVisibility(mMyMonitorInfo.getNetworkQuality().equals(NetworkQuality.UNKNOWN) ? View.GONE : View.VISIBLE);
        }
    }


    /**
     * 检测网络
     *
     * @param networkQualityBeans
     * @param groupNum
     * @return
     */
    private NetworkQuality checkNetworkQuality(LinkedList<UserDataMonitorInfo> networkQualityBeans, int groupNum) {
        double packetLossSum = 0;
        double rttSum = 0;
        NetworkQuality packetQuality = NetworkQuality.UNKNOWN;
        NetworkQuality rttQuality = NetworkQuality.UNKNOWN;
        if (networkQualityBeans.size() >= groupNum) {
            for (UserDataMonitorInfo bean : networkQualityBeans) {
                packetLossSum += convertPacketLossToLevel(bean.getPacketLoss() * 100);
                rttSum += convertRttToLevel(bean.getRtt());
            }
            packetLossSum = packetLossSum / networkQualityBeans.size();
            packetQuality = NetworkQuality.getNetworkQuality((int) Math.round(packetLossSum));
            rttSum = rttSum / networkQualityBeans.size();
            rttQuality = NetworkQuality.getNetworkQuality((int) Math.round(rttSum));
            return packetQuality.value() > rttQuality.value() ? rttQuality : packetQuality;
        }
        return NetworkQuality.UNKNOWN;
    }

    private int convertRttToLevel(long rtt) {
        if (rtt > Constants.BAD_RTT) {
            return NetworkQuality.BAD.value();
        } else if (rtt < Constants.FINE_RTT) {
            return NetworkQuality.GRATE.value();
        } else {
            return NetworkQuality.FINE.value();
        }
    }

    private int convertPacketLossToLevel(double packetLossRate) {
        if (packetLossRate > Constants.BAD_PACKET_LOSS) {
            return NetworkQuality.BAD.value();
        } else if (packetLossRate < Constants.FINE_PACKET_LOSS) {
            return NetworkQuality.GRATE.value();
        } else {
            return NetworkQuality.FINE.value();
        }
    }


    private LinkedList<UserDataMonitorInfo> addToNetworkList(String streamId, UserDataMonitorInfo networkQualityBean) {
        LinkedList<UserDataMonitorInfo> networkQualityBeans;
        if (String.valueOf(mMyInfo.getUserID()).equals(streamId)) {
            networkQualityBeans = myNetworkQualityList;
        } else if (mNetworkQualityMap.containsKey(streamId)) {
            networkQualityBeans = mNetworkQualityMap.get(streamId);
        } else {
            networkQualityBeans = new LinkedList<>();
            mNetworkQualityMap.put(streamId, networkQualityBeans);
        }
        if (networkQualityBeans.size() >= Constants.SAMPLE_NUM_IN_GROUP) {
            networkQualityBeans.remove();
            networkQualityBeans.remove();
        }
        networkQualityBeans.add(networkQualityBean);
        return networkQualityBeans;
    }

    /**
     * 主播说话的波浪效果
     *
     * @param hostInfo
     * @param userVolumeMap
     */
    private void dealWithHostVolume(UserInfo hostInfo, HashMap<String, Float> userVolumeMap) {
        if (hostInfo == null || !hostInfo.isMicOpen() || userVolumeMap == null) {
            mAnimationDrawable.stop();
            mAnimationDrawable.selectDrawable(0);
            return;
        }

        for (String uid : userVolumeMap.keySet()) {
            if (String.valueOf(hostInfo.getUserID()).endsWith(uid)) {
                if (userVolumeMap.get(uid) > 1) {
                    if (!mAnimationDrawable.isRunning()) {
                        mAnimationDrawable.start();
                    }
                } else {
                    mAnimationDrawable.stop();
                    mAnimationDrawable.selectDrawable(0);
                }
                break;
            }
        }
    }


    /**
     * 退出房间
     *
     * @param view
     */
    public void exitRoom(View view) {
        showExitRoomDialog();
    }

    private void showExitRoomDialog() {
        if (mMyInfo.getUserID() == KTVRoomManager.getInstance().getHostInfo().getUserID()) {
            MessageDialog messageDialog = new MessageDialog(this, R.layout.ktv_dialog_msg);
            messageDialog.setMsgTitle("是否退出歌房");
            messageDialog.setMsgContent("你是房主，退出房间后其他人也将一起退出。");
            messageDialog.setSureListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    hostCloseRoom = true;
                    messageDialog.dismiss();
                    KTVRoomManager.getInstance().closeRoom(mMyInfo.getUserID(), mIExecuteCallback);
                    //todo 之前的逻辑是 不管接口调用情况都能退出房间。这是不合理的，改成接口返回成功后再退出
//                    finish();
//                    startActivity(ZegoQualityFactory.getQualityIntent(KTVRoomActivity.this));
                }
            });
            messageDialog.setCancelListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    messageDialog.dismiss();
                }
            });
        } else {
            KTVRoomManager.getInstance().logoutRoom(true);
            finish();
            startActivity(QualityManagerProxy.getQualityIntent(KTVRoomActivity.this));
        }
    }

    /**
     * 调节菜单点击回调
     * @param view
     */
    public void adjustEffects(View view) {
        if (!mMyInfo.isOnstage()) {
            Toast.makeText(this, "请上麦后操作", Toast.LENGTH_SHORT).show();
            return;
        }

        AdjustVolumeDialog adjustVolumeDialog = new AdjustVolumeDialog();
        adjustVolumeDialog.show(getSupportFragmentManager(), adjustVolumeDialog.getTag());
    }

    /**
     * 点歌弹窗
     *
     * @param view
     */
    public void selectSong(View view) {
        if (!mMyInfo.isOnstage()) {
            Toast.makeText(this, "请上麦后点歌", Toast.LENGTH_SHORT).show();
            return;
        }

        if (CheckDoubleClick.isFastDoubleClick()) {
            return;
        }

        SongListDialog songListDialog = new SongListDialog();
        songListDialog.show(getSupportFragmentManager(), songListDialog.getTag());

    }

    /**
     * 切换歌曲
     *
     * @param view
     */
    public void NextSong(View view) {
        //房主可以切歌没有限制，观众只有在当前播放的歌曲是自己点的就可以切歌
        if (mMyInfo.getUserID() == KTVRoomManager.getInstance().getHostInfo().getUserID()
                || mMyInfo.getUserID() == RoomInfoHelp.getSongCreatorId()) {
            KTVRoomManager.getInstance().switchSong(mIExecuteCallback);
        } else {
            Toast.makeText(this, "你没有切歌权限", Toast.LENGTH_SHORT).show();
        }

    }

    /**
     * 麦克风按钮点击
     *
     * @param view
     */
    public void switchMic(View view) {
        if (!mMyInfo.isOnstage()) {
            Toast.makeText(this, "请上麦后操作", Toast.LENGTH_SHORT).show();
            return;
        }

        if (mIsMicMuted) {
            //麦克是关闭状态要开启的时候，需要判断麦克风的权限
            micPermission();
        } else {
            mIsMicMuted = !mIsMicMuted;
            switchMic(true, mIExecuteCallback);
        }
    }

    /**
     * 麦克风切换
     * @param isMuted
     * @param callback
     */
    private void switchMic(boolean isMuted, IExecuteCallback callback) {
        mIsMicMuted = isMuted;
        KTVRoomManager.getInstance().switchMic(mMyInfo.getUserID(), isMuted, callback);
    }

    /**
     * 点击数据界面切换
     * @param view
     */
    public void dataMonitorSwitch(View view) {

        mIsDataMonitorOpen = !mIsDataMonitorOpen;

        if (mIsDataMonitorOpen) {
            dataMonitor.setCompoundDrawablesWithIntrinsicBounds(null, getDrawable(R.mipmap.ktv_data), null, null);
        } else {
            dataMonitor.setCompoundDrawablesWithIntrinsicBounds(null, getDrawable(R.mipmap.ktv_mute_data), null, null);
        }

        stateGuestAdapter.setDataMonitorShowStatus(mIsDataMonitorOpen);
    }

    public void startHostProgressSyncTask() {
        if (mMyInfo.getUserID() != KTVRoomManager.getInstance().getHostInfo().getUserID()) {
            return;
        }
        if (mHostSyncProgressTimer != null) {
            return;
        }
        TimerTask syncTimertask = new TimerTask() {
            @Override
            public void run() {
                try {
//                    if (mZgktvMediaPlayer == null || ZegoExpressEngine.getEngine() == null) {
//                        cancel();
//                    }
//                    JSONObject localMusicProcessStatusJsonObject = new JSONObject();
//                    localMusicProcessStatusJsonObject.put(Constants.KEY_Total_IN_MS, mZgktvMediaPlayer.getDuration());
//                    localMusicProcessStatusJsonObject.put(Constants.KEY_MV_STATE, mZgktvMediaPlayer.getCurrentState().value());
//                    localMusicProcessStatusJsonObject.put(Constants.KEY_PROCESS_IN_MS, mZgktvMediaPlayer.getCurrentDuration());
//                    if (RoomInfoHelp.isSongInfoExist()) {
//                        localMusicProcessStatusJsonObject.put(Constants.KEY_Song_Id, RoomInfoHelp.getSongId());
//                    }
//                    localMusicProcessStatusJsonObject.put(Constants.KEY_ROLE, 3);
//                    ZegoExpressEngine.getEngine().sendSEI(localMusicProcessStatusJsonObject.toString().getBytes());
                } catch (Exception e) {

                }
            }
        };
        if (mHostSyncProgressTimer == null) {
            mHostSyncProgressTimer = new Timer();
        }
        mHostSyncProgressTimer.scheduleAtFixedRate(syncTimertask, 0, 1000);
    }

    /**
     * 发送SEI
     * @param millis
     * @param zegoNetworkTime
     * @param duration
     */
    public void startDataMonitorSync(long millis, long zegoNetworkTime,long duration) {
        L.i(TAG,"startDataMonitorSync: millis:" + millis + " zegoNetworkTime " + zegoNetworkTime);
        if (mHostSyncProgressTimer != null) {
            mHostSyncProgressTimer.cancel();
            mHostSyncProgressTimer = null;
        }
        if (!RoomInfoHelp.isSongInfoExist()) {
            return;
        }
        try {
            JSONObject localMusicProcessStatusJsonObject = new JSONObject();
            L.i("SEI","mMyInfo.getUserID() :" + mMyInfo.getUserID());
            L.i("SEI","KTVRoomManager.getInstance().getHostInfo().getUserID() :" + KTVRoomManager.getInstance().getHostInfo().getUserID());
            if (mMyInfo.getUserID() == KTVRoomManager.getInstance().getHostInfo().getUserID()) {

                if(ZGKTVPlayerManager.getInstance().isPlaying()) {
                    localMusicProcessStatusJsonObject.put(Constants.KEY_MV_STATE, ZegoMediaPlayerState.PLAYING.value());
                }else {
                    localMusicProcessStatusJsonObject.put(Constants.KEY_MV_STATE, ZegoMediaPlayerState.PLAY_ENDED.value());
                }
                localMusicProcessStatusJsonObject.put(Constants.KEY_ROLE, 3);
            }

            localMusicProcessStatusJsonObject.put(Constants.KEY_Song_Id, RoomInfoHelp.getSongId());
            localMusicProcessStatusJsonObject.put(Constants.KEY_PROCESS_IN_MS, millis);
            localMusicProcessStatusJsonObject.put(Constants.KEY_Total_IN_MS, duration);
            localMusicProcessStatusJsonObject.put(Constants.KEY_NTP_TIME_STAMP, zegoNetworkTime);
            L.i("SEI","startDataMonitorSync: sendSEI :" + localMusicProcessStatusJsonObject.toString());
            ZegoExpressEngine.getEngine().sendSEI(localMusicProcessStatusJsonObject.toString().getBytes());
        } catch (JSONException e) {

            L.i("SEI","startDataMonitorSync: JSONException: " + e.toString());
        }
    }


    /**
     * 按钮-选择上台
     *
     * @param view
     */
    public void selectOnstage(View view) {

        NetworkQuality networkQuality = mMyMonitorInfo.getNetworkQuality();
        if (NetworkQuality.BAD == networkQuality) {
            Toast.makeText(KTVRoomActivity.this, "当前网络较差，无法支持KTV合唱", Toast.LENGTH_SHORT).show();
        } else if (NetworkQuality.UNKNOWN == networkQuality) {
            Toast.makeText(KTVRoomActivity.this, "请等待测速完成", Toast.LENGTH_SHORT).show();
        } else {
            if (mMyInfo.getUserID() != KTVRoomManager.getInstance().getHostInfo().getUserID()) {
                // stateGuestAdapter.indexOfMix() 获取最前的空位子 todo 需求改成传 0 （0为随机麦位）
                KTVRoomManager.getInstance().onStageOperation(mMyInfo.getUserID(), mMyInfo.getUserID(), 0, mIExecuteCallback);
            }
        }

    }

    /**
     * 麦克风权限
     */
    private void micPermission() {
        PermissionHelper.onAudioPermissionGranted(this, new PermissionHelper.GrantResult() {
            @Override
            public void onGrantResult(boolean allGranted) {
                if (allGranted) {
                    mIsMicMuted = !mIsMicMuted;
                    switchMic(false, mIExecuteCallback);
                } else {

                    MessageDialog messageDialog = new MessageDialog(KTVRoomActivity.this, R.layout.ktv_dialog_msg);
                    messageDialog.setMsgTitle("无法使用麦克风");
                    messageDialog.setMsgContent("请在系统设置中开启麦克风的使用权限");
                    messageDialog.setRightBtnContent("去设置");
                    messageDialog.setLeftBtnContent("取消");
                    messageDialog.setSureListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            //取消
                            messageDialog.dismiss();
                            Toast.makeText(KTVRoomActivity.this, "打开麦克风失败", Toast.LENGTH_SHORT).show();
                        }
                    });
                    messageDialog.setCancelListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            //去设置
                            messageDialog.dismiss();
                            Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
                                    .setData(Uri.fromParts("package", getPackageName(), null));
                            startActivityForResult(intent, 666);
                        }
                    });

                }
            }
        });
    }

    public static void setNetworkQuality(ImageView view, NetworkQuality quality) {
        if (quality == null) {
            return;
        }
        switch (quality) {
            case BAD:
                view.setImageResource(R.mipmap.ktv_bad_network);
                break;
            case FINE:
                view.setImageResource(R.mipmap.ktv_fine_network);
                break;
            case GRATE:
                view.setImageResource(R.mipmap.ktv_great_network);
                break;
            default:
                break;
        }
    }

    public static void setNetworkQualityContent(TextView view, NetworkQuality quality) {
        if (quality == null) {
            return;
        }
        switch (quality) {
            case UNKNOWN:
                view.setText("本机网络测速中...");
                break;
            case BAD:
                view.setText("本机网络差");
                break;
            case FINE:
                view.setText("本机网络中");
                break;
            case GRATE:
                view.setText("本机网络好");
                break;
            default:
                break;
        }
    }


    /**
     * 网咯变更初始化
     */
    private void initNetwork() {
        NetWorkKtvConManager.getInstance().registerNetworkCallback(this);
        NetWorkKtvConManager.getInstance().setCallBack(new INetworkKtvCallback() {
            @Override
            public void onStageList() {
                //恢复网咯后重新拉取
                cancleToastOnLost();
                reLoadApi();
            }

            @Override
            public void networkOnLost() {
                if (messageDialog == null && onlostDialog == null) {
                    showNetworkOnLostDialog();
                }
            }

            @Override
            public void networkOnLostCloseRoom() {
                cancleToastOnLost();
                showRoomCloseAlertDialog("网络异常", "网络无法连接，请离开房间后再试。");
            }

            @Override
            public void cancleToastOnLost() {
                if (onlostDialog != null) {
                    onlostDialog.dismiss();
                    onlostDialog = null;
                }
            }
        });
    }

    /**
     * 网络断开后
     */
    private void showNetworkOnLostDialog() {
        if (isFinishing() || onlostDialog != null) {
            return;
        }
        onlostDialog = new MessageDialog(this, R.layout.ktv_dialog_toast);
    }

    /**
     * 用于房间界面状态切换
     */
    private void switchMusicPlayerMenu(){

        L.i(TAG,"switchMusicPlayerMenu:" + RoomInfoHelp.isSongInfoExist());
        boolean playingSong = false;
        if(RoomInfoHelp.isSongInfoExist()){
            playingSong = true;
        }
        String songId = RoomInfoHelp.getSongId();

        if(!mMyInfo.isAudience()){
            if(playingSong){
                ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
                String songName = song == null? "":song.getSongName();
                boolean isHasAccompany = song != null && song.isHasAccompany();
                boolean isHasOriginal = song != null && song.isHasOriginal();
                if(song!= null && song.getLyric() != null && song.getLoadState() == ZGKTVSongLoadState.LOAD_COMPLETE && ZGKTVPlayerManager.getInstance().isPlaying()) {
                    mMusicPlayerMenu.startPlaying(songName, isHasAccompany, isHasOriginal, pitchShowMap.get(RoomInfoHelp.getUniqueId()) != null);
                }else {
                    mMusicPlayerMenu.comingSoon(songName, isHasAccompany, isHasOriginal);
                }

                String uniqueId = RoomInfoHelp.getUniqueId();

                if(!TextUtils.isEmpty(uniqueId) && audioTrackCheckMap.get(uniqueId) != null){
                    Boolean check = audioTrackCheckMap.get(uniqueId);
                    if(check != null) {

                        mMusicPlayerMenu.getSongSwitch().setChecked(check);
                    }
                }else if(RoomInfoHelp.isAccompany()){

                    mMusicPlayerMenu.getSongSwitch().setChecked(true);
                }else {

                    mMusicPlayerMenu.getSongSwitch().setChecked(false);
                }
                mMusicPlayerMenu.setSongSwitchVisibility(true);
            }else {
                mMusicPlayerMenu.orderSong();

            }
            selectSong.setVisibility(View.VISIBLE);
            if(KTVRoomOrderSongManager.getInstance().getOrderList() != null && KTVRoomOrderSongManager.getInstance().getOrderList().size() > 0){
                songNum.setVisibility(View.VISIBLE);
            }else {
                songNum.setVisibility(View.GONE);
            }

            selectOnstage.setVisibility(View.GONE);
        }else {
            if(playingSong){
                ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
                String songName = song == null? "":song.getSongName();
                boolean isHasAccompany = song != null && song.isHasAccompany();
                boolean isHasOriginal = song != null && song.isHasOriginal();
                if(ZGKTVLyricManager.getInstance().getLyric(songId) != null && mKState == 1) {
                    mMusicPlayerMenu.startPlaying(songName, isHasAccompany, isHasOriginal,false);
                }else {
                    mMusicPlayerMenu.comingSoon(songName, isHasAccompany, isHasOriginal);
                }
                mMusicPlayerMenu.setSongSwitchVisibility(false);
            }else {
                mMusicPlayerMenu.offState();
            }

            selectSong.setVisibility(View.GONE);
            songNum.setVisibility(View.GONE);
            selectOnstage.setVisibility(View.VISIBLE);
        }


        //设置左下角菜单

        if(mMyInfo.isHost()){
            if(KTVRoomOrderSongManager.getInstance().getOrderList() == null || KTVRoomOrderSongManager.getInstance().getOrderList().size() == 0){
                nextSong.setVisibility(View.GONE);
                adjust.setVisibility(View.GONE);
            }else {
                nextSong.setVisibility(View.VISIBLE);
                adjust.setVisibility(View.VISIBLE);
            }


            mic.setVisibility(View.VISIBLE);

            nextSong.setAlpha(1.0f);
            nextSong.setCompoundDrawablesWithIntrinsicBounds(0, R.mipmap.ktv_next_song, 0, 0);

            if (mIsMicMuted) {
                mic.setCompoundDrawablesWithIntrinsicBounds(0, R.mipmap.ktv_mute_mic, 0, 0);
            } else {
                mic.setCompoundDrawablesWithIntrinsicBounds(0, R.mipmap.ktv_mic, 0, 0);
            }

        }else if(mMyInfo.isStageGuest()){

            mic.setVisibility(View.VISIBLE);

            if(playingSong){
                nextSong.setVisibility(View.VISIBLE);
                adjust.setVisibility(View.VISIBLE);
            }else {
                nextSong.setVisibility(View.GONE);
                adjust.setVisibility(View.GONE);
            }

            if (mMyInfo.getUserID() == RoomInfoHelp.getSongCreatorId()) {
                nextSong.setAlpha(1.0f);
                nextSong.setCompoundDrawablesWithIntrinsicBounds(0, R.mipmap.ktv_next_song, 0, 0);
            } else {
                nextSong.setAlpha(0.5f);
                nextSong.setCompoundDrawablesWithIntrinsicBounds(0, R.mipmap.ktv_nextsong_disable, 0, 0);
            }

            if (mIsMicMuted) {
                mic.setCompoundDrawablesWithIntrinsicBounds(0, R.mipmap.ktv_mute_mic, 0, 0);
            } else {
                mic.setCompoundDrawablesWithIntrinsicBounds(0, R.mipmap.ktv_mic, 0, 0);
            }


        }else{
            nextSong.setVisibility(View.GONE);
            adjust.setVisibility(View.GONE);
            mic.setVisibility(View.GONE);
        }


    }

    /**
     * 更新已点列表
     */
    private void updateOrderList(){
//        String lastSongId = RoomInfoHelp.getSongId();
        String lastUniqueId = RoomInfoHelp.getUniqueId();
        KTVRoomOrderSongManager.getInstance().getOrderSongList(mMyInfo.getUserID(),!mMyInfo.isAudience(), new IGetOrderedSongListCallback() {
            @Override
            public void onGetOrderedSongList(int retcode, OrderedSongInfo userInfoData) {
                if(retcode == 0){
                    String songId = RoomInfoHelp.getSongId();
                    String uniqueId = RoomInfoHelp.getUniqueId();
                    ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songId);
                    if(song != null) {
                        mMusicPlayerMenu.setSongName(song.getSongName());
                    }

                    if(mMyInfo.isStageGuest()){
                        if(!TextUtils.isEmpty(uniqueId) && uniqueId.equals(lastUniqueId)){


                        }else {
                            L.i(TAG,"stopTimerTask 666");
                            ZGKTVPlayerManager.getInstance().stopTimerTask();
                            ZGKTVPlayerManager.getInstance().stop();
                        }
                    }

                    if(mMyInfo.isHost()) {
                        if (TextUtils.isEmpty(uniqueId )) {
                            L.i(TAG,"stopTimerTask 777");
                            ZGKTVPlayerManager.getInstance().stopTimerTask();
                            ZGKTVPlayerManager.getInstance().stop();
                        } else if (uniqueId.equals(lastUniqueId)) {
                            ZGKTVCopyrightedSong zgktvCopyrightedSong = ZGKTVPlayerManager.getInstance().getSong(songId);
                            if (zgktvCopyrightedSong != null) {
                                if (zgktvCopyrightedSong.getState() != ZGKTVSongState.PLAYING) {
                                    startPlaySong();
                                }
                            }
                        }else{
                            startPlaySong();
                        }
                    }

                    switchMusicPlayerMenu();
                }
            }
        });
    }

    private void startPlaySong(){
        L.i(TAG,"stopTimerTask 888");
        ZGKTVPlayerManager.getInstance().stopTimerTask();
        ZGKTVPlayerManager.getInstance().stop();

        long networkTime = RTCSDKManager.getInstance().getNetworkTime();
        long musicPlayTime = networkTime + 3000;
        KTVRoomManager.getInstance().syncTime(musicPlayTime, 0);
    }

    /**
     * 当前播放的歌曲信息更新回调
     * @param song
     */
    @Override
    public void onZGKTVCopyrightedSongUpdate(ZGKTVCopyrightedSong song) {
        if(song != null && RoomInfoHelp.getSongId() != null){
            if(song.getSongID().equals(RoomInfoHelp.getSongId())){
                mMusicPlayerMenu.setSongName(song.getSongName());
                switchMusicPlayerMenu();
            }
        }
    }
    /**
     * 断网重连、sdk重连后 更新房间最新消息（用户信息、房间信息、已点歌曲列表）
     * <p>
     * 1.获取用户信息接口，判断当前房间是否有效
     * 2.无效就弹窗退出，有效就更新房间最新消息（用户信息、房间信息、已点歌曲列表）
     */
    private void reLoadApi() {
        if (CheckDoubleClick.isFastDoubleClick()) {
            return;
        }

        RoomInfo roomInfo = KTVRoomManager.getInstance().getRoomInfo();
        if (roomInfo != null) {
            RTCSDKManager.getInstance().initQualityManager(KTVRoomActivity.this);
            RoomAPI.getUserInfo(mMyInfo.getUserID(), roomInfo.getRoomId(), new IGetUserInfoCallback() {
                @Override
                public void onGetUserInfo(int errorCode, UserInfo userInfoData) {
                    L.d(TAG, "room reLogin, errorCode:" + errorCode);
                    if (errorCode == 0) {
                        if(messageDialog != null){
                            messageDialog.dismiss();
                            messageDialog = null;
                        }
                        mMyInfo = userInfoData;
                        KTVRoomManager.getInstance().setMyUserInfo(mMyInfo);
                        pullUserList(); //麦上人员接口
                        updateOrderList();
                        //获取房间信息
                    } else if (errorCode == ERROR_USER_OFF_LINE || errorCode == ERROR_ROOM_NOT_EXIST) {
                        showRoomCloseAlertDialog("网络异常", "网络无法连接，请离开房间后再试。");
                    }
                }
            });
        }
    }

    /**
     * 本人下麦后更新网络状态为 UNKNOWN
     */
    private void offStageAdapter(){
        if (stateGuestAdapter != null) {
            stateGuestAdapter.updateGuestPlayQuality(NetworkQuality.UNKNOWN);
        }
        if (mHostMonitorInfo != null) {
            mHostMonitorInfo.setNetworkQuality(NetworkQuality.UNKNOWN);
        }
        mNetworkQualityMap.clear();
    }

    private void startKTVService() {
        Intent intent = new Intent(this, KTVRoomService.class);
        if (Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            startForegroundService(intent);
        } else {
            startService(intent);
        }
    }

    private void stopKTVService() {
        Intent intent = new Intent(this, KTVRoomService.class);
        stopService(intent);
    }

}
