package im.zego.ktv.chorus.ktvroom;


import static im.zego.ktv.ErrorCodeConstants.ERROR_USER_OFF_LINE;
import static im.zego.ktv.ErrorCodeConstants.GoChatErrorFailNetwork;

import android.content.Context;
import android.content.Intent;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import im.zego.common.model.LoginInfo;
import im.zego.common.util.L;
import im.zego.ktv.ErrorCodeConstants;
import im.zego.ktv.R;
import im.zego.ktv.chorus.base.Constants;
import im.zego.ktv.chorus.download.FileDownloadManager;
import im.zego.ktv.chorus.ktvroom.callback.IExecuteCallback;
import im.zego.ktv.chorus.ktvroom.callback.IRoomSongStateUpdateListener;
import im.zego.ktv.chorus.model.OrderedSongInfo;
import im.zego.ktv.chorus.model.RoomInfo;
import im.zego.ktv.chorus.model.UserInfo;
import im.zego.ktv.chorus.model.UserInfoData;
import im.zego.ktv.chorus.model.notify.NotifyRoomStateUpdateInfo;
import im.zego.ktv.chorus.protocol.ICommonResponseCallback;
import im.zego.ktv.chorus.protocol.IGetUserListCallback;
import im.zego.ktv.chorus.protocol.ILoginRoomCallback;
import im.zego.ktv.chorus.protocol.RoomAPI;
import im.zego.ktv.chorus.rtc.RTCSDKManager;
import im.zego.ktv.chorus.rtc.ZGKTVCopyrightedMusicManager;
import im.zego.ktv.chorus.rtc.ZGKTVPlayerManager;
import im.zego.ktv.chorus.rtc.callbacks.IRTCCommonCallback;
import im.zego.ktv.utils.ToastUtil;
import im.zego.ktv.chorus.viewmodel.KTVRoomViewModel;


/**
 * 房间管理类
 */
public class KTVRoomManager {

    private static final String TAG = "KTVRoomManager";

    private static volatile KTVRoomManager singleton = null;
    private NotifyRoomStateUpdateInfo mNotifyRoomStateUpdateInfo;
    protected HashMap<Long, UserInfo> mUserModelMap = new HashMap<>(); //麦上人（包含主播）
    private UserInfo mHostUserInfo;
    private List<OrderedSongInfo.OrderedSongItemInfo> mOrderedSongList = new ArrayList<>();
    private LoginInfo mMyLoginInfo;
    private UserInfo mMyUserInfo;
    private KTVRoomViewModel mViewModel;

    private RoomInfo mRoomInfo;
    private IRoomSongStateUpdateListener mRoomSongStateUpdateListener;

    private KTVRoomManager() {
    }

    public static KTVRoomManager getInstance() {
        if (singleton == null) {
            synchronized (KTVRoomManager.class) {
                if (singleton == null) {
                    singleton = new KTVRoomManager();
                }
            }
        }
        return singleton;
    }

    public void setMyUserInfo(UserInfo myUserInfo) {
        this.mMyUserInfo = myUserInfo;
    }

    public UserInfo getmMyUserInfo() {
        return mMyUserInfo;
    }

    public void setHostUserInfo(UserInfo hostUserInfo) {
        this.mHostUserInfo = hostUserInfo;
    }

    public KTVRoomViewModel getViewModel() {
        return mViewModel;
    }

    public void setViewModel(KTVRoomViewModel mViewModel) {
        this.mViewModel = mViewModel;
    }

    /**
     * 房间信息
     *
     * @return
     */
    public RoomInfo getRoomInfo() {
        return mRoomInfo;
    }

    /**
     * 用户列表
     *
     * @return
     */
    public HashMap<Long, UserInfo> getUserModelMap() {
        return mUserModelMap;
    }

    /**
     * 主持人信息
     *
     * @return
     */
    public UserInfo getHostInfo() {
        return mHostUserInfo;
    }


    /**
     * 登录房间
     *
     * @param uid
     * @param roomId
     */
    public void loginRoom(final Context context, long uid, final String roomId,String nickName,String avatar) {
        RoomAPI.loginRoom(uid, roomId,nickName,avatar, new ILoginRoomCallback() {
            // TODO: 2/13/21
            @Override
            public void onLoginRoom(int retcode, NotifyRoomStateUpdateInfo notifyRoomStateUpdateInfo) {
                if (retcode == 0) {

                    RoomInfo roomInfo = new RoomInfo();
                    roomInfo.setRoomId(notifyRoomStateUpdateInfo.getRoomId());
                    roomInfo.setOriginPlay(notifyRoomStateUpdateInfo.getOriginPlay());
                    roomInfo.setCoverImg(notifyRoomStateUpdateInfo.getCoverImg());
                    roomInfo.setCreateTime(notifyRoomStateUpdateInfo.getCreateTime());
                    roomInfo.setCreatorName(notifyRoomStateUpdateInfo.getCreatorName());
                    roomInfo.setSubject(notifyRoomStateUpdateInfo.getSubject());
                    roomInfo.setCreatorId(notifyRoomStateUpdateInfo.getCreatorId());
                    RTCSDKManager.getInstance().setHost(notifyRoomStateUpdateInfo.getCreatorId());
                    setRoomInfo(roomInfo);
                    context.startActivity(new Intent(context, KTVRoomActivity.class));
                } else if (retcode == ErrorCodeConstants.ERROR_ROOM_FULL) {
                    ToastUtil.showWarnToast(R.string.room_list_join_fail);
                } else if (retcode == ErrorCodeConstants.ERROR_ALREADY_IN_ROOM) {
                    ToastUtil.showWarnToast(R.string.room_list_join_fail_common);
                } else if (retcode == ERROR_USER_OFF_LINE) {
                    ToastUtil.showWarnToast(R.string.room_list_join_offline);
                    // ZegoApplication.restartLogin();
                } else if (retcode == GoChatErrorFailNetwork) {
                    ToastUtil.showWarnToast(R.string.fail_network);
                } else if (retcode == ErrorCodeConstants.ERROR_ROOM_NOT_EXIST) {
                    ToastUtil.showWarnToast(R.string.ktv_room_list_not_exist);
                } else {
                    ToastUtil.showWarnToast(R.string.room_list_join_fail_common);
                }
            }
        });
    }

    public void setRoomInfo(RoomInfo roomInfo) {
        mRoomInfo = roomInfo;

        if(mRoomSongStateUpdateListener != null){
            mRoomSongStateUpdateListener.onRoomSongStateUpdate();
        }

//        mNotifyRoomStateUpdateInfo = notifyRoomStateUpdateInfo;
//        Log.e("APIBase","setRoomInfo:" + notifyRoomStateUpdateInfo.getRoomId());
        // 不是所有路径都会走 RoomAPI.loginRoom，但是都会走 setRoomInfo，所以RTC的相关设置在这里
    }

    public void logoutRoom(boolean needQuit) {
        // 保底处理

        System.out.println("退出房间");

        RTCSDKManager.getInstance().stopPublishAudio(null);
        RTCSDKManager.getInstance().logoutRoom();

        if (needQuit) {
            RoomAPI.quitRoom(mMyLoginInfo.getUid(), null);
        }
        mUserModelMap.clear();
        mNotifyRoomStateUpdateInfo = null;
        this.mViewModel = null;
        this.mRoomSongStateUpdateListener = null;
        this.mRoomInfo = null;
        ZGKTVCopyrightedMusicManager.getInstance().clearData();
        FileDownloadManager.getInstance().clearData();
        KTVRoomOrderSongManager.getInstance().clearData();
    }

    /**
     * 拉取台上用户列表
     *
     * @param uid
     * @param callback
     */
    public void getUserList(final long uid, final IExecuteCallback callback) {
        // 先清除之前的列表
        RoomAPI.getUserList(uid, new IGetUserListCallback() {
            @Override
            public void onGetUserList(int retcode, UserInfoData userInfoData) {
                mUserModelMap.clear();
                if (retcode == 0 && userInfoData != null) {
                    List<UserInfo> attendeeList = userInfoData.getAttendeeList();
                    for (UserInfo userInfo : attendeeList) {
                        if (userInfo.isHost()) {
                            mHostUserInfo = userInfo;
                            if(userInfo.getUserID() == uid){
                                mMyUserInfo = userInfo;
                            }
                        } else if(userInfo.getUserID() == uid){
                            mMyUserInfo = userInfo;
//                            mUserModelMap.put(userInfo.getUserID(), userInfo);
                        }
                        //全部在麦上人员信息（包括房主）
                        mUserModelMap.put(userInfo.getUserID(), userInfo);
                    }
                    if (callback != null) {
                        callback.onSuccess(Constants.OperationType.GET_ONSTAGE_USERS);
                    }
                } else {
                    if (callback != null) {
                        callback.onFailed(retcode, Constants.OperationType.GET_ONSTAGE_USERS);
                    }
                }
            }

        });
    }


    /**
     * 关闭房间
     *
     * @param uid
     * @param callback
     */
    public void closeRoom(final long uid, final IExecuteCallback callback) {
        RoomAPI.closeRoom(uid, new ICommonResponseCallback() {
            @Override
            public void onResponse(int retcode) {
                if (retcode == 0) {
                    logoutRoom(false);
                    //结束房间成功，清楚数据
                    if (callback != null) {
                        callback.onSuccess(Constants.OperationType.CLOSE_ROOM);
                    }
                } else {
                    if (callback != null) {
                        callback.onFailed(retcode, Constants.OperationType.CLOSE_ROOM);
                    }
                }
            }
        });
        //todo 接口返回成功后再调
//        logoutRoom(false);
    }


    /**
     *  观众上麦
     * @param myUid
     * @param targetUid
     * @param position
     * @param callback
     */
    public void onStageOperation(long myUid, long targetUid, int position, IExecuteCallback callback) {
        L.i(TAG,"onStageOperation");
        RoomAPI.setStageStatus(myUid, targetUid, 2, position, Constants.OperationType.ON_STAGE.value(), new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if (errorCode == 0) {

                    RTCSDKManager.getInstance().stopNetworkMonitor();
                    //停止了拉混流
                    RTCSDKManager.getInstance().stopPlayingAux();
                    //开始拉麦上合唱者麦克风音频流

                    RTCSDKManager.getInstance().startPublishAudio(new IRTCCommonCallback() {
                           @Override
                           public void onRTCCallback(int errorCode) {
                           }
                    });
                    RTCSDKManager.getInstance().startPlayingExistedStreams();
                    if (callback != null) {
                        callback.onSuccess(Constants.OperationType.ON_STAGE);
                    }

                } else {
                    if (callback != null) {
                        callback.onFailed(errorCode, Constants.OperationType.ON_STAGE);
                    }
                }
            }
        });

    }


    /**
     *  观众上麦
     * @param myUid
     * @param targetUid
     * @param position
     * @param callback
     */
    public void switchMicLocation(long myUid, long targetUid, int position, IExecuteCallback callback) {
        L.i(TAG,"onStageOperation");
        RoomAPI.setStageStatus(myUid, targetUid, 2, position, Constants.OperationType.ON_STAGE.value(), new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if (errorCode == 0) {

                    if (callback != null) {
                        callback.onSuccess(Constants.OperationType.ON_STAGE);
                    }

                } else {
                    if (callback != null) {
                        callback.onFailed(errorCode, Constants.OperationType.ON_STAGE);
                    }
                }
            }
        });

    }

//    /**
//     * 获取已经选择的歌曲
//     * @param callback
//     */
//    public void getOrderedSongList(IExecuteCallback callback) {
//        RoomAPI.getOrderedSongList(mMyLoginInfo.getUid(),  new IGetOrderedSongListCallback() {
//            @Override
//            public void onGetOrderedSongList(int retcode, OrderedSongInfo orderedSongInfo) {
//                if (retcode == 0) {
//                    mOrderedSongList = orderedSongInfo.getSongList();
//                    if (callback != null) {
//                        callback.onSuccess(Constants.OperationType.GET_ORDERED_SONGS);
//                    }
//                } else {
//                    if (callback != null) {
//                        callback.onFailed(retcode, Constants.OperationType.GET_ORDERED_SONGS);
//                    }
//                }
//            }
//        });
//    }

    public List<OrderedSongInfo.OrderedSongItemInfo> getOrderedSongListData() {
        return mOrderedSongList;
    }


    /**
     * 下台
     * @param myUid
     * @param targetUid
     * @param position
     * @param callback
     */
    public void offStageOperation(long myUid, long targetUid, int position, IExecuteCallback callback) {
        L.i(TAG,"offStageOperation");
        RoomAPI.setStageStatus(myUid, targetUid, 1, position, Constants.OperationType.OFF_STAGE.value(), new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if (errorCode == 0) {
//                    RTCSDKManager.getInstance().startNetworkMonitor();
                    if (callback != null) {
                        callback.onSuccess(Constants.OperationType.OFF_STAGE);
                    }

                    //本人下麦了才会拉混流等操作
                    if(myUid == targetUid){
                        stageOperationSetRtc();
                    }

//                    //拉混流
//                    RTCSDKManager.getInstance().startPlayingAux();
//                    //停止拉麦上合唱者麦克风音频流
//                    RTCSDKManager.getInstance().stopPlayingExistedStreams();
//                    //停止媒体播放器
//                    ZGKTVMediaPlayer.getInstance().stop();
//                    //下麦
//                    RTCSDKManager.getInstance().stopPublishAudio(new IRTCCommonCallback() {
//                        @Override
//                        public void onRTCCallback(int errorCode) {
//
//                        }
//                    });
                } else {
                    if (callback != null) {
                        callback.onFailed(errorCode, Constants.OperationType.OFF_STAGE);
                    }
                    L.i(TAG, "onStageOperation errorcode = " + errorCode);
                }
            }
        });

    }

    /**
     * 下麦后流的变更（本人下麦/被主播踢下麦）
     */
    public void stageOperationSetRtc(){
        //拉混流
        RTCSDKManager.getInstance().startPlayingAux();
        //停止拉麦上合唱者麦克风音频流
        RTCSDKManager.getInstance().stopPlayingExistedStreams();
        //停止媒体播放器
        ZGKTVPlayerManager.getInstance().stop();
        //下麦
        RTCSDKManager.getInstance().stopPublishAudio(new IRTCCommonCallback() {
            @Override
            public void onRTCCallback(int errorCode) {

            }
        });
    }

    /**
     * 切换麦克风状态
     * @param myUid
     * @param mIsMicMuted
     * @param callback
     */
    public void switchMic(long myUid, boolean mIsMicMuted, IExecuteCallback callback) {
        RoomAPI.setMicStatus(myUid, myUid, mIsMicMuted ? 1 : 2, new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if (errorCode == 0) {
                    if (callback != null) {
                        callback.onSuccess(Constants.OperationType.MIC);
                    }
                    RTCSDKManager.getInstance().muteMicphone(mIsMicMuted);
                } else {
                    if (callback != null) {
                        callback.onFailed(errorCode, Constants.OperationType.MIC);
                    }
                }
            }
        });
    }

//    /**
//     * 取消选择的歌曲
//     * @param uid
//     * @param targetUid
//     * @param targetNickName
//     * @param position
//     * @param currentTimeMillis
//     * @param callback
//     */
//    public void cancelSong(long uid, long targetUid, String targetNickName, int position, long currentTimeMillis, IExecuteCallback callback) {
//        RoomAPI.setSongStatus(uid, targetUid, targetNickName, position, Constants.OperationType.CANCEL_SONG.value(), currentTimeMillis, new ICommonResponseCallback() {
//            @Override
//            public void onResponse(int errorCode) {
//                if (errorCode == 0) {
//                    getOrderedSongList(callback);
//                } else {
//                    callback.onFailed(errorCode, Constants.OperationType.CANCEL_SONG);
//                }
//
//            }
//        });
//    }

    /**
     * 选择歌曲
     * @param position
     * @param currentTimeMillis
     * @param callback
     */
    public void selectSong(int position, long currentTimeMillis, IExecuteCallback callback) {
        RoomAPI.setSongStatus(mMyLoginInfo.getUid(), mMyLoginInfo.getUid(), mMyLoginInfo.getNickName(), position, Constants.OperationType.SELECT_SONG.value(), currentTimeMillis, new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if (errorCode == 0) {
                    callback.onSuccess(Constants.OperationType.SELECT_SONG);
                }else if(errorCode == GoChatErrorFailNetwork){
                    ToastUtil.showWarnToast("网络异常，请检查网络后重试");
                }else {
                    callback.onFailed(errorCode, Constants.OperationType.SELECT_SONG);
                }

            }
        });
    }


    public void setMyLoginInfo(LoginInfo myLoginInfo) {
        mMyLoginInfo = myLoginInfo;
    }

    public LoginInfo getMyLoginInfo() {
        return mMyLoginInfo;
    }

    public void syncTime(long timeInMillis, int type) {
        RoomAPI.setBroadcastMsg(mMyLoginInfo.getUid(), timeInMillis, type, new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if (errorCode == 0) {

                }else if(errorCode == GoChatErrorFailNetwork){
                    ToastUtil.showWarnToast("网络异常，播放歌曲失败，请检查网络后重试");
                }
            }
        });
    }

    /**
     * 切换歌曲
     * @param callback
     */
    public void switchSong(IExecuteCallback callback) {
        RoomAPI.setSongStatus(mMyLoginInfo.getUid(), mMyLoginInfo.getUid(), mMyLoginInfo.getNickName(), 0, Constants.OperationType.NEXT_SONG.value(), 0, new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if (errorCode == 0) {
                    if (callback != null) {
                        callback.onSuccess(Constants.OperationType.NEXT_SONG);
                    }
                } else {
                    if (callback != null) {
                        callback.onFailed(errorCode, Constants.OperationType.NEXT_SONG);
                    }
                }

            }
        });
    }

    public void setRoomSongStateUpdateListener(IRoomSongStateUpdateListener roomSongStateUpdateListener) {
        this.mRoomSongStateUpdateListener = roomSongStateUpdateListener;
    }
}