package im.zego.ktv.chorus.ktvroom;


import static im.zego.ktv.ErrorCodeConstants.GoChatErrorFailNetwork;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import im.zego.common.util.L;
import im.zego.common.util.T;
import im.zego.ktv.chorus.download.FileDownloadManager;
import im.zego.ktv.chorus.ktvroom.callback.IOrderSongListUpdate;
import im.zego.ktv.chorus.ktvroom.callback.IPlayingSongUpdateCallback;
import im.zego.ktv.chorus.ktvroom.callback.ISongInfoUpdateListener;
import im.zego.ktv.chorus.login.LoginInfoManager;
import im.zego.ktv.chorus.model.OrderSongInfo;
import im.zego.ktv.chorus.model.OrderedSongInfo;
import im.zego.ktv.chorus.model.RoomInfo;
import im.zego.ktv.chorus.model.ktv.SongInfoInfo;
import im.zego.ktv.chorus.model.notify.NotifyOrderListUpdateInfo;
import im.zego.ktv.chorus.protocol.ICommonResponseCallback;
import im.zego.ktv.chorus.protocol.IGetOrderedSongListCallback;
import im.zego.ktv.chorus.protocol.IGetRoomInfoCallback;
import im.zego.ktv.chorus.protocol.IOrderSongCallback;
import im.zego.ktv.chorus.protocol.KTVRequestAPI;
import im.zego.ktv.chorus.protocol.RoomAPI;
import im.zego.ktv.chorus.protocol.ktv.ISongInfoCallback;
import im.zego.ktv.chorus.rtc.ZGKTVCopyrightedMusicManager;
import im.zego.ktv.chorus.rtc.ZGKTVCopyrightedSong;
import im.zego.ktv.chorus.rtc.ZGKTVLyricManager;
import im.zego.ktv.chorus.rtc.ZGKTVPlayerManager;
import im.zego.ktv.chorus.rtc.ZGKTVSongLoadState;
import im.zego.ktv.utils.ToastUtil;

public class KTVRoomOrderSongManager {

    private static final String TAG = "KTVRoomOrderSongManager";

    private static volatile KTVRoomOrderSongManager mInstance = new KTVRoomOrderSongManager();

    /**
     * songId对应的song_uni_id列表
     */
    private Map<String, List<String>> orderMap = new HashMap<>();

    /**
     * 已点列表
     */
    private List<OrderedSongInfo.OrderedSongItemInfo> orderList = new ArrayList<>();

    private IOrderSongListUpdate mOrderSongListUpdate;
    private IOrderSongListUpdate mKTVOrderSongListUpdate;
    private IPlayingSongUpdateCallback mPlayingSongUpdateCallback;
    private ISongInfoUpdateListener mSongInfoUpdateListener;

    private KTVRoomOrderSongManager(){
    }


    public static KTVRoomOrderSongManager getInstance() {
        if (mInstance == null) {
            synchronized (KTVRoomOrderSongManager.class) {
                if (mInstance == null) {
                    mInstance = new KTVRoomOrderSongManager();
                }
            }
        }
        return mInstance;
    }

    public void getOrderSongList(long uid,IGetOrderedSongListCallback callback){
        getOrderSongList(uid,false,callback);
    }

    /**
     * 获取点歌列表
     * @param uid
     * @param callback
     */
    public void getOrderSongList(long uid,boolean isLoad,IGetOrderedSongListCallback callback){
        L.i(TAG,"getOrderSongList:uid:" + uid);

        RoomAPI.getRoomInfo(uid, new IGetRoomInfoCallback() {
            @Override
            public void onGetRoomInfo(int retcode, RoomInfo roomInfo) {
                if(retcode == 0){
                    if(roomInfo.getSongInfo() != null && roomInfo.getSongInfo().getSongId() != null){
                        RoomInfo.SongInfoDTO song = roomInfo.getSongInfo();

                        OrderedSongInfo.OrderedSongItemInfo info = new OrderedSongInfo.OrderedSongItemInfo();
                        info.setUniqueId(song.getUniqueId());
//                        info.setOrder(song.getOrder());
                        info.setOrder(Integer.MIN_VALUE/2);
                        info.setSongId(song.getSongId());
                        info.setShareToken(song.getShareToken());
                        info.setKrcToken(song.getKrcToken());
                        info.setCreatorNickName(song.getCreatorNickName());
                        info.setCreator(song.getCreator());
                        info.setCreateTime(song.getCreateTime());
                        info.setReadyStatus(song.getReadyStatus());


                        if(mPlayingSongUpdateCallback != null){
                            mPlayingSongUpdateCallback.onPlayingSongUpdateCallback(roomInfo);
                        }


                        RoomAPI.getOrderedSongList(uid, new IGetOrderedSongListCallback() {
                            @Override
                            public void onGetOrderedSongList(int retcode, OrderedSongInfo orderedSongInfo) {
                                if(retcode == 0) {
                                    List<OrderedSongInfo.OrderedSongItemInfo> songslist = orderedSongInfo.getSongList();
                                    if(songslist == null){
                                        songslist = new ArrayList<>();
                                    }
                                    songslist.add(0,info);
                                    updateInfo(songslist);
                                    orderList = songslist;
                                }
                                if(callback != null) {
                                    callback.onGetOrderedSongList(retcode, orderedSongInfo);
                                }

                                if(retcode == 0) {

                                    if (mOrderSongListUpdate != null) {
                                        mOrderSongListUpdate.OrderSongListUpdate(orderList);
                                    }
                                    if (mKTVOrderSongListUpdate != null) {
                                        mKTVOrderSongListUpdate.OrderSongListUpdate(orderList);
                                    }

                                    updateSongInfo();

                                    for (OrderedSongInfo.OrderedSongItemInfo info : orderList) {

                                        ZGKTVCopyrightedSong copyrightedSong = ZGKTVPlayerManager.getInstance().getSong(info.getSongId());
                                        if (isLoad && (copyrightedSong == null || (copyrightedSong.getLoadState() == ZGKTVSongLoadState.NO_LOAD || copyrightedSong.getLoadState() == ZGKTVSongLoadState.LOAD_ERROR))) {
                                            FileDownloadManager.getInstance().downloadFileByToken(info.getSongId(), info.getShareToken());
                                        }

                                        if (ZGKTVLyricManager.getInstance().getLyric(info.getSongId()) == null) {
                                            ZGKTVCopyrightedMusicManager.getInstance().getKrcLyricForSongID(info.getSongId(), null);
                                        }
                                    }
                                }else if(retcode == GoChatErrorFailNetwork){
                                    ToastUtil.showWarnToast("网络异常，更新已点列表失败");
                                }

                            }
                        });
                    }else {
                        orderList.clear();

                        if(mPlayingSongUpdateCallback != null){
                            mPlayingSongUpdateCallback.onPlayingSongUpdateCallback(roomInfo);
                        }

                        if(callback != null) {
                            OrderedSongInfo orderedSongInfo = new OrderedSongInfo();
                            orderedSongInfo.setSongList(new ArrayList<OrderedSongInfo.OrderedSongItemInfo>());
                            callback.onGetOrderedSongList(retcode, orderedSongInfo);
                        }

                        if (mOrderSongListUpdate != null) {
                            mOrderSongListUpdate.OrderSongListUpdate(orderList);
                        }
                        if (mKTVOrderSongListUpdate != null) {
                            mKTVOrderSongListUpdate.OrderSongListUpdate(orderList);
                        }



                    }

                }else {

                    if(retcode == GoChatErrorFailNetwork){
                        ToastUtil.showWarnToast("网络异常，更新已点列表失败");
                    }

                    OrderedSongInfo orderedSongInfo = new OrderedSongInfo();
                    orderedSongInfo.setSongList(new ArrayList<OrderedSongInfo.OrderedSongItemInfo>());
                    if(callback != null) {
                        callback.onGetOrderedSongList(retcode, orderedSongInfo);
                    }
                }
            }
        });



    }

    //待播列表增加歌曲
    public void addOrderSongList(List<OrderedSongInfo.OrderedSongItemInfo> infoList){


        if(infoList == null){
            L.i(TAG,"addOrderSongList:infoList == NULL");
            return;
        }
        L.i(TAG,"addOrderSongList:infoList:" + infoList.size());
        updateInfo(infoList);
        orderList.addAll(infoList);
        Collections.sort(orderList);

        if(mOrderSongListUpdate != null){
            mOrderSongListUpdate.OrderSongListUpdate(orderList);
        }
        if(mKTVOrderSongListUpdate != null){
            mKTVOrderSongListUpdate.OrderSongListUpdate(orderList);
        }
        updateSongInfo();
    }

    //待播列表增加歌曲
    public void addOrderSongListForNotify(List<NotifyOrderListUpdateInfo.SongsDTO> infoList){

        if(infoList == null){
            return;
        }

        addOrderSongList(notifyListToInfoList(infoList));
    }

    //取消歌
    public void cancelOrderSongList(List<OrderedSongInfo.OrderedSongItemInfo> infoList){
        if(infoList == null){
            L.i(TAG,"cancelOrderSongList:infoList == NULL");
            return;
        }
        L.i(TAG,"cancelOrderSongList:infoList:" + infoList.size());

        for(OrderedSongInfo.OrderedSongItemInfo info : infoList){
            for(int i=0; i < orderList.size();i++){
                if(orderList.get(i).getUniqueId().equals(info.getUniqueId())){
                    orderList.remove(i);
                    break;
                }
            }
        }

        Collections.sort(orderList);


        if(mOrderSongListUpdate != null){
            mOrderSongListUpdate.OrderSongListUpdate(orderList);
        }
        if(mKTVOrderSongListUpdate != null){
            mKTVOrderSongListUpdate.OrderSongListUpdate(orderList);
        }
    }

    //取消歌
    public void cancelOrderSongListForNotify(List<NotifyOrderListUpdateInfo.SongsDTO> infoList){

        if(infoList == null){
            return;
        }

        cancelOrderSongList(notifyListToInfoList(infoList));
    }


    //歌曲顺序变化
    public void reorderSongList(List<OrderedSongInfo.OrderedSongItemInfo> infoList){
        if(infoList == null){
            L.i(TAG,"reorderSongList:infoList == NULL");
            return;
        }
        L.i(TAG,"reorderSongList:infoList:" + infoList.size());

        for(OrderedSongInfo.OrderedSongItemInfo info : infoList){
            for(int i=0; i < orderList.size();i++){
                if(orderList.get(i).getUniqueId().equals(info.getUniqueId())){
                    orderList.get(i).setOrder(info.getOrder());
                    break;
                }
            }
        }

        Collections.sort(orderList);


        if(mOrderSongListUpdate != null){
            mOrderSongListUpdate.OrderSongListUpdate(orderList);
        }
        if(mKTVOrderSongListUpdate != null){
            mKTVOrderSongListUpdate.OrderSongListUpdate(orderList);
        }
    }

    //歌曲顺序变化
    public void reorderSongListForNotify(List<NotifyOrderListUpdateInfo.SongsDTO> infoList){
        if(infoList == null){
            return;
        }

        reorderSongList(notifyListToInfoList(infoList));
    }


    //当前播放歌曲发生切换或就绪
    public void playOrderSongList(List<OrderedSongInfo.OrderedSongItemInfo> infoList){

        if(infoList == null){
            L.i(TAG,"playOrderSongList:infoList == NULL");
            return;
        }
        L.i(TAG,"playOrderSongList:infoList:" + infoList.size());

        for(OrderedSongInfo.OrderedSongItemInfo info : infoList) {
            info.setOrder(Integer.MIN_VALUE/2);
            if (orderList.size() == 0) {
                updateInfo(info);
                orderList.add(info);
            } else {
                if(orderList.get(0).getUniqueId().equals(info.getUniqueId())){
                    orderList.get(0).setOrder(Integer.MIN_VALUE/2);
                }else {

                    orderList.remove(0);

                    if (orderList.size() == 0) {
                        updateInfo(info);
                        orderList.add(info);
                    }else if (orderList.get(0).getUniqueId().equals(info.getUniqueId())) {
                        orderList.get(0).setOrder(Integer.MIN_VALUE/2);
                    } else {
                        // TODO: 10/27/21 已点列表和服务器版本对应不上，需要更新
                        getOrderSongList(LoginInfoManager.getInstance().getLoginInfo().getUid(),null);
                    }
                }
            }
        }



        Collections.sort(orderList);


        if(mOrderSongListUpdate != null){
            mOrderSongListUpdate.OrderSongListUpdate(orderList);
        }
        if(mKTVOrderSongListUpdate != null){
            mKTVOrderSongListUpdate.OrderSongListUpdate(orderList);
        }
        updateSongInfo();
    }

    //当前播放歌曲发生切换或就绪
    public void playOrderSongListForNotify(List<NotifyOrderListUpdateInfo.SongsDTO> infoList){
        if(infoList == null){
            return;
        }

        playOrderSongList(notifyListToInfoList(infoList));
    }


    private List<OrderedSongInfo.OrderedSongItemInfo> notifyListToInfoList(List<NotifyOrderListUpdateInfo.SongsDTO> infoList){
        List<OrderedSongInfo.OrderedSongItemInfo> temp = new ArrayList<>();

        for(NotifyOrderListUpdateInfo.SongsDTO song:infoList){
            OrderedSongInfo.OrderedSongItemInfo info = new OrderedSongInfo.OrderedSongItemInfo();
            info.setUniqueId(song.getUniqueId());
            info.setCreateTime(song.getCreateTime());
            info.setCreator(song.getCreator());
            info.setCreatorNickName(song.getCreatorNickName());
            info.setOrder(song.getOrder());
            info.setShareToken(song.getShareToken());
            info.setKrcToken(song.getKrcToken());
            info.setSongId(song.getSongId());
            info.setReadyStatus(song.getReadyStatus());
            temp.add(info);
        }

        return temp;
    }

    private OrderedSongInfo.OrderedSongItemInfo updateInfo(OrderedSongInfo.OrderedSongItemInfo info){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(info.getSongId());
        if(song != null) {
            info.setSongName(song.getSongName());
            info.setAlbumImageUri(song.getAlbumImageUri());
        }
        return info;

    }

    private List<OrderedSongInfo.OrderedSongItemInfo> updateInfo(List<OrderedSongInfo.OrderedSongItemInfo> infoList){
        for(OrderedSongInfo.OrderedSongItemInfo info : infoList) {
            ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(info.getSongId());
            if(song != null) {
                info.setSongName(song.getSongName());
                info.setAlbumImageUri(song.getAlbumImageUri());
            }
        }
        return infoList;

    }

    /**
     * 点歌
     * @param uid
     * @param songId
     * @param shareToken
     */
    public void orderSong(long uid, String songId, String shareToken, String krcToken) {
        L.i(TAG, "orderSong() called with: uid = [" + uid + "], songId = [" + songId + "], shareToken = [" + shareToken + "], krcToken = [" + krcToken + "]");

        RoomAPI.orderSong(uid, songId, shareToken, krcToken, new IOrderSongCallback() {
            @Override
            public void onOrderSong(int errorCode, OrderSongInfo orderSongInfo) {

                if (errorCode == 0) {

                }else if(errorCode == GoChatErrorFailNetwork){
                    ToastUtil.showWarnToast("网络异常，请检查网络后重试");
                }
//                String songUniId = orderSongInfo.getSongUniId();
//                List<String> songUniIdrList = orderMap.get(songId);
//                if(songUniIdrList == null){
//                    songUniIdrList = new ArrayList<>();
//                    orderMap.put(songId,songUniIdrList);
//                }
//                songUniIdrList.add(songUniId);

            }
        });
    }

    /**
     * 麦上成员回复点歌
     * @param uid
     * @param songUniId
     */
    public void orderSongNotifyAck(long uid,String songId,String songUniId,ICommonResponseCallback callback){
        L.i(TAG,"orderSongNotifyAck:uid:" + uid+":songId:" + songId + ":songUniId:"+ songUniId);
        if(songId == null || songUniId == null){
            return;
        }

        List<String> orderList = orderMap.get(songId);
        if(orderList == null){
            orderList = new ArrayList<>();
            orderMap.put(songId,orderList);
        }
        orderList.add(songUniId);

        RoomAPI.orderSongNotifyAck(uid, songUniId, new ICommonResponseCallback() {
            @Override
            public void onResponse(int errorCode) {
                if(errorCode == 0){
                    if(callback != null){
                        callback.onResponse(errorCode);
                    }
                }
            }
        });
    }


    /**
     * 歌曲下载完成上报(加入已点列表)
     * @param uid
     * @param songId
     */
    public void songDownloadReport(long uid,String songId){
        L.i(TAG,"songDownloadReport:uid:" + uid+":songId:" + songId );

        List<String> orderList = orderMap.remove(songId);
        if(orderList == null || orderList.isEmpty()){
            return;
        }
        for(String songUniId : orderList) {
            RoomAPI.songDownloadReport(uid, songUniId, new ICommonResponseCallback() {
                @Override
                public void onResponse(int errorCode) {
                    if (errorCode == 0) {

                    }else if(errorCode == GoChatErrorFailNetwork){
                        ToastUtil.showWarnToast("网络异常，请检查网络后重试");
                    }
                }
            });
        }
    }

    public void clearData(){
        orderMap.clear();
        orderList.clear();
        mOrderSongListUpdate = null;
        mKTVOrderSongListUpdate = null;
        mPlayingSongUpdateCallback = null;
        mSongInfoUpdateListener = null;
    }

    public void setOrderSongListUpdate(IOrderSongListUpdate orderSongListUpdate) {
        this.mOrderSongListUpdate = orderSongListUpdate;
    }

    public void setPlayingSongUpdateCallback(IPlayingSongUpdateCallback playingSongUpdateCallback) {
        this.mPlayingSongUpdateCallback = playingSongUpdateCallback;
    }

    public void setSongInfoUpdateListener(ISongInfoUpdateListener songInfoUpdateListener) {
        this.mSongInfoUpdateListener = songInfoUpdateListener;
    }

    public List<OrderedSongInfo.OrderedSongItemInfo> getOrderList() {
        return orderList;
    }

    /**
     * 从服务器获取歌曲信息
     */
    public void updateSongInfo(){
        List<String> songIDList = new ArrayList<>();
        for(OrderedSongInfo.OrderedSongItemInfo info :orderList){
            if(info.getSongName() == null || info.getAlbumImageUri() == null){
                songIDList.add(info.getSongId());

            }

            ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(info.getSongId());
            if (song != null) {
                song.setKrcToken(info.getKrcToken());
            }

            if (ZGKTVLyricManager.getInstance().getLyric(info.getSongId()) == null) {
                ZGKTVCopyrightedMusicManager.getInstance().getKrcLyricForSongID(info.getSongId(), null);
            }
        }

        if(songIDList.size() == 0){
            return;
        }

        List<String> temp = new ArrayList<>();
        int j = 0;
        for(int i = 0; i < songIDList.size();i++)
        {

            if(j < 10) {
                temp.add(songIDList.get(i));
                j++;
                if((i+1 < songIDList.size())) {
                    continue;
                }
            }
            j = 0;
            KTVRequestAPI.songInfo(temp, new ISongInfoCallback<SongInfoInfo>() {
                @Override
                public void onSongInfo(int errorCode, SongInfoInfo songInfoInfo) {

                    if(errorCode != 0){

                        if(ZGKTVCopyrightedMusicManager.isMusicSDKErrorCode(errorCode)){
                            T.show(ZGKTVCopyrightedMusicManager.getErrorInfo(errorCode));
                        }
                        return;
                    }

                    for(SongInfoInfo.SongsDTO info : songInfoInfo.getSongs()) {
                        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(info.getSongId());
                        if(song == null){
                            song = new ZGKTVCopyrightedSong();
                            L.i("songId","putSong222 : " + " : : " + song.getLoadState());
                            ZGKTVPlayerManager.getInstance().putSong(info.getSongId(),song);
                        }
                        song.setSongID(info.getSongId());
                        song.setSongName(info.getSongName());
                        song.setSingerName(info.getSingerName());
                        song.setAlbumImageUri(info.getAlbumImgMini());
                        song.setAlbumID(info.getAlbumId());
                        song.setAlbumName(info.getAlbumName());
                        if(info.getDuration() != 0 && song.getDuration() == 0) {
                            song.setDuration(info.getDuration());
                        }
                    }


                    for(OrderedSongInfo.OrderedSongItemInfo info :orderList){
                        if(info.getSongName() == null || info.getAlbumImageUri() == null){
                            ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(info.getSongId());
                            if(song != null) {
                                info.setSongName(song.getSongName());
                                info.setAlbumImageUri(song.getAlbumImageUri());
                            }
                        }
                    }

                    if(mOrderSongListUpdate != null){
                        mOrderSongListUpdate.OrderSongListUpdate(orderList);
                    }
                    if(mKTVOrderSongListUpdate != null){
                        mKTVOrderSongListUpdate.OrderSongListUpdate(orderList);
                    }

                    if(mSongInfoUpdateListener != null){
                        mSongInfoUpdateListener.onSongInfoUpdate();
                    }

                }
            });
            temp.clear();
        }




    }

    public void updateSongInfo(String songID){
        if(songID == null){
            return;
        }

        List<String> songIDList = new ArrayList<>();
        songIDList.add(songID);
        if(ZGKTVLyricManager.getInstance().getLyric(songID) == null) {
            ZGKTVCopyrightedMusicManager.getInstance().getKrcLyricForSongID(songID, null);
        }


        KTVRequestAPI.songInfo(songIDList, new ISongInfoCallback<SongInfoInfo>() {
            @Override
            public void onSongInfo(int errorCode, SongInfoInfo songInfoInfo) {

                for(SongInfoInfo.SongsDTO info : songInfoInfo.getSongs()) {
                    ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(info.getSongId());
                    if(song == null){
                        song = new ZGKTVCopyrightedSong();
                        L.i("songId","putSong333 : " + " : : " + song.getLoadState());
                        ZGKTVPlayerManager.getInstance().putSong(info.getSongId(),song);
                    }
                    song.setSongID(info.getSongId());
                    song.setSongName(info.getSongName());
                    song.setSingerName(info.getSingerName());
                    song.setAlbumImageUri(info.getAlbumImgMini());
                    song.setAlbumID(info.getAlbumId());
                    song.setAlbumName(info.getAlbumName());
                }

                if(mOrderSongListUpdate != null){
                    mOrderSongListUpdate.OrderSongListUpdate(orderList);
                }

                if(mKTVOrderSongListUpdate != null){
                    mKTVOrderSongListUpdate.OrderSongListUpdate(orderList);
                }

                if(mSongInfoUpdateListener != null){
                    mSongInfoUpdateListener.onSongInfoUpdate();
                }

            }
        });

    }


    public void skipSong(String songId){
        if(orderList.size() > 0 && orderList.get(0).getSongId().equals(songId)){
            orderList.remove(0);
        }

        if(mOrderSongListUpdate != null){
            mOrderSongListUpdate.OrderSongListUpdate(orderList);
        }

        if(mKTVOrderSongListUpdate != null){
            mKTVOrderSongListUpdate.OrderSongListUpdate(orderList);
        }
    }

    public void setKTVOrderSongListUpdate(IOrderSongListUpdate kTVOrderSongListUpdate) {
        this.mKTVOrderSongListUpdate = kTVOrderSongListUpdate;
    }
}
