package im.zego.ktv.chorus.ktvroom;

import android.text.TextUtils;

import im.zego.common.util.L;
import im.zego.ktv.chorus.model.RoomInfo;
import im.zego.ktv.chorus.model.notify.NotifyOrderListUpdateInfo;

public class RoomInfoHelp {

    public static final String TAG = "RoomInfoHelp";

    /**
     * 是否存在当前播放歌曲
     * @return true 表示存在
     */
    public static boolean isSongInfoExist(){
        if(KTVRoomManager.getInstance().getRoomInfo() == null){
            return false;
        }

        if(KTVRoomManager.getInstance().getRoomInfo().getSongInfo() == null){
            return false;
        }

        return !TextUtils.isEmpty(KTVRoomManager.getInstance().getRoomInfo().getSongInfo().getSongId());
    }


    /**
     * 获取当前播放歌曲的songId(可能返回null)
     * @return
     */
    public static String getSongId() {
        if (KTVRoomManager.getInstance().getRoomInfo() == null) {
            return null;
        }

        if (KTVRoomManager.getInstance().getRoomInfo().getSongInfo() == null) {
            return null;
        }

        if(TextUtils.isEmpty(KTVRoomManager.getInstance().getRoomInfo().getSongInfo().getSongId())){
            return null;
        }else {
            return KTVRoomManager.getInstance().getRoomInfo().getSongInfo().getSongId();
        }
    }

    /**
     * 获取当前播放歌曲的uniqueId(可能返回null)
     * @return
     */
    public static String getUniqueId() {
        if (KTVRoomManager.getInstance().getRoomInfo() == null) {
            return null;
        }

        if (KTVRoomManager.getInstance().getRoomInfo().getSongInfo() == null) {
            return null;
        }

        if(TextUtils.isEmpty(KTVRoomManager.getInstance().getRoomInfo().getSongInfo().getUniqueId())){
            return null;
        }else {
            return KTVRoomManager.getInstance().getRoomInfo().getSongInfo().getUniqueId();
        }
    }

    /**
     * 获取当前播放歌曲的token(可能返回null)
     * @return
     */
    public static String getToken() {
        if (KTVRoomManager.getInstance().getRoomInfo() == null) {
            return null;
        }

        if (KTVRoomManager.getInstance().getRoomInfo().getSongInfo() == null) {
            return null;
        }

        if(TextUtils.isEmpty(KTVRoomManager.getInstance().getRoomInfo().getSongInfo().getShareToken())){
            return null;
        }else {
            return KTVRoomManager.getInstance().getRoomInfo().getSongInfo().getShareToken();
        }


    }

    /**
     * 更新当前播放的歌曲
     * @param song
     */
    public static void updateSongInfo(NotifyOrderListUpdateInfo.SongsDTO song){
        RoomInfo roomInfo = KTVRoomManager.getInstance().getRoomInfo();
        if(roomInfo == null || song == null){
            return;
        }
        L.i(TAG,"updateSongInfo" + song);
        RoomInfo.SongInfoDTO songInfo = roomInfo.getSongInfo();
        if(songInfo == null){
            songInfo = new RoomInfo.SongInfoDTO();
        }
        songInfo.setSongId(song.getSongId());
        songInfo.setCreateTime(song.getCreateTime());
        songInfo.setCreator(song.getCreator());
        songInfo.setCreatorNickName(song.getCreatorNickName());
        songInfo.setShareToken(song.getShareToken());
        songInfo.setKrcToken(song.getKrcToken());
        songInfo.setUniqueId(song.getUniqueId());
        roomInfo.setSongInfo(songInfo);
        KTVRoomManager.getInstance().setRoomInfo(roomInfo);
    }

    /**
     * 更新房间信息
     * @param roomInfo
     */
    public static void updateRoomInfo(RoomInfo roomInfo){
        L.i(TAG,"updateRoomInfo" + roomInfo);
        KTVRoomManager.getInstance().setRoomInfo(roomInfo);
    }

    /**
     * 移除当前播放的歌曲
     */
    public static void removeSongInfo(){
        L.i(TAG,"removeSongInfo");
        RoomInfo roomInfo = KTVRoomManager.getInstance().getRoomInfo();
        if(roomInfo == null){
            return;
        }
        roomInfo.setSongInfo(null);
    }


    /**
     * 更新原唱还是伴奏//1：不是，  2：是
     */
    public static void updateOriginPlay(int originPlay){
        L.i(TAG,"updateOriginPlay" + originPlay);
        RoomInfo roomInfo = KTVRoomManager.getInstance().getRoomInfo();
        if(roomInfo == null){
            return;
        }
        roomInfo.setOriginPlay(originPlay);
        KTVRoomManager.getInstance().setRoomInfo(roomInfo);
    }

    public static boolean isAccompany(){
        RoomInfo roomInfo = KTVRoomManager.getInstance().getRoomInfo();
        if(roomInfo == null){
            return true;
        }
        return roomInfo.getOriginPlay() == 0;
    }

    public static long getSongCreatorId(){
        RoomInfo roomInfo = KTVRoomManager.getInstance().getRoomInfo();
        if(roomInfo == null){
            return -1;
        }
        if(roomInfo.getSongInfo() == null){
            return -2;
        }
        return roomInfo.getSongInfo().getCreator();

    }

}
