package im.zego.ktv.chorus.ktvroom.adapter;

import android.graphics.Color;
import android.graphics.Typeface;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;

import im.zego.ktv.R;
import im.zego.ktv.chorus.model.EffectsItemInfo;

public class EffectsAdjustAdapter extends RecyclerView.Adapter<EffectsAdjustAdapter.EffectsViewHolder> {
    private ArrayList<EffectsItemInfo> mAdjustItems = new ArrayList<>();
    private OnClickListener mListener;


    @NonNull
    @Override
    public EffectsViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.ktv_effects_item, parent, false);
        return new EffectsViewHolder(view);
    }

    public void setListener(OnClickListener mListener) {
        this.mListener = mListener;
    }

    @Override
    public void onBindViewHolder(@NonNull EffectsViewHolder holder, int position) {
        EffectsItemInfo effectsItemInfo = mAdjustItems.get(position);

        if (effectsItemInfo.getIsSelected()) {
            holder.tvEffectName.setTextColor(Color.parseColor("#ffff3670"));
            holder.tvEffectName.setTypeface(Typeface.defaultFromStyle(Typeface.BOLD));
        } else {
            holder.tvEffectName.setTextColor(Color.parseColor("#CCFFFFFF"));
            holder.tvEffectName.setTypeface(Typeface.defaultFromStyle(Typeface.NORMAL));
        }
        holder.ivEffects.setImageResource(effectsItemInfo.getResId());
        holder.tvEffectName.setText(effectsItemInfo.getItemName());

        if(effectsItemInfo.getIsSelected()){
            holder.effectsBg.setVisibility(View.VISIBLE);
        }else {
            holder.effectsBg.setVisibility(View.GONE);
        }

        holder.ivEffects.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(mListener != null){
                    mListener.onClick(v, holder.getAdapterPosition());
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        return mAdjustItems.size();
    }

    public void initEffectItems(ArrayList list) {
        mAdjustItems = list;
        notifyDataSetChanged();
    }

    static class EffectsViewHolder extends RecyclerView.ViewHolder {

        private ImageView ivEffects;
        private ImageView effectsBg;
        private TextView tvEffectName;

        public EffectsViewHolder(@NonNull View itemView) {
            super(itemView);
            ivEffects = itemView.findViewById(R.id.iv_effects);
            effectsBg = itemView.findViewById(R.id.effects_bg);
            tvEffectName = itemView.findViewById(R.id.tv_effect_name);
        }

    }
    public interface OnClickListener{
         void onClick(View v, int position);

    }
}
