package im.zego.ktv.chorus.ktvroom.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;

import java.util.ArrayList;
import java.util.List;

import im.zego.ktv.R;
import im.zego.ktv.custom.SongCoverImageView;
import im.zego.ktv.chorus.download.FileDownloadManager;
import im.zego.ktv.chorus.model.ktv.TopSongInfo;
import im.zego.ktv.chorus.rtc.ZGKTVSongLoadState;
import im.zego.ktv.utils.CheckDoubleClick;

public class SongListContentAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private static final String TAG = "SongListContentAdapter";

    private List<TopSongInfo.SongsDTO> mData = new ArrayList<>();
    private Context context;

    public SongListContentAdapter(Context context) {
        this.context = context;
    }

    public void addData(List<TopSongInfo.SongsDTO> mData) {
        this.mData.addAll(mData);
    }

    public void clearData() {
        this.mData.clear();
    }

    public void clearAndAddData(List<TopSongInfo.SongsDTO> mData) {
        this.mData.clear();
        this.mData.addAll(mData);
    }

    public List<TopSongInfo.SongsDTO> getmData() {
        return mData;
    }

    public void updateDownloadState(String songId, ZGKTVSongLoadState state, RecyclerView recyclerView) {
        SongListViewHolder viewHolder = queryViewHolder(songId, recyclerView);
        if (viewHolder != null) {
            viewHolder.updateDownloadState(state);
        }
    }

    public void updateDownloadProgress(String songID, float progressRate, RecyclerView recyclerView) {
        SongListViewHolder viewHolder = queryViewHolder(songID, recyclerView);
        if (viewHolder != null) {
            viewHolder.updateProgress(progressRate);
        }
    }

    private SongListViewHolder queryViewHolder(String songId, RecyclerView recyclerView) {
        int position = queryViewIndexByTag(songId);
        if (position >= 0) {
            SongListViewHolder viewHolder = (SongListViewHolder) recyclerView.findViewHolderForLayoutPosition(position);
            return viewHolder;
        }
        return null;
    }

    private int queryViewIndexByTag(String songId) {
        List<TopSongInfo.SongsDTO> songList = getmData();
        for (int i = 0; i < songList.size(); i++) {
            if (songId.equals(songList.get(i).getSongId())) {
                return i;
            }
        }
        return -1;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.ktv_item_song_list, parent, false);
        return new SongListViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {

        TopSongInfo.SongsDTO bean = mData.get(position);

        ((SongListViewHolder) holder).onBind(bean, position);

    }

    @Override
    public int getItemCount() {
        if (mData == null) {
            return 0;
        }
        return mData.size();
    }


    public class SongListViewHolder extends RecyclerView.ViewHolder {

        private SongCoverImageView mIvSongImg;
        private TextView mTvSongName;
        private TextView mTvSinger;
        private ProgressBar mPbSongDown;
        private TextView mSelectSong;
        private TextView mTvProgressTip;
        private ImageView mIvYsdIcon;

        private SongListViewHolder(View view) {
            super(view);

            mIvSongImg = view.findViewById(R.id.iv_song_img);
            mTvSongName = view.findViewById(R.id.tv_song_name);
            mTvSinger = view.findViewById(R.id.tv_singer);
            mSelectSong = view.findViewById(R.id.select_song);
            mPbSongDown = view.findViewById(R.id.pb_song_down);
            mTvProgressTip = view.findViewById(R.id.tv_progress_tip);
            mIvYsdIcon = view.findViewById(R.id.iv_ysd_icon);
        }

        /**
         * 更新进度条
         *
         * @param progress
         */
        public void updateProgress(float progress) {
            int progress_int = (int) (progress * 100);
            mPbSongDown.setProgress(progress_int);
        }

        /**
         * 更新下载进度
         *
         * @param state
         */
        public void updateDownloadState(ZGKTVSongLoadState state) {
            switch (state) {
                case LOADING:
                    //下载中
                    if (mPbSongDown.getVisibility() != View.VISIBLE) {
                        mSelectSong.setVisibility(View.GONE);
                        mPbSongDown.setVisibility(View.VISIBLE);
                        mTvProgressTip.setVisibility(View.VISIBLE);
                        mTvProgressTip.setText("下载中");
                    }
                    break;
                case LOAD_COMPLETE:
                    Toast.makeText(context, "点歌成功，已加入队列", Toast.LENGTH_SHORT).show();
                    mSelectSong.setVisibility(View.VISIBLE);
                    mPbSongDown.setVisibility(View.GONE);
                    mTvProgressTip.setVisibility(View.GONE);
                    mSelectSong.setText("点歌");
                    break;
                case LOAD_ERROR:
                    //下载完成
                    mSelectSong.setVisibility(View.VISIBLE);
                    mPbSongDown.setVisibility(View.GONE);
                    mTvProgressTip.setVisibility(View.GONE);
                    mSelectSong.setText("点歌");
                    break;
                default:
                    break;

            }
        }

        public void onBind(TopSongInfo.SongsDTO bean, int position) {
            if (bean == null) {
                return;
            }

            Glide.with(context)
                    .load(bean.getAlbumImgSmall())
                    .dontAnimate()
                    .circleCrop()
                    .diskCacheStrategy(DiskCacheStrategy.ALL) // 设置缓存的策略
                    .into(mIvSongImg);

            mTvSongName.setText(bean.getSongName());
            mTvSinger.setText("歌手：" + bean.getSingerName());

            //是音速达的标志(bean.getCopyright().getChannel() == 0)
            if(bean.getCopyright() != null){
                mIvYsdIcon.setVisibility(bean.getCopyright().getChannel() == 0 ? View.VISIBLE : View.GONE);
            }

            if (bean.getLoadState() == ZGKTVSongLoadState.LOAD_COMPLETE.value()) { //已经下载过了
                mSelectSong.setVisibility(View.VISIBLE);
                mPbSongDown.setVisibility(View.GONE);
                mTvProgressTip.setVisibility(View.GONE);
                mSelectSong.setText("点歌");
            } else if (FileDownloadManager.getInstance().isDownloading(bean.getSongId())) { //下载中
                mSelectSong.setVisibility(View.GONE);
                mPbSongDown.setVisibility(View.VISIBLE);
                mTvProgressTip.setVisibility(View.VISIBLE);
                mTvProgressTip.setText("下载中");
            } else {
                mSelectSong.setVisibility(View.VISIBLE);
                mPbSongDown.setVisibility(View.GONE);
                mTvProgressTip.setVisibility(View.GONE);
                mSelectSong.setText("点歌");
            }

            mSelectSong.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {

                    if (CheckDoubleClick.isFastDoubleClick()) {
                        return;
                    }

                    if (bean.getLoadState() == ZGKTVSongLoadState.NO_LOAD.value()) {
                        mSelectSong.setVisibility(View.VISIBLE);
                        mPbSongDown.setVisibility(View.GONE);
                        mTvProgressTip.setVisibility(View.GONE);
                        mSelectSong.setText("等待下载");
                    }

                    if (onSongItemClickListener != null) {
                        onSongItemClickListener.onClick(bean);
                    }

                }
            });
        }

    }


    private OnSongItemClickListener onSongItemClickListener;

    public void setOnSongItemClickListener(OnSongItemClickListener onSongItemClickListener) {
        this.onSongItemClickListener = onSongItemClickListener;
    }

    public interface OnSongItemClickListener {
        void onClick(TopSongInfo.SongsDTO bean);
    }

}
