package im.zego.ktv.chorus.ktvroom.adapter;

import static im.zego.ktv.ErrorCodeConstants.GoChatErrorFailNetwork;

import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;

import java.util.ArrayList;
import java.util.List;

import im.zego.ktv.R;
import im.zego.ktv.chorus.ktvroom.KTVRoomManager;
import im.zego.ktv.chorus.model.OrderedSongInfo;
import im.zego.ktv.chorus.model.RoomInfo;
import im.zego.ktv.chorus.protocol.ICommonResponseCallback;
import im.zego.ktv.chorus.protocol.RoomAPI;
import im.zego.ktv.utils.ToastUtil;
import im.zego.ktv.chorus.viewmodel.KTVRoomViewModel;

public class WaitListAdapter  extends RecyclerView.Adapter<WaitListAdapter.MyViewHolder> {


    private List<OrderedSongInfo.OrderedSongItemInfo> data = new ArrayList<>();

    private Context context;

    public WaitListAdapter(Context context) {
        this.context = context;
    }

    @NonNull
    @Override
    public MyViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.ktv_item_choose_song, parent, false);

        return new MyViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull MyViewHolder holder, int position) {
        if(data.size() <= position){
            return;
        }
        OrderedSongInfo.OrderedSongItemInfo info = data.get(position);


        holder.mSongName.setText(info.getSongName());
        holder.mCreateName.setText("点唱者：" + info.getCreatorNickName());

        Glide.with(context)
                .load(info.getAlbumImageUri())
                .dontAnimate()
                .circleCrop()
                .diskCacheStrategy(DiskCacheStrategy.ALL) // 设置缓存的策略
                .into(holder.mSongCover);


        KTVRoomViewModel mViewModel = KTVRoomManager.getInstance().getViewModel();
        RoomInfo.SongInfoDTO songInfoDTO = KTVRoomManager.getInstance().getRoomInfo().getSongInfo();
        String songId = null;
        if(songInfoDTO != null){
            songId = songInfoDTO.getSongId();
        }
        if(mViewModel != null) {
            if (position == 0 && info.getSongId().equals(songId)) {
                holder.mCancelSong.setVisibility(View.GONE);
                holder.mTopSong.setVisibility(View.GONE);
                holder.mPlayingSong.setVisibility(View.VISIBLE);
            }else if(mViewModel.getMyInfo().getValue().isHost() || mViewModel.getMyInfo().getValue().getUserID() == info.getCreator()){
                holder.mCancelSong.setVisibility(View.VISIBLE);
                if(position == 1){
                    holder.mTopSong.setVisibility(View.GONE);
                }else {
                    holder.mTopSong.setVisibility(View.VISIBLE);
                }
                holder.mPlayingSong.setVisibility(View.GONE);
            }else {
                holder.mCancelSong.setVisibility(View.GONE);
                holder.mTopSong.setVisibility(View.GONE);
                holder.mPlayingSong.setVisibility(View.GONE);
            }
        }

        holder.mCancelSong.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                RoomAPI.setCancelSong(mViewModel.getMyInfo().getValue().getUserID(), info.getCreator(), info.getCreatorNickName(), info.getCreateTime(), info.getUniqueId(), new ICommonResponseCallback() {
                    @Override
                    public void onResponse(int errorCode) {
                        if(errorCode == 0){
                            Toast.makeText(context,"取消歌曲成功",Toast.LENGTH_SHORT).show();
                        }else if(errorCode == GoChatErrorFailNetwork){
                            ToastUtil.showWarnToast("网络异常，请检查网络后重试");
                        }else {
                            Toast.makeText(context,"取消歌曲失败",Toast.LENGTH_SHORT).show();
                        }
                    }
                });
            }
        });

        holder.mTopSong.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                RoomAPI.setTopSong(mViewModel.getMyInfo().getValue().getUserID(), info.getCreator(), info.getCreatorNickName(), info.getCreateTime(), info.getUniqueId(), new ICommonResponseCallback() {
                    @Override
                    public void onResponse(int errorCode) {
                        if(errorCode == 0){
                            Toast.makeText(context,"置顶歌曲成功",Toast.LENGTH_SHORT).show();
                        }else if(errorCode == GoChatErrorFailNetwork){
                            ToastUtil.showWarnToast("网络异常，请检查网络后重试");
                        } else {
                            Toast.makeText(context,"置顶歌曲失败",Toast.LENGTH_SHORT).show();
                        }
                    }
                });
            }
        });

    }

    @Override
    public int getItemCount() {
        return data.size();
    }

    public void setData(List<OrderedSongInfo.OrderedSongItemInfo> data) {
        this.data = data;
        for(OrderedSongInfo.OrderedSongItemInfo info : data){
            Log.e("APIBase","info:getSongId:" + info.getSongId() + ":order:" + info.getOrder());
        }
//        notifyDataSetChanged();
    }

    static class MyViewHolder extends  RecyclerView.ViewHolder{
        private View view;
        private TextView mSongName;
        private TextView mCreateName;
        private ImageView mSongCover;
        private ImageView mTopSong;
        private ImageView mCancelSong;
        private View mPlayingSong;

        public MyViewHolder(@NonNull View itemView) {
            super(itemView);
            view = itemView;
            mSongName = itemView.findViewById(R.id.song_name);
            mCreateName = itemView.findViewById(R.id.create_name);
            mSongCover = itemView.findViewById(R.id.song_cover);
            mTopSong = itemView.findViewById(R.id.top_song);
            mCancelSong = itemView.findViewById(R.id.cancel_song);
            mPlayingSong = itemView.findViewById(R.id.playing_song);
        }

    }
}
