package im.zego.ktv.chorus.ktvroom.fragment;

import static im.zego.ktv.ErrorCodeConstants.GoChatErrorFailNetwork;
import static im.zego.ktv.Constants.STATE_ERROR;
import static im.zego.ktv.Constants.STATE_FINISH;
import static im.zego.ktv.Constants.STATE_NORMAL;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import java.util.List;

import im.zego.ktv.R;
import im.zego.ktv.custom.KTVLoadMoreView;
import im.zego.ktv.custom.ProcessView;
import im.zego.ktv.chorus.download.FileDownloadManager;
import im.zego.ktv.chorus.download.impl.FileDownloadStateListener;
import im.zego.ktv.chorus.ktvroom.adapter.SongListContentAdapter;
import im.zego.ktv.chorus.model.ktv.TopSongInfo;
import im.zego.ktv.chorus.protocol.KTVRequestAPI;
import im.zego.ktv.chorus.protocol.ktv.ITopSongCallback;
import im.zego.ktv.chorus.roomlist.adapter.EmptyAndLoadMoreAdapterWrapper;
import im.zego.ktv.chorus.rtc.ZGKTVSongLoadState;
import im.zego.ktv.utils.ToastUtil;

/**
 * 热歌推荐
 */
public class HitSongFragment extends Fragment {

    private View view;
    private RecyclerView mSongListContent;  //歌曲类型列表
    private SongListContentAdapter mSongListContentAdapter;

    private ConstraintLayout mClLoadBtn;
    private TextView mTvReloading;

    private KTVLoadMoreView mLoadMoreView;
    private EmptyAndLoadMoreAdapterWrapper mAdapterWrapper;
    private static final int PAGE_NUM = 10;
    private int pageNum = 1;
    private int mState = STATE_NORMAL;
    private ProcessView mProcessView;
    private View mSongListView;
    private String topId = "8888";
    private int position = 0;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (view == null) {
            view = inflater.inflate(R.layout.ktv_fragment_song, container, false);
            initView(view);
            initRecycleView();
            loadData();
        }
        return view;
    }

    private void initView(View view) {
        mSongListContent = view.findViewById(R.id.song_list_content);
        mClLoadBtn = view.findViewById(R.id.cl_load_btn);
        mTvReloading = view.findViewById(R.id.tv_reloading);
        mProcessView = new ProcessView(getActivity());
    }

    public void setContext(View mSongListView, int position) {
        this.mSongListView = mSongListView;
        this.position = position;
    }

    @Override
    public void onResume() {
        super.onResume();
        if (mSongListContentAdapter.getmData().size() == 0 && mClLoadBtn.getVisibility() == View.GONE) {
            loading();
        }
    }

    private void initRecycleView() {

        String[] categoryIdList = getResources().getStringArray(R.array.ktv_category_id);
        topId = categoryIdList[position];

        this.mSongListContentAdapter = new SongListContentAdapter(getContext());
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getContext());
        this.mSongListContent.setLayoutManager(linearLayoutManager);

        //解决刷新闪烁问题
        ((SimpleItemAnimator) this.mSongListContent.getItemAnimator()).setSupportsChangeAnimations(false);

        View emptyView = LayoutInflater.from(getContext()).inflate(R.layout.ktv_search_song_empty, mSongListContent, false);
        mTvReloading.setOnClickListener(v -> {
            loading();
            mState = STATE_NORMAL;
            loadData();
        });

        mLoadMoreView = new KTVLoadMoreView(getContext());
        mAdapterWrapper = new EmptyAndLoadMoreAdapterWrapper(mSongListContentAdapter);
        mAdapterWrapper.setEmptyView(emptyView);
        mAdapterWrapper.setLoadMoreView(mLoadMoreView);
        this.mSongListContent.setAdapter(this.mAdapterWrapper);

        //加载更多
        mAdapterWrapper.setOnLoadMoreListener(new EmptyAndLoadMoreAdapterWrapper.OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                if (mState == STATE_FINISH) {
                    mLoadMoreView.showFinish();
                } else {
                    pageNum++;
                    mLoadMoreView.showLoading();
                    loadData();
                }

            }
        });

        mSongListContentAdapter.setOnSongItemClickListener(new SongListContentAdapter.OnSongItemClickListener() {
            @Override
            public void onClick(TopSongInfo.SongsDTO bean) {
                FileDownloadManager.getInstance().downloadFile(getActivity(), bean.getSongId());
                registerDownloadProgress(bean);
            }
        });

    }

    private void registerDownloadProgress(TopSongInfo.SongsDTO bean) {
        String songId = bean.getSongId();
        FileDownloadManager.getInstance().registerDownloadListener(getActivity(), songId, new FileDownloadStateListener() {
            @Override
            public void onDownloadState(ZGKTVSongLoadState state) {
                mSongListContentAdapter.updateDownloadState(songId, state, mSongListContent);
                if (state == ZGKTVSongLoadState.LOAD_COMPLETE) {
                    bean.setLoadState(ZGKTVSongLoadState.LOAD_COMPLETE.value());
                } else if (state == ZGKTVSongLoadState.LOADING) {
                    if (bean.getLoadState() != ZGKTVSongLoadState.LOADING.value()) {
                        bean.setLoadState(ZGKTVSongLoadState.LOADING.value());
                    }
                }
            }

            @Override
            public void onDownloadProgress(float progress) {
                mSongListContentAdapter.updateDownloadProgress(songId, progress, mSongListContent);
            }
        });
    }

    private void loading() {
        mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mSongListView.getRootView());
    }

    /**
     * 加载更多（通过sdk获取列表数据）
     */
    private void loadData() {
        gainSongListCategory(topId, pageNum, PAGE_NUM);
    }

    public void gainSongListCategory(String topId, int page, int size) {

        if (mState == STATE_NORMAL) {

            KTVRequestAPI.topSong(topId, page, size, new ITopSongCallback<TopSongInfo>() {
                @Override
                public void onTopSong(int errorCode, TopSongInfo topSongInfo) {
                    if (mProcessView != null) {
                        mProcessView.dismiss();
                    }
                    if (errorCode == 0) {  //返回成功
                        if (topSongInfo != null && topSongInfo.getSongs() != null) {
                            List<TopSongInfo.SongsDTO> mData = topSongInfo.getSongs();
                            //返回来的数据不固定，不受size 控制。只有返回的数据为0的时候才不加载下一页
                            if (mData.size() == 0) {
                                mState = STATE_FINISH;
                                mLoadMoreView.showFinish();
                                loadNoneData();
                            } else {
                                mState = STATE_NORMAL;
                                mClLoadBtn.setVisibility(View.GONE);
                            }
                            mSongListContentAdapter.addData(topSongInfo.getSongs());
                            mAdapterWrapper.notifyDataSetChanged();
                        } else {
                            mState = STATE_ERROR;
                            mLoadMoreView.showFinish();
                            loadNoneData();
                        }
                    } else {
                        if (errorCode == GoChatErrorFailNetwork) {
                            ToastUtil.showWarnToast(R.string.fail_network);
                        }
                        mState = STATE_ERROR;
                        mLoadMoreView.showFinish();
                        loadNoneData();
                    }
                }
            });
        }
    }

    private void loadNoneData(){
        if(pageNum == 1){
            mClLoadBtn.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        FileDownloadManager.getInstance().unRegisterDownloadListener(getActivity());
    }
}
