package im.zego.ktv.chorus.ktvroom.fragment;

import static im.zego.ktv.ErrorCodeConstants.GoChatErrorFailNetwork;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import im.zego.ktv.R;
import im.zego.ktv.custom.ProcessView;
import im.zego.ktv.chorus.download.FileDownloadManager;
import im.zego.ktv.chorus.download.impl.FileDownloadStateListener;
import im.zego.ktv.chorus.ktvroom.adapter.SongListContentAdapter;
import im.zego.ktv.chorus.model.ktv.SongCopyrightInfo;
import im.zego.ktv.chorus.model.ktv.SongInfoInfo;
import im.zego.ktv.chorus.model.ktv.TopSongInfo;
import im.zego.ktv.chorus.protocol.KTVRequestAPI;
import im.zego.ktv.chorus.protocol.ktv.ISongCopyrightCallback;
import im.zego.ktv.chorus.protocol.ktv.ITopSongCallback;
import im.zego.ktv.chorus.rtc.ZGKTVSongLoadState;
import im.zego.ktv.utils.ToastUtil;


/**
 * 推荐歌曲
 */
public class RecommendSongFragment extends Fragment {

    private View view;
    private RecyclerView mSongListContent;  //歌曲类型列表
    private SongListContentAdapter mSongListContentAdapter;

    private ConstraintLayout mClLoadBtn;
    private TextView mTvReloading;

    private ProcessView mProcessView;

    private View mSongListView;
    private int position = 0;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (view == null) {
            view = inflater.inflate(R.layout.ktv_fragment_song, container, false);
            initView(view);
            initRecycleView();
            loadData();
        }
        return view;
    }

    private void initView(View view) {
        mSongListContent = view.findViewById(R.id.song_list_content);
        mClLoadBtn = view.findViewById(R.id.cl_load_btn);
        mTvReloading = view.findViewById(R.id.tv_reloading);
        mProcessView = new ProcessView(getActivity());
    }

    public void setContext(View mSongListView, int position) {
        this.mSongListView = mSongListView;
        this.position = position;
    }

    private void initRecycleView() {

        this.mSongListContentAdapter = new SongListContentAdapter(getContext());
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getContext());
        this.mSongListContent.setLayoutManager(linearLayoutManager);
        //解决刷新闪烁问题
        ((SimpleItemAnimator) this.mSongListContent.getItemAnimator()).setSupportsChangeAnimations(false);
        this.mSongListContent.setAdapter(this.mSongListContentAdapter);

        mTvReloading.setOnClickListener(v -> {
            loading();
            loadData();
        });

        mSongListContentAdapter.setOnSongItemClickListener(new SongListContentAdapter.OnSongItemClickListener() {
            @Override
            public void onClick(TopSongInfo.SongsDTO bean) {
                FileDownloadManager.getInstance().downloadFile(getActivity(), bean.getSongId());
                registerDownloadProgress(bean);
            }
        });

    }

    private void registerDownloadProgress(TopSongInfo.SongsDTO bean) {
        String songId = bean.getSongId();
        FileDownloadManager.getInstance().registerDownloadListener(getActivity(), songId, new FileDownloadStateListener() {
            @Override
            public void onDownloadState(ZGKTVSongLoadState state) {
                mSongListContentAdapter.updateDownloadState(songId, state, mSongListContent);
                if (state == ZGKTVSongLoadState.LOAD_COMPLETE) {
                    bean.setLoadState(ZGKTVSongLoadState.LOAD_COMPLETE.value());
                } else if (state == ZGKTVSongLoadState.LOADING) {
                    if (bean.getLoadState() != ZGKTVSongLoadState.LOADING.value()) {
                        bean.setLoadState(ZGKTVSongLoadState.LOADING.value());
                    }
                }
            }

            @Override
            public void onDownloadProgress(float progress) {
                mSongListContentAdapter.updateDownloadProgress(songId, progress, mSongListContent);
            }
        });
    }

    private void loading() {
        mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mSongListView.getRootView());
    }


    /**
     * 加载数据
     */
    private void loadData() {
        loading();
        songCopyright();
    }

    /**
     * 获取歌曲权限
     */
    private void songCopyright() {

        KTVRequestAPI.songCopyright(recommendList(), new ISongCopyrightCallback<SongCopyrightInfo>() {
            @Override
            public void onSongCopyright(int errorCode, List<String> songCopyrightInfo, ConcurrentHashMap<String, Integer> songCopyrightChannel) {
                if (errorCode == 0) {
                    gainSongListCategory(songCopyrightInfo,songCopyrightChannel);
                } else {
                    ToastUtil.showWarnToast(R.string.fail_network);
                    if (mProcessView != null) {
                        mProcessView.dismiss();
                    }
                    mClLoadBtn.setVisibility(View.VISIBLE);
                }
            }
        });

    }

    /**
     * 获取歌曲信息
     *
     * @param songCopyrightInfo
     */
    private void gainSongListCategory(List<String> songCopyrightInfo,ConcurrentHashMap<String, Integer> songCopyrightChannel) {

        if(songCopyrightInfo == null || songCopyrightChannel == null)
        {
            ToastUtil.showWarnToast(R.string.fail_network);
            return;
        }

        KTVRequestAPI.songRecommendInfo(songCopyrightInfo, songCopyrightChannel,new ITopSongCallback<SongInfoInfo>() {
            @Override
            public void onTopSong(int errorCode, TopSongInfo topSongInfo) {
                if (mProcessView != null) {
                    mProcessView.dismiss();
                }

                if (errorCode == 0) {  //返回成功
                    if (topSongInfo != null && topSongInfo.getSongs() != null) {
                        List<TopSongInfo.SongsDTO> mData = topSongInfo.getSongs();
                        //返回来的数据不固定，不受size 控制。只有返回的数据为0的时候才不加载下一页
                        if (mData.size() == 0) {
                            mClLoadBtn.setVisibility(View.VISIBLE);
                        } else {
                            mClLoadBtn.setVisibility(View.GONE);
                        }
                        mSongListContentAdapter.addData(topSongInfo.getSongs());
                        mSongListContentAdapter.notifyDataSetChanged();
                    } else {
                        mClLoadBtn.setVisibility(View.VISIBLE);
                    }
                } else {
                    if (errorCode == GoChatErrorFailNetwork) {
                        ToastUtil.showWarnToast(R.string.fail_network);
                    }
                    mClLoadBtn.setVisibility(View.VISIBLE);
                }
            }

        });
    }


    /**
     * 获取全部
     *
     * @return
     */
    private List<String> recommendList() {
        String[] recommendIdList = getResources().getStringArray(R.array.ktv_recommend_id);
        List<String> resultList = new ArrayList<>(recommendIdList.length);
        Collections.addAll(resultList, recommendIdList);
        return resultList;
    }


}
