package im.zego.ktv.chorus.protocol;


import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import im.zego.ktv.chorus.model.ktv.PlaylistCategoryInfo;
import im.zego.ktv.chorus.model.ktv.PlaylistClassifyInfo;
import im.zego.ktv.chorus.model.ktv.PlaylistSongInfo;
import im.zego.ktv.chorus.model.ktv.SearchTipsInfo;
import im.zego.ktv.chorus.model.ktv.SongCopyrightInfo;
import im.zego.ktv.chorus.model.ktv.SongInfoInfo;
import im.zego.ktv.chorus.model.ktv.TopSongInfo;
import im.zego.ktv.chorus.protocol.ktv.IPlayListCategoryCallback;
import im.zego.ktv.chorus.protocol.ktv.IPlayListClassifyCallback;
import im.zego.ktv.chorus.protocol.ktv.IPlayListSongCallback;
import im.zego.ktv.chorus.protocol.ktv.ISearchTipsCallback;
import im.zego.ktv.chorus.protocol.ktv.ISongCopyrightCallback;
import im.zego.ktv.chorus.protocol.ktv.ISongInfoCallback;
import im.zego.ktv.chorus.protocol.ktv.ITopSongCallback;
import im.zego.ktv.chorus.utils.DataFilteringUtil;

public class KTVRequestAPI {

    /**
     * 歌单分类列表
     *
     * @param callback
     */
    public static void playlistCategory(@NotNull final IPlayListCategoryCallback<PlaylistCategoryInfo> callback) {
        String reqJson = APIBase.builder()
                .build();

        String url = "/playlist/category";

        APIBase.postSDKRequest(url, reqJson, PlaylistCategoryInfo.class, new IPostSDKRequestCallback<PlaylistCategoryInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, PlaylistCategoryInfo responseJsonBean) {
                if (callback != null) {
                    callback.onPlayListCategory(errorCode, responseJsonBean);
                }
            }
        });
    }


    /**
     * 歌单列表
     *
     * @param callback
     */
    public static void playlistClassify(String categoryID, int page, int size, @NotNull final IPlayListClassifyCallback<PlaylistClassifyInfo> callback) {
        String reqJson = APIBase.builder()
                .addParams("category_id", categoryID)
                .addParams("page", page)
                .addParams("size", size)
                .build();

        String url = "/playlist/classify";

        APIBase.postSDKRequest(url, reqJson, PlaylistClassifyInfo.class, new IPostSDKRequestCallback<PlaylistClassifyInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, PlaylistClassifyInfo responseJsonBean) {
                if (callback != null) {
                    callback.onPlayListClassify(errorCode, responseJsonBean);
                }
            }
        });
    }


    /**
     * 歌单歌曲
     *
     * @param playlistID
     * @param page
     * @param size       无效参数，测试发现是固定大小
     * @param callback
     */
    public static void playlistSong(String playlistID, int page, int size, @NotNull final IPlayListSongCallback<PlaylistSongInfo> callback) {
        String reqJson = APIBase.builder()
                .addParams("playlist_id", playlistID)
                .addParams("page", page)
                .addParams("size", size)
                .build();

        String url = "/playlist/song";

        APIBase.postSDKRequest(url, reqJson, PlaylistSongInfo.class, new IPostSDKRequestCallback<PlaylistSongInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, PlaylistSongInfo responseJsonBean) {
                if (callback != null) {
                    callback.onPlayListSong(errorCode, responseJsonBean);
                }
            }
        });
    }

    /**
     * 批量查询歌曲信息
     *
     * @param callback
     */
    public static void songInfo(List<String> songIDList, @NotNull final ISongInfoCallback<SongInfoInfo> callback) {
        String reqJson = APIBase.builder()
                .addParams("songs_id", songIDList)
                .build();

        String url = "/song/infos";

        APIBase.postSDKRequest(url, reqJson, SongInfoInfo.class, new IPostSDKRequestCallback<SongInfoInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, SongInfoInfo responseJsonBean) {
                if (callback != null) {
                    callback.onSongInfo(errorCode, responseJsonBean);
                }
            }
        });
    }


    /**
     * 搜索提示
     *
     * @param callback
     */
    public static void searchTips(String keyword, @NotNull final ISearchTipsCallback<SearchTipsInfo> callback) {
        String reqJson = APIBase.builder()
                .addParams("keyword", keyword)
                .build();

        String url = "/search/tips";

        APIBase.postSDKRequest(url, reqJson, SearchTipsInfo.class, new IPostSDKRequestCallback<SearchTipsInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, SearchTipsInfo responseJsonBean) {
                if (callback != null) {
                    callback.onSearchTips(errorCode, responseJsonBean);
                }
            }
        });
    }

    /**
     * 搜索歌曲
     *
     * @param keyword
     * @param page
     * @param size     无效参数，测试发现是固定大小
     * @param callback
     */
    public static void searchSong(String keyword, int page, int size, @NotNull final ITopSongCallback<TopSongInfo> callback) {
        String reqJson = APIBase.builder()
                .addParams("keyword", keyword)
                .addParams("page", page)
                .addParams("size", size)
                .build();

        String url = "/search/song";

        APIBase.postSDKRequest(url, reqJson, TopSongInfo.class, new IPostSDKRequestCallback<TopSongInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, TopSongInfo responseJsonBean) {
                if (callback != null) {
                    callback.onTopSong(errorCode, responseJsonBean == null ? null : DataFilteringUtil.topSongInfoFilter(responseJsonBean));
                }
            }
        });
    }

    /**
     * 榜单歌曲
     *
     * @param topID
     * @param page
     * @param size     无效参数，测试发现是固定大小
     * @param callback
     */
    public static void topSong(String topID, int page, int size, @NotNull final ITopSongCallback<TopSongInfo> callback) {
        String reqJson = APIBase.builder()
                .addParams("top_id", topID)
                .addParams("page", page)
                .addParams("size", size)
                .build();

        String url = "/top/song";

        APIBase.postSDKRequest(url, reqJson, TopSongInfo.class, new IPostSDKRequestCallback<TopSongInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, TopSongInfo responseJsonBean) {
                if (callback != null) {
                    callback.onTopSong(errorCode, responseJsonBean == null ? null : DataFilteringUtil.topSongInfoFilter(responseJsonBean));
                }
            }
        });
    }


    /**
     * 歌曲权限
     *
     * @param callback
     */
    public static void songCopyright(List<String> songsID, @NotNull final ISongCopyrightCallback<SongCopyrightInfo> callback) {
        String reqJson = APIBase.builder()
                .addParams("songs_id", songsID)
                .build();

        String url = "/song/copyright";

        APIBase.postSDKRequest(url, reqJson, SongCopyrightInfo.class, new IPostSDKRequestCallback<SongCopyrightInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, SongCopyrightInfo responseJsonBean) {
                if (callback != null) {
                    callback.onSongCopyright(errorCode, responseJsonBean == null ? null : DataFilteringUtil.songCopyrightInfoFilter(responseJsonBean),responseJsonBean == null ? null :DataFilteringUtil.songCopyrightChannelFilter(responseJsonBean));
                }
            }
        });
    }



    /**
     * 批量查询歌曲信息
     * @param callback
     */
    public static void songRecommendInfo(List<String> songIDList, ConcurrentHashMap<String, Integer> songCopyrightChannel, @NotNull final ITopSongCallback<SongInfoInfo> callback) {
        String reqJson = APIBase.builder()
                .addParams("songs_id",songIDList)
                .build();

        String url = "/song/infos";

        APIBase.postSDKRequest(url, reqJson, TopSongInfo.class, new IPostSDKRequestCallback<TopSongInfo>() {
            @Override
            public void onResponse(int errorCode, @NotNull String message, TopSongInfo responseJsonBean) {
                if(callback != null){
                    callback.onTopSong(errorCode,DataFilteringUtil.topSongAddChannel(responseJsonBean,songCopyrightChannel));
                }
            }
        });
    }


}
