package im.zego.ktv.chorus.roomlist.activity;


import static im.zego.ktv.ErrorCodeConstants.ERROR_USER_OFF_LINE;
import static im.zego.ktv.ErrorCodeConstants.GoChatErrorFailNetwork;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;

import java.util.Locale;

import im.zego.common.application.ZegoApplication;
import im.zego.common.model.LoginInfo;
import im.zego.ktv.R;
import im.zego.ktv.chorus.base.ZGBaseActivity;
import im.zego.ktv.custom.MessageDialog;
import im.zego.ktv.custom.ProcessView;
import im.zego.ktv.chorus.ktvroom.KTVRoomActivity;
import im.zego.ktv.chorus.ktvroom.KTVRoomManager;
import im.zego.ktv.chorus.login.LoginInfoManager;
import im.zego.ktv.chorus.model.RoomInfo;
import im.zego.ktv.chorus.model.notify.NotifyRoomStateUpdateInfo;
import im.zego.ktv.chorus.protocol.ICreateRoomCallback;
import im.zego.ktv.chorus.protocol.RoomAPI;
import im.zego.ktv.chorus.rtc.RTCSDKManager;
import im.zego.ktv.chorus.rtc.ZGKTVCopyrightedMusicManager;
import im.zego.ktv.chorus.rtc.ZGKTVPlayerManager;
import im.zego.ktv.chorus.rtc.callbacks.IRTCCommonCallback;
import im.zego.ktv.utils.GlideImageLoader;
import im.zego.ktv.utils.ToastUtil;
import im.zego.ktv.utils.PermissionHelper;

public class RoomListActivity extends ZGBaseActivity {
    private FrameLayout mFlRoomList;
    private FrameLayout mFlRoomListFragment;

    private ConstraintLayout titleLayout;

    private ProcessView mProcessView;

    private BottomSheetDialog mBottomSheetDialog;

    private LoginInfo mMyInfo;

    private String mTmpRoomName;
    private String mTmpHostName;

    private TextView mCreateKtvRoom;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mMyInfo = LoginInfoManager.getInstance().getLoginInfo();
//        ScreenAdaptationUtil.init(this.getApplicationContext(), 375);
//        ScreenAdaptationUtil.setCustomDensity(this);
        setContentView(R.layout.ktv_activity_roomlist);
        RTCSDKManager.getInstance().init(ZegoApplication.getAppContext());
        initView();
        initListener();
        initFragment();

    }


    private void initView() {
        mFlRoomList = findViewById(R.id.fl_room_list);
        mFlRoomListFragment = findViewById(R.id.fl_room_list_fragment);
        titleLayout = findViewById(R.id.custom_title);
        mCreateKtvRoom = findViewById(R.id.tv_create_ktv_room);
        mProcessView = new ProcessView(RoomListActivity.this);
        mBottomSheetDialog = new BottomSheetDialog(RoomListActivity.this, R.style.KTVBottomSheetDialog);
    }

    private void initListener() {
        TextView mCreateRoom = (TextView) titleLayout.getViewById(R.id.right_icon);
        mCreateRoom.setVisibility(View.GONE);
        titleLayout.getViewById(R.id.title_text).setVisibility(View.VISIBLE);

        mCreateKtvRoom.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                micPermission();
            }
        });
        titleLayout.findViewById(R.id.left_icon).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mBottomSheetDialog.dismiss();
                finish();
            }
        });

    }

    private void initFragment() {
        Fragment fragment = getSupportFragmentManager().findFragmentById(R.id.fl_room_list_fragment);
        if (fragment == null) {
            FragmentTransaction beginTransaction = getSupportFragmentManager().beginTransaction();
            RoomListFragment roomListFragment = new RoomListFragment();
            beginTransaction.add(R.id.fl_room_list_fragment, roomListFragment);
            beginTransaction.commit();
        }
    }


    /**
     * 创建房间的弹窗
     */
    private void showBottomSheetDialog() {
        View view = View.inflate(RoomListActivity.this, R.layout.ktv_dialog_create_room, null);
        view.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                InputMethodManager inputMethodManager = (InputMethodManager) RoomListActivity.this.getSystemService(Activity.INPUT_METHOD_SERVICE);
                inputMethodManager.hideSoftInputFromWindow(view.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
            }
        });
        final EditText mEtRoomName = view.findViewById(R.id.et_room_name);
        final EditText mEtNickName = view.findViewById(R.id.et_nick_name);
        int randomNum = getKTVRandomNum();
        mEtNickName.setText(getKTVRandomHostName(randomNum));
        mEtRoomName.setText(getKTVRandomRoomName(randomNum));
        final Button mBtCreateRoom = view.findViewById(R.id.bt_create_room);

        view.findViewById(R.id.v_top_line).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mBottomSheetDialog.dismiss();
            }
        });

        mEtRoomName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                mTmpRoomName = mEtRoomName.getText().toString();
            }
        });

        mEtNickName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                mTmpHostName = mEtNickName.getText().toString();
            }
        });

        mBtCreateRoom.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mEtRoomName.getText() == null || "".equals(mEtRoomName.getText().toString().trim())) {
                    Toast.makeText(RoomListActivity.this, "请输入房间名", Toast.LENGTH_SHORT).show();
                    return;
                }
                if (mEtNickName.getText() == null || "".equals(mEtNickName.getText().toString().trim())) {
                    Toast.makeText(RoomListActivity.this, "请输入用户名", Toast.LENGTH_SHORT).show();
                    return;
                }
                String avatar = GlideImageLoader.getInstance().getAvatar();
                mBottomSheetDialog.dismiss();
                mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mFlRoomList.getRootView());
                RTCSDKManager.getInstance().initQualityManager(RoomListActivity.this);
                mMyInfo.setNickName(mEtNickName.getText().toString().trim());
                mMyInfo.setAvatar(avatar);
                RTCSDKManager.getInstance().setUserID(mMyInfo.getUid(), mMyInfo.getNickName());
                RoomAPI.createRoom(mMyInfo.getUid(), mEtNickName.getText().toString().trim(), avatar, mEtRoomName.getText().toString().trim(), avatar, new ICreateRoomCallback() {
                    @Override
                    public void onCreateRoom(int retcode, NotifyRoomStateUpdateInfo createdNotifyRoomStateUpdateInfo) {
                        if (retcode == 0) {
                            mProcessView.dismiss();

                            RoomInfo roomInfo = new RoomInfo();
                            roomInfo.setRoomId(createdNotifyRoomStateUpdateInfo.getRoomId());
                            roomInfo.setOriginPlay(createdNotifyRoomStateUpdateInfo.getOriginPlay());
                            roomInfo.setCoverImg(createdNotifyRoomStateUpdateInfo.getCoverImg());
                            roomInfo.setCreateTime(createdNotifyRoomStateUpdateInfo.getCreateTime());
                            roomInfo.setCreatorName(createdNotifyRoomStateUpdateInfo.getCreatorName());
                            roomInfo.setSubject(createdNotifyRoomStateUpdateInfo.getSubject());
                            roomInfo.setCreatorId(createdNotifyRoomStateUpdateInfo.getCreatorId());
                            KTVRoomManager.getInstance().setRoomInfo(roomInfo);
                            mMyInfo.setNickName(mEtNickName.getText().toString().trim());
                            mMyInfo.setAvatar(avatar);
                            LoginInfoManager.getInstance().setLoginInfo(mMyInfo);
                            initUserData(new IRTCCommonCallback() {
                                @Override
                                public void onRTCCallback(int errorCode) {
                                    startActivity(new Intent(RoomListActivity.this, KTVRoomActivity.class));
                                }
                            });

                        } else if (retcode == GoChatErrorFailNetwork) {
                            mProcessView.dismiss();
                            ToastUtil.showWarnToast(R.string.room_list_join_fail_network);
                        } else if (retcode == ERROR_USER_OFF_LINE) {
                            mProcessView.dismiss();
                            ToastUtil.showWarnToast("已掉线 请退出重新登陆");
                        } else {
                            mProcessView.dismiss();
                            ToastUtil.showWarnToast(String.format("加入房间失败（%s）", retcode));
                        }
                    }
                });
            }
        });
        mBottomSheetDialog.setContentView(view);
        final BottomSheetBehavior mDialogBehavior = BottomSheetBehavior.from((View) view.getParent());
        mDialogBehavior.addBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                    mBottomSheetDialog.dismiss();
                    mDialogBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {
            }
        });
        Window localWindow = mBottomSheetDialog.getWindow();
        localWindow.setGravity(Gravity.BOTTOM);
        localWindow.setWindowAnimations(R.style.BottomSheetDialogAnim);

        mBottomSheetDialog.show();

    }


    private void initUserData(IRTCCommonCallback callback){
        RTCSDKManager.getInstance().setUserID(mMyInfo.getUid(), mMyInfo.getNickName());
        ZGKTVCopyrightedMusicManager.getInstance().setUserID(mMyInfo.getUid(), mMyInfo.getNickName());
        ZGKTVCopyrightedMusicManager.getInstance().init(ZegoApplication.getAppContext(), new IRTCCommonCallback() {
            @Override
            public void onRTCCallback(int errorCode) {
                callback.onRTCCallback(errorCode);
            }
        });
    }

    @Override
    protected void onPause() {
        super.onPause();
        if(isFinishing()){
            LoginInfoManager.getInstance().logoutUser(null);
            ZGKTVPlayerManager.getInstance().uninit();
            ZGKTVCopyrightedMusicManager.getInstance().uninit();
            RTCSDKManager.getInstance().uninit();
        }
    }

    @Override
    protected void onDestroy() {
//        ScreenAdaptationUtil.unInit(this.getApplicationContext());
        if(!isFinishing()) {
            LoginInfoManager.getInstance().logoutUser(null);
            ZGKTVPlayerManager.getInstance().uninit();
            ZGKTVCopyrightedMusicManager.getInstance().uninit();
            RTCSDKManager.getInstance().uninit();
        }
        super.onDestroy();
    }

    /**
     * 麦克风权限
     */
    private void micPermission() {
        PermissionHelper.onAudioPermissionGranted(this, new PermissionHelper.GrantResult() {
            @Override
            public void onGrantResult(boolean allGranted) {
                if (allGranted) {
                    showBottomSheetDialog();
                } else {

                    MessageDialog messageDialog = new MessageDialog(RoomListActivity.this, R.layout.ktv_dialog_msg);
                    messageDialog.setMsgTitle("无法使用麦克风");
                    messageDialog.setMsgContent("请在系统设置中开启麦克风的使用权限");
                    messageDialog.setRightBtnContent("去设置");
                    messageDialog.setLeftBtnContent("取消");
                    messageDialog.setSureListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            //取消
                            messageDialog.dismiss();
                            Toast.makeText(RoomListActivity.this, "打开麦克风失败", Toast.LENGTH_SHORT).show();
                        }
                    });
                    messageDialog.setCancelListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            //去设置
                            messageDialog.dismiss();
                            Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
                                    .setData(Uri.fromParts("package", getPackageName(), null));
                            startActivityForResult(intent, 666);
                        }
                    });

                }
            }
        });
    }

    static public String getKTVRandomRoomName(int randomNum) {
        return String.format(Locale.ENGLISH, "在线KTV房间 %03d", randomNum);
    }

    static public String getKTVRandomHostName(int randomNum) {
        return String.format(Locale.ENGLISH, "房主 %03d", randomNum);
    }

    static public int getKTVRandomNum() {
        int randomNum = (int) (Math.random() * 1000);
        return randomNum;
    }
}
