package im.zego.ktv.chorus.roomlist.activity;

import static im.zego.ktv.ErrorCodeConstants.GoChatErrorFailNetwork;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import im.zego.common.application.ZegoApplication;
import im.zego.common.model.LoginInfo;
import im.zego.common.util.MiscUtils;
import im.zego.ktv.R;
import im.zego.ktv.chorus.ktvroom.KTVRoomManager;
import im.zego.ktv.chorus.login.LoginInfoManager;
import im.zego.ktv.chorus.model.RoomListInfo;
import im.zego.ktv.chorus.protocol.IGetRoomListCallback;
import im.zego.ktv.chorus.protocol.RoomAPI;
import im.zego.ktv.chorus.roomlist.adapter.EmptyAndLoadMoreAdapterWrapper;
import im.zego.ktv.chorus.roomlist.adapter.RoomListAdapter;
import im.zego.ktv.chorus.roomlist.view.LoadMoreView;
import im.zego.ktv.chorus.rtc.RTCSDKManager;
import im.zego.ktv.chorus.rtc.ZGKTVCopyrightedMusicManager;
import im.zego.ktv.chorus.rtc.callbacks.IRTCCommonCallback;
import im.zego.ktv.utils.GlideImageLoader;
import im.zego.ktv.utils.ToastUtil;
import im.zego.ktv.chorus.roomlist.view.GridLayoutItemDecoration;

public class RoomListFragment extends Fragment {

    private static final int STATE_NORMAL = 0;
    private static final int STATE_REFRESH = 1;
    private static final int STATE_LOADING = 2;
    private static final int STATE_FINISH = 3;
    private static final int STATE_ERROR = 4;

    private static final int BEGIN_TIME = 0;

    private static final int COUNT_SIZE = 10;
    private long mTimestamp = BEGIN_TIME;

    private SwipeRefreshLayout mSrlRoomList;
    private RecyclerView mRvRoomList;

    private RoomListAdapter mAdapter;
    private EmptyAndLoadMoreAdapterWrapper emptyAndLoadMoreAdapterWrapper;

    private LoadMoreView loadMoreView;

    private int mState;

    private View view;

    private LoginInfo mMyInfo;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRetainInstance(true);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        if (view == null) {
            view = inflater.inflate(R.layout.ktv_fragment_roomlist, null, false);
            initView(view);
            initRecycleView();
            initListener();
            initLoadData();
        }
        return view;
    }


    private void initView(View view) {
        mSrlRoomList = view.findViewById(R.id.srl_room_list);
        mRvRoomList = view.findViewById(R.id.rv_room_list);
    }

    private void initRecycleView() {

        GridLayoutManager layoutManager = new GridLayoutManager(getActivity(), 2);
        layoutManager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {

            @Override
            public int getSpanSize(int position) {
                if ((position == 0 && layoutManager.getItemCount() == 1) || layoutManager.getItemCount() - 1 == position) {
                    return layoutManager.getSpanCount();
                } else {
                    return 1;
                }
            }
        });
        mRvRoomList.setLayoutManager(layoutManager);

        mAdapter = new RoomListAdapter();

        View emptyView = LayoutInflater.from(getContext()).inflate(R.layout.ktv_empty_room_list, mSrlRoomList, false);

        loadMoreView = new LoadMoreView(getContext());

        emptyAndLoadMoreAdapterWrapper = new EmptyAndLoadMoreAdapterWrapper(mAdapter);
        emptyAndLoadMoreAdapterWrapper.setEmptyView(emptyView);
        emptyAndLoadMoreAdapterWrapper.setLoadMoreView(loadMoreView);
        mRvRoomList.addItemDecoration(new GridLayoutItemDecoration(getActivity(), MiscUtils.dp2px(getActivity(), 16),
                MiscUtils.dp2px(getActivity(), 18), true, false));
        mRvRoomList.setAdapter(emptyAndLoadMoreAdapterWrapper);
    }


    private void initListener() {
        mSrlRoomList.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                RoomListFragment.this.onRefresh();
            }
        });

        mAdapter.setOnClickListener(new RoomListAdapter.OnClickListener() {
            @Override
            public void onClick(View v, int position, RoomListInfo.RoomListDTO data) {
                if (!LoginInfoManager.getInstance().isLogin()) {
                    Toast.makeText(RoomListFragment.this.getContext(), "未登陆 请退出重新登陆", Toast.LENGTH_SHORT).show();
                } else {

                    String audienceName = getKTVRandomAudienceName();
                    String avatar = GlideImageLoader.getInstance().getAvatar();
                    mMyInfo = new LoginInfo();
                    mMyInfo.setNickName(audienceName);
                    mMyInfo.setAvatar(avatar);
                    mMyInfo.setUid(LoginInfoManager.getInstance().getLoginInfo().getUid());
                    LoginInfoManager.getInstance().setLoginInfo(mMyInfo);
                    initUserData(new IRTCCommonCallback() {
                        @Override
                        public void onRTCCallback(int errorCode) {
                            RTCSDKManager.getInstance().initQualityManager(getActivity());
                            RTCSDKManager.getInstance().setHost(data.getCreatorId());
                            KTVRoomManager.getInstance().loginRoom(getActivity(), mMyInfo.getUid(), data.getRoomId(), mMyInfo.getNickName(), mMyInfo.getAvatar());

                        }
                    });
                }
            }
        });

        emptyAndLoadMoreAdapterWrapper.setOnLoadMoreListener(new EmptyAndLoadMoreAdapterWrapper.OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                RoomListFragment.this.onLoadMore();
            }
        });
    }

    private void initUserData(IRTCCommonCallback callback) {
        RTCSDKManager.getInstance().setUserID(mMyInfo.getUid(), mMyInfo.getNickName());
        ZGKTVCopyrightedMusicManager.getInstance().setUserID(mMyInfo.getUid(), mMyInfo.getNickName());
        ZGKTVCopyrightedMusicManager.getInstance().init(ZegoApplication.getAppContext(), new IRTCCommonCallback() {
            @Override
            public void onRTCCallback(int errorCode) {
                callback.onRTCCallback(errorCode);
            }
        });
    }

    private void initLoadData() {
        mSrlRoomList.setRefreshing(true);
//        onRefresh();
    }

    private void onRefresh() {
        mState = STATE_REFRESH;

        mTimestamp = BEGIN_TIME;
        RoomAPI.getRoomList(LoginInfoManager.getInstance().getLoginInfo().getUid(), COUNT_SIZE, mTimestamp, new IGetRoomListCallback() {
            @Override
            public void onGetRoomList(int retcode, RoomListInfo roomInfoList) {
                mSrlRoomList.setRefreshing(false);
                if (retcode == 0) {
                    if (roomInfoList != null && roomInfoList.getRoomList() != null) {

                        List<RoomListInfo.RoomListDTO> list = roomInfoList.getRoomList();
                        if (list.size() < COUNT_SIZE) {
                            mState = STATE_FINISH;
                            loadMoreView.showFinish();
                        } else {
                            mState = STATE_NORMAL;
                        }

                        if (list.size() > 0) {
                            mTimestamp = list.get(list.size() - 1).getCreateTime()-1;
                        }

                        mAdapter.refreshData(roomInfoList.getRoomList());
                        emptyAndLoadMoreAdapterWrapper.notifyDataSetChanged();
                    } else {
                        mState = STATE_NORMAL;
                        mAdapter.refreshData(new ArrayList<RoomListInfo.RoomListDTO>());
                        emptyAndLoadMoreAdapterWrapper.notifyDataSetChanged();
                    }
                } else {
                    if (retcode == GoChatErrorFailNetwork) {
                        ToastUtil.showWarnToast(R.string.fail_network);

                    }
                    mState = STATE_ERROR;
                }
            }
        });
    }


    @Override
    public void onResume() {
        super.onResume();
        onRefresh();
    }

    /**
     * 上拉加载更多
     */
    private void onLoadMore() {
        if (mState == STATE_REFRESH) {
            return;
        }

        if (mState == STATE_NORMAL) {
            mState = STATE_LOADING;
            loadMoreView.showLoading();

            RoomAPI.getRoomList(LoginInfoManager.getInstance().getLoginInfo().getUid(), COUNT_SIZE, mTimestamp, new IGetRoomListCallback() {
                @Override
                public void onGetRoomList(int retcode, RoomListInfo roomInfoList) {
                    if (retcode == 0) {
                        if (roomInfoList != null && roomInfoList.getRoomList() != null) {

                            List<RoomListInfo.RoomListDTO> list = roomInfoList.getRoomList();
                            if (list.size() < COUNT_SIZE) {
                                mState = STATE_FINISH;
                                loadMoreView.showFinish();
                            } else {
                                mState = STATE_NORMAL;
                            }

                            if (list.size() > 0) {
                                mTimestamp = list.get(list.size() - 1).getCreateTime()-1;
                            }

                            mAdapter.addData(roomInfoList.getRoomList());
                            emptyAndLoadMoreAdapterWrapper.notifyItemRangeInserted(mAdapter.getItemCount() - roomInfoList.getRoomList().size(), roomInfoList.getRoomList().size());
                        } else {
                            mState = STATE_ERROR;
                            loadMoreView.showFinish();
                        }
                    } else {
                        if (retcode == GoChatErrorFailNetwork) {
                            ToastUtil.showWarnToast(R.string.fail_network);
                        }

                        mState = STATE_ERROR;
                        loadMoreView.showFinish();
                    }
                }
            });

        }
    }

    static public String getKTVRandomAudienceName() {
        int randomNum = (int) (Math.random() * 1000);
        return String.format(Locale.ENGLISH, "观众 %03d", randomNum);
    }

}
