package im.zego.ktv.chorus.roomlist.adapter;

import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

public class EmptyAndLoadMoreAdapterWrapper  extends RecyclerView.Adapter<RecyclerView.ViewHolder>{

    public static final int ITEM_TYPE_EMPTY = Integer.MAX_VALUE-1;
    public static final int ITEM_TYPE_LOAD_MORE = Integer.MAX_VALUE-2;

    private RecyclerView.Adapter<RecyclerView.ViewHolder> mAdapter;
    private View mEmptyView;
    private View mLoadMoreView;

    public EmptyAndLoadMoreAdapterWrapper(RecyclerView.Adapter adapter)
    {
        mAdapter = adapter;
    }

    private boolean isEmpty()
    {
        return mEmptyView != null && mAdapter.getItemCount() == 0;
    }

    private boolean isLoadMore(int position)
    {
        return mLoadMoreView != null && position >= mAdapter.getItemCount();
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        if (isEmpty())
        {
            return new EmptyViewHolder(mEmptyView);
        }

        if(viewType == ITEM_TYPE_LOAD_MORE)
        {
            return new LoadMoreViewHolder(mLoadMoreView);
        }

        return mAdapter.onCreateViewHolder(parent, viewType);
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        if (isEmpty())
        {
            return;
        }

        if (isLoadMore(position))
        {
            if (mOnLoadMoreListener != null)
            {
                mOnLoadMoreListener.onLoadMore();
            }
            return;
        }

        mAdapter.onBindViewHolder(holder, position);
    }

    @Override
    public int getItemCount() {
        if(isEmpty())
            return 1;
        return mAdapter.getItemCount() + 1;
    }

    @Override
    public int getItemViewType(int position)
    {
        if (isEmpty())
        {
            return ITEM_TYPE_EMPTY;
        }
        if (isLoadMore(position))
        {
            return ITEM_TYPE_LOAD_MORE;
        }
        return mAdapter.getItemViewType(position);
    }

    public void setEmptyView(View view)
    {
        mEmptyView = view;
    }

    public View getEmptyView()
    {
        return mEmptyView;
    }

    public void setLoadMoreView(View view)
    {
        mLoadMoreView = view;
    }

    public View getLoadMoreView()
    {
        return mLoadMoreView;
    }


    public static class EmptyViewHolder extends RecyclerView.ViewHolder{

        public EmptyViewHolder(@NonNull View itemView) {
            super(itemView);
        }
    }

    public static class LoadMoreViewHolder extends RecyclerView.ViewHolder{

        public LoadMoreViewHolder(@NonNull View itemView) {
            super(itemView);
        }
    }

    private OnLoadMoreListener mOnLoadMoreListener;
    public interface OnLoadMoreListener
    {
        void onLoadMore();
    }

    public void setOnLoadMoreListener(OnLoadMoreListener loadMoreListener)
    {
        mOnLoadMoreListener = loadMoreListener;
    }

}
