package im.zego.ktv.chorus.roomlist.adapter;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.List;

import im.zego.ktv.R;
import im.zego.ktv.chorus.model.RoomListInfo;
import im.zego.ktv.utils.GlideImageLoader;


public class RoomListAdapter extends RecyclerView.Adapter<RoomListAdapter.RoomListViewHolder> {

    private List<RoomListInfo.RoomListDTO> mData = new ArrayList<>();

    public RoomListAdapter() {
    }

    @NonNull
    @Override
    public RoomListViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.ktv_item_room_list, parent, false);
        return new RoomListViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final @NonNull RoomListViewHolder holder, final int position) {
        RoomListInfo.RoomListDTO info = mData.get(holder.getAdapterPosition());
        holder.mTvRoomName.setText(info.getSubject());
        holder.mTvPeopleNum.setText(String.valueOf(info.getOnlineCount()));
        holder.mTvMicNum.setText(String.valueOf(info.getStageCount()));
        holder.mIvBg.setBackgroundResource(GlideImageLoader.getInstance().getImageByAvatar(info.getCoverImg()));

        holder.mClRoomListItem.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mOnClickListener != null) {
                    mOnClickListener.onClick(v, holder.getAdapterPosition(), mData.get(holder.getAdapterPosition()));
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        if (mData == null) {
            return 0;
        }
        return mData.size();
    }

    class RoomListViewHolder extends RecyclerView.ViewHolder {

        public ConstraintLayout mClRoomListItem;
        public TextView mTvRoomName;
        public TextView mTvMicNum;
        public TextView mTvPeopleNum;
        private ImageView mIvBg;

        private RoomListViewHolder(View itemView) {
            super(itemView);
            mClRoomListItem = itemView.findViewById(R.id.cl_room_list_item);
            mTvRoomName = itemView.findViewById(R.id.tv_room_name);
            mTvMicNum = itemView.findViewById(R.id.tv_mic_num);
            mTvPeopleNum = itemView.findViewById(R.id.tv_people_num);
            mIvBg = itemView.findViewById(R.id.iv_bg);
        }
    }

    public void refreshData(List<RoomListInfo.RoomListDTO> newData) {
        mData = newData;
    }


    public void addData(List<RoomListInfo.RoomListDTO> newData) {
        mData.addAll(newData);

    }

    private OnClickListener mOnClickListener;

    public interface OnClickListener {
        void onClick(View v, int position, RoomListInfo.RoomListDTO data);
    }

    public void setOnClickListener(OnClickListener onClickListener) {
        mOnClickListener = onClickListener;
    }

}
