package im.zego.ktv.chorus.rtc;

import static im.zego.zegoexpress.constants.ZegoRoomState.CONNECTED;
import static im.zego.zegoexpress.constants.ZegoRoomState.DISCONNECTED;
import static im.zego.zegoexpress.constants.ZegoUpdateType.ADD;
import static im.zego.zegoexpress.constants.ZegoUpdateType.DELETE;

import android.app.Application;
import android.content.Context;
import android.graphics.Rect;
import android.os.Environment;

import org.jetbrains.annotations.NotNull;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import im.zego.common.proxy.QualityManagerProxy;
import im.zego.common.util.L;
import im.zego.common.util.SystemUtil;
import im.zego.common.util.T;
import im.zego.ktv.AuthConstants;
import im.zego.ktv.ErrorCodeConstants;
import im.zego.ktv.chorus.base.Constants;
import im.zego.ktv.chorus.login.LoginInfoManager;
import im.zego.ktv.chorus.model.HostSEIInfo;
import im.zego.ktv.chorus.model.notify.NotifyCustomBroadcastInfo;
import im.zego.ktv.chorus.model.notify.NotifyOrderListUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyPreOrderUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyRoomStateUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyUserListUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyUserStateUpdateInfo;
import im.zego.ktv.chorus.protocol.APIBase;
import im.zego.ktv.chorus.rtc.callbacks.IMediaPlayerEventCallback;
import im.zego.ktv.chorus.rtc.callbacks.IRTCCommonCallback;
import im.zego.ktv.chorus.rtc.callbacks.IRTCEventCallback;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.callback.IZegoDestroyCompletionCallback;
import im.zego.zegoexpress.callback.IZegoEventHandler;
import im.zego.zegoexpress.callback.IZegoMixerStartCallback;
import im.zego.zegoexpress.constants.ZegoAECMode;
import im.zego.zegoexpress.constants.ZegoANSMode;
import im.zego.zegoexpress.constants.ZegoAudioChannel;
import im.zego.zegoexpress.constants.ZegoAudioCodecID;
import im.zego.zegoexpress.constants.ZegoAudioConfigPreset;
import im.zego.zegoexpress.constants.ZegoAudioRoute;
import im.zego.zegoexpress.constants.ZegoAudioSourceType;
import im.zego.zegoexpress.constants.ZegoLanguage;
import im.zego.zegoexpress.constants.ZegoMixerInputContentType;
import im.zego.zegoexpress.constants.ZegoNetworkMode;
import im.zego.zegoexpress.constants.ZegoNetworkSpeedTestType;
import im.zego.zegoexpress.constants.ZegoPlayerState;
import im.zego.zegoexpress.constants.ZegoPublishChannel;
import im.zego.zegoexpress.constants.ZegoPublisherState;
import im.zego.zegoexpress.constants.ZegoReverbPreset;
import im.zego.zegoexpress.constants.ZegoRoomState;
import im.zego.zegoexpress.constants.ZegoScenario;
import im.zego.zegoexpress.constants.ZegoStreamAlignmentMode;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoAudioConfig;
import im.zego.zegoexpress.entity.ZegoBroadcastMessageInfo;
import im.zego.zegoexpress.entity.ZegoCustomAudioConfig;
import im.zego.zegoexpress.entity.ZegoEngineConfig;
import im.zego.zegoexpress.entity.ZegoEngineProfile;
import im.zego.zegoexpress.entity.ZegoMixerAudioConfig;
import im.zego.zegoexpress.entity.ZegoMixerInput;
import im.zego.zegoexpress.entity.ZegoMixerOutput;
import im.zego.zegoexpress.entity.ZegoMixerTask;
import im.zego.zegoexpress.entity.ZegoNetworkSpeedTestConfig;
import im.zego.zegoexpress.entity.ZegoNetworkSpeedTestQuality;
import im.zego.zegoexpress.entity.ZegoNetworkTimeInfo;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublisherConfig;
import im.zego.zegoexpress.entity.ZegoRoomConfig;
import im.zego.zegoexpress.entity.ZegoStream;
import im.zego.zegoexpress.entity.ZegoUser;

public class RTCSDKManager {
    private static final String TAG = "RTCSDKManager";

    private int mHeadphoneMonitorVolumeNum = 50;
    private int mCaptureVolumeNum = 100;
    private boolean mLoopbackSwitch = false;
    private int mVerbPresetPos = 3;


    private RTCSDKManager() {
    }


    public void initAudioProcessParams() {
        setCaptureVolume(100);  //设置推流端采集音量，范围为 0 ~ 200，默认值为 100
        setLoopBackVolume(50);
        enableHeadphoneMonitor(false); //耳返开关默认关闭
        mSDKEngine.enableAEC(true);  //是否开启回声消除；true 表示开启；false 表示关闭
        mSDKEngine.enableAGC(false);  //是否开启自动增益控制；true 表示开启；false 表示关闭
        mSDKEngine.enableANS(true);  //是否开启噪声抑制；true 表示开启噪声抑制；false 表示关闭噪声抑制
        mSDKEngine.setAECMode(ZegoAECMode.MEDIUM); //设置回声消除模式
        mSDKEngine.setANSMode(ZegoANSMode.MEDIUM); //设置音频噪声抑制模式
        mSDKEngine.enableHeadphoneMonitor(false);  //开启耳返， false: 关闭耳返
        mSDKEngine.setReverbPreset(ZegoReverbPreset.RECORDING_STUDIO); //通过预设枚举设置混响  RECORDING_STUDIO - 录音棚混响效果
    }

    //是否开启回声消除；true 表示开启；false 表示关闭
    public void enableAEC(boolean enable) {
        L.i(TAG, "enableAEC :" + enable);
        mSDKEngine.enableAEC(enable);
    }

    //是否开启噪声抑制；true 表示开启噪声抑制；false 表示关闭噪声抑制
    public void enableANS(boolean enable) {
        L.i(TAG, "enableANS :" + enable);
        mSDKEngine.enableANS(enable);
    }

    //是否开启自动增益控制；true 表示开启；false 表示关闭
    public void enableAGC(boolean enable) {
        L.i(TAG, "enableAGC :" + enable);
        mSDKEngine.enableAGC(enable);
    }

    //开启耳返， false: 关闭耳返
    public void enableHeadphoneMonitor(boolean enable) {
        L.i(TAG, "enableHeadphoneMonitor :" + enable);
        this.mLoopbackSwitch = enable;
        mSDKEngine.enableHeadphoneMonitor(enable);
    }

    //设置混响
    public void setVerbPresetMode(ZegoReverbPreset mode, int position) {
        L.i(TAG, "setVerbPresetMode :" + mode);
        this.mVerbPresetPos = position;
        mSDKEngine.setReverbPreset(mode);
    }

    //开启网络测速
    public void startNetworkMonitor() {
        ZegoNetworkSpeedTestConfig zegoNetworkSpeedTestConfig = new ZegoNetworkSpeedTestConfig();
        zegoNetworkSpeedTestConfig.testUplink = true;
        zegoNetworkSpeedTestConfig.expectedUplinkBitrate = 128;
        mSDKEngine.startNetworkSpeedTest(zegoNetworkSpeedTestConfig);
//        mSDKEngine.startPerformanceMonitor(1000);
    }

    //停止网络测速
    public void stopNetworkMonitor() {
        mSDKEngine.stopNetworkSpeedTest();
    }

    private static final class Holder {
        private static final RTCSDKManager INSTANCE = new RTCSDKManager();
    }

    public static RTCSDKManager getInstance() {
        return Holder.INSTANCE;
    }

    private ZegoExpressEngine mSDKEngine = null;
    private IRTCEventCallback mEventCallbackList;
    private ZegoUser mUserInfo = null;
    private long mHostID;
    private String mRoomID = null;
    private IMediaPlayerEventCallback mMediaPlayerEventCallback;
    private ZegoPublisherState mMyPublisherState = ZegoPublisherState.NO_PUBLISH;
    private int mRetryTime = 3;
    public static final int MAX_USER_COUNT = 10000000;

    private IRTCCommonCallback mLoginCallback = null;


    //台上用户和对应的流
    private final HashMap<Long, String> mUserStreamIDMap = new HashMap<>();
    //声浪集合
    private final HashMap<String, Float> mUserVolumeMap = new HashMap<>();
    //是否首次登录
    private boolean mHasFirstLoginRoom = false;

    public void addListener(IRTCEventCallback callback) {
        mEventCallbackList = callback;
    }

    //重新开始推伴奏流
    public void restartMvStream() {
        L.i(TAG, "restartMvStream");
        if (mHostID == Long.parseLong(mUserInfo.userID)) {

            mSDKEngine.stopPublishingStream(ZegoPublishChannel.AUX);
            QualityManagerProxy.getInstance().setPublishStreamID(getMVStreamId());
//            mSDKEngine.startPublishingStream(getMVStreamId(), ZegoPublishChannel.AUX);
            mSDKEngine.setStreamAlignmentProperty(1, ZegoPublishChannel.AUX);
            ZegoPublisherConfig config = new ZegoPublisherConfig();
            config.forceSynchronousNetworkTime = 1;
            config.roomID = mRoomID;
            mSDKEngine.startPublishingStream(getMVStreamId(), config, ZegoPublishChannel.AUX);
            updateMixStream();
        }
    }

    //初始化引擎
    public void init(@NotNull Context application) {
        if (mSDKEngine != null) {
            mSDKEngine.startPerformanceMonitor(3000);
            return;
        }
        preEngineConfig(application);

        ZegoEngineProfile profile = new ZegoEngineProfile();
        profile.appID = AuthConstants.APP_ID;
        profile.appSign = AuthConstants.APP_SIGN;
        profile.scenario = ZegoScenario.GENERAL;
        profile.application = (Application) application.getApplicationContext();

        mSDKEngine = ZegoExpressEngine.createEngine(profile, mEventHandler);


//        mSDKEngine = ZegoExpressEngine.createEngine(AuthConstants.APP_ID, AuthConstants.APP_SIGN, false, ZegoScenario.GENERAL, (Application) application.getApplicationContext(), mEventHandler);
        if(mSDKEngine != null) {
            engineConfig();
            ZGKTVCopyrightedMusicManager.getInstance().init(application, null);
        }else {
            T.show("引擎创建失败，请检查appID ，appSign是否存在问题");
        }
    }

    //引擎配置
    private void engineConfig() {
        //不需要视频功能 关闭视频功能
        mSDKEngine.startPerformanceMonitor(3000);
        mSDKEngine.enableCamera(false);
        mSDKEngine.enableCamera(false, ZegoPublishChannel.AUX);

        //第二路流配置媒体播放器
        ZegoCustomAudioConfig zegoCustomAudioConfig = new ZegoCustomAudioConfig();
        zegoCustomAudioConfig.sourceType = ZegoAudioSourceType.MEDIA_PLAYER;
        mSDKEngine.enableCustomAudioIO(true, zegoCustomAudioConfig, ZegoPublishChannel.AUX);
        mSDKEngine.setDebugVerbose(true, ZegoLanguage.CHINESE);
        ZegoAudioConfig audioConfig = new ZegoAudioConfig(ZegoAudioConfigPreset.HIGH_QUALITY);
        audioConfig.codecID = ZegoAudioCodecID.LOW3;
        mSDKEngine.setAudioConfig(audioConfig, ZegoPublishChannel.MAIN);
        mSDKEngine.setAudioConfig(audioConfig, ZegoPublishChannel.AUX);

    }

    //初始化引擎前配置
    private void preEngineConfig(Context application) {
        //配置日志路径
        ZegoEngineConfig engineConfig = new ZegoEngineConfig();
        if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
            if (application.getExternalFilesDir(null) != null) {
                engineConfig.logConfig.logPath = application.getExternalFilesDir(null).getAbsolutePath() + File.separator + "zegologs";
            } else {
                engineConfig.logConfig.logPath = application.getFilesDir().getAbsolutePath() + File.separator + "zegologs";
            }
        }
        //配置
        engineConfig.advancedConfig.put("prefer_play_ultra_source", "1");
        //开启超低延迟模式
        engineConfig.advancedConfig.put("ultra_low_latency", "true");
        //开启超低采集和渲染耗时模式
        engineConfig.advancedConfig.put("enforce_audio_loopback_in_sync", "true");
        //关闭高通滤波，保证高音质
        engineConfig.advancedConfig.put("prep_high_pass_filter", "false");
//        engineConfig.advancedConfig.put("publish_stream_alignment", "true");变为方法方式设置

        engineConfig.advancedConfig.put("allow_verbose_print_high_frequency_content", "true");
        engineConfig.advancedConfig.put("enable_callback_verbose", "true");

        //5.25后SDK版本不需要设置这个
//        engineConfig.advancedConfig.put("mix_stream_alignment", "true");
        //推流质量回调周期
        engineConfig.advancedConfig.put("publish_quality_interval", "500");
        //拉流质量回调周期
        engineConfig.advancedConfig.put("play_quality_interval", "500");
        ZegoExpressEngine.setEngineConfig(engineConfig);
    }

    //反初始化
    public void uninit() {
        if (mSDKEngine == null) {
            return;
        }
        mEventCallbackList = null;
        mMediaPlayerEventCallback = null;

//        mSDKEngine.startPerformanceMonitor(3000);
        mSDKEngine.setEventHandler(null);
        ZegoExpressEngine.destroyEngine(new IZegoDestroyCompletionCallback() {
            @Override
            public void onDestroyCompletion() {

            }
        });
        mSDKEngine = null;
    }

    //设置用户信息，需在登录前设置
    public void setUserID(long userID, @NotNull String userName) {
        mUserInfo = new ZegoUser(String.valueOf(userID), userName);
    }

    //设置房主信息
    public void setHost(long userID) {
        mHostID = userID;
    }

    //获得房主信息
    public long getHostID() {
        return mHostID;
    }

    //登录房间
    public void loginRoom(@NotNull String roomID, IRTCCommonCallback callback) {
        if (mSDKEngine == null) {
            return;
        }
        mSDKEngine.setEventHandler(null);
        mSDKEngine.setEventHandler(mEventHandler);
        mSDKEngine.muteMicrophone(true);
        initAudioProcessParams();
        mRetryTime = Constants.LOGIN_MAX_TRY_TIME;
        if (mSDKEngine == null) {
            callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_NOT_INITIALIZED);
            return;
        } else if (mUserInfo == null) {
            callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_INVALID_USERID);
            return;
        }

        mUserStreamIDMap.clear();
        mRoomID = roomID;
        QualityManagerProxy.getInstance().setRoomID(roomID);
        QualityManagerProxy.getInstance().setUserID(mUserInfo.userID);
        QualityManagerProxy.getInstance().setUserName(mUserInfo.userName);
        QualityManagerProxy.getInstance().setProductName("goenjoy");
        String language = SystemUtil.getSystemLanguage();
        L.i(TAG, "系统语言：" + language);
        if (language.equals("en")) {
            QualityManagerProxy.getInstance().setLanguageType(false);
        } else {
            QualityManagerProxy.getInstance().setLanguageType(true);
        }
        QualityManagerProxy.getInstance().setLoginOnStart();

        mHasFirstLoginRoom = false;
        mLoginCallback = callback;

        ZegoRoomConfig roomConfig = new ZegoRoomConfig();
        roomConfig.maxMemberCount = MAX_USER_COUNT;
        roomConfig.isUserStatusNotify = true;
        mSDKEngine.logoutRoom();
        mSDKEngine.loginRoom(roomID, mUserInfo, roomConfig);
//        if (callback != null) {
//            callback.onRTCCallback(0);
//        }

        mSDKEngine.startSoundLevelMonitor(500);
    }

    //退出房间
    public void logoutRoom() {
        if (mSDKEngine == null) {
            return;
        }
        stopNetworkMonitor();

        if (mRoomID != null) {
            mSDKEngine.logoutRoom(mRoomID);
//            mSDKEngine.setEventHandler(null);
            mRoomID = null;
            mUserStreamIDMap.clear();
            mHasFirstLoginRoom = false;
        }
        if (mSDKEngine != null) {
            mSDKEngine.stopSoundLevelMonitor();
        }
    }

    //开始推音频流
    public void startPublishAudio(@NotNull IRTCCommonCallback callback) {
        L.i(TAG, "startPublishAudio");
        stopNetworkMonitor();
        if (mSDKEngine == null) {
            L.e(TAG, "ERROR_RTC_SDK_NOT_INITIALIZED MAIN");
            callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_NOT_INITIALIZED);
            return;
        } else if (mUserInfo == null) {
            L.e(TAG, "ERROR_RTC_SDK_INVALID_USERID MAIN");
            callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_INVALID_USERID);
            return;
        } else if (mRoomID == null) {
            L.e(TAG, "ERROR_RTC_SDK_NOT_IN_ROOM MAIN");
            callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_NOT_IN_ROOM);
            return;
        }
        mSDKEngine.setStreamAlignmentProperty(1, ZegoPublishChannel.MAIN);

        QualityManagerProxy.getInstance().setPublishStreamID(getMainStreamId());
        ZegoPublisherConfig config = new ZegoPublisherConfig();
        config.forceSynchronousNetworkTime = 1;
        config.roomID = mRoomID;
        L.i(TAG, "startPublishAudio MAIN");
        mSDKEngine.startPublishingStream(getMainStreamId(), config, ZegoPublishChannel.MAIN);
        if (callback != null) {
            callback.onRTCCallback(0);
        }
    }

    //开始推伴奏流
    public void startPublishMV(@NotNull IRTCCommonCallback callback) {
        L.i(TAG, "startPublishMV");
        if (mSDKEngine == null) {
            L.e(TAG, "ERROR_RTC_SDK_NOT_INITIALIZED AUX");
            callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_NOT_INITIALIZED);
            return;
        } else if (mUserInfo == null) {
            L.e(TAG, "ERROR_RTC_SDK_INVALID_USERID AUX");
            callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_INVALID_USERID);
            return;
        } else if (mRoomID == null) {
            L.e(TAG, "ERROR_RTC_SDK_NOT_IN_ROOM AUX");
            callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_NOT_IN_ROOM);
            return;
        }
        mSDKEngine.setStreamAlignmentProperty(1, ZegoPublishChannel.AUX);

        QualityManagerProxy.getInstance().setPublishStreamID(getMVStreamId());
        ZegoPublisherConfig config = new ZegoPublisherConfig();
        config.forceSynchronousNetworkTime = 1;
        config.roomID = mRoomID;
        L.i(TAG, "startPublishAudio AUX");
        mSDKEngine.startPublishingStream(getMVStreamId(), config, ZegoPublishChannel.AUX);
        if (callback != null) {
            callback.onRTCCallback(0);
        }
    }

    //获得伴奏流的id
    public String getMVStreamId() {
        return String.format("%s%s", mUserInfo.userID, Constants.MV_STREAM_SURFIX);
    }

    //获得台上推流的id
    public String getMainStreamId() {
        return mUserInfo.userID;
    }

    //更新混流
    private void updateMixStream() {
        L.i(TAG, "updateMixStream");
        ZegoMixerTask mixerTask = new ZegoMixerTask(getMixStreamID());
        mixerTask.enableSoundLevel(true);

        //变为 setStreamAlignmentMode 方式实现
//        HashMap<String, String> advancedConfig = new HashMap<>();
//        advancedConfig.put("is_stream_alignment", "true");
//        mixerTask.setAdvancedConfig(advancedConfig);

        mixerTask.setStreamAlignmentMode(ZegoStreamAlignmentMode.TRY);


        ZegoMixerAudioConfig audioConfig = new ZegoMixerAudioConfig();
        audioConfig.bitrate = 128;
        audioConfig.codecID = ZegoAudioCodecID.LOW3;
        audioConfig.channel = ZegoAudioChannel.STEREO;
        mixerTask.setAudioConfig(audioConfig);

        ArrayList<ZegoMixerInput> inputs = new ArrayList<>();
        int length = 0;
        Rect audioRect = new Rect(0, 0, 0, 0);
        ZegoMixerInput publisherInput = new ZegoMixerInput(getMainStreamId(), ZegoMixerInputContentType.AUDIO, audioRect);
        length = getMainStreamId().length();
        if (length >= Constants.SOUND_ID_SURFIX_NUM) {
            publisherInput.soundLevelID = Integer.parseInt(getMainStreamId().substring(length - Constants.SOUND_ID_SURFIX_NUM, length));
        }
        inputs.add(publisherInput);

        ZegoMixerInput publisherMusicInput = new ZegoMixerInput(getMVStreamId(), ZegoMixerInputContentType.AUDIO, audioRect);
        inputs.add(publisherMusicInput);

        ZegoMixerInput playStreamInput;
        for (String playStreamID : mUserStreamIDMap.values()) {
            playStreamInput = new ZegoMixerInput(playStreamID, ZegoMixerInputContentType.AUDIO, audioRect);
            length = playStreamID.length();
            if (length >= Constants.SOUND_ID_SURFIX_NUM) {
                playStreamInput.soundLevelID = Integer.parseInt(playStreamID.substring(length - Constants.SOUND_ID_SURFIX_NUM, length));
            }
            inputs.add(playStreamInput);
        }

        mixerTask.setInputList(inputs);

        ZegoMixerOutput output = new ZegoMixerOutput(getMixStreamID());
        ArrayList<ZegoMixerOutput> outputs = new ArrayList<>();
        outputs.add(output);
        mixerTask.setOutputList(outputs);

        mSDKEngine.startMixerTask(mixerTask, new IZegoMixerStartCallback() {
            @Override
            public void onMixerStartResult(int resultCode, JSONObject jsonObject) {
                if (resultCode != 0) {

                }
            }
        });
    }

    //获得混流的id
    public String getMixStreamID() {
        return mHostID + Constants.MIX_STREAM_SURFIX;
    }

    //停止推流
    public void stopPublishAudio(IRTCCommonCallback callback) {
        L.i(TAG, "stopPublishAudio");
        if (mSDKEngine == null) {
            if (callback != null) {
                callback.onRTCCallback(ErrorCodeConstants.ERROR_RTC_SDK_NOT_INITIALIZED);
            }
            return;
        }

        mSDKEngine.stopPublishingStream();
        if (callback != null) {
            callback.onRTCCallback(0);
        }
    }

    //获得ntp时间
    public long getNetworkTime() {
        ZegoNetworkTimeInfo zegoNetworkTimeInfo = mSDKEngine.getNetworkTimeInfo();
        if (zegoNetworkTimeInfo != null) {
//            L.i(TAG, "getNetworkTime:" + zegoNetworkTimeInfo.timestamp);
            return zegoNetworkTimeInfo.timestamp;
        }
        return 0;
    }

    //停止拉混流
    public void stopPlayingAux() {
        L.i(TAG, "stopPlayingAux");
        mSDKEngine.stopPlayingStream(getMixStreamID());
    }

    //开始拉混流
    public void startPlayingAux() {
        //拉混流
        //设置拉流jitter buffer大小
        L.i(TAG, "startPlayingAux");
        mSDKEngine.setPlayStreamBufferIntervalRange(getMixStreamID(), -1, -1);
        QualityManagerProxy.getInstance().setPlayerStreamOnStart(getMixStreamID());
        mSDKEngine.startPlayingStream(getMixStreamID(), null);
    }

    //设置耳返音量
    public void setLoopBackVolume(int volume) {
        this.mHeadphoneMonitorVolumeNum = volume;
        mSDKEngine.setHeadphoneMonitorVolume(volume * 2);
    }

    //设置推流端采集音量
    public void setCaptureVolume(int volume) {
        this.mCaptureVolumeNum = volume;
        mSDKEngine.setCaptureVolume(volume);
    }

    //是否静音（关闭麦克风）[true] 表示静音（关闭麦克风）；[false] 表示开启麦克风。
    public void muteMicphone(boolean isMuted) {
        mSDKEngine.muteMicrophone(isMuted);
    }

    private final IZegoEventHandler mEventHandler = new IZegoEventHandler() {

        @Override
        public void onDebugError(int errorCode, String funcName, String info) {
            super.onDebugError(errorCode, funcName, info);

            if(errorCode == 1017041){
                T.show(info);
            }

            L.e(TAG, "onDebugError-:errorCode:" + errorCode + ":funcName:" + funcName);
            L.e(TAG, "onDebugError-:errorCode:" + errorCode + ":info:" + info);
        }

        @Override
        public void onPublisherStateUpdate(String streamID, ZegoPublisherState state, int errorCode, JSONObject extendedData) {
            L.e(TAG,"onPublisherStateUpdate: streamID :" + streamID +": state :" + state + ": errorCode :" + errorCode);
            mMyPublisherState = state;
            if (state == ZegoPublisherState.PUBLISHING) {
                if (mHostID == Long.parseLong(mUserInfo.userID)) {
                    updateMixStream();
                }
            }
        }

        @Override
        public void onPlayerStateUpdate(String streamID, ZegoPlayerState state, int errorCode, JSONObject extendedData) {
            L.e(TAG,"onPlayerStateUpdate: streamID :" + streamID +": state :" + state + ": errorCode :" + errorCode);
//            if (streamID.equals(getMixStreamID())) {
//                startNetworkMonitor();
//            }
        }

        @Override
        public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData) {
            if (mSDKEngine == null) {
                // ERROR_RTC_SDK_NOT_INITIALIZED
                return;
            } else if (mRoomID == null) {
                // ERROR_RTC_SDK_NOT_IN_ROOM
                return;
            }
            if (mEventCallbackList != null) {
                mEventCallbackList.onRoomStreamUpdate(roomID, updateType, streamList, extendedData);
            }

            for (ZegoStream stream : streamList) {
                L.e(TAG,"onRoomStreamUpdate:" + " : updateType : " + updateType + " : streamID : " + stream.streamID);
                if (updateType == ADD) {
                    if (!stream.streamID.contains("_")) {
                        if (mMyPublisherState != ZegoPublisherState.NO_PUBLISH) {
                            mSDKEngine.setPlayStreamBufferIntervalRange(stream.streamID, 0, 30);
                            QualityManagerProxy.getInstance().setPlayerStreamOnStart(stream.streamID);
                            mSDKEngine.startPlayingStream(stream.streamID, null);
                        }
                        mUserStreamIDMap.put(Long.parseLong(stream.user.userID), stream.streamID);
                    }
                } else if (updateType == DELETE) {

                    if (!stream.streamID.contains("_")) {
                        mSDKEngine.stopPlayingStream(stream.streamID);
                        mUserStreamIDMap.remove(Long.parseLong(stream.user.userID));
                    }
                }
            }
            if (mHostID == Long.parseLong(mUserInfo.userID)) {
                updateMixStream();
            }
        }

        @Override
        public void onPublisherQualityUpdate(String streamID, ZegoPublishStreamQuality quality) {
            //推流质量回调
            if (mEventCallbackList != null) {
                ZegoNetworkTimeInfo zegoNetworkTimeInfo = mSDKEngine.getNetworkTimeInfo();
                mEventCallbackList.onPublisherQualityUpdate(streamID, quality, zegoNetworkTimeInfo);
            }
        }

        @Override
        public void onIMRecvCustomCommand(String roomID, ZegoUser fromUser, String command) {
            super.onIMRecvCustomCommand(roomID, fromUser, command);
            try {
                JSONObject jsonObject = new JSONObject(command);
                int cmd = jsonObject.getInt("cmd");
                L.i(TAG, "onIMRecvCustomCommand :" + command);
                if (cmd == 6010) {
                    NotifyPreOrderUpdateInfo notifyPreOrderUpdateInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyPreOrderUpdateInfo.class);
                    if (mEventCallbackList != null) {
                        mEventCallbackList.onPreOrderUpdateInfo(notifyPreOrderUpdateInfo);
                    }
                }
            } catch (JSONException jsonException) {
                jsonException.printStackTrace();
            }
        }

        @Override
        public void onIMRecvBroadcastMessage(String roomID, ArrayList<ZegoBroadcastMessageInfo> messageList) {
            super.onIMRecvBroadcastMessage(roomID, messageList);


            if (messageList == null || messageList.size() == 0) {
                return;
            }

            L.i(TAG, "onIMRecvBroadcastMessage:" + messageList.get(0).message);

            for (ZegoBroadcastMessageInfo extraInfo : messageList) {
                try {
                    JSONObject jsonObject = new JSONObject(extraInfo.message);
                    int cmd = jsonObject.getInt("cmd");
                    L.i(TAG, String.format("cmd:%d loginUserID:%d", cmd, LoginInfoManager.getInstance().getLoginInfo().getUid()));
                    L.i(TAG, "data:" + jsonObject.getString("data"));
                    if (cmd == 6002) {  // 成员状态变更通知
                        NotifyUserStateUpdateInfo notifyInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyUserStateUpdateInfo.class);
                        L.d(TAG, "type:" + notifyInfo.type);  // 1、举手操作 2、角色变更 3、麦克风操作

                        if (mEventCallbackList != null) {
                            mEventCallbackList.onUserStateUpdateNotify(notifyInfo);
                        }
                    } else if (cmd == 6003) { // 成员列表变更通知
                        NotifyUserListUpdateInfo notifyInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyUserListUpdateInfo.class);
                        if (mEventCallbackList != null) {
                            mEventCallbackList.onUserListUpdateNotify(notifyInfo);
                        }
                    } else if (cmd == 6006) { //房间关闭
                        if (mEventCallbackList != null) {
                            mEventCallbackList.onRoomCloseNotify();
                        }
                    } else if (cmd == 6007) { //房间状态变更
                        NotifyRoomStateUpdateInfo notifyInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyRoomStateUpdateInfo.class);
                        if (mEventCallbackList != null) {
                            mEventCallbackList.onRoomStateNotify(notifyInfo);
                        }
                    } else if (cmd == 6008) { //终端透传广播消息，通知歌曲在某个ntp时间点开始播放
                        NotifyCustomBroadcastInfo notifyCustomBroadcastInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyCustomBroadcastInfo.class);
                        if (mEventCallbackList != null) {
                            mEventCallbackList.onCustomBroadCastUpdate(notifyCustomBroadcastInfo);
                        }
                    } else if (cmd == 6009) { //点歌列表变更通知
                        NotifyOrderListUpdateInfo notifyOrderListUpdateInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyOrderListUpdateInfo.class);
                        if (mEventCallbackList != null) {
                            mEventCallbackList.onOrderListUpdate(notifyOrderListUpdateInfo);
                        }
                    } else if (cmd == 6010) { //预点歌通知
                        NotifyPreOrderUpdateInfo notifyPreOrderUpdateInfo = APIBase.getGson().fromJson(jsonObject.getString("data"), NotifyPreOrderUpdateInfo.class);
                        if (mEventCallbackList != null) {
                            mEventCallbackList.onPreOrderUpdateInfo(notifyPreOrderUpdateInfo);
                        }
                    }
                } catch (JSONException jsonException) {
                    jsonException.printStackTrace();
                }
            }
        }

        /**
         * 本地采集音频声浪回调
         *
         * 回调通知周期为 100 ms。
         * @param soundLevel 本地采集的声浪值，取值范围为 0.0 ~ 100.0
         */
        @Override
        public void onCapturedSoundLevelUpdate(float soundLevel) {
//            L.i(TAG,"onCapturedSoundLevelUpdate soundLevel:" + soundLevel);
            if (mSDKEngine.isMicrophoneMuted()) {
                return;
            }
            mUserVolumeMap.clear();
            if (mUserInfo.userID == null) {
                return;
            }
            int length = mUserInfo.userID.length();
            String key = String.valueOf(mUserInfo.userID);
            if (length >= Constants.SOUND_ID_SURFIX_NUM) {
                key = (key.substring(length - Constants.SOUND_ID_SURFIX_NUM, length));
            }
            mUserVolumeMap.put(key, soundLevel);
//            mUserVolumeMap.put(mUserInfo.userID, soundLevel);
            if (mEventCallbackList != null) {
                mEventCallbackList.onUserVolumeUpdate(mUserVolumeMap);
            }
        }

        @Override
        public void onMixerSoundLevelUpdate(HashMap<Integer, Float> soundLevels) {
            mUserVolumeMap.clear();
            // 根据流id 更新对应的user的音量
            for (Map.Entry<Integer, Float> pair : soundLevels.entrySet()) {
//                L.i(TAG,"onMixerSoundLevelUpdate key : " + pair.getKey() + ": value :" + pair.getValue());
                mUserVolumeMap.put(pair.getKey().toString(), pair.getValue());
            }

            if (mEventCallbackList != null) {
                mEventCallbackList.onUserVolumeUpdate(mUserVolumeMap);
            }
        }

        @Override
        public void onPlayerRecvSEI(String streamID, byte[] data) {
            try {
                L.i("SEI", "onPlayerRecvSEI SEI:" + new String(data));
                HostSEIInfo hostSEIInfo = APIBase.getGson().fromJson(new String(data), HostSEIInfo.class);
                if (mEventCallbackList != null) {
                    mEventCallbackList.onPlayerRecvSEI(streamID, hostSEIInfo);
                }
            } catch (Exception e) {

            }

        }

        @Override
        public void onPlayerQualityUpdate(String streamID, ZegoPlayStreamQuality quality) {
            //拉流质量回调
            if (mEventCallbackList != null) {
                mEventCallbackList.onPlayerQualityUpdate(streamID, quality);
            }
        }

        /**
         * 远端拉流音频声浪回调
         *
         * 回调通知周期为 100 ms。
         * @param soundLevels 远端的声浪键值对，key 为流 ID，value 为对应的流的声浪值，value 取值范围为 0.0 ~ 100.0
         */
        @Override
        public void onRemoteSoundLevelUpdate(HashMap<String, Float> soundLevels) {
            mUserVolumeMap.clear();
            // 根据流id 更新对应的user的音量
            for (Map.Entry<String, Float> pair : soundLevels.entrySet()) {
                if (!pair.getKey().endsWith(Constants.MIX_STREAM_SURFIX)) {
                    int length = pair.getKey().length();
                    String key = pair.getKey();
                    if (length >= Constants.SOUND_ID_SURFIX_NUM) {
                        key = (key.substring(length - Constants.SOUND_ID_SURFIX_NUM, length));
                    }
//                    L.i(TAG,"onRemoteSoundLevelUpdate key : " + key + ": value :" + pair.getValue());
                    mUserVolumeMap.put(key, pair.getValue());

                }
            }

            if (mEventCallbackList != null) {
                mEventCallbackList.onUserVolumeUpdate(mUserVolumeMap);
            }
        }

        /**
         * 音频设备路由变更通知
         * @param audioRoute
         */
        @Override
        public void onAudioRouteChange(ZegoAudioRoute audioRoute) {
            L.i(TAG, "onAudioRouteChange: " + audioRoute);
            if (mEventCallbackList != null) {
                mEventCallbackList.onAudioRouteChange(audioRoute);
            }
        }

        @Override
        public void onNetworkSpeedTestQualityUpdate(ZegoNetworkSpeedTestQuality onNetworkSpeedTestQualityUpdate, ZegoNetworkSpeedTestType type) {
            //网络测速质量回调通知
            if (mEventCallbackList != null) {
                mEventCallbackList.onNetworkSpeedTestQualityUpdate(onNetworkSpeedTestQualityUpdate);
            }
        }

        @Override
        public void onNetworkSpeedTestError(int errorCode, ZegoNetworkSpeedTestType type) {

        }

        @Override
        public void onNetworkModeChanged(ZegoNetworkMode mode) {
            super.onNetworkModeChanged(mode);
        }

        @Override
        public void onRoomStateUpdate(String roomID, ZegoRoomState state, int errorCode, JSONObject extendedData) {

            if (state == CONNECTED) {
                mRetryTime = 0;
                boolean isReconnected = false;
                if (!mHasFirstLoginRoom) {


                    if(mLoginCallback != null)
                    {
                        mLoginCallback.onRTCCallback(errorCode);
                        mLoginCallback = null;
                    }

                    QualityManagerProxy.getInstance().setLoginOnFinish();
                    //观众进房默认拉混流 上台后停止拉混流
                    if (mHostID != Long.parseLong(mUserInfo.userID)) {
                        startPlayingAux();
                        startNetworkMonitor();
                    } else {
                        startPublishMV(null);
                        startPublishAudio(null);
                    }
                    mHasFirstLoginRoom = true;
                } else {
                    isReconnected = true;
                }
                if (isReconnected) {
                    L.i(TAG, "detect RTC Room reconnected");
                    if (mEventCallbackList != null) {
                        mEventCallbackList.onRoomReconnected();
                    }
                }
            } else if (state == DISCONNECTED) {

                if (!mHasFirstLoginRoom) {
                    if(mLoginCallback != null)
                    {
                        mLoginCallback.onRTCCallback(errorCode);
                        mLoginCallback = null;
                    }

                    mHasFirstLoginRoom = true;
                }

                if (errorCode == 1002001) {
                    logoutRoom();
                }
                if (errorCode != 0) {
                    if (mRetryTime-- > 0) {
                        loginRoom(roomID, null);
                    }
                }
            }
        }

        @Override
        public void onRecvExperimentalAPI(String content) {
            super.onRecvExperimentalAPI(content);
//            System.out.println("日志:"+content);
            QualityManagerProxy.getInstance().parsingLog(content);
        }

        @Override
        public void onPlayerRecvVideoFirstFrame(String streamID) {
            super.onPlayerRecvVideoFirstFrame(streamID);
            QualityManagerProxy.getInstance().setPlayerStreamOnFirstFrame(streamID);
        }

        @Override
        public void onPlayerRecvAudioFirstFrame(String streamID) {
            super.onPlayerRecvAudioFirstFrame(streamID);
            QualityManagerProxy.getInstance().setPlayerStreamOnFirstFrame(streamID);
        }
        //        @Override
//        public void onPerformanceStatusUpdate(ZegoPerformanceStatus status) {
//            super.onPerformanceStatusUpdate(status);
//            System.out.println("onPerformanceStatusUpdate:"+status.memoryUsageApp);
//        }
    };

    public void stopPlayingExistedStreams() {
        for (String streamId : mUserStreamIDMap.values()) {
            mSDKEngine.stopPlayingStream(streamId);
        }
    }

    public void startPlayingExistedStreams() {
        for (String streamId : mUserStreamIDMap.values()) {
            //设置拉流jitter buffer
            mSDKEngine.setPlayStreamBufferIntervalRange(streamId, 0, 30);
            //拉麦上合唱者流
            QualityManagerProxy.getInstance().setPlayerStreamOnStart(streamId);
            mSDKEngine.startPlayingStream(streamId, null);
        }
    }

    /**
     * 初始化质量报告本地插件
     */
    public void initQualityManager(Context context) {
        QualityManagerProxy.getInstance().init(context.getApplicationContext());
        QualityManagerProxy.getInstance().setAppVersion(SystemUtil.getAppVersion(context.getApplicationContext()));
    }

    public int getHeadphoneMonitorVolumeNum() {
        return mHeadphoneMonitorVolumeNum;
    }

    public int getCaptureVolumeNum() {
        return mCaptureVolumeNum;
    }

    public boolean isLoopbackSwitch() {
        return mLoopbackSwitch;
    }

    public int getVerbPresetPos() {
        return mVerbPresetPos;
    }
}
