package im.zego.ktv.chorus.rtc;


import android.content.Context;
import android.text.TextUtils;

import com.google.gson.Gson;

import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Map;

import im.zego.common.util.L;
import im.zego.ktv.chorus.rtc.callbacks.IRTCCommonCallback;
import im.zego.ktv.chorus.rtc.callbacks.ISDKRequestCallback;
import im.zego.ktv.chorus.rtc.callbacks.IZGKTVGetLyricCallback;
import im.zego.ktv.chorus.rtc.callbacks.IZGKTVRequestMusicCallback;
import im.zego.ktv.chorus.rtc.model.ZGKTVRequestSong;
import im.zego.lyricview.model.ZGKTVLyric;
import im.zego.pitchview.model.ZGKTVPitch;
import im.zego.zegoexpress.ZegoCopyrightedMusic;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicGetKrcLyricByTokenCallback;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicGetLrcLyricCallback;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicGetMusicByTokenCallback;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicGetStandardPitchCallback;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicInitCallback;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicRequestAccompanimentCallback;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicRequestSongCallback;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicSendExtendedRequestCallback;
import im.zego.zegoexpress.constants.ZegoCopyrightedMusicBillingMode;
import im.zego.zegoexpress.entity.ZegoCopyrightedMusicConfig;
import im.zego.zegoexpress.entity.ZegoCopyrightedMusicRequestConfig;
import im.zego.zegoexpress.entity.ZegoUser;


/**
 * 音乐管理
 */
public class ZGKTVCopyrightedMusicManager {

    private static final String TAG = "CopyrightedMusicManager";

    private static volatile ZGKTVCopyrightedMusicManager singleton = null;

    private volatile boolean initialized = false;

    public static ZGKTVCopyrightedMusicManager getInstance() {
        if (singleton == null) {
            synchronized (ZGKTVCopyrightedMusicManager.class) {
                if (singleton == null) {
                    singleton = new ZGKTVCopyrightedMusicManager();
                }
            }
        }
        return singleton;
    }


    private ZegoCopyrightedMusic mCopyrightedMusic = null;
    private ZegoUser mUserInfo = null;

    private Gson gson = new Gson();

    private Map<String,IZGKTVGetLyricCallback> onGetLyricCallbackMap = new HashMap<>();
    /**
     * 初始化
     *
     * @param application
     */
    public void init(@NotNull Context application, IRTCCommonCallback callback) {
        ZegoExpressEngine mSDKEngine = ZegoExpressEngine.getEngine();
        if (initialized || mSDKEngine == null) {
            if(callback != null) {
                callback.onRTCCallback(0);
            }
            return;
        }
        mCopyrightedMusic = mSDKEngine.createCopyrightedMusic();

//        mCopyrightedMusic.setEventHandler(mEventHandler);

        ZegoCopyrightedMusicConfig config = new ZegoCopyrightedMusicConfig();
        config.user = mUserInfo;
//        Log.d(TAG, mUserInfo.userID + "--------" + mUserInfo.userName);
        L.i(TAG,"ZGKTVCopyrightedMusicManager init start: " + System.currentTimeMillis());
        mCopyrightedMusic.initCopyrightedMusic(config, new IZegoCopyrightedMusicInitCallback() {
            @Override
            public void onInitCallback(int i) {
                L.i(TAG,"ZGKTVCopyrightedMusicManager init end: " + System.currentTimeMillis() );
                if (i == 0) {
                    initialized = true;
                    ZGKTVPlayerManager.getInstance().init();
                    L.i(TAG, "Init ZegoCopyrightedMusic success");
                } else {
                    L.i(TAG, "Init ZegoCopyrightedMusic fail, errorCode:%d" + i);
                }
                if(callback != null) {
                    callback.onRTCCallback(i);
                }
            }
        });

    }

    public void uninit(){
        ZegoExpressEngine mSDKEngine = ZegoExpressEngine.getEngine();
        if (!initialized || mSDKEngine == null) {
            initialized = false;
            return;
        }
        mSDKEngine.destroyCopyrightedMusic(mCopyrightedMusic);
        mCopyrightedMusic = null;
        initialized = false;
    }

    public boolean isInitialized(){
        return initialized && (mCopyrightedMusic != null);
    }

    public void setUserID(long userID, @NotNull String userName) {
        mUserInfo = new ZegoUser(String.valueOf(userID), userName);
    }

//        @Override
//        public void onPlaybackStateUpdate(ZegoCopyrightedMusic copyrightedMusic, String resourceID, ZegoCopyrightedMusicPlaybackState state, int errorCode) {
//            super.onPlaybackStateUpdate(copyrightedMusic, resourceID, state, errorCode);
//            if(mPlayerEventHandler != null){
//                mPlayerEventHandler.onPlaybackStateUpdate(copyrightedMusic, resourceID, state, errorCode);
//            }
//        }
//
//        @Override
//        public void onPlaybackProgressUpdate(ZegoCopyrightedMusic copyrightedMusic, String resourceID, long progress) {
//            super.onPlaybackProgressUpdate(copyrightedMusic, resourceID, progress);
//            if(mPlayerEventHandler != null){
//                mPlayerEventHandler.onPlaybackProgressUpdate(copyrightedMusic, resourceID, progress);
//            }
//        }
//
//        @Override
//        public void onLoadProgressUpdate(ZegoCopyrightedMusic copyrightedMusic, String resourceID, float progressRate) {
//            super.onLoadProgressUpdate(copyrightedMusic, resourceID, progressRate);
//            if(mPlayerEventHandler != null){
//                mPlayerEventHandler.onLoadProgressUpdate(copyrightedMusic, resourceID, progressRate);
//            }
//        }



    public ZegoCopyrightedMusic getCopyrightedMusic(){
        return mCopyrightedMusic;
    }

    /**
     * 通过token请求歌曲
     * @param token
     * @param callback
     */
    public void getMusicByToken(String token, IZGKTVRequestMusicCallback callback){
        L.i(TAG,"getMusicByToken");
        mCopyrightedMusic.getMusicByToken(token, new IZegoCopyrightedMusicGetMusicByTokenCallback() {
            @Override
            public void onGetMusicByTokenCallback(int errorCode, String info) {
                L.i(TAG,"getMusicByToken:errorCode:" + errorCode + ":info:" + info);
                if(errorCode == 0){
                    ZGKTVRequestSong requestSong = gson.fromJson(info,ZGKTVRequestSong.class);

                    if(requestSong.getCode() == 0 && requestSong.getData()!= null){
                        ZGKTVRequestSong.DataDTO data = requestSong.getData();
                        String songID = data.getSongId();
                        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
                        if(song == null){
                            song = new ZGKTVCopyrightedSong();
                        }
                        song.setSongID(data.getSongId());
                        L.i(TAG,"song.setSongID:" + errorCode + ":song.getSongID:" + song.getSongID());
                        if(data.getResources() != null && data.getResources().size() > 0){

                            song.setResourceID(data.getResources().get(0).getResourceId());
                            song.setToken(data.getResources().get(0).getShareToken());
                        }
                        song.setKrcToken(data.getKrcToken());
                        song.setDuration(data.getDuration());
                        if(data.getIsAccompany() == 0){
                            song.setHasOriginal(true);
                            song.setHasAccompany(false);
                        }else {
                            song.setHasAccompany(true);

                            if(data.getHasOriginal() == 0) {
                                song.setHasOriginal(false);
                            }else {
                                song.setHasOriginal(true);
                            }

                        }
                        L.i("songId","putSong444 : " + " : : " + song.getLoadState());
                        ZGKTVPlayerManager.getInstance().putSong(songID,song);

                        if(callback != null){
                            callback.onRequestMusicCallback(errorCode,"",song);
                        }
                    }else {
                        if(callback != null){
                            callback.onRequestMusicCallback(requestSong.getCode(),"requestSongErrorCode()",null);
                        }
                    }
                }else {
                    if(callback != null){
                        callback.onRequestMusicCallback(errorCode,"errorCode:" + errorCode,null);
                    }
                }

            }
        });
    }

    /**
     * 请求伴奏
     * 此方法才会返回多音轨文件
     * @param songID
     */
    public void requestAccompaniment(String songID, IZGKTVRequestMusicCallback callback){
        ZegoCopyrightedMusicRequestConfig config = new ZegoCopyrightedMusicRequestConfig();
        config.songID = songID;
        config.mode = ZegoCopyrightedMusicBillingMode.COUNT;

        L.i(TAG,"requestAccompaniment");
        mCopyrightedMusic.requestAccompaniment(config, new IZegoCopyrightedMusicRequestAccompanimentCallback() {
            @Override
            public void onRequestAccompanimentCallback(int errorCode, String info) {
                L.i(TAG,"requestAccompaniment:errorCode:" + errorCode + ":info:" + info);
                if(errorCode == 0){
                    ZGKTVRequestSong requestSong = gson.fromJson(info,ZGKTVRequestSong.class);

                    if(requestSong.getCode() == 0 && requestSong.getData()!= null){
                        ZGKTVRequestSong.DataDTO data = requestSong.getData();
                        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
                        if(song == null){
                            song = new ZGKTVCopyrightedSong();
                        }
                        song.setSongID(data.getSongId());
                        if(data.getResources() != null && data.getResources().size() > 0){

                            song.setResourceID(data.getResources().get(0).getResourceId());
                            song.setToken(data.getResources().get(0).getShareToken());
                        }
                        song.setDuration(data.getDuration());
                        song.setKrcToken(data.getKrcToken());
                        if(data.getIsAccompany() == 0){
                            song.setHasOriginal(true);
                            song.setHasAccompany(false);
                        }else {

                            song.setHasAccompany(true);

                            if(data.getHasOriginal() == 0) {
                                song.setHasOriginal(false);
                            }else {
                                song.setHasOriginal(true);
                            }

                        }
                        L.i("songId","putSong555 : " + " : : " + song.getLoadState());
                        ZGKTVPlayerManager.getInstance().putSong(songID,song);

                        if(callback != null){
                            callback.onRequestMusicCallback(errorCode,"",song);
                        }
                    }else {
                        if(callback != null){
                            callback.onRequestMusicCallback(requestSong.getCode(),"requestSongErrorCode()",null);
                        }
                    }
                }else {
                    if(callback != null){
                        callback.onRequestMusicCallback(errorCode,"errorCode:" + errorCode,null);
                    }
                }
            }
        });
    }

    /**
     * 请求原唱
     * 此方法不会返回多音轨文件
     * @param songID
     */
    public void requestSong(String songID, IZGKTVRequestMusicCallback callback){
        ZegoCopyrightedMusicRequestConfig config = new ZegoCopyrightedMusicRequestConfig();
        config.songID = songID;
        config.mode = ZegoCopyrightedMusicBillingMode.COUNT;
        L.i(TAG,"requestSong");
        mCopyrightedMusic.requestSong(config, new IZegoCopyrightedMusicRequestSongCallback() {
            @Override
            public void onRequestSongCallback(int errorCode, String info) {
                L.i(TAG,"requestSong:errorCode:" + errorCode + ":info:" + info);
                if(errorCode == 0){
                    ZGKTVRequestSong requestSong = gson.fromJson(info,ZGKTVRequestSong.class);

                    if(requestSong.getCode() == 0 && requestSong.getData()!= null){
                        ZGKTVRequestSong.DataDTO data = requestSong.getData();
                        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
                        if(song == null){
                            song = new ZGKTVCopyrightedSong();
                        }
                        song.setSongID(data.getSongId());
                        if(data.getResources() != null && data.getResources().size() > 0){

                            song.setResourceID(data.getResources().get(0).getResourceId());
                            song.setToken(data.getResources().get(0).getShareToken());
                        }
                        song.setDuration(data.getDuration());
                        if(data.getIsAccompany() == 0){
                            song.setHasOriginal(true);
                            song.setHasAccompany(false);
                        }else {
                            song.setHasAccompany(true);

                            if(data.getHasOriginal() == 0) {
                                song.setHasOriginal(false);
                            }else {
                                song.setHasOriginal(true);
                            }

                        }
                        L.i("songId","putSong666 : " + " : : " + song.getLoadState());
                        ZGKTVPlayerManager.getInstance().putSong(songID,song);

                        if(callback != null){
                            callback.onRequestMusicCallback(errorCode,"",song);
                        }
                    }else {
                        if(callback != null){
                            callback.onRequestMusicCallback(requestSong.getCode(),"requestSongErrorCode()",null);
                        }
                    }
                }else {
                    if(callback != null){
                        callback.onRequestMusicCallback(errorCode,"errorCode:" + errorCode,null);
                    }
                }
            }
        });
    }

    /**
     * 获得歌词
     * @param songID
     */
    public void getLyricForSongID(String songID, IZGKTVGetLyricCallback callback){
        L.i(TAG,"getLyricForSongID");
        mCopyrightedMusic.getLrcLyric(songID, new IZegoCopyrightedMusicGetLrcLyricCallback() {
            @Override
            public void onGetLrcLyricCallback(int errorCode, String info) {
                L.i(TAG,"getLyricForSongID:errorCode:" + errorCode + ":info:" + info);
                if(errorCode == 0){
                    ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
                    if(song == null){
                        song = new ZGKTVCopyrightedSong();
                        song.setSongID(songID);
                        L.i("songId","putSong777 : " + " : : " + song.getLoadState());
                        ZGKTVPlayerManager.getInstance().putSong(songID,song);
                    }
                    ZGKTVLyric lyric = null;

                    try {
                        lyric  = gson.fromJson(info,ZGKTVLyric.class);
                    }catch (Exception e){
                    }finally {
                        ZGKTVLyricManager.getInstance().putLyric(songID, lyric);
                    }

                    if(callback != null){
                        callback.onGetLyricCallback(errorCode,"",lyric);
                    }
                }else if (ZGKTVCopyrightedMusicManager.isMusicSDKErrorCode(errorCode)) {

                    ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
                    if(song == null){
                        song = new ZGKTVCopyrightedSong();
                        song.setSongID(songID);
                        L.i("songId","putSong888 : " + " : : " + song.getLoadState());
                        ZGKTVPlayerManager.getInstance().putSong(songID,song);
                    }
                    ZGKTVLyric lyric = null;
                    ZGKTVLyricManager.getInstance().putLyric(songID, lyric);


                    if(callback != null){
                        callback.onGetLyricCallback(0,"",lyric);
                    }

                }  else {
                    if(callback != null){
                        callback.onGetLyricCallback(errorCode,"",null);
                    }
                }
            }
        });
    }



    /**
     * 获取逐字歌词
     * @param songID
     * @param callback
     */
    public void getKrcLyricForSongID(String songID, IZGKTVGetLyricCallback callback) {
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);

        if (song != null) {

            if(TextUtils.isEmpty(song.getKrcToken())){

                L.e(TAG, "getKrcLyricForSongID : KrcToken is NULL :");
                if(callback != null) {
                    callback.onGetLyricCallback(-1, "KrcToken 为空", null);
                }
                return;
            }

            if(callback != null){
                onGetLyricCallbackMap.put(songID,callback);
            }


            mCopyrightedMusic.getKrcLyricByToken(song.getKrcToken(), new IZegoCopyrightedMusicGetKrcLyricByTokenCallback() {
                @Override
                public void onGetKrcLyricByTokenCallback(int errorCode, String lyrics) {
                    L.i(TAG, "getKrcLyricForSongID, errorCode=" + errorCode + ": songID :" + songID + ": lyrics=" + lyrics);
                    ZGKTVLyric lyric = gson.fromJson(lyrics, ZGKTVLyric.class);
                    if(lyric != null && lyric.getLines() != null && lyric.getLines().size() > 0) {
                        lyric.setHasKrcFormat(true);
                    }
                    ZGKTVLyricManager.getInstance().putLyric(songID, lyric);

                    IZGKTVGetLyricCallback izgktvGetLyricCallback = onGetLyricCallbackMap.remove(songID);

                    if (izgktvGetLyricCallback != null) {
                        L.i(TAG,"onGetKrcLyricByTokenCallback:" + callback);
                        izgktvGetLyricCallback.onGetLyricCallback(0, "", lyric);
                    }
                }
            });
        } else {
            L.e(TAG, "getKrcLyricForSongID : songID is NULL :");
            if (callback != null) {
                callback.onGetLyricCallback(-1, "song 为空", null);
            }
        }
    }

    /**
     * 获取标准音高线
     * @param resourceID
     */
    public void getStandardPitch(String resourceID,IZegoCopyrightedMusicGetStandardPitchCallback callback){
        if (resourceID != null) {
            mCopyrightedMusic.getStandardPitch(resourceID, new IZegoCopyrightedMusicGetStandardPitchCallback() {
                @Override
                public void onGetStandardPitchCallback(int errorCode, String pitch) {
                    callback.onGetStandardPitchCallback(errorCode,pitch);
                    L.e(TAG,"onGetStandardPitchCallback : errorCode : " + errorCode);
                    L.e(TAG,"onGetStandardPitchCallback : pitch : " + pitch);
                }
            });
        } else {
            L.e(TAG,"getStandardPitch resourceID is NULL");
        }
    }

    /**
     * 获取标准音高线
     * @param songID
     */
    public void getStandardPitchWithSongID(String songID){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if (song != null) {
            mCopyrightedMusic.getStandardPitch(song.getResourceID(), new IZegoCopyrightedMusicGetStandardPitchCallback() {
                @Override
                public void onGetStandardPitchCallback(int errorCode, String pitch) {

                    ZGKTVPitch zgktvPitch = gson.fromJson(pitch, ZGKTVPitch.class);
                    ZGKTVPitchManager.getInstance().putPitch(songID, zgktvPitch);
                    L.e(TAG,"onGetStandardPitchCallback : errorCode : " + errorCode);
                    L.e(TAG,"onGetStandardPitchCallback : pitch : " + pitch);
                }
            });
        } else {
            L.e(TAG,"getStandardPitch songID is NULL");
        }
    }

    /**
     * 开始打分
     * @param resourceID
     */
    public void startScore(String resourceID){
        if (resourceID != null) {
            L.e(TAG,"startScore:resourceID" + resourceID);
            mCopyrightedMusic.startScore(resourceID,60);
        } else {
            L.e(TAG,"startScore resourceID is NULL");
        }
    }

    /**
     * 暂停打分
     * @param resourceID
     */
    public void pauseScore(String resourceID){
        if (resourceID != null) {
            L.e(TAG,"pauseScore:resourceID" + resourceID);
            mCopyrightedMusic.pauseScore(resourceID);
        } else {
            L.e(TAG,"pauseScore resourceID is NULL");
        }
    }

    /**
     * 恢复打分
     * @param resourceID
     */
    public void resumeScore(String resourceID){
        if (resourceID != null) {
            L.e(TAG,"resumeScore:resourceID" + resourceID);
            mCopyrightedMusic.resumeScore(resourceID);
        } else {
            L.e(TAG,"pauseScore resourceID is NULL");
        }
    }

    /**
     * 结束打分
     * @param resourceID
     */
    public void stopScore(String resourceID){
        if (resourceID != null) {
            L.e(TAG,"stopScore:resourceID" + resourceID);
            mCopyrightedMusic.stopScore(resourceID);
        } else {
            L.e(TAG,"stopScore resourceID is NULL");
        }
    }

    /**
     * 重置打分
     * @param resourceID
     */
    public void resetScore(String resourceID){
        if (resourceID != null) {
            L.e(TAG,"resetScore:resourceID" + resourceID);
            mCopyrightedMusic.resetScore(resourceID);
        } else {
            L.e(TAG,"resetScore resourceID is NULL");
        }
    }

    /**
     * 获取前一句分数
     * @param resourceID
     * @return
     */
    public int getPreviousScore(String resourceID){
        if (resourceID != null) {
            L.e(TAG,"getPreviousScore:resourceID" + resourceID);
            int score = mCopyrightedMusic.getPreviousScore(resourceID);
            L.e(TAG,"getPreviousScore:score:" + score);
            return score;
        } else {
            L.e(TAG,"getPreviousScore resourceID is NULL");
            return -1;
        }
    }

    /**
     * 获取当前音高值
     * @param resourceID
     * @return
     */
    public int getCurrentPitch(String resourceID){

        if (resourceID != null) {
//            L.e(TAG,"getPreviousScore:resourceID:" + resourceID);
            int pitch = mCopyrightedMusic.getCurrentPitch(resourceID);
//            L.e(TAG,"getPreviousScore:pitch1:" + pitch);
            return pitch;
        } else {
            L.e(TAG,"getCurrentPitch resourceID is NULL");
            return -1;
        }

    }

    /**
     * 获取平均分数
     * @param resourceID
     * @return
     */
    public int getAverageScore(String resourceID){
        if (resourceID != null) {
            L.e(TAG,"getAverageScore:resourceID" + resourceID);
            int score =  mCopyrightedMusic.getAverageScore(resourceID);
            L.e(TAG,"getAverageScore:score:" + score);
            return score;
        } else {
            L.e(TAG,"getAverageScore resourceID is NULL");
            return -1;
        }
    }

    /**
     * 获取总分数
     * @param resourceID
     * @return
     */
    public int getTotalScore(String resourceID){
        if (resourceID != null) {
            L.e(TAG,"getTotalScore:resourceID" + resourceID);
            int score = mCopyrightedMusic.getTotalScore(resourceID);
            L.e(TAG,"getTotalScore:score:" + score);
            return score;
        } else {
            L.e(TAG,"getTotalScore resourceID is NULL");
            return -1;
        }
    }

    /**
     * 开始打分
     * @param songID
     */
    public void startScoreWithSongID(String songID){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if (song != null) {
            startScore(song.getResourceID());
        } else {
            L.e(TAG,"startScore song is NULL");
        }
    }

    /**
     * 暂停打分
     * @param songID
     */
    public void pauseScoreWithSongID(String songID){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if (song != null) {
            pauseScore(song.getResourceID());
        } else {
            L.e(TAG,"pauseScore song is NULL");
        }
    }

    /**
     * 恢复打分
     * @param songID
     */
    public void resumeScoreWithSongID(String songID){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if (song != null) {
            resumeScore(song.getResourceID());
        } else {
            L.e(TAG,"pauseScore song is NULL");
        }
    }

    /**
     * 结束打分
     * @param songID
     */
    public void stopScoreWithSongID(String songID){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if (song != null) {
            stopScore(song.getResourceID());
        } else {
            L.e(TAG,"stopScore song is NULL");
        }
    }

    /**
     * 重置打分
     * @param songID
     */
    public void resetScoreWithSongID(String songID){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if (song != null) {
            resetScore(song.getResourceID());
        } else {
            L.e(TAG,"resetScore song is NULL");
        }
    }

    /**
     * 获取前一句分数
     * @param songID
     * @return
     */
    public int getPreviousScoreWithSongID(String songID){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if (song != null) {
            return getPreviousScore(song.getResourceID());
        } else {
            return -1;
        }
    }

    /**
     * 获取平均分数
     * @param songID
     * @return
     */
    public int getAverageScoreWithSongID(String songID){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if (song != null) {
            return getAverageScore(song.getResourceID());
        } else {
            return -1;
        }
    }

    /**
     * 获取总分数
     * @param songID
     * @return
     */
    public int getTotalScoreWithSongID(String songID){
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if (song != null) {
            return getTotalScore(song.getResourceID());
        } else {
            return -1;
        }
    }

    /**
     * 获取缓存大小
     * @return
     */
    public long getCacheSize(){
        return mCopyrightedMusic.getCacheSize();
    }

    /**
     * 清除缓存
     */
    public void clearCache(){
        mCopyrightedMusic.clearCache();
    }


    public <T> void sendExtendedRequest(String url,String reqJson, final ISDKRequestCallback reqCallback){
        mCopyrightedMusic.sendExtendedRequest(url, reqJson, new IZegoCopyrightedMusicSendExtendedRequestCallback() {
            @Override
            public void onSendExtendedRequestCallback(int errorCode, String url, String result) {
                if(reqCallback != null){
                    reqCallback.onResponse(errorCode,url,result);
                }
            }
        });
    }


    public void clearData(){
        onGetLyricCallbackMap.clear();
        ZGKTVPlayerManager.getInstance().clearData();
        ZGKTVLyricManager.getInstance().clearData();
        ZGKTVPitchManager.getInstance().clearData();
    }


    public static boolean isMusicSDKErrorCode(int errorCode){

        if(errorCode >= 1017000 && errorCode <= 1017099){
            return true;
        }
        return false;

    }
    public static String getErrorInfo(int errorCode){
        switch (errorCode){
            case 1002005:
                return "userId 为空";
            case 1002008:
                return "username 为空";
            case 1017000:
                return "输入的 command 参数为空";
            case 1017001:
                return "输入的 params 参数为空";
            case 1017002:
                return "输入的 song_id 参数为空";
            case 1017003:
                return "输入的 song_token 参数为空";
            case 1017004:
                return "输入的 resource_id 参数为空";
            case 1017005:
                return "输入的 start_position 参数无效";
            case 1017006:
                return "输入的 position 参数无效";
            case 1017007:
                return "输入的 Volume 参数无效";
            case 1017008:
                return "输入的 krcToken 参数无效";
            case 1017010:
                return "请求版权服务失败（服务响应body解析失败）";
            case 1017011:
                return "本地磁盘空间不足";
            case 1017012:
                return "正在下载中";
            case 1017013:
                return "资源文件丢失";
            case 1017014:
                return "过期的资源文件";
            case 1017015:
                return "无效的资源文件";
            case 1017016:
                return "当前资源不支持切换音轨";
            case 1017017:
                return "媒体未开始播放";
            case 1017018:
                return "资源ID 未授权";
            case 1017020:
                return "媒体播放器错误";
            case 1017030:
                return "音乐无版权";
            case 1017031:
                return "音乐无词曲权限";
            case 1017032:
                return "非包月会员";
            case 1017033:
                return "没有对应的伴奏";
            case 1017034:
                return "找不到资源";
            case 1017040:
                return "参数非法";
            case 1017041:
                return "appid 不可用";
            case 1017042:
                return "不支持的付费类型";
            case 1017043:
                return "会员按次访问";
            case 1017044:
                return "分享 Token 过期";
            case 1017045:
                return "分享 Token 非法";
            case 1017046:
                return "krcToken 非法";
            case 1017047:
                return "krcToken 过期";
            case 1017048:
                return "获取歌词失败";
            case 1017049:
                return "获取标准音高线失败";
            case 1017096:
                return "版权音乐模块未初始化";
            case 1017097:
                return "系统繁忙";
            case 1017098:
                return "网络错误";
            case 1017099:
                return "可能原因：内部未知错误";
        }
        L.e("KTV","errorCode" + errorCode);
        return "";
    }


}
