package im.zego.ktv.chorus.rtc;

import im.zego.ktv.chorus.rtc.callbacks.IZGKTVCopyrightedSongCallback;
import im.zego.ktv.chorus.rtc.callbacks.IZGKTVCopyrightedSongUpdateListener;
import im.zego.lyricview.model.ZGKTVLyric;

public class ZGKTVCopyrightedSong {

    private String songID;
    private String songName;
    private String resourceID;
    private String krcToken;
    private String token;
    private String singerName;
    private String albumID;
    private String albumName;
    private String albumImageUri;
    private boolean hasAccompany;
    private boolean hasOriginal;
    private long duration;
    private ZGKTVLyric lyric;

    private boolean audioTrackIsAccompany = true;

    private ZGKTVSongState state = ZGKTVSongState.NO_LOAD;
    private ZGKTVSongLoadState loadState = ZGKTVSongLoadState.NO_LOAD;
    private float progressRate;

    private IZGKTVCopyrightedSongCallback mCallback;
    private IZGKTVCopyrightedSongUpdateListener mSongUpdateListener;

    public void load(){
        ZGKTVPlayerManager.getInstance().load(songID);
    }

    public void pause(){
        ZGKTVPlayerManager.getInstance().pause();
    }

    public void resume(){
        ZGKTVPlayerManager.getInstance().resume();
    }

    public void stop(){
        ZGKTVPlayerManager.getInstance().stop();
    }

    public void setAudioOrigin( ){
        if(audioTrackIsAccompany){
            if(hasOriginal) {
                audioTrackIsAccompany = false;
                ZGKTVPlayerManager.getInstance().setAudioOrigin(false);
            }
        }else {
            if(hasAccompany) {
                audioTrackIsAccompany = true;
                ZGKTVPlayerManager.getInstance().setAudioOrigin(true);
            }
        }
    }


    public void playAtPosition(long position){
        ZGKTVPlayerManager.getInstance().play(songID,position);
    }

    public void seekToPosition(long position){
        ZGKTVPlayerManager.getInstance().seekTo(position);
    }

    public ZGKTVLyric getLyric(){
        return ZGKTVLyricManager.getInstance().getLyric(songID);
    }

    public void setLyric(ZGKTVLyric zgktvLyric){
        lyric = zgktvLyric;
        if(mSongUpdateListener != null){
            mSongUpdateListener.onZGKTVCopyrightedSongUpdate(this);
        }
    }


    public String getSongID() {
        return songID;
    }

    public void setSongID(String songID) {
        this.songID = songID;
    }

    public String getSongName() {
        return songName;
    }

    public void setSongName(String songName) {
        this.songName = songName;
        if(mSongUpdateListener != null){
            mSongUpdateListener.onZGKTVCopyrightedSongUpdate(this);
        }
    }

    public String getResourceID() {
        return resourceID;
    }

    public void setResourceID(String resourceID) {
        this.resourceID = resourceID;
        ZGKTVPlayerManager.getInstance().putResourceIDSong(resourceID,songID);
    }

    /**
     * 用于获取版权歌词的token
     * @return
     */
    public String getKrcToken() {
        return krcToken;
    }

    public void setKrcToken(String krcToken) {
        this.krcToken = krcToken;
    }

    public String getToken() {
        return token;
    }

    public void setToken(String token) {
        this.token = token;
    }

    public String getSingerName() {
        return singerName;
    }

    public void setSingerName(String singerName) {
        this.singerName = singerName;
    }

    public String getAlbumID() {
        return albumID;
    }

    public void setAlbumID(String albumID) {
        this.albumID = albumID;
    }

    public String getAlbumName() {
        return albumName;
    }

    public void setAlbumName(String albumName) {
        this.albumName = albumName;
    }

    public String getAlbumImageUri() {
        return albumImageUri;
    }

    public void setAlbumImageUri(String albumImageUri) {
        this.albumImageUri = albumImageUri;
    }

    public boolean isHasAccompany() {
        return hasAccompany;
    }

    public void setHasAccompany(boolean hasAccompany) {
        this.hasAccompany = hasAccompany;
        audioTrackIsAccompany = hasAccompany;
    }

    public boolean isHasOriginal() {
        return hasOriginal;
    }

    public void setHasOriginal(boolean hasOriginal) {
        this.hasOriginal = hasOriginal;
    }

    public long getDuration() {
        return duration;
    }

    public void setDuration(long duration) {
        this.duration = duration;
    }

    public void setCallback(IZGKTVCopyrightedSongCallback callback){
        mCallback = callback;
    }

    public IZGKTVCopyrightedSongCallback getCallback(){
        return mCallback;
    }

    public IZGKTVCopyrightedSongUpdateListener getSongUpdateListener() {
        return mSongUpdateListener;
    }

    public void setSongUpdateListener(IZGKTVCopyrightedSongUpdateListener songUpdateListener) {
        this.mSongUpdateListener = songUpdateListener;
    }

    public ZGKTVSongState getState() {

        return state;
    }

    public void setState(ZGKTVSongState state) {
        this.state = state;
    }

    public ZGKTVSongLoadState getLoadState() {

        return loadState;
    }

    public void setLoadState(ZGKTVSongLoadState state) {
        this.loadState = state;

        if(state == ZGKTVSongLoadState.LOAD_COMPLETE){
            if(mSongUpdateListener != null){
                mSongUpdateListener.onZGKTVCopyrightedSongUpdate(this);
            }
        }

    }

    public float getProgressRate() {
        return progressRate;
    }

    public void setProgressRate(float progressRate) {
        this.progressRate = progressRate;
    }
}
