package im.zego.ktv.chorus.rtc;



import im.zego.common.util.L;
import im.zego.ktv.chorus.rtc.callbacks.IZGKTVCopyrightedSongLoadCallback;

public class ZGKTVSongLoadManager {

    private static final String TAG = "ZGKTVSongLoadManager";

    private IZGKTVCopyrightedSongLoadCallback callback;
    private static volatile ZGKTVSongLoadManager mInstance = new ZGKTVSongLoadManager();


    private ZGKTVSongLoadManager(){
    }


    public static ZGKTVSongLoadManager getInstance() {
        if (mInstance == null) {
            synchronized (ZGKTVSongLoadManager.class) {
                if (mInstance == null) {
                    mInstance = new ZGKTVSongLoadManager();
                }
            }
        }
        return mInstance;
    }

    public void startLoad(String songID){
        L.i(TAG,"startLoad:songID:" + songID);
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if(song != null){
            song.setLoadState(ZGKTVSongLoadState.WAITING);
            if(callback != null){
                callback.onLoadUpdate(songID,song.getLoadState(),song.getProgressRate());
            }
        }

    }

    public void loading(String songID,float progressRate){
        L.i(TAG,"loading:songID:" + songID + ":progressRate:" + progressRate);
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if(song != null) {
            if (progressRate >= 1) {
                song.setLoadState(ZGKTVSongLoadState.LOAD_COMPLETE);
            }else {
                song.setLoadState(ZGKTVSongLoadState.LOADING);
            }
            song.setProgressRate(progressRate);
            if(callback != null){
                callback.onLoadUpdate(songID,song.getLoadState(),song.getProgressRate());
            }
        }
    }

    public void loadError(String songID,int errorCode){
        L.i(TAG,"loadError:songID:" + songID);
        ZGKTVCopyrightedSong song = ZGKTVPlayerManager.getInstance().getSong(songID);
        if(song != null) {
            song.setLoadState(ZGKTVSongLoadState.LOAD_ERROR);
            if(callback != null){
                callback.onLoadUpdate(songID,song.getLoadState(),song.getProgressRate());
            }
        }
    }


    public IZGKTVCopyrightedSongLoadCallback getLoadCallback() {
        return callback;
    }

    public void setLoadCallback(IZGKTVCopyrightedSongLoadCallback callback) {
        this.callback = callback;
    }
}
