package im.zego.ktv.chorus.rtc.callbacks;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;

import im.zego.ktv.chorus.model.HostSEIInfo;
import im.zego.ktv.chorus.model.notify.NotifyCustomBroadcastInfo;
import im.zego.ktv.chorus.model.notify.NotifyOrderListUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyPreOrderUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyRoomStateUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyUserListUpdateInfo;
import im.zego.ktv.chorus.model.notify.NotifyUserStateUpdateInfo;
import im.zego.zegoexpress.constants.ZegoAudioRoute;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoNetworkSpeedTestQuality;
import im.zego.zegoexpress.entity.ZegoNetworkTimeInfo;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoStream;

public interface IRTCEventCallback {
    /**
     * 成员列表变更通知
     */
    void onUserListUpdateNotify( NotifyUserListUpdateInfo notifyInfo);

    /**
     * 成员状态变更通知
     */
    void onUserStateUpdateNotify(NotifyUserStateUpdateInfo notifyInfo);

    /**
     * 房间关闭通知
     */
    void onRoomCloseNotify();

    /**
     * 房间状态通知
     */
    void onRoomStateNotify(NotifyRoomStateUpdateInfo notifyRoomStateInfo);

    /**
     * 借助RTC SDK判断房间断线重连
     */
    void onRoomReconnected();

    /**
     * 声浪信息更新
     * @param userVolumeMap
     */
    void onUserVolumeUpdate(HashMap<String, Float> userVolumeMap);

    /**
     * 音频设备路由变更通知  当有耳机插拔、扬声器和听筒切换等音频路由发生变化时会抛出此回调。
     * @param zegoAudioRoute
     */
    void onAudioRouteChange(ZegoAudioRoute zegoAudioRoute);

    /**
     *网络测速质量回调通知
     * @param zegoNetworkSpeedTestQuality
     */
    void onNetworkSpeedTestQualityUpdate(ZegoNetworkSpeedTestQuality zegoNetworkSpeedTestQuality);

    /**
     * 终端透传广播消息
     * @param notifyCustomBroadcastInfo
     */
    void onCustomBroadCastUpdate(NotifyCustomBroadcastInfo notifyCustomBroadcastInfo);

    /**
     * SEI消息回调
     * @param streamId
     * @param info
     */
    void onPlayerRecvSEI(String streamId, HostSEIInfo info);

    /**
     * 拉流质量更新
     * @param streamID
     * @param quality
     */
    void onPlayerQualityUpdate(String streamID, ZegoPlayStreamQuality quality);

    /**
     * 相同房间内其他用户推的流增加或减少的通知
     * @param roomID
     * @param updateType
     * @param streamList
     * @param extendedData
     */
    void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData);

    /**
     * 推流质量更新
     * @param streamID
     * @param quality
     * @param zegoNetworkTimeInfo
     */
    void onPublisherQualityUpdate(String streamID, ZegoPublishStreamQuality quality, ZegoNetworkTimeInfo zegoNetworkTimeInfo);


    /**
     * 点歌列表变更通知
     * @param notifyOrderListUpdateInfo
     */
    void onOrderListUpdate(NotifyOrderListUpdateInfo notifyOrderListUpdateInfo);

    /**
     * 预点歌通知
     * @param notifyPreOrderUpdateInfo
     */
    void onPreOrderUpdateInfo(NotifyPreOrderUpdateInfo notifyPreOrderUpdateInfo);

}
