package im.zego.ktv.chorus.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import im.zego.ktv.chorus.model.ktv.SongCopyrightInfo;
import im.zego.ktv.chorus.model.ktv.TopSongInfo;

/**
 * 数据过滤
 */
public class DataFilteringUtil {

    /**
     * 歌曲过滤掉无词曲权限的
     *
     * @param responseJsonBean
     * @return
     */
    public static TopSongInfo topSongInfoFilter(TopSongInfo responseJsonBean) {

        List<TopSongInfo.SongsDTO> mData = responseJsonBean.getSongs();

        if (null == mData || mData.size() == 0) {
            return null;
        }

        Iterator<TopSongInfo.SongsDTO> it = mData.iterator();
        TopSongInfo.SongsDTO model = null;
        while (it.hasNext()) {
            model = it.next();
            if (model.getCopyright().getSongLyric() == 0) {
                //词曲权限 非0有权 所以有0就剔除
                it.remove();
            }
        }

        TopSongInfo mTopSongInfo = new TopSongInfo();

        mTopSongInfo.setTotal(responseJsonBean.getTotal());
        mTopSongInfo.setSongs(mData);

        return mTopSongInfo;

    }


    /**
     * 歌曲过滤掉无词曲权限的
     *
     * @param songCopyrightInfo
     * @return
     */
    public static List<String> songCopyrightInfoFilter(SongCopyrightInfo songCopyrightInfo) {

        List<SongCopyrightInfo.CopyrightDTO> mData = songCopyrightInfo.getCopyright();

        if (null == mData || mData.size() == 0) {
            return null;
        }

        List<String> idList = new ArrayList<>();

        Iterator<SongCopyrightInfo.CopyrightDTO> it = mData.iterator();
        SongCopyrightInfo.CopyrightDTO model = null;
        while (it.hasNext()) {
            model = it.next();
            if (model.getSubright().getSongLyric() == 0) {
                //词曲权限 非0有权 所以有0就剔除
                it.remove();
            } else {
                idList.add(model.getSongId());
            }
        }

        return idList;

    }


    /**
     * 获取对应的 channel
     *
     * @param songCopyrightInfo
     * @return
     */
    public static ConcurrentHashMap<String, Integer> songCopyrightChannelFilter(SongCopyrightInfo songCopyrightInfo) {

        List<SongCopyrightInfo.CopyrightDTO> mData = songCopyrightInfo.getCopyright();

        if (null == mData || mData.size() == 0) {
            return null;
        }

        ConcurrentHashMap<String, Integer> channel = new ConcurrentHashMap<>();

        Iterator<SongCopyrightInfo.CopyrightDTO> it = mData.iterator();
        SongCopyrightInfo.CopyrightDTO model = null;
        while (it.hasNext()) {
            model = it.next();
            if (model.getSubright().getSongLyric() == 0) {
                //词曲权限 非0有权 所以有0就剔除
                it.remove();
            } else {
                channel.put(model.getSongId(), model.getSubright().getChannel());
            }
        }

        return channel;

    }


    /**
     * 添加音速达标识
     * @param responseJsonBean
     * @return
     */
    public static TopSongInfo topSongAddChannel(TopSongInfo responseJsonBean, ConcurrentHashMap<String, Integer> songCopyrightChannel) {

        if(responseJsonBean == null){
            return null;
        }

        List<TopSongInfo.SongsDTO> mData = responseJsonBean.getSongs();

        if (null == mData || mData.size() == 0) {
            return null;
        }


        for (int i = 0; i < mData.size(); i++) {
            TopSongInfo.SongsDTO info = mData.get(i);
            TopSongInfo.CopyrightData copyrightData = new TopSongInfo.CopyrightData();
            copyrightData.setChannel(songCopyrightChannel.get(info.getSongId()));
            info.setCopyright(copyrightData);
        }


        TopSongInfo mTopSongInfo = new TopSongInfo();
        mTopSongInfo.setTotal(responseJsonBean.getTotal());
        mTopSongInfo.setSongs(mData);

        return mTopSongInfo;


    }


}
