package im.zego.ktv.custom;

import android.animation.ObjectAnimator;
import android.animation.ValueAnimator;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.View;
import android.view.animation.LinearInterpolator;

import androidx.annotation.Nullable;

import im.zego.common.util.MiscUtils;
import im.zego.ktv.R;

public class KTVLoadMoreTagView extends View {

    private Paint mPaint;

    private float mProgress = 0;
    private int mDuration = 600;
    private Bitmap mBitmapFront;
    private Bitmap mBitmapBehind;
    private int mScope = 24;//dp
    private float mBehindLeft = 3.5f;//dp
    private float mFrontLeft = 10f;//dp

    ObjectAnimator mProgressAnimator;

    public KTVLoadMoreTagView(Context context) {
        this(context,null);
    }

    public KTVLoadMoreTagView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public KTVLoadMoreTagView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        Drawable drawableFront = getResources().getDrawable(R.mipmap.ktv_load_more_front);
        Drawable drawableBehind = getResources().getDrawable(R.mipmap.ktv_load_more_behind);
        mBitmapFront = ((BitmapDrawable) drawableFront).getBitmap();
        mBitmapBehind = ((BitmapDrawable) drawableBehind).getBitmap();

        mPaint = new Paint();
        mPaint.setAntiAlias(true);

        mBehindLeft = MiscUtils.dp2px(context,3.5f);
        mFrontLeft = MiscUtils.dp2px(context,10);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        int scope = MiscUtils.dp2px(getContext(),mScope);
        if(mBitmapFront != null && mBitmapBehind != null) {
            canvas.drawBitmap(mBitmapBehind,mBehindLeft + (getWidth() - scope)/2.0f,getBehindDuration(mProgress) + (getHeight() - scope)/2.0f,mPaint);
            canvas.drawBitmap(mBitmapFront,mFrontLeft + (getWidth() - scope)/2.0f,getFrontDuration(mProgress) + (getHeight() - scope)/2.0f,mPaint);
        }

    }


    private float getFrontDuration(float progress){
        if(mBitmapFront == null){
            return 0;
        }

        int duration = MiscUtils.dp2px(getContext(),mScope) - mBitmapFront.getHeight();
        if(progress <= 0.5){
            return  duration - progress * duration * 2;
        }else {
            return duration - (1- progress) * duration * 2;
        }
    }


    private float getBehindDuration(float progress){
        if(mBitmapBehind == null){
            return 0;
        }

       int duration = MiscUtils.dp2px(getContext(),mScope) - mBitmapBehind.getHeight();
       if(progress <= 0.5){
           return  progress * duration * 2;
        }else {
           return (1- progress) * duration * 2;
       }
    }

    public void startAnim(){
        if(mProgressAnimator == null) {
            mProgressAnimator = ObjectAnimator.ofFloat(this, "progress", 0f, 1f);
            mProgressAnimator.setInterpolator(new LinearInterpolator());
            mProgressAnimator.setDuration(mDuration);
            mProgressAnimator.setRepeatCount(ValueAnimator.INFINITE);
        }
        mProgressAnimator.start();
    }

    public void cancelAnim(){
        if(mProgressAnimator == null)
        {
            return;
        }
        mProgressAnimator.cancel();
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        startAnim();
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        cancelAnim();
    }

    public float getProgress() {
        return mProgress;
    }

    public void setProgress(float progress) {
        this.mProgress = progress;
        invalidate();
    }
}
