package im.zego.ktv.custom;

import android.app.Dialog;
import android.content.Context;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.TextView;

import androidx.annotation.NonNull;

import im.zego.ktv.R;


public class MessageDialog extends Dialog {

    private TextView mTitle;
    private TextView mContent;
    private Button mCancelButton;
    private Button mConfirmButton;

    public MessageDialog(@NonNull Context context, int layoutId) {
        this(context, R.style.KTVMessageDialogStyle, layoutId);
    }

    private MessageDialog(@NonNull Context context, int themeResId, int layoutId) {
        super(context, themeResId);
        init(context, layoutId);
    }

    private void init(Context context, int layoutId) {
        //设置dialog位置
        Window window = getWindow();
        if (window != null) {
            window.setContentView(layoutId);
            mTitle = window.findViewById(R.id.title);
            mContent = window.findViewById(R.id.content);
            mCancelButton = window.findViewById(R.id.cancel);
            mConfirmButton = window.findViewById(R.id.confirm);
        }

        setCancelable(false);
        setCanceledOnTouchOutside(false);

        show();
    }

    public void setMsgTitle(String title) {
        mTitle.setText(title);
    }

    public void setMsgContent(String content) {
        mContent.setText(content);
    }

    public void setRightBtnContent(String content) {
        mCancelButton.setText(content);
    }

    public void setLeftBtnContent(String content) {
        mConfirmButton.setText(content);
    }

    public void setSureListener(View.OnClickListener listener) {
        mConfirmButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                listener.onClick(view);
            }
        });
    }

    public void setCancelListener(View.OnClickListener listener) {
        mCancelButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                listener.onClick(view);
            }
        });
    }

}
