package im.zego.ktv.custom;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapShader;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Shader;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.widget.ImageView;

import androidx.annotation.Nullable;

import im.zego.common.util.MiscUtils;

@SuppressLint("AppCompatCustomView")
public class SongCoverImageView extends ImageView {

    private Paint mPaint;
    private Paint mCirclePaint;
    //圆形图片的半径
    private int mRadius;
    //图片的宿放比例
    private float mScale;

    public SongCoverImageView(Context context) {
        this(context,null);
    }

    public SongCoverImageView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public SongCoverImageView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        mPaint = new Paint();
        mPaint.setAntiAlias(true);


        mCirclePaint = new Paint();

        mCirclePaint.setAntiAlias(true);

    }


    @Override
    protected void onDraw(Canvas canvas) {

        mCirclePaint.setColor(Color.WHITE);
        mCirclePaint.setAlpha(128);
        canvas.drawCircle(mRadius,mRadius,mRadius,mCirclePaint);
        mCirclePaint.setAlpha(256);

        Drawable drawable = getDrawable();
        if (null != drawable) {

            Bitmap bitmap = ((BitmapDrawable) drawable).getBitmap();

            //初始化BitmapShader，传入bitmap对象
            BitmapShader bitmapShader = new BitmapShader(bitmap, Shader.TileMode.CLAMP, Shader.TileMode.CLAMP);
            //计算缩放比例
            mScale = ((mRadius * 2.0f))/ Math.min(bitmap.getHeight(), bitmap.getWidth());

            Matrix matrix = new Matrix();
            matrix.setScale(mScale, mScale);
            bitmapShader.setLocalMatrix(matrix);
            mPaint.setShader(bitmapShader);
            //画圆形，指定好坐标，半径，画笔
            canvas.drawCircle(mRadius, mRadius, mRadius - (MiscUtils.dp2px(getContext(),1)/2.0f), mPaint);



        } else {
//            super.onDraw(canvas);
        }

        mCirclePaint.setColor(Color.WHITE);
        float whiteRadius  = (MiscUtils.dp2px(getContext(),14))/2.0f;
        canvas.drawCircle(mRadius,mRadius,whiteRadius,mCirclePaint);

        mCirclePaint.setColor(Color.parseColor("#FF120326"));
        float blackRadius  = (MiscUtils.dp2px(getContext(),7))/2.0f;
        canvas.drawCircle(mRadius,mRadius,blackRadius,mCirclePaint);

    }


    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
        int size = Math.min(getMeasuredWidth(), getMeasuredHeight());
        mRadius = size / 2;
        setMeasuredDimension(size, size);
    }
}
