package im.zego.ktv.custom;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Color;
import android.util.AttributeSet;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.FrameLayout;
import android.widget.Switch;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import im.zego.ktv.R;

public class SongSwitchView extends FrameLayout {

    @SuppressLint("UseSwitchCompatOrMaterialCode")
    private Switch mSongSwitch;

    private TextView mSongOriginal;
    private TextView mSongAccompany;

    private boolean mHasAccompany;
    private boolean mHasOriginal;

    private CompoundButton.OnCheckedChangeListener mOnCheckedChangeListener;

    public SongSwitchView(@NonNull Context context) {
        this(context,null);
    }

    public SongSwitchView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public SongSwitchView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        View view = View.inflate(context, R.layout.ktv_view_song_switch,this);
        mSongSwitch = view.findViewById(R.id.view_song_switch);
        mSongOriginal = view.findViewById(R.id.view_song_original);
        mSongAccompany = view.findViewById(R.id.view_song_accompany);
//        setEnabled(false);
        setChecked(true);
        mSongSwitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if(mOnCheckedChangeListener != null){
                    mOnCheckedChangeListener.onCheckedChanged(buttonView,isChecked);
                }
            }
        });
    }


    @Override
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        mSongSwitch.setEnabled(enabled);
        refreshState();
    }

    public void setChecked(boolean checked){
        if(!isEnabled()){
            checked = true;
        }

        mSongSwitch.setChecked(checked);
        refreshState();
    }

    public boolean isChecked() {
        return mSongSwitch.isChecked();
    }

    private void refreshState(){
        if(isEnabled()){
            mSongOriginal.setTextColor(Color.parseColor("#FFFFFFFF"));
            mSongAccompany.setTextColor(Color.parseColor("#FFFFFFFF"));
        }else {
            if(isChecked()){
                mSongOriginal.setTextColor(Color.parseColor("#80FFFFFF"));
                mSongAccompany.setTextColor(Color.parseColor("#FFFFFFFF"));
            }else {
                mSongOriginal.setTextColor(Color.parseColor("#FFFFFFFF"));
                mSongAccompany.setTextColor(Color.parseColor("#80FFFFFF"));
            }
        }
    }

    /**
     * 设置
     * @param hasAccompany 是否有伴奏
     * @param hasOriginal 是否有原唱
     */
    public void setSongType(boolean hasAccompany,boolean hasOriginal){
        mHasAccompany = hasAccompany;
        mHasOriginal = hasOriginal;

        if(hasAccompany && hasOriginal){
            setEnabled(true);
        }else {
            setEnabled(false);
        }
        if(hasAccompany){
            setChecked(true);
        }else {
            setChecked(false);
        }
    }

    public void setOnCheckedChangeListener( CompoundButton.OnCheckedChangeListener listener) {
        mOnCheckedChangeListener = listener;
    }




}
