package im.zego.ktv.utils;

import android.widget.ImageView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;

import java.util.Random;

import im.zego.ktv.R;

public class GlideImageLoader {

    private int[] res = {
            R.mipmap.ktv_user1, R.mipmap.ktv_user2, R.mipmap.ktv_user3, R.mipmap.ktv_user4, R.mipmap.ktv_user5,
            R.mipmap.ktv_user6, R.mipmap.ktv_user7, R.mipmap.ktv_user8, R.mipmap.ktv_user9, R.mipmap.ktv_user10,
            R.mipmap.ktv_user11, R.mipmap.ktv_user12, R.mipmap.ktv_user13, R.mipmap.ktv_user14, R.mipmap.ktv_user15,
            R.mipmap.ktv_user16};


    /**
     * 单例
     */
    private static volatile GlideImageLoader mInstance;

    private GlideImageLoader() {
    }

    public static GlideImageLoader getInstance() {
        if (mInstance == null) {
            synchronized (GlideImageLoader.class) {
                if (mInstance == null) {
                    mInstance = new GlideImageLoader();
                    return mInstance;
                }
            }
        }
        return mInstance;
    }

    /**
     * 在一个imageView里面异步展示一个图片
     *
     * @param resourceId
     * @param imageView
     */
    private void displayImage(int resourceId, ImageView imageView) {
        Glide.with(imageView.getContext())
                .load(resourceId)
                .dontAnimate()
                .circleCrop()
                .diskCacheStrategy(DiskCacheStrategy.NONE) // 设置缓存的策略
                .into(imageView);
    }


    public void displayImageByAvatar(String avatar, ImageView imageView) {
        try {
            int a = Integer.parseInt(avatar);
            if (a >= 0 && a < res.length) {
                displayImage(res[a], imageView);
            } else {
                displayImage(res[0], imageView);
            }
        } catch (Exception e) {
            displayImage(res[0], imageView);
        }
    }

    /**
     * 获取图片
     *
     * @param avatar
     * @return
     */
    public int getImageByAvatar(String avatar) {
        int resourceId = res[0];
        try {
            int a = Integer.parseInt(avatar);
            if (a >= 0 && a < res.length) {
                resourceId = res[a];
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return resourceId;
    }

    public String getAvatar() {
        //nextInt 【0，16）
        int avatar = new Random().nextInt(16);
        return String.valueOf(avatar);
    }

}
