package im.zego.ktv.utils;


import android.content.res.Resources;
import android.os.Build;
import android.os.Handler;
import android.os.Looper;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import im.zego.common.application.ZegoApplication;
import im.zego.ktv.R;


public class ToastUtil {

    private static final Handler toastHandler = new Handler(Looper.getMainLooper());

    public enum ToastMessageType {
        NORMAL, WARN;
    }

    public static void showNormalToast(String message) {
        showToast(ToastMessageType.NORMAL,message);
    }

    public static void showNormalToast(int res) {
        showToast(ToastMessageType.NORMAL,ZegoApplication.getAppContext().getString(res));
    }

    public static void showWarnToast(String message) {
        showToast(ToastMessageType.WARN,message);
    }

    public static void showWarnToast(int res) {
        showToast(ToastMessageType.WARN,ZegoApplication.getAppContext().getString(res));
    }

    private static void showToast(ToastMessageType type, String message)
    {
        if (ZegoApplication.isOnBackground() && Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            toastHandler.post(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(ZegoApplication.getAppContext(), message, Toast.LENGTH_LONG).show();
                }
            });
        } else {
            toastHandler.post(new Runnable() {
                @Override
                public void run() {
                    if(type == ToastMessageType.NORMAL)
                    {
                        showColorToast(ToastMessageType.NORMAL, message);
                    }else {
                        showColorToast(ToastMessageType.WARN, message);
                    }

                }
            });
        }
    }

    private static void showColorToast(ToastMessageType type, String message) {
        View view = LayoutInflater.from(ZegoApplication.getAppContext()).inflate(R.layout.ktv_view_top_toast, null);
        TextView mTvToastMassage = (TextView) view.findViewById(R.id.tv_toast_message);
        mTvToastMassage.setText(message);
        if (type == ToastMessageType.NORMAL) {
            mTvToastMassage.setBackgroundColor(ZegoApplication.getAppContext().getResources().getColor(R.color.ktv_bg_top_toast_normal));
        } else {
            mTvToastMassage.setBackgroundColor(ZegoApplication.getAppContext().getResources().getColor(R.color.ktv_bg_top_toast_warn));
        }


        Resources resources = ZegoApplication.getAppContext().getResources();
        DisplayMetrics dm = resources.getDisplayMetrics();
        int width = dm.widthPixels;
        mTvToastMassage.setWidth(width);

        Toast toast = new Toast(ZegoApplication.getAppContext());
        toast.setGravity(Gravity.TOP | Gravity.FILL_HORIZONTAL, 0, 0);
        toast.setDuration(Toast.LENGTH_LONG);
        toast.setView(view);
        toast.setMargin(0,0);
        toast.show();
    }


}
