package im.zego.pitchview;

import android.graphics.Color;

import java.util.ArrayList;
import java.util.List;

import im.zego.pitchview.model.MusicPitch;
import im.zego.pitchview.model.ZGKTVPitch;

public class ZegoPitchViewHelper {

    /**
     * 将音高线原始数据模型转为控件数据模型
     * @param zgktvPitch 音高线原始model
     */
    public static List<MusicPitch> parsePitch(ZGKTVPitch zgktvPitch) {
        List<MusicPitch> dataList = new ArrayList<>();
        if(zgktvPitch != null && zgktvPitch.getCode() == 0 && zgktvPitch.getData() != null && zgktvPitch.getData().getPitch() != null){
            List<ZGKTVPitch.DataModel.PitchModel> pitchModelList = zgktvPitch.getData().getPitch();
            for(ZGKTVPitch.DataModel.PitchModel model : pitchModelList){
                if(model != null) {
                    dataList.add(new MusicPitch(model.getBeginTime(), model.getDuration(), model.getValue()));
                }
            }
        }
        return dataList;
    }

    public static ZegoPitchViewUIConfigBuilder getConfigBuilder() {
        return new ZegoPitchViewUIConfigBuilder();
    }

    public static class ZegoPitchViewUIConfigBuilder {

        private int standardPitchColor = Color.parseColor("#FF5D3B94");  //默认音高线颜色
        private int hitPitchColor = Color.parseColor("#FF3751");  // 击中音高线颜色
        private int pitchIndicatorColor = Color.parseColor("#FFFFFF");  // 音调指示器颜色
        private int staffColor = Color.parseColor("#33FFFFFF"); //五线谱横线颜色
        private int verticalLineColor = Color.parseColor("#FFA87BF1"); //竖线颜色
        private int scoreTextColor = Color.WHITE; //分数文本颜色

        /**
         * 设置默认音高线颜色
         */
        public ZegoPitchViewUIConfigBuilder setStandardPitchColor(int standardPitchColor) {
            this.standardPitchColor = standardPitchColor;
            return this;
        }

        /**
         * 设置击中音高线颜色
         */
        public ZegoPitchViewUIConfigBuilder setHitPitchColor(int hitPitchColor) {
            this.hitPitchColor = hitPitchColor;
            return this;
        }

        /**
         * 设置音调指示器颜色
         */
        public ZegoPitchViewUIConfigBuilder setPitchIndicatorColor(int pitchIndicatorColor) {
            this.pitchIndicatorColor = pitchIndicatorColor;
            return this;
        }

        /**
         * 设置五线谱横线颜色
         */
        public ZegoPitchViewUIConfigBuilder setStaffColor(int staffColor) {
            this.staffColor = staffColor;
            return this;
        }

        /**
         * 设置竖线颜色
         */
        public ZegoPitchViewUIConfigBuilder setVerticalLineColor(int verticalLineColor) {
            this.verticalLineColor = verticalLineColor;
            return this;
        }

        /**
         * 设置分数文本颜色
         */
        public ZegoPitchViewUIConfigBuilder setScoreTextColor(int scoreTextColor) {
            this.scoreTextColor = scoreTextColor;
            return this;
        }

        public ZegoPitchViewUIConfig build(){
            ZegoPitchViewUIConfig config = new ZegoPitchViewUIConfig();
            config.setStandardPitchColor(standardPitchColor);
            config.setHitPitchColor(hitPitchColor);
            config.setPitchIndicatorColor(pitchIndicatorColor);
            config.setStaffColor(staffColor);
            config.setVerticalLineColor(verticalLineColor);
            config.setScoreTextColor(scoreTextColor);
            return config;
        }
    }

}
